<?PHP
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This file is part of Travelsized CMS
 *              A content management system with modules, based on wiki syntax
 *
 * Author: Dan Jensen <admin@leinir.dk>
 * Copyright 2003/2004
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The GNU General Public License is available at: http://www.gnu.org/copyleft/
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

global $setup_folder, $i18nstrings, $page_help, $permissionlevels_array; // Used for if the language changes later

// Import i18n even if language is English - people might want to change strings!
if( file_exists( "$setup_folder/i18n/$language/i18n.inc" ) )
	include( "$setup_folder/i18n/$language/i18n.inc" );

//Run through the modules and get their i18n, if available
$fp = opendir("$setup_folder/modules");
while ($file = readdir($fp)) {
	if( substr( $file, 0, 1 ) != "." )
		if( file_exists( "$setup_folder/modules/$file/i18n.$language.inc" ) )
			include( "$setup_folder/modules/$file/i18n.$language.inc" );
}
closedir($fp);

// Help {
$page_help["//<br />
/<br />
txt1<br />
txt2<br />
/<br />
txt3<br />
txt4<br />
txt5<br />
//"] = i18n("Will produce this table:") . "<table><tr><td>txt1</td><td>txt2</td></tr><tr><td>txt3</td><td>txt4</td><td>txt5</td></tr></table>";
$page_help["* txt1\n** txt2\n# txt3"] = i18n("Will produce three list items: txt1 will be first level in a bulleted list, txt2 will be second level in a bulleted list, and txt3 will be first level in an ordered (numbered) list");
$page_help["{txt1||txt2||txt3||txt4}"] = i18n("Will produce an image for the uploaded file txt1, and the picture will be given the title txt2. The picture will be scaled to the width txt3, and floated to txt4 (correct values are left and right)");
$page_help["{txt1||txt2||txt3}"] = i18n("Will produce an image for the uploaded file txt1, and the picture will be given the title txt2. The picture will be floated to txt3 (correct values are left and right)");
$page_help["{txt1||txt2}"] = i18n("Will produce an image for the uploaded file txt1, and the picture will be given the title txt2 (this will make most browsers show a tooltip)");
$page_help["{{txt1||txt2}}"] = i18n("Will produce a link to an uploaded file txt1 with the title txt2");
$page_help["%%txt1||txt2%%"] = i18n("Will produce a link to the profile page for the user with id txt1, showing it as txt2");
$page_help["%%txt1%%"] = i18n("Will produce a link to the profile page for the user with id txt1, replacing txt1 with the user's full name");
$page_help[":txt1"] = i18n("Will put txt1 in a preformatted format, for example for showing code");
$page_help[">>txt1<br />||txt2"] = i18n("Right aligns txt1 and centers txt2");
$page_help["//txt1//<br />**txt2**<br />__txt3__"] = i18n("These will make the text txt1 emphasized (italic), txt2 bold and txt3 underlined. Please note that emphasized text cannot contain links.");
$page_help["!txt1<br />
!!txt2<br />
!!!txt3<br />
!!!!txt4<br />
!!!!!txt5"] = i18n("When written as the first thing on a line, this will make the line into a header of the level indicated by the number of exclamation marks");
$page_help["[[txt1||txt2||{txt3||txt4||txt5}]]<br />
((txt1||txt2||{txt3||txt4||txt5}))"] = i18n("Will produce links as explained above, but with an image showing in stead (as shown below)");
$page_help["[[txt1||txt2||txt3]]"] = i18n("Will produce a link to an external resource txt1 (for example http://uberghey.sf.net/), but show the link at txt3, and with the tooltip txt2");
$page_help["[[txt1||txt2]]"] = i18n("Will produce a link to an external resource txt1 (for example http://uberghey.sf.net/), but show the link as txt2");
$page_help["[[txt1]]"] = i18n("Will produce a link to an external resource txt1 (for example ##0##)", array("http://leinir.dk/travelsized/"));
$page_help["(((txt1||txt2)))"] = i18n("Will create a description link, where the text shown would be txt1, and hovering over it with the mouse would show txt2");
$page_help["((txt1||txt2))"] = i18n("Will produce a link to a page with the title txt1, but show the link as txt2");
$page_help["((txt1))"] = i18n("Will produce a link to a page with the title txt1");

$page_help = array_reverse($page_help);
// Help }

// This really ought to be in auth.php, but we need the i18n function for it... so here it is
$permissionlevels_array = array(
			0 => i18n("Everybody"),
			1 => i18n("Registered users only"),
			2 => i18n("Registered users, except blacklisted"),
			3 => i18n("Friends only"),
			4 => i18n("Noone, except administrator"));	

?>