<?PHP
class blogger_entries
{
	var $entries;                     ///< Array of references to entries in users, sorted by timestamp (and then by username)...
	var $users;                       ///< Array of blogger_user instances fitting the constructor's description of entries to load
	var $categories;                  ///< Array of categories - if null, show everything
	var $edit = false;                ///< Boolean describing wether we are in edit mode or not
	var $limit = 10;                  ///< The pagination limit
	var $limitTiny = null;            ///< From which item to start making entries tiny - if null, none are tiny
	
	// users == null -> all users
	// categories == null -> everything
	function blogger_entries( $users = null, $categories = null )
	{
		if( is_array( $users ) )
			$tempUsers = $users;
		else
			$tempUsers = allUsersArray();
		foreach( $tempUsers as $key => $value )
			$this->users[$value] = $key;
		
		$this->categories = $categories;
		
		$this->load();
	}
	
	function load()
	{
		// (re)initialise instances of all users
		$instances = array();
		foreach( $this->users as $key => $value )
			$instances[$key] = new blogger_user( &$this, $key );
		$this->users = $instances;
		
		// Get references to entries out of all users and put them into $this->entries["timestamp/username"]
		$this->entries = array();
		foreach( $this->users as $key => $value )
		{
			foreach( $this->users[$key]->entries as $key2 => $value2 )
			{
				$entryKey = $value2->timestamp . "/" . $key;
				$this->entries[$entryKey] = &$this->users[$key]->entries[$key2];
			}
		}
		krsort($this->entries);
	}
	
	/**
	 * Render a feed for the blog entries contained in the current instance
	 *
	 * @param    type                Which type of feed to create. Defaults to RSS2, accepts any type accepted by Feed
	 *
	 * @return   The rendered feed according to type
	 */
	function renderFeed( $type = null )
	{
		global $page_title;
		$title = $page_title;
		
		$link = thisPageURL();
		
		if( count( $this->users ) == count( allUsersArray() ) )
			$theUsers = i18n("all users, ");
		else
		{
			foreach( $this->users as $key => $value )
				$theUsers .= $value->username . ", ";
		}
		
		if( $this->categories == null )
			$theUsers = i18n("all categories, ");
		else
		{
			foreach( $this->categories as $key => $value )
				$theCategories .= "$value, ";
		}
		
		$description = i18n("A syndication of the entries by ##0##in the category or categories ##1##on ##2##", array( $theUsers, $theCategories, $title ) );
		
		$feed = new Feed( $title, $link, $description, time() );
		
		$i = 0;
		foreach( $this->entries as $key => $entry )
		{
			$feed->addItem( $entry->getFeedItem() );
			if( $i++ > $this->limit )
				break;
		}
		
		return $feed->render( $type );
	}
	
	/**
	 * Render the entries for an optional user and ditto timestamp
	 *
	 * @param    username          Optional. If present, render the user in question
	 * @param    timestamp         Optional. If present, render the blog entry with the user above and the timestamp in question
	 *
	 * @return   The rendered data
	 */
	function render( $username = null, $timestamp = null )
	{
		$renderedData = "";
		if( $username != null )
		{
			// No need to check for null here as that is also what's checked the other end
			$renderedData = $this->users[$username]->render($timestamp);
		}
		else
		{
			if( count( $this->entries ) > 0 )
			{
				$entryCount = 0;
				$shownEntry = 0;
				$currentPage = array_key_exists( "currentpage", $_REQUEST) ? $_REQUEST["currentpage"] : 0;
				$lowerBound = $currentPage * $this->limit;
				$upperBound = $lowerBound + $this->limit;
				// Loop through all entries...
				foreach( $this->entries as $key => $value )
				{
					if( $value->published )
					{
						if( $lowerBound <= $entryCount && $entryCount < $upperBound )
						{
							$shownEntry++;
							if( $this->limitTiny !== null && $shownEntry < $this->limitTiny )
								$renderedData .= $value->renderTiny();
							else
								$renderedData .= $value->renderBlock();
						}
						$entryCount++;
					}
				}
			}
			if( $renderedData == "" )
				$renderedData = renderInformationBox( i18n("No entries"), i18n("There are no published blog entries available.") );
			else
				$renderedData = renderNavigator($entryCount, $currentPage, $lowerBound, $upperBound, $this->limit) . $renderedData;
		}
		return $renderedData;
	}
}
?>