<?PHP
/**
 * Dialog for editing a blog entry
 */
class blogger_entryEditor
{
	var $entry;              ///< Reference to the instance of blogger_entry we are editing
	
	/**
	 * Creates a new instance of the blogger entry editor for the entry passed
	 *
	 * @param    entry            The blogger_entry instance that the editor should work on
	 */
	function blogger_entryEditor( $entry )
	{
		$this->entry = &$entry;
	}
	
	function render()
	{
		global $permissionlevels_array, $userinfo_folder, $blogcategories;
		
		$availableCategories = array();
		
		$usercategories = $this->entry->parentUser->getParam("categories");
		foreach( $blogcategories as $key => $value );
			$availableCategories["system($value)"] = array( "title" => i18n("##0## (system category)", array( i18n( $value ) ) ), "description" => "" );
		
		foreach( $usercategories as $key => $value );
			$availableCategories["user($value)"] = array( "title" => $value, "description" => "" );
		
		$theDefaultCategories = array();
		$defaultCategories = $this->entry->parentUser->getParam("defaultCategories");
		foreach( $defaultCategories["system"] as $key => $value )
			$theDefaultCategories["system($key)"] = $value;
		foreach( $defaultCategories["user"] as $key => $value )
			$theDefaultCategories["user($key)"] = $value;
		
		$dialog = new optionsPanel( "bloggerEntry" );
		
		$dialog->showHeader = false;
		$dialog->saveTitle = i18n("Save");
		$dialog->addCommand( i18n("Preview"), "preview" );
		$dialog->revertTitle = i18n("Discard changes");
		
		$dialog->addOption( i18n("Title"), i18n("The title of the blog entry. This is what is shown in listings when no content is shown"), $this->entry->title, "title", "text" );
		$dialog->addOption( i18n("Published"), i18n("Wether or not to show this entry to the world - until the entry is published, it does not show up in any lists for anyone but the author"), $this->entry->published, "published", "checkbox" );
		$dialog->addOption( i18n("Time"), i18n("The time the entry was created."), $this->entry->timestamp, "timestamp", "datetime" );
		$dialog->addOption( i18n("Personal"), i18n("Is this entry personal?"), $this->entry->personal, "personal", "checkbox" );
		$dialog->addOption( i18n("Viewing allowance level"), i18n("Who, except for the owner, is able to view this entry"), $this->entry->viewLevel, "viewLevel", "select", $permissionlevels_array );
		$dialog->addOption( i18n("Commenting allowance level"), i18n("Who, except for the owner, is able to comment on this entry"), $this->entry->commentLevel, "commentLevel", "select", $permissionlevels_array );
		$dialog->addOption( i18n("Categories"), i18n("What categories do you wish this entry to show up under?"), $theDefaultCategories, "categories", "multicheck", $availableCategories );
		$dialog->addOption( i18n("Content"), i18n("The content of the entry"), $this->entry->content, "content", "pagedata" );
		
		$renderedData = "";
		
		if( $dialog->submitted )
		{
			if( $this->entry->lock() )
			{
				// Reload the entry's data...
				$this->entry->load();
				$this->entry->title = $dialog->getValue("title");
				$this->entry->published = $dialog->getValue("published");
				$this->entry->personal = $dialog->getValue("personal");
				$this->entry->viewLevel = $dialog->getValue("viewLevel");
				$this->entry->commentLevel = $dialog->getValue("commentLevel");
				$this->entry->categories = $dialog->getValue("categories");
				$this->entry->content = $dialog->getValue("content");
				// This needs to be handled - don't leave old entries behind! (in both subscriptions and data layer)
				$timestampChanged = ( $this->entry->timestamp != $dialog->getValue("timestamp") ) ? $this->entry->timestamp : 0;
				$this->entry->timestamp = $dialog->getValue("timestamp");
				
				if( $dialog->submitted == "preview" )
				{
					// Make sure the user knows their data is NOT saved
					$renderedData .= renderWarningBox( i18n("Not saved"), i18n("Please note that this is only a preview and that the entry is NOT saved.\nThe preview shows the listing version of the entry, and the full version of the entry. If there is a horizontal ruler in the content, it further shows the block version of the entry.") );
					// Show preview - if text includes "\n---\n" then we should show the block version in addition to the full entry and the tiny version
					$renderedData .= $this->entry->render( false, true );
					if( strpos( $this->entry->content, "---" ) > 0 )
						$renderedData .= $this->entry->render(true);
					$renderedData .= $this->entry->render();
				}
				else
				{
					// Make sure we don't leave old, stale entries behind...
					if( $timestampChanged > 0 )
					{
						// ...in subscriptions
						///FIXME
						// ...on data layer
						unlink( $this->entry->filename );
						$this->entry->filename = $userinfo_folder . "/" . $this->entry->username . "/blogger/" . $this->entry->timestamp . ".php";
						// Just to mane sure that we don't run into duplicate entries on the same day...
						while( file_exists( $this->entry->filename ) )
						{
							// Increase by one second if it already exists...
							$dialog->options["timestamp"]["current"]++;
							$this->entry->timestamp = $dialog->getValue("timestamp");
							$this->entry->filename = $userinfo_folder . "/" . $this->entry->username . "/" . $this->entry->timestamp . ".php";
						}
					}
					
					// Save the entry to disk
					$this->entry->save();
					// then redirect back to editing overview
					header( "Location: " . globalIDtoURL("user/" . $this->entry->username . "/blogger/edit") );
				}
				$this->entry->unlock();
			}
			else
				$renderedData .= renderErrorBox( i18n("Not locked!"), i18n("The entry could not be locked! Please try again.") );
		}
		
		$renderedData .= $dialog->render();
		
		return $renderedData;
	}
}
?>