<?PHP
require_once( "$module_folder/lastrss/lastRSS.php" );

class blogger_user
{
	var $parentEntries;               ///< Reference to the containing blogger_entries instance
	var $username;                    ///< The username of the user to get entries and such for
	
	var $entries;                     ///< Array of blogger_entry instances fitting the constructor's description of entries to load, sorted by timestamp
	
	var $externalCache;               ///< Array containing the checking cache for external feeds/entries
	
	// Settings
	var $allowsviewpersonal;
	var $viewlevel;
	var $commentlevel;
	var $personal;
	var $categories;
	var $defaultCategories;
	
	function blogger_user( $entries, $username )
	{
		$this->parentEntries = &$entries;
		$this->username = $username;
		
		$this->load();
	}
	
	function load()
	{
		global $userinfo_folder, $username;
		$this->entries = array();
		// cycle through all files in user's blog dir
		if( file_exists( $userinfo_folder . "/" . $this->username . "/blogger" ) )
		{
			$fp = opendir( $userinfo_folder . "/" . $this->username .  "/blogger" );
			while ($file = readdir($fp))
			{
			// create blogger_entry instances for each file, but only if they're called somethingorother.php (then they're not a lock file and not a directory entry)
				$suffix = substr($file, -4);
				$title = substr($file, 0, -4);
				if( $suffix == ".php" )
				{
					$theEntry = new blogger_entry( &$this, $title );
					if( $theEntry->userCanView( $username ) )
					{
						$this->entries[$title] = $theEntry;
					}
					else if( $theEntry->personal == false || userAllows( $this->username, $this->getParam("allowsviewpersonal") ) )
					{
						$showEntry = false;
						if( $this->parentEntries->categories == null )
							$showEntry = true;
						else
						{
							foreach( $this->parentEntries->categories as $key => $value )
								if( in_array( $value, $theEntry->categories ) )
									$showEntry = true;
						}
						
						if( $showEntry )
							$this->entries[$title] = $theEntry;
					}
				}
			}
			closedir($fp);
		}
		
		$this->fetchExternal();
		krsort( $this->entries );
	}
	
	function fetchExternal()
	{
		global $userinfo_folder, $module_folder;
		$timelapse = 900; // Minimum a quarter of an hour between each update
		$oldTime = $this->getParam( "oldTime" );
		
		if( $oldTime == null || time() - $oldTime > $timelapse )
		{
			$update = false;
			// Create lastRSS object
			$rss = new lastRSS;
			
			// Set cache dir and cache time limit (1200 seconds)
			// (don't forget to chmod cahce dir to 777 to allow writing)
			$rss->cache_dir = "$module_folder/lastrss/temp";
			$rss->cache_time = 1200;
			
			// Update the external cache
			$this->getParam("externalCache");
			$external = $this->getParam("external");
			foreach( $external as $key => $entry )
			{
				if( $rs = $rss->get( $entry ) )
				{
					if( is_array( $rs["items"] ) )
					{
						foreach( $rs["items"] as $item )
						{
							// Cache items by way of their URL ($item["link"])
							if( ! array_key_exists( $item["link"], $this->externalCache[$entry] ) )
							{
								$newEntry = new blogger_entry( &$this );
								$newEntry->title = $item["title"];
								// Should probably add the link (in a sort of "This came from here (link)" kind of way)
								$newEntry->content = $item["description"];
								
								// If there is no timestamp - we fix it...
								if( $item["pubDate"] == false )
								{
									$timestamp = time();
									while( array_key_exists( $timestamp, $this->entries ) )
										$timestamp++;
								}
								else
									$timestamp = strtotime( $item["pubDate"] );
								$newEntry->timestamp = $timestamp;
								
								$newEntry->personal = $this->getParam("personal");
								$newEntry->viewLevel = $this->getParam("viewlevel");
								$newEntry->commentLevel = $this->getParam("commentlevel");
								$newEntry->categories = $this->getParam("defaultCategories");
								$newEntry->published = true;
								
								$newEntry->createFilename();
								$newEntry->save();
								$this->entries[$newEntry->timestamp] = $newEntry;
								// Let's futureproof this a bit - we might want to prune this by added-time at a later point in time, who knows ;)
								$this->externalCache[$entry][$item["link"]] = time();
								$updated = true;
							}
						}
					}
				}
			}
			$this->setParam( "oldTime", time() );
			$this->setParam( "externalCache", $this->externalCache );
			if( $updated )
				header( "Location: " . thisPageURL() );
		}
	}
	
	function save()
	{
		// Dummy entry... we don't actually have anything saveable per se...
	}
	
	function getParam( $paramName )
	{
		if( $this->$paramName == null )
		{
			$this->$paramName = getUserInfo( $this->username, "blogger_" . $paramName );
			if( $paramName == "categories" || $paramName == "external" || $paramName == "defaultCategories" || $paramName == "externalCache" )
			{
				$this->$paramName = unserialize( $this->$paramName );
				if( !is_array( $this->$paramName ) )
					$this->$paramName = array();
			}
			if( $this->$paramName == i18n("No ##0## information\n", array( "blogger_" . $paramName ) ) )
				$this->$paramName = null;
		}
		return $this->$paramName;
	}
	
	function setParam( $paramName, $value )
	{
		$this->$paramName = $value;
		saveUserInfo( $this->username, "blogger_" . $paramName, $value );
	}
	
	function getGlobalID( $aswhat = "view" )
	{
		if( $aswhat == "view" )
			$aswhat = "";
		else
			$aswhat = "/" . $aswhat;
		
		return "user/" . $this->username . "/blogger" . $aswhat;
	}
	
	function renderEditor()
	{
		global $globalID;
		// user/(username)/blogger/edit/(section)/(item)/(action)
		$splitID = explode( "/", $globalID );
		switch( $splitID[4] )
		{
		// section: options
		case "options":
			$dialog = new blogger_userOptions( &$this, $splitID[5], $splitID[6] );
			if( array_key_exists( 7, $splitID ) )
				$dialog->item = $splitID[7];
			$renderedContent = $dialog->render();
			break;
		// section: entry (and none)
		case "entry":
		default:
			//  action: new
			if( $splitID[5] == "new" )
			{
				$entry = new blogger_entry( &$this );
				$entry->title = i18n("New entry");
				$entry->content = i18n("Enter content for your blog entry here.");
				$entry->personal = $this->getParam("personal");
				$entry->viewLevel = $this->getParam("viewlevel");
				$entry->commentLevel = $this->getParam("commentlevel");
				$entry->categories = $this->getParam("defaultCategories");
				$entry->save();
				header( "Location:" . globalIDtoURL( $this->getGlobalID( "edit/entry/" . $entry->timestamp ) ) );
			}
			//  otherwise pass on to blogger_entry instance
			else if( is_numeric( $splitID[5] ) )
			{
				$entry = &$this->entries[$splitID[5]];
				if( $entry != null )
					$renderedContent = $entry->renderEditor();
				else
					$renderedContent = renderInformationBox( i18n("Error editing!"), i18n("An error occured while attempting to edit the new entry. Please press your browser's back button, refresh the page, and try again (or edit the new, empty entry).") );
			}
			// ARGH! BAIL! (or, rather, return back up one level...)
			else if( $splitID[5] == "entry" )
			{
				header( "Location: " . $this->getGlobalID("edit") );
			}
			// Just list entries in block format...
			else
			{
				if( count($this->entries) < 1 )
				{
					$renderedContent = renderInformationBox( i18n("No entries"), i18n("There are no blog entries to show. Please click on ##0## to create a new entry, or go to the ##1## dialog to change your options before you begin", array( drawCommand( i18n("New entry"), i18n("Create a new blog entry"), globalIDtoURL( $this->getGlobalID("edit/entry/new") ) ), drawCommand( i18n("Options"), i18n("Change the options for the blogging module"), globalIDtoURL( $this->getGlobalID("edit/options") ) ) ) ) );
				}
				else
				{
					foreach( $this->entries as $key => $entry )
						$renderedContent .= $this->entries[$key]->renderTiny();
				}
			}
			break;
		}
		return $renderedContent;
	}
	
	function renderFeed()
	{
	}
	
	function render( $timestamp = null )
	{
		if( $this->parentEntries->edit )
			$renderedContent = $this->renderEditor();
		else
		{
			if( $timestamp == null )
			{
				foreach( $this->entries as $key => $entry )
					$renderedContent .= $this->entries[$key]->renderTiny();
			}
			else
				$renderedContent = $this->entries[$timestamp]->render();
		}
		return $renderedContent;
	}
}
?>