<?php
class blogger_userOptions
{
	var $parentUser;
	var $section;
	var $subSection;
	var $item;
	
	var $title;
	
	function blogger_userOptions( $parentUser, $section, $subSection )
	{
		$this->parentUser = &$parentUser;
		$this->section = $section;
		$this->subSection = $subSection;
	}

	function renderOptions()
	{
		global $permissionlevels_array, $blogcategories;
		
		$dialog = new optionsPanel("blogger_options");
		$dialog->showHeader = false;
		
		$dialog->addOption( i18n("Personal allowance level"), i18n("Who, except for the owner, is able to view entries marked as personal"), $this->parentUser->getParam("allowsviewpersonal"), "allowsviewpersonal", "select", $permissionlevels_array );
		
		$dialog->addHeader( i18n("Default settings for new blog entries"), "defaults" );
		$dialog->addOption( i18n("Personal"), i18n("Is this entry personal?"), $this->parentUser->getParam("personal"), "personal", "checkbox" );
		$dialog->addOption( i18n("Viewing allowance level"), i18n("Who, except for the owner, is able to view this entry"), $this->parentUser->getParam("viewlevel"), "viewlevel", "select", $permissionlevels_array );
		$dialog->addOption( i18n("Commenting allowance level"), i18n("Who, except for the owner, is able to comment on this entry"), $this->parentUser->getParam("commentlevel"), "commentlevel", "select", $permissionlevels_array );
		
		$dialog->addHeader( i18n("Default categories for new blog entries"), "categories" );
		
		$defaultCategories = $this->parentUser->getParam("defaultCategories");
		if( !is_array( $defaultCategories ) )
			$defaultCategories = array( "system" => array(), "user" => array() );
		
		// Don't need to check for blogcategories being an array and actually containing stuff - we ALWAYS have system_category
		foreach( $blogcategories as $key => $value )
			$categories[$value] = array( "title" => i18n($value), "description" => "" );
		$dialog->addOption( i18n("Sitewide categories"), i18n("Categories defined by the site administrators"), $defaultCategories["system"], "categoriesSystem", "multicheck", $categories );
		
		$usercategories = $this->parentUser->getParam("categories");
		if( !is_array( $usercategories ) )
			$usercategories = array();
		if( count( $usercategories ) > 0 )
		{
			$categories = array();
			foreach( $usercategories as $key => $value )
				$categories[$value] = array( "title" => i18n($value), "description" => "" );
			$dialog->addOption( i18n("User defined categories"), i18n("Categories defined by the the owner of the blog"), $defaultCategories["user"], "categoriesUser", "multicheck", $categories );
		}
		
		if( $dialog->submitted )
		{
			$this->parentUser->setParam( "allowsviewpersonal", $dialog->getValue("allowsviewpersonal") );
			$this->parentUser->setParam( "personal", $dialog->getValue("personal") );
			$this->parentUser->setParam( "viewlevel", $dialog->getValue("viewlevel") );
			$this->parentUser->setParam( "commentlevel", $dialog->getValue("commentlevel") );
			
			// Re-initialise the default categories array...
			$defaultCategories = array( "system" => array(), "user" => array() );
			$newCategoriesSystem = $dialog->getValue("categoriesSystem");
			foreach( $blogcategories as $key => $value )
				$defaultCategories["system"][$value] = $newCategoriesSystem[$value];
			$newCategoriesUser = $dialog->getValue("categoriesUser");
			foreach( $usercategories as $key => $value )
				$defaultCategories["user"][$value] = $newCategoriesUser[$value];
			$this->parentUser->setParam( "defaultCategories", $defaultCategories );
		}
		
		return $dialog->render();
	}
	
	function renderCategories()
	{
		$dialog = new SimpleListEditor( $this->parentUser->getGlobalID("edit/options/categories"), $this->subSection, $this->parentUser->getParam("categories"), $this->item );
		$dialog->title = i18n("Category");
		$dialog->description = i18n("A string containing the name of the category. Please note that any spaces will be translated to underscores.");
		$renderedContent = $dialog->render();
		if( $dialog->changed )
		{
			foreach( $dialog->items as $key => $value )
				$dialog->items[$key] = str_replace( " ", "_", $value );
			$this->parentUser->setParam( "categories", $dialog->items );
		}
		return $renderedContent;
	}
	
	function renderExternal()
	{
		$dialog = new SimpleListEditor( $this->parentUser->getGlobalID("edit/options/external"), $this->subSection, $this->parentUser->getParam("external"), $this->item );
		$dialog->title = i18n("URL");
		$dialog->description = i18n("The URL of the external feed you wish to add. Please note this should be an RSS file.");
		$renderedContent = $dialog->render();
		$oldExternalCache = $this->parentUser->getParam("externalCache");
		if( $dialog->changed )
		{
			foreach( $dialog->items as $key => $value )
			{
				$dialog->items[$key] = str_replace( " ", "%20", $value );
				if( substr( strtolower($dialog->items[$key]), 0, 7 ) != "http://" &&
					substr( strtolower($dialog->items[$key]), 0, 8 ) != "https://" &&
					substr( strtolower($dialog->items[$key]), 0, 6 ) != "ftp://" )
					$dialog->items[$key] = "http://" . $dialog->items[$key];
					
					// Copy the old cache over if the item aleady exists...
					if( array_key_exists( $dialog->items[$key], $oldExternalCache ) )
						$tempExternalCache[$dialog->items[$key]] = $oldExternalCache[$dialog->items[$key]];
					// And create the cache if it doesn't already exist...
					else
						$tempExternalCache[$dialog->items[$key]] = array();
			}
			// Save the list of external feeds
			$this->parentUser->setParam( "external", $dialog->items );
			// Save the updated external feeds cache
			$this->parentUser->setParam( "externalCache", $tempExternalCache );
		}
		return $renderedContent;
	}
	
	function render()
	{
		switch( $this->section )
		{
		case "external":
			$renderedContent = $this->renderExternal();
			$this->title = i18n("External Feeds");
			break;
		case "categories":
			$renderedContent = $this->renderCategories();
			$this->title = i18n("Categories");
			break;
		case "options":
		default:
			$renderedContent = $this->renderOptions();
			$this->title = i18n("Options");
			break;
		}
		
		$renderedContent = "<form method=\"POST\" action=\"" . thisPageURL() . "\">$renderedContent</form>";
		
		return renderInformationBox( $this->title, $renderedContent, false );
	}
}
?>