<?PHP
/**
 * The Discuss forum module for Travelsized CMS
 *
 * @Author	Dan Jensen <admin@leinir.dk>
 * @Version	0.1
 * @Date	2nd July, 2006
 *
 * Discuss directory structure
 *
 * modules/
 *   discuss/
 *     options.php
 *     sections/
 *       sectionposition/
 *         options.php
 *         forums/
 *           forumposition/
 *             options.php
 *             threads/
 *               threadid.php
 *
 * modules/discuss/options.php:
 *   discuss_options = array(
 *     "page_id" => ID of the page which the main forum will go on,
 *     "header" => The optional header (page data) which will be shown on the main forum page,
 *     "footer" => The optional footer (page data) which will be shown on the main forum page,
 *     "sectionPositions" => The order in which to show the sections on the forum main page,
 *     "forumdefaults" => array(
 *       "viewLevel" => The default view level for forums,
 *       "postLevel" => The default post level for forums,
 *       ),
 *     "sectiondefaults" => array(
 *       "viewLevel" => The default view level for sections,
 *       ),
 *    )
 *
 * modules/discuss/sections/sectionid/options.php
 *   section_options = array(
 *     "sectionid" => The section's ID,
 *     "title" => The section's title,
 *     "description" => The section's description,
 *     "viewLevel" => The section's view level,
 *     "forumPositions" => The position of the contained forums
 *   )
 *
 * modules/discuss/sections/sectionid/forums/forumid/options.php
 *   forum_options = array(
 *     "forumid" => The forum's ID,
 *     "title" => The forum's title,
 *     "moderators" => array containing usernames,
 *     "description" => The forum's description,
 *     "viewLevel" => The forum's viewing level,
 *     "postLevel" => The forum's posting level,
 *     "lastChanged" => The timestamp for the forum's last change,
 *   )
 */

$modules_names[] = "Discuss";
$mopules_versions[] = "0.1";
$modules_descriptions[] = "Manages discussion forums";

$modules_setup[] = "modulesetup_discuss";

$modules_globalID[] = "globalID_discuss";

$profilemodules[] = "profilemodule_discuss";

$modules[] = "parse_discusssection";
$modules[] = "parse_discussforum";
$modules[] = "parse_discussthread";

$page_help["\\discuss(txt1)"] = "This will show the discussion section named txt1";
$page_help["\\discussforum(txt1,txt2)"] = "This will show the forum named txt2 in the section txt1";
$page_help["\\discussthread(txt1,txt2,txt3)"] = "This will show the thread with ID txt3 in the forum named txt2 in the section txt1";

$meta_headers .= "
	<link rel=\"StyleSheet\" href=\"" . siteURL(true) . "$module_folder/discuss/discuss.css\" type=\"text/css\" />";

// Usergroup defaults {
$usergroups_global_default["modules_discuss_admin"] = false; // Let users view posts in new forums by default
// Usergroup defaults }

$discussDirectory = "$module_folder/discuss";
$discussContentDirectory  = "$discussDirectory/sections";

if( file_exists( "$discussDirectory/options.php" ) )
{
	include( "$discussDirectory/options.php" );
}
else
{
	$discuss_options = array(
		"page_id" => 12, // If the user's not set anything up yet - show him the way back to module setup!
		"title" => "Travelsized Discussions",
		"header" => ">>//Where people can talk//",
		"footer" => "---\nDiscussions for Travelsized Websites",
		"sectionPositions" => array(),
		"forumdefaults" => array(
			"viewLevel" => 1,
			"postLevel" => 2,
		),
		"sectiondefaults" => array(
			"viewLevel" => 1,
		),
	);
}

$discuss_allowanceLevels = array(
	1 => "Everybody",
	2 => "Only registered users",
	3 => "Only moderators"
);

$discuss_defaultoptions = array(
	"user" => array(
		"viewLevel" => 0,
		"replyLevel" => 1,
		"editLevel" => 4
		)
);

include("$module_folder/discuss/code/discuss_userPosts.php");
include("$module_folder/discuss/code/discussThread.php");
include("$module_folder/discuss/code/discussForum.php");
include("$module_folder/discuss/code/discussSection.php");

function profilemodule_discuss()
{
	global $tabwidget, $globalID, $setup_folder, $userinfo_folder, $language, $languages, $permissionlevels_array, $blogcategories, $formatting_toolbar, $discuss_options, $viewUser;
	
	$splitID = explode( "/", $globalID );
	
	if( $splitID[3] == "edit" && ( isAllowed( "global_user_edit" ) || ( $viewUser != "" && $viewUser == currentUser() ) ) )
	{
		$tabwidget->addCommand( i18n("View"), i18n("View ##0##'s most recent posts", array( currentUser() ) ), globalIDtoURL("user/$viewUser/discuss") );
		
		// Set up options
			// Default options for my new posts (also, check for with remember these settings during post creation)
			// View options- discuss_verboseListings
		
		$discuss_verboseListings = getUserInfo( $viewUser, "discuss_verboseListings" );
		$discussOptions["viewLevel"] = getUserInfo( $viewUser, "discuss_viewLevel" );
		$discussOptions["replyLevel"] = getUserInfo( $viewUser, "discuss_replyLevel" );
		$discussOptions["editLevel"] = getUserInfo( $viewUser, "discuss_editLevel" );
		if( $_POST["saveOptions"] )
		{
			$discuss_verboseListings = $_POST["discussOptions"]["verboseListings"] ? 1 : 0;
			saveUserInfo( $viewUser, "discuss_verboseListings", $discuss_verboseListings );
			
			$discussOptions["viewLevel"] = $_POST["discussOptions"]["viewLevel"];
			saveUserInfo( $viewUser, "discuss_viewLevel", $discussOptions["viewLevel"] );
			
			$discussOptions["replyLevel"] = $_POST["discussOptions"]["replyLevel"];
			saveUserInfo( $viewUser, "discuss_replyLevel", $discussOptions["replyLevel"] );
			
			$discussOptions["editLevel"] = $_POST["discussOptions"]["editLevel"];
			saveUserInfo( $viewUser, "discuss_editLevel", $discussOptions["editLevel"] );
		}
		
		$discussProfileOptions = new optionsPanel( "discussOptions" );
		
		$discussProfileOptions->addOption(
				i18n("Hide verbose listings"),
				i18n("Hide the verbose information in the forum listings, such as last poster, creator, "),
				$discuss_verboseListings,
				"verboseListings",
				"checkbox"
			);
		
		$discussProfileOptions->addOption(
				i18n("Default view level for new posts"),
				i18n("The view level that is used for when you create new posts in the forum"),
				$discussOptions["viewLevel"],
				"viewLevel",
				"select",
				$permissionlevels_array
			);
		$discussProfileOptions->addOption(
				i18n("Default reply level for new posts"),
				i18n("The reply level that is used for when you create new posts in the forum"),
				$discussOptions["replyLevel"],
				"replyLevel",
				"select",
				$permissionlevels_array
			);
		$discussProfileOptions->addOption(
				i18n("Default edit level for new posts"),
				i18n("The edit level that is used for when you create new posts in the forum"),
				$discussOptions["editLevel"],
				"editLevel",
				"select",
				$permissionlevels_array
			);
		
		$data =
				"<form action=\"" . thisPageURL() . "\" method=\"POST\">" .
				$discussProfileOptions->render() .
				"</form>";
	}
	else
	{
		$tabwidget->addCommand( i18n("Options"), i18n("Options for Discuss" ), globalIDtoURL( "user/$viewUser/discuss/edit") );
		// Get all of the threads listed...
		$tempPosts = new discuss_userPosts( $viewUser );
		
		// Render each thread into $data
		if( $tempPosts->postcount() < 1 )
			$data = renderInformationBox( i18n("No posts"), i18n("This user has not yet made any posts in the forum. When they do, this will show a list of the posts made by them or replied to by them.") );
		else
		{
			$data = renderInformationBox( i18n("Last ##0## posts", array( $tempPosts->postcount() ) ), i18n("Bellow is a list of the ##0## posts made by ##1##. Click on each of them to go to the post.", array( $tempPosts->postcount(), getUserInfo( currentUser(), "name" ) ) ) );
			foreach( $tempPosts->posts as $key => $post )
			{
				$tempURL = globalIDtoURL("discuss/" . $post["sectionID"] . "/" . $post["forumID"] . "/" . $post["threadID"]);
				
				if( $post["commentID"] != "" )
					$tempURL .= "#" . $post["commentID"];
				
				$data .= "<div class=\"discuss_threadHeader\"><a
					href=\"$tempURL\" class=\"discuss_threadHeader\"><span
					class=\"discuss_threadLink\">" . i18n("Go") . " &raquo;</span><span
					class=\"discuss_threadTitle\">{$post['title']}</span><span
					class=\"discuss_threadDescription\">" . i18n("Posted on: ##0##", array(formatTime($post["timestamp"])) ) . "</span></a></div>";
			}
		}
	}
	return $data;
}

function parse_discusssection( $page_data )
{
	global $setup_folder;

	$moduleCommandName = "discuss";
	while( $parameters = spotModuleCommand( $page_data, $moduleCommandName, &$loopcontrol ) )
	{
		$theSection = new discussSection($parameters);
		$replaceData = $theSection->render();
		$page_data = str_replace( "\\$moduleCommandName($parameters)", $replaceData, $page_data );
	}
	return $page_data;
}

function parse_discussforum( $page_data )
{
	global $setup_folder;

	$moduleCommandName = "discussforum";
	while( ( $parameters = spotModuleCommand( $page_data, $moduleCommandName, &$loopcontrol ) ) === true )
	{
		$replaceData = "";
		$page_data = str_replace( "\\$moduleCommandName($parameters)", $replaceData, $page_data );
	}
	return $page_data;
}

function parse_discussthread( $page_data )
{
	global $setup_folder;

	$moduleCommandName = "discussthread";
	while( ( $parameters = spotModuleCommand( $page_data, $moduleCommandName, &$loopcontrol ) ) === true )
	{
		$replaceData = "";
		$page_data = str_replace( "\\$moduleCommandName($parameters)", $replaceData, $page_data );
	}
	return $page_data;
}

/**
 * Returns the url that corresponds to the globalID for a discuss item, if appropriate
 *
 * @param     $splitID       The global ID to check
 *
 * @return    string         A string containing the URL corresponding to the globalID, if existing
 */
function globalID_discuss( $splitID )
{
	global $discuss_options;
	$pageID = null;
	
	list($toplevel, $sectionID, $forumID, $threadID ) = split( "/", $globalID );
	
	if( $splitID[0] == "discuss" )
	{
		$_REQUEST["page_id"] = $discuss_options["page_id"];
		if( $splitID[1] != "" )
		{
			$_REQUEST["discuss_sectionID"] = $splitID[1];
			if( $splitID[2] != "" )
			{
				$_REQUEST["discuss_forumID"] = $splitID[2];
				if( $splitID[3] != "" )
					$_REQUEST["discuss_threadID"] = $splitID[3];
			}
		}
		$pageID = $_REQUEST["page_id"];
	}
	else if( $splitID[0] == "user" && $splitID[2] == "discuss" )
	{
		$pageID = "13";
		$_REQUEST["module"] = "profilemodule_discuss";
	}
	
	return $pageID;
}

/**
 * Rebuild the cache for all threads in all forums in all sections
 *
 * @return void
 */
function discuss_rebuildUserCache()
{
	global $discuss_options;
	
	// First, clear out all existing cache
	$allUsers = allUsersArray();
	foreach( $allUsers as $key => $username )
	{
		$userposts[$username] = new discuss_userposts( $username );
		$userposts[$username]->posts = array();
	}
	
	// Go through each section
	foreach( $discuss_options["sectionPositions"] as $position => $sectionID )
	{
		$theSection = new discussSection( $sectionID );
		// and each forum in that section
		foreach( $theSection->forums as $key => $theForum )
		{
			$theForum->loadThreads();
			
			//And rebuild the cache for all posts in those, item by item.
			foreach( $theForum->threads as $position => $thread )
			{
				// We can only do this... if the user is registered!
				if( array_key_exists( $thread->owner, $userposts ) )
				{
					$postdata["sectionID"] = $thread->sectionID;
					$postdata["forumID"] = $thread->forumID;
					$postdata["threadID"] = $thread->threadID;
					$postdata["title"] = $thread->title;
					$postdata["timestamp"] = $thread->lastChanged;
					$postdata["viewLevel"] = $thread->viewLevel;
					$userposts[$thread->owner]->rotate($postdata);
				}
				
				// Make sure to get the replies as well!
				discuss_rebuildDeep( &$userposts, $thread->replies, &$thread );
			}
		}
	}
}

/**
 * Rebuild the cache for each reply in a thread.
 *
 * @return void
 */
function discuss_rebuildDeep($userposts, $replies, $thread, $commentID = "")
{
	foreach( $replies as $timestamp => $comment )
	{
		// We can only do this... if the user is registered!
		if( array_key_exists( $comment["name"], $userposts ) )
		{
			$postdata["sectionID"] = $thread->sectionID;
			$postdata["forumID"] = $thread->forumID;
			$postdata["threadID"] = $thread->threadID;
			$postdata["commentID"] = $commentID . "[$timestamp]";
			$postdata["title"] = i18n("[Commented] ##0##", array($thread->title));
			$postdata["viewLevel"] = $thread->viewLevel;
			$postdata["timestamp"] = $timestamp;
			
			$userposts[$comment["name"]]->rotate( $postdata );
		}
		
		if( array_key_exists( "comments", $comment ) )
			discuss_rebuildDeep( &$userposts, $comment["comments"], &$thread, $commentID . "[$timestamp][comments]" );
	}
}

function discuss_saveOptions()
{
	global $discussDirectory, $discuss_options;
	
	if( file_exists( "$discussDirectory/options.php" ) )
		unlink( "$discussDirectory/options.php" );
	
	if( !is_dir( $discussDirectory ) )
		RecursiveMkdir( $discussDirectory );
	
	file_put_contents( "$discussDirectory/options.php", array_export( $discuss_options, "discuss_options" ) );
}

function modulesetup_discuss()
{
	global $pagectl, $setup_folder, $page_folder, $recent_file, $menu_folder, $discussDirectory, $discussContentDirectory, $discuss_options, $discuss_allowanceLevels, $language, $default_language, $theModuleSection, $theModuleSubsection, $theModuleAction;
	if( isAllowed( "modules_discuss_admin" ) )
	{
		$discussSetup = new TabWidget;
		$discussSetup->tabbar->addTab(i18n("Forum Setup"), globalIDtoURL("setup/modulesetup/discuss/setup"));
		$discussSetup->tabbar->addTab(i18n("Options"), globalIDtoURL("setup/modulesetup/discuss/options"));
		
		switch( $theModuleSection )
		{
		// Import
		case "import":
			switch( $theModuleSubsection )
			{
			// First, ask user confirmation to clear everything already in the forum... Need a clean slate to work with
			case "":
				break;
			// Then get which importer to use
			case "":
				break;
			// Perform actions according to that importer
			default:
				break;
			}
			break;
		// Options:
		case "options":
			$discussSetup->tabbar->setCurrent( i18n( "Options" ) );
			
			if( $theModuleAction == "create_forum_page" )
			{
				// Create the page to save the forum to!
				$edit_page["id"] = $pagectl->getFreePageID($page_folder, $pagectl);
				$edit_page["title"] = $discuss_options["title"];
				$edit_page["menu"] = $discuss_options["menu_id"];
				$edit_page["language"] = $default_language;
				$edit_page["content"] = i18n("Write the forum structure to this page by going to the Discuss management in Site Setup...");
				$edit_page["author"] = getUserInfo(currentUser(), "name");
				$edit_page["email"] = getUserInfo(currentUser(), "email");
				
				unlink_multiple( "$page_folder/{$edit_page['id']}.*" );
				
				if ($pagectl->savePageData($page_folder, $edit_page["id"], $edit_page["language"], $edit_page["author"], $edit_page["email"], $discuss_options["menu_id"], $edit_page["title"], $edit_page["content"])) {
					$page_content = parse_page_data(i18n("The page was saved."));
					update_getpage_id($page_folder, $setup_folder, $pagectl);
					rotate_recent($edit_page["id"], $edit_page["title"], $edit_page["language"], date("j/n/y"), $recent_file);
					
					$discuss_options["page_id"] = $edit_page["id"];
					if( file_exists( "$discussDirectory/options.php" ) )
						unlink( "$discussDirectory/options.php" );
					
					if( !is_dir( $discussDirectory ) )
						RecursiveMkdir( $discussDirectory );
					
					file_put_contents( "$discussDirectory/options.php", array_export( $discuss_options, "discuss_options" ) );
					
					header("Location: " . globalIDtoURL("setup/modulesetup/discuss/options"));
				} else {
					$error_description = parse_page_data(i18n("The discuss page could not be saved!"));
				}
			}
			else if( $_POST["saveOptions"] )
			{
				$discuss_options["page_id"] = $_POST["discuss"]["page_id"];
				$discuss_options["menu_id"] = $_POST["discuss"]["menu_id"];
				$discuss_options["sectiondefaults"]["viewLevel"] = $_POST["discuss"]["sectiondefaults"]["viewLevel"] ? 1 : 0;
				$discuss_options["forumdefaults"]["viewLevel"] = $_POST["discuss"]["forumdefaults"]["viewLevel"];
				$discuss_options["forumdefaults"]["postLevel"] = $_POST["discuss"]["forumdefaults"]["postLevel"];
				$discuss_options["title"] = stripslashes($_POST["discuss"]["title"]);
				$discuss_options["header"] = stripslashes($_POST["discuss"]["header"]);
				$discuss_options["footer"] = stripslashes($_POST["discuss"]["footer"]);
				
				discuss_saveOptions();
			}
			
			$discussOptionsPanel = new optionsPanel("discuss");
			
			$pagelist = $pagectl->fetchPageList($page_folder);
			$temp_pagelist[12] = i18n("Please select a page...");
			foreach( $pagelist as $key => $value )
			{
				$temp_pagelist[$key] = $value["title"][0];
			}
			$discussOptionsPanel->addOption(
				i18n("Forum page"),
				i18n("This is the page that will be used to save the forum structure to. If you want to create a new page to put the information into, ##0##simply click here##1## and this will be done for you.", array("<a href=\"" . globalIDtoURL("setup/modulesetup/discuss/options/na/create_forum_page") . "\">", "</a>")),
				$discuss_options["page_id"],
				"page_id",
				"select",
				$temp_pagelist
			);
			
			$menulist = $pagectl->fetchMenuMenuList($menu_folder);
			foreach( $menulist as $key => $value )
			{
				$temp_menulist[$value["id"]] = $value["title"][0];
			}
			$discussOptionsPanel->addOption(
				i18n("Forum menu"),
				i18n("This is the menu that will be shown on the Forum page"),
				$discuss_options["menu_id"],
				"menu_id",
				"select",
				$temp_menulist
			);
			
			$discussOptionsPanel->addOption(
				i18n("Anonymous can view new sections by default"),
				i18n("When creating a new section, this viewing allowance level will be assigned to it"),
				$discuss_options["sectiondefaults"]["viewLevel"],
				"sectiondefaults][viewLevel",
				"checkbox"
			);
			
			$discussOptionsPanel->addOption(
				i18n("Who can view forums by default"),
				i18n("When creating a new forum, this viewing allowance level will be assigned to it"),
				$discuss_options["forumdefaults"]["viewLevel"],
				"forumdefaults][viewLevel",
				"select",
				$discuss_allowanceLevels
			);
			
			$discussOptionsPanel->addOption(
				i18n("Who can post to forums by default"),
				i18n("When creating a new forum, this posting allowance level will be assigned to it"),
				$discuss_options["forumdefaults"]["postLevel"],
				"forumdefaults][postLevel",
				"select",
				$discuss_allowanceLevels
			);
			
			$discussOptionsPanel->addOption(
				i18n("Forum Title"),
				i18n("The title of the forum page"),
				$discuss_options["title"],
				"title",
				"text"
			);
			
			$discussOptionsPanel->addOption(
				i18n("Forum header"),
				i18n("The header that is shown above the forum sections on the main forum page - it is also shown above all loaded threads, so be careful how much you write here!"),
				$discuss_options["header"],
				"header",
				"textbox"
			);
			
			$discussOptionsPanel->addOption(
				i18n("Forum Footer"),
				i18n("The footer that is shown below the forum sections on the main forum page - it is also shown below all loaded threads, so be careful how much you write here!"),
				$discuss_options["footer"],
				"footer",
				"textbox"
			);
			
			
			$discussSetup->contents = "<form action=\"" . globalIDtoURL("setup/modulesetup/discuss/options") . "\" method=\"POST\">" . $discussOptionsPanel->render() . "</form>";
			break;
		case "setup":
		default:
			$discussSetup->tabbar->setCurrent( i18n( "Forum Setup" ) );
			
			$discussSetup->addCommand(i18n("Rebuild cache"), i18n("Rebuild the user post cache, used for showing what posts the user has created."), globalIDtoURL("setup/modulesetup/discuss/setup/na/rebuild_user_cache") );
			$discussSetup->addCommand(i18n("Add Section"), i18n("Add a section to the end of the forum"), globalIDtoURL("setup/modulesetup/discuss/setup/na/add_section"));
			if( $discuss_options["page_id"] != 12 )
			{
				$discussSetup->addCommand(i18n("Write section order"), i18n("Write the order of the sections to the page defined in options"), globalIDtoURL("setup/modulesetup/discuss/setup/na/write_forum") );
				$discussSetup->addCommand(i18n("Go to forum page"), i18n("Go to the page defined in options as the forum page"), globalIDtoURL("content/$language/" . $discuss_options["page_id"] ));
			}
			
			switch( $theModuleAction )
			{
			case "rebuild_user_cache":
				discuss_rebuildUserCache();
				header("Location: " . globalIDtoURL("setup/modulesetup/discuss/setup"));
				break;
			case "add_section":
				// We do this until there is no directory called that already (this avoids two people attempting to create a new section with the same ID at the same time)
				do {
					// Increase the lastsectionid by one (thus also making sure the first is 1)
					$discuss_options["counters"]["lastSectionID"]++;
					// Assign this value to the be the new section's ID
					$newSectionID = $discuss_options["counters"]["lastSectionID"];
				} while( is_dir( "$discussContentDirectory/$newSectionID" ) );
				
				$section_options["title"] = i18n("Section title");
				$section_options["description"] = i18n("Section description - a one line text describing the section. Can be empty.");
				$section_options["viewLevel"] = $discuss_options["sectiondefaults"]["viewLevel"];
				$section_options["forumPositions"] = array();
				
				RecursiveMkdir( "$discussContentDirectory/$newSectionID" );
				
				file_put_contents( "$discussContentDirectory/$newSectionID/options.php", array_export( $section_options, "section_options" ) );
				
				$discuss_options["sectionPositions"][] = $newSectionID;
				discuss_saveOptions();
				
				header("Location: " . globalIDtoURL("setup/modulesetup/discuss/setup"));
				
				break;
			case "delete_section":
				foreach( $discuss_options["sectionPositions"] as $position => $positionID )
					if( $positionID == $_REQUEST["discuss_sectionID"] )
						unset( $discuss_options["sectionPositions"][$position] );
				
				// Update the array with the new position indexes... stops confusion when moving sections up and down
				foreach( $discuss_options["sectionPositions"] as $position => $positionID )
					$temp_positions[] = $positionID;
				$discuss_options["sectionPositions"] = $temp_positions;
				
				discuss_saveOptions();
				
				rmdirr( $discussContentDirectory . "/" . $_REQUEST["discuss_sectionID"] ); // Recursively remove the section data itself...
				
				header("Location: " . globalIDtoURL("setup/modulesetup/discuss/setup"));
			
				break;
			case "move_section_down":
				foreach( $discuss_options["sectionPositions"] as $position => $positionID )
				{
					if( $positionID == $_REQUEST["discuss_sectionID"] )
					{
						$temp_sectionID = $discuss_options["sectionPositions"][$position + 1];
						$discuss_options["sectionPositions"][$position + 1] = $positionID;
						$discuss_options["sectionPositions"][$position] = $temp_sectionID;
					}
				}
				discuss_saveOptions();
				header("Location: " . globalIDtoURL("setup/modulesetup/discuss/setup"));
				break;
			case "move_section_up":
				foreach( $discuss_options["sectionPositions"] as $position => $positionID )
				{
					if( $positionID == $_REQUEST["discuss_sectionID"] )
					{
						$temp_sectionID = $discuss_options["sectionPositions"][$position - 1];
						$discuss_options["sectionPositions"][$position - 1] = $positionID;
						$discuss_options["sectionPositions"][$position] = $temp_sectionID;
					}
				}
				discuss_saveOptions();
				header("Location: " . globalIDtoURL("setup/modulesetup/discuss/setup"));
				break;
			case "add_forum":
				// We do this until there is no directory called that already (this avoids two people attempting to create a new forum with the same ID at the same time)
				do {
					// Increase the lastsectionid by one (thus also making sure the first is 1) - we have loads of IDs to use, no need to save this information with the section
					$discuss_options["counters"]["lastForumID"]++;
					// Assign this value to the be the new forum's ID
					$newForumID = $discuss_options["counters"]["lastForumID"];
				} while( is_dir( "$discussContentDirectory/{$_REQUEST['discuss_sectionID']}/forums/$newForumID" ) );
				
				RecursiveMkdir( "$discussContentDirectory/{$_REQUEST['discuss_sectionID']}/forums/$newForumID" );
				
				$forum_options["title"] = i18n("Forum title");
				$forum_options["description"] = i18n("Forum description - a one line text describing the forum. Can be empty.");
				$forum_options["postLevel"] = $discuss_options["forumdefaults"]["postLevel"];
				$forum_options["viewLevel"] = $discuss_options["forumdefaults"]["viewLevel"];
				
				file_put_contents( "$discussContentDirectory/{$_REQUEST['discuss_sectionID']}/forums/$newForumID/options.php", array_export( $forum_options, "forum_options" ) );
				
				// Add the new forum to the section's forum list...
				$theSection = new discussSection($_REQUEST["discuss_sectionID"]);
				$theSection->forumPositions[] = $newForumID;
				$theSection->save();
				
				discuss_saveOptions();
				
				header("Location: " . globalIDtoURL("setup/modulesetup/discuss/setup"));
				
				break;
			case "delete_forum":
				$theSection = new discussSection($_REQUEST["discuss_sectionID"]);
				foreach( $theSection->forumPositions as $position => $positionID )
					if( $positionID == $_REQUEST["discuss_forumID"] )
						unset( $theSection->forumPositions[$position] );
				
				// Update the array with the new position indexes... stops confusion when moving forums up and down
				foreach( $theSection->forumPositions as $position => $positionID )
					$temp_positions[] = $positionID;
				$theSection->forumPositions = $temp_positions;
				
				$theSection->save();
				
				rmdirr( $discussContentDirectory . "/" . $_REQUEST["discuss_sectionID"] . "/forums/" . $_REQUEST["discuss_forumID"] ); // Recursively remove the section data itself...
				
				header("Location: " . globalIDtoURL("setup/modulesetup/discuss/setup"));
			
				break;
			case "move_forum_down":
				$theSection = new discussSection($_REQUEST["discuss_sectionID"]);
				foreach( $theSection->forumPositions as $position => $positionID )
				{
					if( $positionID == $_REQUEST["discuss_forumID"] )
					{
						$temp_forumID = $theSection->forumPositions[$position + 1];
						$theSection->forumPositions[$position + 1] = $positionID;
						$theSection->forumPositions[$position] = $temp_forumID;
					}
				}
				$theSection->save();
				header("Location: " . globalIDtoURL("setup/modulesetup/discuss/setup"));
				break;
			case "move_forum_up":
				$theSection = new discussSection($_REQUEST["discuss_sectionID"]);
				foreach( $theSection->forumPositions as $position => $positionID )
				{
					if( $positionID == $_REQUEST["discuss_forumID"] )
					{
						$temp_forumID = $theSection->forumPositions[$position - 1];
						$theSection->forumPositions[$position - 1] = $positionID;
						$theSection->forumPositions[$position] = $temp_forumID;
					}
				}
				$theSection->save();
				header("Location: " . globalIDtoURL("setup/modulesetup/discuss/setup"));
				break;
			case "write_forum":
				
				// Create the page to save the forum to!
				$edit_page["id"] = $discuss_options["page_id"];
				$edit_page["menu"] = $discuss_options["menu_id"];
				$edit_page["title"] = $discuss_options["title"];
				$edit_page["language"] = $default_language;
				$edit_page["author"] = getUserInfo(currentUser(), "name");
				$edit_page["email"] = getUserInfo(currentUser(), "email");
				
				$edit_page["content"] = $discuss_options["header"] . "\n";
				if( is_array( $discuss_options["sectionPositions"] ) )
				{
					foreach( $discuss_options["sectionPositions"] as $position => $positionID )
						$edit_page["content"] .= "\\\\discuss($positionID)\n";
				}
				$edit_page["content"] .= $discuss_options["footer"] . "\n";
				
				unlink_multiple("$page_folder/{$discuss_options["page_id"]}.*");
				
				if ($pagectl->savePageData($page_folder, $edit_page["id"], $edit_page["language"], $edit_page["author"], $edit_page["email"], $discuss_options["menu_id"], $edit_page["title"], $edit_page["content"])) {
					$page_content = parse_page_data(i18n("The page was saved."));
					update_getpage_id($page_folder, $setup_folder, $pagectl);
					rotate_recent($edit_page["id"], $edit_page["title"], $edit_page["language"], date("j/n/y"), $recent_file);
					
					$error_description = renderInformationBox(
							i18n("The discuss page was saved successfully"),
							i18n("The section structure was saved to the page described by you in the options panel. If you want, you can go to that page right now, ##0##simply by clicking right here##1##.", array( "<a href=\"" . globalIDtoURL("content/$language/" . $discuss_options["page_id"]) . "\">", "</a>" ) )
						);
				} else {
					$error_description = renderInformationBox(
							i18n("The discuss page could not be saved!"),
							i18n("An error occured while attempting to save the section structure to the forum page. Please check any error messages found in the browser and act accordingly. There is a high risk it is due to errors in file permissions, or that you have run out of disc space.")
						);
				}
				break;
			default:
				// Naaafink
			}
			
			if( count( $discuss_options["sectionPositions"] ) > 0 ){
				foreach( $discuss_options["sectionPositions"] as $key => $value )
				{
				// Section
					$theSection = new discussSection($value);
					
					if( $_REQUEST["discuss_sectionDeleteID"] == $value )
						$discussSetup->contents .= "
					<div class=\"discussSetup_sectionContainerDelete\">
						<p class=\"wikicenteralign\"><a name=\"deleteConfirmPanel\" class=\"namedanchor\"> </a>" . i18n("Please confirm below that you really wish to delete this section. Note that you will be unable to undo this, and all threads in all forums contained in the section will be irreversibly deleted!") . "</p>
						<div class=\"wikicenteralign\">
						" . drawCommand( i18n("Confirm delete"), i18n("Confirm delete"), globalIDtoURL("setup/modulesetup/discuss/setup/na/delete_section", array("discuss_sectionID"=>$value) ) ) . "
						" . drawCommand( i18n("Abort delete"), i18n("Abort delete"), globalIDtoURL("setup/modulesetup/discuss/setup") ) . "
						</div>
						<hr />";
					else
						$discussSetup->contents .= "
					<div class=\"discussSetup_sectionContainer\">";
						
					if( $_REQUEST["discuss_editSectionID"] == $value )
					{
						$temp_viewLevel = "";
						if( $theSection->viewLevel == 1 )
							$temp_viewLevel = " checked";
						
						$discussSetup->contents .= "
						<form action=\"" . globalIDtoURL("setup/modulesetup/discuss/setup") . "#discussSectionEdit\" method=\"post\">
						<input type=\"hidden\" name=\"discussSectionEdit[sectionID]\" value=\"$value\" />
						<div class=\"discussSetup_sectionHeader\">
							<span class=\"discussSetup_sectionTitle\">" . i18n("Section title") . ":<a name=\"discussSectionEdit\" class=\"namedanchor\"> </a></span><input class=\"discussSetup_sectionTitle\" type=\"text\" name=\"discussSectionEdit[title]\" value=\"{$theSection->title}\" />
							<span class=\"discussSetup_sectionDescription\">" . i18n("Section description") . ":</span><input class=\"discussSetup_sectionDescription\" type=\"text\" name=\"discussSectionEdit[description]\" value=\"{$theSection->description}\" />
							<span class=\"discussSetup_sectionViewlevel\">" . i18n("Viewable by") . ":</span><label class=\"discussSetup_sectionViewlevel\"><input type=\"checkbox\" name=\"discussSectionEdit[viewLevel]\"$temp_viewLevel>Anonymous can view new sections by default</label>
							<input style=\"width: 99%\" type=\"submit\" name=\"save_section\" value=\"" . i18n("Save") . "\"/>
						</div>
						</form>";
						$editThisSection = "
							" . drawCommand( i18n("Abort delete"), i18n("Abort delete"), globalIDtoURL("setup/modulesetup/discuss/setup") );
					}
					else
					{
						$temp_viewLevel = "";
						if( $theSection->viewLevel == 1 )
							$temp_viewLevel = i18n("Everybody");
						else
							$temp_viewLevel = i18n("Only registered users");
						
						$discussSetup->contents .= "
						<div class=\"discuss_sectionHeader\">
							<span class=\"discuss_sectionTitle\">{$theSection->title}</span>
							<span class=\"discuss_sectionDescription\">{$theSection->description}</span>
							<span class=\"discuss_sectionViewlevel\">" . i18n("##0## can view this section", array( $temp_viewLevel ) ) . "</span>
						</div>";
						$editThisSection = "
							" . drawCommand( i18n("Edit"), i18n("Edit this section"), globalIDtoURL("setup/modulesetup/discuss/setup", array("discuss_editSectionID" => "$value#discussSectionEdit") ) );
					}
					
					if( $key > 0 )
						$moveSectionUp = drawCommand( i18n("Move up"), i18n("Move the section up one step"), globalIDtoURL("setup/modulesetup/discuss/setup/na/move_section_up", array("discuss_sectionID" => $value ) ) );
					else
						$moveSectionUp = "<acronym title=\"" . i18n("Alrady topmost") . "\">[" . i18n("Move up") . "]</acronym>";
					
					if( count( $discuss_options["sectionPositions"] ) > $key + 1 )
						$moveSectionDown = drawCommand(i18n("Move down") ,i18n("Move the section down one step") , globalIDtoURL("setup/modulesetup/discuss/setup/na/move_section_down", array("discuss_sectionID"=>$value) ) );
					else
						$moveSectionDown = "<acronym title=\"" . i18n("Alrady bottommost") . "\">[" . i18n("Move down") . "]</acronym>";
					
					$discussSetup->contents .= "
						<div class=\"discussSetup_sectionAddForum\">
							" . drawCommand( i18n("Add a new forum to this section"), i18n("Add a new forum to this section"), globalIDtoURL("setup/modulesetup/discuss/setup/na/add_forum", array("discuss_sectionID" => $value ) ) ) . " &nbsp; &nbsp;
							" . drawCommand( i18n("Delete"), i18n("Delete this section"), globalIDtoURL("setup/modulesetup/discuss/setup",  array( "discuss_sectionDeleteID" => "$value#deleteConfirmPanel" ) ) ) . " &nbsp; &nbsp;
							$moveSectionUp
							$moveSectionDown &nbsp; &nbsp; 
							$editThisSection
						</div>";
					if( count( $theSection->forumPositions ) > 0 )
					{
						foreach( $theSection->forumPositions as $key2 => $value2 )
						{
							$temp_title = $theSection->forums[$theSection->forumByID($value2)]->title;
							$temp_description = $theSection->forums[$theSection->forumByID($value2)]->description;
							
							if( $_POST["add_moderator"] )
							{
								$theSection->forums[$theSection->forumByID($value2)]->moderators[] = $_POST["discussForumEdit"]["newModerator"];
								$theSection->forums[$theSection->forumByID($value2)]->save();
							}
							else if( $_POST["remove_moderator"] )
							{
								unset( $theSection->forums[$theSection->forumByID($value2)]->moderators[$_POST["discussForumEdit"]["moderators"]] );
								$theSection->forums[$theSection->forumByID($value2)]->save();
							}
							
							$theModerators = array();
							if( is_array( $theSection->forums[$theSection->forumByID($value2)]->moderators ) && count( $theSection->forums[$theSection->forumByID($value2)]->moderators ) > 0 )
								$theModerators = $theSection->forums[$theSection->forumByID($value2)]->moderators;
							
							if( is_array( $theModerators ) && count( $theModerators ) > 0 )
							{
								foreach( $theModerators as $key3 => $value3 )
								{
									if( $_REQUEST["discuss_editForumID"] == $value2 || ( $_POST["discussForumEdit"]["forumID"] == $value2 && ( $_POST["add_moderator"] || $_POST["remove_moderator"] ) ) )
										$temp_moderators .= "
										<option value=\"$key3\">" . getUserInfo( $value3, "name" ) . "</option>";
									else
										$temp_moderators .= parse_profilelinks("%%$value3%%") . ", ";
								}
								if( ! ($_REQUEST["discuss_editForumID"] == $value2 || ( $_POST["discussForumEdit"]["forumID"] == $value2 && ( $_POST["add_moderator"] || $_POST["remove_moderator"] ) ) ) )
									$temp_moderators = substr($temp_moderators, 0, -2); // Remove the last comma...
							}
							else
							{
								if( $_REQUEST["discuss_editForumID"] == $value2 || ( $_POST["discussForumEdit"]["forumID"] == $value2 && ( $_POST["add_moderator"] || $_POST["remove_moderator"] ) ) )
								{
									$temp_moderators_disabled = " disabled";
									$temp_moderators = "
									<option>" . i18n("No moderators assigned to this forum") . "</option>";
								}
								else
									$temp_moderators = i18n("No moderators assigned to this forum");
							}
							
							if( $_REQUEST["discuss_editForumID"] == $value2 || ( $_POST["discussForumEdit"]["forumID"] == $value2 && ( $_POST["add_moderator"] || $_POST["remove_moderator"] ) ) )
							{
								$temp_users = allUsersArray();
								foreach( $temp_users as $user )
									$allUsersList .= "<option value=\"$user\">" . getUserInfo( $user, "name" ) . "</option>";
								
								foreach( $discuss_allowanceLevels as $level => $description )
								{
									if( $level == $theSection->forums[$theSection->forumByID($value2)]->postLevel )
										$postlevel_selected = " selected";
									else
										$postlevel_selected = "";
									
									$temp_setuppostlevel .= "<option value=\"$level\"$postlevel_selected>$description</option>";
									
									if( $level == $theSection->forums[$theSection->forumByID($value2)]->viewLevel )
										$viewlevel_selected = " selected";
									else
										$viewlevel_selected = "";
									
									$temp_setupviewlevel .= "<option value=\"$level\"$viewlevel_selected>$description</option>";
								}
								
								$discussSetup->contents .= "
								<form action=\"" . globalIDtoURL("setup/modulesetup/discuss/setup") . "#discussForumEdit\" method=\"post\">
								<input type=\"hidden\" name=\"discussForumEdit[sectionID]\" value=\"$value\" />
								<input type=\"hidden\" name=\"discussForumEdit[forumID]\" value=\"$value2\" />
								<div class=\"discussSetup_forumHeader\">
									<span class=\"discussSetup_forumTitle\">" . i18n("Forum title") . ":<a name=\"discussForumEdit\" class=\"namedanchor\"> </a></span><input class=\"discussSetup_forumTitle\" type=\"text\" name=\"discussForumEdit[title]\" value=\"$temp_title\" />
									<span class=\"discussSetup_forumDescription\">" . i18n("Forum title") . ": </span><input class=\"discussSetup_forumDescription\" type=\"text\" name=\"discussForumEdit[description]\" value=\"$temp_description\" />
									<span class=\"discussSetup_forumPostLevel\">" . i18n("Can post in the forum") . ": </span><select class=\"discussSetup_forumPostLevel\" name=\"discussForumEdit[postLevel]\">$temp_setuppostlevel</select>
									<span class=\"discussSetup_forumViewLevel\">" . i18n("Can view the forum") . ": </span><select class=\"discussSetup_forumViewLevel\" name=\"discussForumEdit[viewLevel]\">$temp_setupviewlevel</select>
									<input style=\"width: 99%\" type=\"submit\" name=\"save_forum\" value=\"" . i18n("Save") . "\"/>
									<span class=\"discussSetip_forumModeratorsHeader\">" . i18n("Moderators") . "</span>
									<span class=\"discussSetup_forumModerators\">" . i18n("Current moderators") . ": </span><select class=\"discussSetup_forumModerators\" name=\"discussForumEdit[moderators]\" $temp_moderators_disabled>$temp_moderators</select><input class=\"discussSetup_forumModeratorsRemove\" type=\"submit\" name=\"remove_moderator\" value=\"" . i18n("Remove selected") . "\"$temp_moderators_disabled />
									<span class=\"discussSetup_forumModeratorsAdd\">" . i18n("Add a moderator") . ": </span><select class=\"discussSetup_forumModeratorsAdd\" name=\"discussForumEdit[newModerator]>$allUsersList</select><input type=\"submit\" class=\"discussSetup_forumModeratorsAdd\" name=\"add_moderator\" value=\"" . i18n("Add selected") . "\" />
								</div>";
								$editThisForum = "
									" . drawCommand(i18n("Abort Edit") ,i18n("Abort Edit") , globalIDtoURL("setup/modulesetup/discuss/setup") );
							}
							else
							{
								$temp_postlevel = $discuss_allowanceLevels[$theSection->forums[$theSection->forumByID($value2)]->postLevel];
								$temp_viewlevel = $discuss_allowanceLevels[$theSection->forums[$theSection->forumByID($value2)]->viewLevel];
								
								if( $_REQUEST["discuss_forumDeleteID"] == $value2 )
									$discussSetup->contents .= "
								<div class=\"discussSetup_forumHeader\">
									<p class=\"wikicenteralign\"><a name=\"deleteConfirmPanel\" class=\"namedanchor\"> </a>" . i18n("Please confirm below that you really wish to delete this forum. Note that you will be unable to undo this, and all threads in the forum will be irreversibly deleted!") . "</p>
									<div class=\"wikicenteralign\">
									" . drawCommand(i18n("Confirm delete") ,i18n("Confirm delete") , globalIDtoURL("setup/modulesetup/discuss/setup/na/delete_forum", array("discuss_sectionID" => $value, "discuss_forumID" => $value2 ) ) ) . "
									" . drawCommand(i18n("Abort delete") ,i18n("Abort delete") , globalIDtoURL("setup/modulesetup/discuss/setup") ) . "
									</div>
									<hr />";
								else
									$discussSetup->contents .= "
								<div class=\"discuss_forumHeader\">
								";
								
								$discussSetup->contents .= "
									<span class=\"discuss_forumTitle\">$temp_title</span>
									<span class=\"discuss_forumDescription\">$temp_description</span>
									<span class=\"discuss_forumModerators\">" . i18n("Moderators") . ": $temp_moderators</span>
									<span class=\"discuss_forumLevels\">" . i18n("##0## can post new threads and ##1## can view existing threads in the forum.", array( $temp_postlevel, $temp_viewlevel ) ) . "</span>
								</div>";
								$editThisForum = "
									" . drawCommand(i18n("Edit") ,i18n("Edit this forum") , globalIDtoURL("setup/modulesetup/discuss/setup", array("discuss_editForumID" => "$value2#discussForumEdit" ) ) );
							}
							
							if( $key2 > 0 )
								$moveForumUp = drawCommand(i18n("Move up") ,i18n("Move the forum up one step") , globalIDtoURL("setup/modulesetup/discuss/setup/na/move_forum_up", array("discuss_sectionID" => $value, "discuss_forumID" => $value2 ) ) );
							else
								$moveForumUp = "<acronym title=\"" . i18n("Alrady topmost") . "\">[" . i18n("Move up") . "]</acronym>";
							
							if( count( $theSection->forums ) > $key2 + 1 )
								$moveForumDown = drawCommand(i18n("Move down") ,i18n("Move down") , globalIDtoURL("setup/modulesetup/discuss/setup/na/move_forum_down", array("discuss_sectionID" => $value, "discuss_forumID" => $value2 ) ) );
							else
								$moveForumDown = "<acronym title=\"" . i18n("Alrady bottommost") . "\">[" . i18n("Move down") . "]</acronym>";
							
							$discussSetup->contents .= "
								<span class=\"discussSetup_forumCommands\">
									<acronym title=\"Not implemented, sorry\">[Move to another section]</acronym> &nbsp; &nbsp;
									" . drawCommand( i18n("Delete"), i18n("Delete"), globalIDtoURL("setup/modulesetup/discuss/setup", array("discuss_sectionID" => $value, "discuss_forumDeleteID" => "$value2#deleteConfirmPanel" ) ) ) . " &nbsp; &nbsp;
									$moveForumUp
									$moveForumDown &nbsp; &nbsp;
									$editThisForum
								</span>";
						}
					}
					else
					{
						$discussSetup->contents .= "
						<small class=\"comment\">" . i18n("There are currently no forums in this section.") . "</small>";
					}
					$discussSetup->contents .= "
					</div>";
				}
				$discussSetup->contents = $error_description . $discussSetup->contents;
			}
			else
			{
				$discussSetup->contents = renderInformationBox(
						i18n("Welcome to the Discuss forum setup"),
						i18n("Here is where you set up forums, sections and the options retaining to them, such as viewing and posting allowance levels, moderators and so on. You have not yet added any sections. Please do so by clicking on this ##0##[Add Section]##1## command link. Please also make sure to check out ##2##the options##1## before doing anything further.", array("<a class=\"command\" href=\"" . globalIDtoURL("setup/modulesetup/discuss/setup/na/add_section") . "\">", "</a>", "<a href=\"" . globalIDtoURL("setup/modulesetup/discuss/options") . "\">"))
					);
			}
		}
		return $discussSetup->renderTabWidget();
	}
	else
	{
		return "<div>D'oh! Who do you think you are, you are not authorised to come in here - and you can only click your way here if you're allowed. Hacker!</div>";
	}
}


/**
 * Return an array of data about the thumbnail for a global ID
 * As the discussions don't have thumbnails per se, we simply pass a fitting image directly back
 *
 * @param     splitID        The global ID to check
 *
 * @return    Array containing array
 */
function globalIDThumbnail_discuss( $splitID )
{
	global $module_folder;
	$data = array( "uri" => siteURL(true) . "$module_folder/discuss/thumb.png", "width" => 64, "height" => 64 );
	return $data;
}
?>
