<?PHP
/**
 * The discussForum class
 * GlobalID = discuss/sectionID/forumID
 */
class discussForum extends lockableClass
{
	var $sectionID;      ///< long            The ID of the section containing the forum
	var $forumID;        ///< long            The forum's ID
	var $globalID;       ///< string          The forum's global ID
	
	var $title;          ///< string          The title of the forum
	var $description;    ///< string          The description of the forum - a single line of text, without any type of formatting
	
	var $threads;        ///< array           The threads contained in this forum - array of discussThread objects
	var $threadsCount;   ///< integer         The amount of threads (for speed reasons - if we're not showing the threads' data, don't load them)
	
	var $moderators;     ///< array           Contains the usernames of the forum's moderators
	var $postLevel;      ///< integer         Comment level - Anonymous/Registered only/Moderators only (defaults to registered)
	var $viewLevel;      ///< integer         Viewing level - Anonymous/Registered only/Moderators only (defaults to everybody)
	
	var $lastChanged;    ///< long            Timestamp - when did the last change happen?
	var $lastChangedBy;  ///< string          Username of the person who last changed the forum (added a thread or comment, edited a thread or comment)
	var $isChanged;      ///< bool            Has the forum been changed?
	var $parentSection;  ///< discussSection  If there is a parent section to the forum, it can be accessed through here
	var $thisForumURL;   ///< string          The URL used to access this forum
	
	/**
	 * discussForum Constructor
	 *
	 * @param sectionID     Which section are we loading
	 * @param forumID       (optional) Which forum are we loading - Omit this if you are creating a new forum
	 */
	function discussForum($sectionID, $forumID)
	{
		global $discussContentDirectory;
		$this->sectionID = $sectionID;
		$this->forumID = $forumID;
		$this->lockfile = "$discussContentDirectory/{$this->sectionID}/forums/{$this->forumID}/lock";
		
		if( $this->lock() )
		{
			$this->load();
			if( isAllowed("modules_discuss_admin") && $_POST["discussForumEdit"]["sectionID"] == $sectionID && $_POST["discussForumEdit"]["forumID"] == $forumID && $_POST["save_forum"] )
			{
				$this->title = stripslashes($_POST["discussForumEdit"]["title"]);
				$this->description = stripslashes($_POST["discussForumEdit"]["description"]);
				$this->postLevel = $_POST["discussForumEdit"]["postLevel"];
				$this->viewLevel = $_POST["discussForumEdit"]["viewLevel"];
				$this->save();
			}
			$this->unlock();
		}
		
		$this->globalID = "discuss/$sectionID/$forumID";
		$this->thisForumURL = globalIDtoURL($this->globalID);
	}
	
	/**
	 * Load the forum options and such
	 */
	function load()
	{
		global $discussContentDirectory;
		
		if( file_exists( "$discussContentDirectory/{$this->sectionID}/forums/{$this->forumID}/options.php" ) )
			include( "$discussContentDirectory/{$this->sectionID}/forums/{$this->forumID}/options.php" );
		
		$this->title = $forum_options["title"];
		$this->moderators = $forum_options["moderators"];
		$this->description = $forum_options["description"];
		$this->postLevel = $forum_options["postLevel"];
		$this->viewLevel = $forum_options["viewLevel"];
		$this->lastChanged = $forum_options["lastChanged"];
		$this->lastChangedBy = $forum_options["lastChangedBy"];
		// Make sure we at least have something /resembling/ a post count... Even if there's not been any posts made yet
		$this->threadsCount = ($forum_options["threadsCount"] > 0) ? $forum_options["threadsCount"] : 0;
	}
	
	function save()
	{
		global $discussContentDirectory;
		if( file_exists( "$discussContentDirectory/{$this->sectionID}/forums/{$this->forumID}/options.php" ) )
			unlink( "$discussContentDirectory/{$this->sectionID}/forums/{$this->forumID}/options.php" );
		
		$forum_options["title"] = $this->title;
		$forum_options["description"] = $this->description;
		$forum_options["moderators"] = $this->moderators;
		$forum_options["postLevel"] = $this->postLevel;
		$forum_options["viewLevel"] = $this->viewLevel;
		$forum_options["lastChanged"] = $this->lastChanged;
		$forum_options["lastChangedBy"] = $this->lastChangedBy;
		$forum_options["threadsCount"] = $this->threadsCount;
		
		file_put_contents( "$discussContentDirectory/{$this->sectionID}/forums/{$this->forumID}/options.php", array_export( $forum_options, "forum_options" ) );
	}
	
	/**
	 * Load the threads contained within the forum
	 */
	function loadThreads()
	{
		global $discussContentDirectory;
		
		$fPath = "$discussContentDirectory/{$this->sectionID}/forums/{$this->forumID}/threads";
		
		$this->threads = array();
		if( is_dir( $fPath ) )
		{
			// The following snippet by web at fischenich dot org on the php.net manual
			if( $handle=opendir($fPath) )
			{
				$filetlist = array();
				// Get file listing, sorted file filemtime and ID {
				while( $file = readdir( $handle ) )
				{
					if( substr($file, 0, 1) == '.' )
						continue;
					
					// The position is the filemtime and the ID
					$position = filemtime( $fPath . '/' . $file ) . "." . substr( $file, 0, -4 );
					$filetlist[$position] = $file;
				}
				krsort($filetlist);
				// Get file listing, sorted file filemtime and ID }
				
				while( list($key, $val) = each($filetlist) )
					$this->threads[$key] = new discussThread( $this->sectionID, $this->forumID, substr( $val, 0, -4 ), &$this );
			}
		}
		
		if( $_POST["discussForumEdit"]["sectionID"] == $sectionID && $_POST["discussForumEdit"]["forumID"] == $forumID && $_REQUEST["discuss_deleteSuccess"] == "true" )
		{
			if( $this->lock() )
			{
				$this->load();
				$this->threadsCount = count( $this->threads );
				$this->save();
				$this->unlock();
			}
		}
	}
	
	/**
	 * Returns true if the current user is a moderator
	 */
	function isModerator()
	{
		if( count( $this->moderators ) > 0 )
			foreach( $this->moderators as $key => $value )
				if( $key == currentUser() )
					return true;
		
		// Admin can always moderate
		if( isAllowed( currentUser(), "global_admin" ) )
			return true;
		
		return false;
	}
	
	function userCanView()
	{
		// Everybody can view, even anonymous users - no need to check further...
		if( $this->viewLevel == 1 )
			return true;
		
		// User is logged in, and only registered users can view...
		if( isAuth() && $this->viewLevel == 2 )
			return true;
		
		// User is moderator...
		if( $this->isModerator() )
			return true;
		
		// Admin can always view
		if( isAllowed( currentUser(), "global_admin" ) )
			return true;
		
		// Otherwise...
		return false;
	}
	
	function userCanPost()
	{
		// Everybody can post, even anonymous users - no need to check further...
		if( $this->postLevel == 1 )
			return true;
		
		// User is logged in, and only registered users can post...
		if( isAuth() && $this->postLevel == 2 )
			return true;
		
		// User is moderator...
		if( $this->isModerator() )
			return true;
		
		// Admin can always post
		if( isAllowed( currentUser(), "global_admin" ) )
			return true;
		
		// Otherwise...
		return false;
	}
	
	/**
	 * Render small block for showing in the section overview
	 */
	function renderBlock()
	{
		global $discuss_allowanceLevels;
		
		if( $this->userCanView() )
		{
			if( is_array( $this->moderators ) && count( $theModerators ) > 0 )
			{
				foreach( $theModerators as $key3 => $value3 )
					$temp_moderators .= parse_page_data("%%$value3%%") . ", ";
				
				$temp_moderators = substr($temp_moderators, 0, -2); // Remove the last comma...
			}
			else
				$temp_moderators = i18n("No moderators assigned to this forum");
			
			$temp_postlevel = $discuss_allowanceLevels[$this->postLevel];
			$temp_viewlevel = $discuss_allowanceLevels[$this->viewLevel];
			
			$data = "
					<div class=\"discuss_forumHeader\">
						<a href=\"" . globalIDtoURL($this->globalID) . "\" class=\"discuss_forumHeader\">
						<span class=\"discuss_forumLink\">" . i18n("Go") . " &raquo; </span>
						<span class=\"discuss_forumTitle\">{$this->title}</span>
						<span class=\"discuss_forumDescription\">{$this->description}</span></a>
						<span class=\"discuss_forumNumberOfThreads\">" . i18n("Number of threads: ##0##", array($this->threadsCount) ) . "</span>
						<span class=\"discuss_forumLastChanged\">" . i18n("Last changed by ##0## on ##1##", array( parse_profilelinks("%%" . $this->lastChangedBy . "%%"), formatTime($this->lastChanged) ) ) . "</span>";
			
			// Check wether the user wants to show the details or not...
			if( getUserInfo( currentUser(), "discuss_verboseListings" ) != 1 )
			{
				$data .= "
						<span class=\"discuss_forumModerators\">" . i18n("Moderators") . ": $temp_moderators</span>
						<span class=\"discuss_forumLevels\">" . i18n("##0## can post new threads and ##1## can view existing threads in the forum.", array( $temp_postlevel, $temp_viewlevel ) ) . "</span>";
			}
			
			$data.= "</div>";
				
		
			return $data;
		}
	}
	
	/**
	 * Renders a dialog for adding a new thread to the current forum
	 */
	function renderNewThread()
	{
		global $discuss_options, $discuss_defaultoptions;
		// If the thread is to be previewed, do this, and fill the editing fields with the data
		if( $_REQUEST["discuss_preview"] )
		{
			$thread_data["title"] = stripslashes($_POST["discussEdit"]["title"]);
			$thread_data["description"] = stripslashes($_POST["discussEdit"]["description"]);
			$thread_data["content"] = stripslashes($_POST["discussEdit"]["content"]);
			$thread_data["viewLevel"] = $_POST["discussEdit"]["viewLevel"];
			$thread_data["replyLevel"] = $_POST["discussEdit"]["replyLevel"];
			$thread_data["editLevel"] = $_POST["discussEdit"]["editLevel"];
			if( $this->isModerator() )
				$thread_data["isSticky"] = $_POST["discussEdit"]["isSticky"] ? 1 : 0;
			else
				$thread_data["isSticky"] = false;
		}
		else
		{
			$thread_data["title"] = i18n("The thread's title");
			$thread_data["description"] = ""; // This is optional
			$thread_data["content"] = i18n("Enter your post's content here.") . "\n---\n" . getUserInfo(currentUser(), "signature", "" );
			$thread_data["isSticky"] = false;
			$thread_data["viewLevel"] = getUserInfo( currentUser(), "discuss_viewLevel", $discuss_defaultoptions["user"]["viewLevel"] );
			$thread_data["replyLevel"] = getUserInfo( currentUser(), "discuss_replyLevel", $discuss_defaultoptions["user"]["replyLevel"] );
			$thread_data["editLevel"] = getUserInfo( currentUser(), "discuss_editLevel", $discuss_defaultoptions["user"]["editLevel"] );
		}
		$thread_data["owner"] = currentUser();
		
		// If the thread is to be published, do this and redirect to showing it...
		if( $_POST["discuss_saveThread"] )
		{
			$timestamp = time();
			
			$discuss_options["counters"]["thread"]++;
			$newThreadID = $discuss_options["counters"]["thread"];
			discuss_saveOptions();
			
			$newThreadKey = $timestamp . "." . $newThreadID;
			
			if( $this->lock() )
			{
				$this->load();
				$this->threads[$newThreadKey] = new discussThread($this->sectionID, $this->forumID);
				$this->threads[$newThreadKey]->parentForum = &$this;
				
				if( isAuth() )
					$this->threads[$newThreadKey]->owner = stripslashes($_POST["discussEdit"]["owner"]);
				else
					$this->threads[$newThreadKey]->owner = i18n("Anonymous user");
				
				$this->threads[$newThreadKey]->threadID = $newThreadID;
				$this->threads[$newThreadKey]->makeFilename();
				$this->threads[$newThreadKey]->created = $timestamp;
				$this->threads[$newThreadKey]->title = stripslashes($_POST["discussEdit"]["title"]);
				$this->threads[$newThreadKey]->description = stripslashes($_POST["discussEdit"]["description"]);
				$this->threads[$newThreadKey]->content = stripslashes($_POST["discussEdit"]["content"]);
				$this->threads[$newThreadKey]->viewLevel = $_POST["discussEdit"]["viewLevel"];
				$this->threads[$newThreadKey]->replyLevel = $_POST["discussEdit"]["replyLevel"];
				$this->threads[$newThreadKey]->editLevel = $_POST["discussEdit"]["editLevel"];
				$this->threads[$newThreadKey]->lastReplyBy = $this->threads[$newThreadKey]->owner;
				$this->threads[$newThreadKey]->lastChangedBy = $this->threads[$newThreadKey]->owner;
				$this->threads[$newThreadKey]->replies = array();
				$this->threads[$newThreadKey]->repliesCount = 0;
				$this->threads[$newThreadKey]->lastChanged = $this->threads[$newThreadKey]->created;
				$this->threads[$newThreadKey]->newThread = true;
				
				if( $this->isModerator() )
					$this->threads[$newThreadKey]->isSticky = $_POST["discussEdit"]["isSticky"] ? 1 : 0;
				else
					$this->threads[$newThreadKey]->isSticky = false;
				
				$this->threads[$newThreadKey]->save(true);
				
				$this->threadsCount++;
				$this->save();
				$this->unlock();
				
				header("Location: " . globalIDtoURL("discuss/" . $this->sectionID . "/" . $this->forumID . "/" . $newThreadID) );
			}
			else
				die( renderInformationBox( i18n("Timeout error!"), i18n("Too many people are attempting to create a new thread in the forum. Please try again.") ) );
			return true;
		}
		
		// Otherwise, simply show the editing box
		return discussThread::renderThreadEditor($thread_data, $this);
	}
	
	/**
	 * Render the complete forum
	 */
	function render()
	{
		global $subscriptions;
		
		if( $this->userCanView() )
		{
			if( $_REQUEST["discuss_sectionID"] == $this->sectionID && $_REQUEST["discuss_forumID"] == $this->forumID && isset( $_REQUEST["discuss_threadID"] ) ) // Is there a thread to be shown?
			{
				$theThread = new discussThread( $this->sectionID, $this->forumID, $_REQUEST["discuss_threadID"], &$this );
				return $theThread->render();
			}
			else if( $_REQUEST["discuss_sectionID"] == $this->sectionID && $_REQUEST["discuss_forumID"] == $this->forumID && isset( $_REQUEST["discuss_newThread"] ) ) // Are we going to create a new thread?
			{
				return "<span
						class=\"discuss_Breadcrumbs\">
							" . i18n("You are here:") . "
							<a href=\"" . globalIDtoURL("discuss") . "\" class=\"discuss_forumBreadcrumbsToTop\">" . i18n("Forum home") . "</a> &raquo;
							<a href=\"" . globalIDtoURL($this->globalID) . "\" class=\"discuss_forumBreadcrumbsForum\">{$this->title}</a> &raquo;
							" . i18n("New thread") . "
							</span><span
						class=\"discuss_Navigator\">&nbsp;
						$rendered_navigator
					</span>" . $this->renderNewThread();
			}
			else if( ! isset( $_REQUEST["discuss_threadID"] ) ) // Unless we're showing some thread in another forum, render the forum proper...
			{
				// Show forum header (title, description, navigator if paginated, breadcrumbs)
				if( true ) // Navigator is rendered when...
				{
					$rendered_navigator = "";
				}
				
				if( $this->userCanPost() )
					$postControl = "
						<span class=\"discuss_forumAddThread\"><a href=\"" . thisPageURL( false, true, array( "discuss_newThread" => "true" ) ) . "\" class=\"command discuss_forumNewThread\">[" . i18n("New thread") . "]</a></span>";
				
				$rendered_content = "<span
						class=\"discuss_Breadcrumbs\">
							" . i18n("You are here:") . "
							<a href=\"" . globalIDtoURL("discuss") . "\" class=\"discuss_forumBreadcrumbsToTop\">" . i18n("Forum home") . "</a> &raquo;
							{$this->title}
							</span><span
						class=\"discuss_Navigator\">&nbsp;
						$rendered_navigator
					</span>
					<div class=\"discuss_forumHeader\"><span
						class=\"discuss_forumTitle\">{$this->title}</span><span
						class=\"discuss_forumDescription\">{$this->description}
					</span></div>
					<div class=\"discuss_forumCommands\">
						" . $subscriptions->renderSubscribeControl( $this->globalID, false, " discussSubscribeControl" ) . "$postControl
					</div>";
				// Render threads in whatever limit imposed by pagination
				if( $this->threadsCount > 0 )
				{
					/*$keyBegin = 0;
					$keyEnd = endKey($this->threads);
					for( $i = $keyBegin; $i <= $keyEnd; $i++ )*/
					foreach( $this->threads as $i => $thread )
						$rendered_content .= $this->threads[$i]->renderBlock();
				}
				else if( $this->userCanPost() )
					$rendered_content .= renderInformationBox(
							i18n("This forum is empty"),
							i18n("This is where you would see the threads, if this forum contained any. If you want to be the first person to create a thread in here, ##0##please click here##1## and start filling out the form!", array( "<a href=\"" . thisPageURL( false, true, array( "discuss_newThread" => "true" ) ) . "\">", "</a>" ) )
						);
				else
					$rendered_content .= renderInformationBox( i18n("This forum is empty"), i18n("This is where you would see the threads, if this forum contained any.") );
				
				return $rendered_content;
			}
		}
		else
			return "";
	}
}
?>