<?PHP
/**
 * The discussSection class
 * GlobalID = discuss/sectionID
 */
class discussSection
{
	var $sectionID;     ///< long     The section's ID
	var $globalID;      ///< string   The globalID
	var $title;         ///< string   The title of the forum
	var $description;   ///< string   The description of the forum - a single line of text, without any type of formatting
	var $viewLevel;     ///< integer  Viewing allowance level - Anonymous/Registered only (defaults to everybody)
	var $forums;        ///< array    The forums contained in this section, in order of appearance
	var $forumPositions;///< array    The order of the forums contained in this section (this is the option)
	/// \private
	var $forumKeys;     ///< array    Internally used backways lookup table for forums...
	
	function discussSection($sectionID)
	{
		if( $_POST["discussSectionEdit"]["sectionID"] == $sectionID && $_POST["save_section"] && isAllowed("modules_discuss_admin") ) // Then we're saving edited information...
		{
			$this->sectionID = $_POST["discussSectionEdit"]["sectionID"];
			$this->load();
			$this->title = stripslashes($_POST["discussSectionEdit"]["title"]);
			$this->description = stripslashes($_POST["discussSectionEdit"]["description"]);
			if( $_POST["discussSectionEdit"]["viewLevel"] )
				$this->viewLevel = 1;
			else
				$this->viewLevel = 0;
			
			$this->save();
		}
		else
		{
			$this->sectionID = $sectionID;
			$this->load();
		}
		$this->globalID = "discuss/$sectionID";
	}
	
	function load()
	{
		global $discussContentDirectory;
		
		if( file_exists( "$discussContentDirectory/{$this->sectionID}/options.php" ) )
			include( "$discussContentDirectory/{$this->sectionID}/options.php" );
		
		$this->title = $section_options["title"];
		$this->description = $section_options["description"];
		$this->viewLevel = $section_options["viewLevel"];
		$this->forumPositions = $section_options["forumPositions"];
		
		foreach( $this->forumPositions as $key => $value )
		{
			$this->forumKeys[$value] = $key; // For reverse lookup...
			$this->forums[$key] = new discussForum( $this->sectionID, $value );
			$this->forums[$key]->parentSection =& $this;
		}
	}

	function save()
	{
		global $discussContentDirectory;
		
		if( file_exists( "$discussContentDirectory/{$this->sectionID}/options.php" ) )
			unlink( "$discussContentDirectory/{$this->sectionID}/options.php" );
		
		$section_options["title"] = $this->title;
		$section_options["description"] = $this->description;
		$section_options["viewLevel"] = $this->viewLevel;
		$section_options["forumPositions"] = $this->forumPositions;
		
		file_put_contents( "$discussContentDirectory/{$this->sectionID}/options.php", array_export( $section_options, "section_options" ) );
	}
	
	/**
	 * Internal convenience function for reverse forum id lookup
	 */
	function forumByID( $forumID )
	{
		return $this->forumKeys[$forumID];
	}
	
	function userCanView()
	{
		// Everybody can view, even anonymous users - no need to check further...
		if( $this->viewLevel == 1 )
			return true;
		
		// Only registered users can view this section
		if( isAuth() )
			return true;
		
		// Otherwise...
		return false;
	}
	
	function render()
	{
		global $subscriptions;
		
		if( $_REQUEST["discuss_moveSuccess"] == "true" )
			discuss_rebuildUserCache();
		
		if( $this->userCanView() )
		{
			if( $_REQUEST["discuss_sectionID"] == $this->sectionID && isset( $_REQUEST["discuss_forumID"] )) // Is there a forum to be shown? (threads are handled in the forum renderer)
			{
				if( !isset( $_REQUEST["discuss_threadID"] ) )
					$this->forums[$this->forumByID($_REQUEST["discuss_forumID"])]->loadThreads();
				return $this->forums[$this->forumByID($_REQUEST["discuss_forumID"])]->render();
			}
			else if( ! isset( $_REQUEST["discuss_forumID"] ) ) // Unless we're showing some forum in another section, render the section proper...
			{
				// Render the section header
				$rendered_content = "
				<div class=\"discuss_sectionContainer\">
					<div class=\"discuss_sectionHeader\">
						<span class=\"discuss_sectionSubscribe\">" . $subscriptions->renderSubscribeControl( $this->globalID, false, " discussSubscribeControl" ) . "</span>
						<span class=\"discuss_sectionTitle\">{$this->title}</span>
						<span class=\"discuss_sectionDescription\">{$this->description}</span>
					</div>";
				// Render the forum blocks for each forum contained within
				foreach( $this->forumPositions as $position => $positionID )
					$rendered_forums .= $this->forums[$this->forumByID($positionID)]->renderBlock();
				
				// End the section container
				$rendered_content .= "$rendered_forums
				</div>";
			
				// This means there's no visible forums in there - that means there's nothing to see in the section. So... don't show it ;)
				if( $rendered_forums == "" )
					$rendered_content = "";
			}
		}
		return $rendered_content;
	}
}
?>