<?PHP
/**
 * The discussThread class
 * GlobalID = discuss/sectionID/forumID/threadID
 */
class discussThread extends lockableClass
{
	var $sectionID;             ///< long         The ID of the section containing the forum containing the thread
	var $forumID;               ///< long         The ID of the forum containing the thread
	var $threadID;              ///< string       The thread's ID - counter in $discuss_options
	var $globalID;              ///< string       The thread's globalID
	
	var $title;                 ///< string       The title of the thread
	var $content;               ///< string       The contents of the initial post
	var $replies;               ///< array        Comment data
	var $lastReplyBy;           ///< string       Username of the last user that commented, or Anonymous
	var $repliesCount;          ///< integer      For performance reasons, for use with renderBlock() so that we don't have to load the whole comments class each time
	
	var $created;               ///< long         Timestamp - When was the thread created?
	var $lastChanged;           ///< long         Timestamp - When was the thread last changed?
	var $lastChangedBy;         ///< string       The username of the last user who edited the thread's top level post
	var $isSticky;              ///< bool         Is this a sticky thread?
	
	var $owner;                 ///< string       The owner - view, reply and edit level are relative to this
	var $viewLevel;             ///< integer      Viewing level (defaults to everybody - 0)
	var $replyLevel;            ///< integer      Comment level (defaults to registered - 1)
	var $editLevel;             ///< integer      Editing level (defaults to admin - 4)
	
	var $filename;              ///< string       The thread's filename
	var $thisThreadURL;         ///< string       The thread's URL, if there's a parent forum
	var $isChanged = false;     ///< bool         Has the thread been changed?
	var $changeIsReply = false; ///< bool         Is the thread's change (above) a reply?
	var $newThread = false;     ///< bool         Is this a new thread? Set by the parent forum
	var $parentForum = 0;       ///< discussForum If the thread is called by a forum, this should be set ByRef by the calling forum
	
	/**
	 * discussThread Constructor
	 *
	 * @param sectionID     The section which contains the forum which contains the thread
	 * @param forumID       The forum which contains the thread
	 * @param threadID      (optional) The thread's ID - Omit this if you are creating a new thread
	 *
	 * @return void
	 */
	function discussThread($sectionID, $forumID, $threadID = 0, $parentForum = null)
	{
		global $globalID, $discussContentDirectory;
		
		$this->sectionID = $sectionID;
		$this->forumID = $forumID;
		$this->threadID = $threadID;
		
		if( $parentForum != null )
			$this->parentForum = &$parentForum;
		
		$this->makeFilename();
		if( ! $this->lock() )
			die( renderInformationBox( i18n("Timeout!"), i18n("Too many users are attempting to access this thread. Please try again.") ) );
		$this->load();
		
		if( $this->userCanEdit() && $_POST["discussEdit"]["threadID"] == $this->threadID ) // Are we editing this thread?
		{
			$this->title = stripslashes($_POST["discussEdit"]["title"]);
			$this->description = stripslashes($_POST["discussEdit"]["description"]);
			$this->content = stripslashes($_POST["discussEdit"]["content"]);
			$this->isSticky = $_POST["discussEdit"]["isSticky"] ? 1 : 0;
			$this->owner = $_POST["discussEdit"]["owner"];
			$this->viewLevel = $_POST["discussEdit"]["viewLevel"];
			$this->replyLevel = $_POST["discussEdit"]["replyLevel"];
			$this->editLevel = $_POST["discussEdit"]["editLevel"];
			$this->lastChanged = time();
			$this->lastChangedBy = currentUser();
			
			if( $_POST["discuss_saveThread"] )
				$this->isChanged = true;
		}
		if( $threadID != 0 )
			$this->save();
		$this->unlock();
		
		$this->globalID = "discuss/$sectionID/$forumID/$threadID";
		$this->thisThreadURL = globalIDtoURL($this->globalID);
		
		if( $this->userCanEdit() && $_REQUEST["discuss_deleteThreadConfirm"] == "true" && $globalID == $this->globalID )
		{
			$errorHandler = new errorHandler( $this->thisThreadURL );
			$errorHandler->handle( i18n("Attempting to delete the thread"), $this->delete() );
			if( $errorHandler->handled == false )
				header( "Location: " . str_replace( "&amp;", "&", globalIDtoURL( "discuss/$sectionID/$forumID", array( "discuss_deleteSuccess" => "true" ) ) ) );
			else
				$this->content = $errorHandler->render() . $this->content;
		}
		else if( $this->parentForum != null && $this->parentForum->isModerator() && $_POST["move_thread"] && $this->parentForum->globalID != $_POST["moveto"] )
		{
			list( $top, $otherSectionID, $otherForumID ) = explode("/", $_POST["moveto"]);
			$otherForum = new discussForum($otherSectionID, $otherForumID);
			// You can't move a post to a forum you are not moderator of... Just for checking, really, but still applies!
			if( $otherForum->isModerator() )
			{
				$movefrom = "$discussContentDirectory/{$this->sectionID}/forums/{$this->forumID}/threads/{$this->threadID}.php";
				$moveto = "$discussContentDirectory/{$otherForum->sectionID}/forums/{$otherForum->forumID}/threads/{$this->threadID}.php";
				
				RecursiveMkdir("$discussContentDirectory/{$otherForum->sectionID}/forums/{$otherForum->forumID}/threads");
				
				if( rename( $movefrom, $moveto ) )
				{
					if( $this->parentForum->lock() )
					{
						if( $otherForum->lock() )
						{
							$this->parentForum->threadsCount--;
							$this->parentForum->lastChanged = time();
							$this->parentForum->lastChangedBy = currentUser();
							$this->parentForum->save();
							if( $this->parentForum->threadsCount == 0 )
								rmdir("$discussContentDirectory/{$this->sectionID}/forums/{$this->forumID}/threads");
							
							$otherForum->threadsCount++;
							$otherForum->lastChanged = time();
							$otherForum->lastChangedBy = currentUser();
							$otherForum->save();
							$otherForum->unlock();
						}
						else
							die( renderInformationBox( i18n("Timeout error!"), i18n("Too many people are attempting to access the forum. Please try again.") ) );
						
						$this->parentForum->unlock();
					}
					else
						die( renderInformationBox( i18n("Timeout error!"), i18n("Too many people are attempting to access the forum. Please try again.") ) );
					header("Location: " . $otherForum->thisForumURL . "&discuss_moveSuccess=true");
				}
			}
		}
	}
	
	/**
	 * Loads the thread data from disc
	 *
	 * @return  bool Wether we successfully loaded the thread data
	 */
	function load()
	{
		if( file_exists( $this->filename ) )
		{
			include( $this->filename );
			$this->title = $thread_data["title"];
			$this->description = $thread_data["description"];
			$this->content = $thread_data["content"];
			$this->replies = $thread_data["replies"];
			$this->lastReplyBy = $thread_data["lastReplyBy"];
			$this->repliesCount = $thread_data["repliesCount"];
			$this->isSticky = $thread_data["isSticky"];
			$this->owner = $thread_data["owner"];
			$this->viewLevel = $thread_data["viewLevel"];
			$this->replyLevel = $thread_data["replyLevel"];
			$this->editLevel = $thread_data["editLevel"];
			$this->created = $thread_data["created"];
			$this->lastChanged = $thread_data["lastChanged"];
			$this->lastChangedBy = $thread_data["lastChangedBy"];
			return true;
		}
		else
		{
			return false;
		}
	}
	
	
	/**
	 * Saves the thread back to disc, when edited, or in any other way worked on...
	 *
	 * @return bool Wether we successfully saved the thread (or if there were no changes to save)
	 */
	function save( $parentLocked = false )
	{
		global $subscriptions;
		if( $this->isChanged )
		{
			if( $parentLocked || $this->parentForum->lock() )
			{
				$thread_data["title"] = $this->title;
				$thread_data["description"] = $this->description;
				$thread_data["content"] = $this->content;
				$thread_data["replies"] = $this->replies;
				$thread_data["lastReplyBy"] = $this->lastReplyBy;
				$thread_data["repliesCount"] = $this->repliesCount;
				$thread_data["isSticky"] = $this->isSticky;
				$thread_data["owner"] = $this->owner;
				$thread_data["viewLevel"] = $this->viewLevel;
				$thread_data["replyLevel"] = $this->replyLevel;
				$thread_data["editLevel"] = $this->editLevel;
				$thread_data["created"] = $this->created;
				$thread_data["lastChanged"] = $this->lastChanged;
				$thread_data["lastChangedBy"] = $this->lastChangedBy;
				
				if( file_exists( $this->filename ) )
					if( !unlink( $this->filename ) )
						return false;
				
				$postdata = array(
					"title" => $thread_data["title"],
					"viewLevel" => $thread_data["viewLevel"],
					"timestamp" => $thread_data["lastChanged"],
					"sectionID" => $this->sectionID,
					"forumID" => $this->forumID,
					"threadID" => $this->threadID
				);
				
				if( $this->newThread )
				{
					$subject = i18n( "The thread ##0## was created (in ##1##)", array( $this->title, $this->parentForum->title ) );
					$message = i18n( "The text in ##0## is as follows:", array( $this->title ) ) . "\n" . $this->content . "\n---\n" . globalIDtoURL($this->globalID);
				}
				else if( $this->changeIsReply )
				{
					$subject = i18n( "The thread ##0## has a new comment by ##1## (in ##2##)", array( $this->title, getUserInfo( $this->lastReplyBy, "name" ), $this->parentForum->title ) );
					$message = i18n( "The comment text is as follows:" ) . "\n" . $this->lastReply . "\n---\n" . globalIDtoURL($this->globalID . "#" . $_REQUEST["newcommentid"]);
					$postdata["title"] = i18n("[Commented] ##0##", array( $postdata["title"] ) );
					$postdata["commentID"] = $_REQUEST["newcommentid"];
				}
				else
				{
					$subject = i18n( "The thread ##0## was edited (in ##1##)", array( $this->title, $this->parentForum->title ) );
					$message = i18n( "The new text in ##0## is as follows:", array( $this->title ) ) . "\n" . $this->content . "\n---\n" . globalIDtoURL($this->globalID);
					$postdata["title"] = i18n("[Edited] ##0##", array( $postdata["title"] ) );
				}
				$subscriptions->handle( $this->globalID, $this->owner, $this->viewLevel, $subject, $message );
				
				if( isAuth() )
				{
					$userposts = new discuss_userPosts( $this->owner );
					$userposts->rotate( $postdata );
					unset( $userposts );
				}
				
				$this->parentForum->load();
				$this->parentForum->lastChanged = $this->lastChanged;
				$this->parentForum->lastChangedBy = $this->lastChangedBy;
				$this->parentForum->save();
				// We really should not be unlocking te parent forum if the caller locked the forum... could cause some horrible stuff :)
				if( $parentLocked == false )
					$this->parentForum->unlock();
			}
			else
				die( renderInformationBox( i18n("Timeout error!"), i18n("Too many people are attempting to access the forum. Please try again.") ) );
			
			return file_put_contents( $this->filename, array_export( $thread_data, "thread_data" ) );
		}
	}
	
	/**
	 * Deletes the thread from disc
	 * NB: Remember to delete it from the currently shown data as well, otherwise
	 * we'll show non-existant data (for example, reload a forum, or delete the thread item directly)
	 *
	 * @return bool Wether we successfully deleted the thread
	 */
	function delete()
	{
		global $subscriptions;
		$subscriptions->removeGlobalID( $this->globalID );
		
		if( file_exists( $this->filename ) )
			return unlink( $this->filename );
	}
	
	/**
	 * Moves the thread to a different forum
	 *
	 * @param  whereto      Which forum to we move to?
	 *
	 * @return bool Was it successfully moved?
	 */
	function move($whereto)
	{
		if( $this->delete() )
		{
			$this->forumID = $whereto;
			$this->isChanged = true;
			$this->makeFilename();
			return $this->save();
		}
		return false;
	}
	
	/**
	 * Function used to determine wether the current user is allowed to edit this...
	 */
	function userCanEdit()
	{
		if( $this->owner == currentUser() )
			return true;
		
		if( userAllows($this->owner, $this->editLevel ) )
			return true;
		
		if( $this->parentForum != 0 && $this->parentForum->isModerator() )
			return true;
		
		// Admin can always edit
		if( isAllowed( "global_admin" ) )
			return true;
		
		// Nothing was caught, the user is not allowed to edit...
		return false;
	}
	
	/**
	 * Function used to determine wether the current user is allowed to edit this...
	 */
	function userCanView()
	{
		if( $this->owner == currentUser() )
			return true;
		
		if( userAllows($this->owner, $this->viewLevel ) )
			return true;
		
		if( $this->parentForum != 0 && $this->parentForum->isModerator() )
			return true;
		
		// Admin can always edit
		if( isAllowed( "global_admin" ) )
			return true;
		
		// Nothing was caught, the user is not allowed to edit...
		return false;
	}
	
	/**
	 * Internal convenience function for generating the thread's filename
	 */
	function makeFilename()
	{
		global $discussContentDirectory;
		if( ! is_dir( "$discussContentDirectory/{$this->sectionID}/forums/{$this->forumID}/threads" ) )
			RecursiveMkdir( "$discussContentDirectory/{$this->sectionID}/forums/{$this->forumID}/threads" );
		
		$this->filename = "$discussContentDirectory/{$this->sectionID}/forums/{$this->forumID}/threads/{$this->threadID}.php";
		$this->lockfile = "$discussContentDirectory/{$this->sectionID}/forums/{$this->forumID}/threads/{$this->threadID}.lock";
		$this->globalID = "discuss/{$this->sectionID}/{$this->forumID}/{$this->threadID}";
	}
	
	/**
	 * Convenience function which draws the post editing dialog
	 */
	function renderThreadEditor($thread_data = "", $parentForum = "") {
		global $formatting_toolbar, $permissionlevels_array;
		
		// If the thread data is "", then we're called from $this, so use $this' data
		if( $thread_data == "" )
		{
			$title = i18n("Editing existing thread");
			$commandTitle = i18n("Save thread");
			$isModerator = $this->parentForum->isModerator();
			$thread_data["title"] = $this->title;
			$thread_data["description"] = $this->description; // This is optional
			$thread_data["content"] = $this->content;
			$thread_data["isSticky"] = $this->isSticky;
			$thread_data["owner"] = $this->owner;
			$thread_data["viewLevel"] = $this->viewLevel;
			$thread_data["replyLevel"] = $this->replyLevel;
			$thread_data["editLevel"] = $this->editLevel;
			$thread_info = "<input type=\"hidden\" name=\"discussEdit[threadID]\" value=\"{$this->threadID}\" />";
		}
		else
		{
			$isModerator = $parentForum->isModerator();
			$title = i18n("Creating a new thread");
			$commandTitle = i18n("Create thread");
			$thread_info = "<input type=\"hidden\" name=\"discuss_newThread\" value=\"true\" />";
			
		}
		
		foreach( $permissionlevels_array as $level => $description )
		{
			$viewlevel_selected = ( $level == $thread_data["viewLevel"] ) ? " selected" : "";
			$temp_viewLevel .= "<option value=\"$level\"$viewlevel_selected>$description</option>";
			
			$replylevel_selected = ( $level == $thread_data["replyLevel"] ) ? " selected" : "";
			$temp_replyLevel .= "<option value=\"$level\"$replylevel_selected>$description</option>";
			
			$editlevel_selected = ( $level == $thread_data["editLevel"] ) ? " selected" : "";
			$temp_editLevel .= "<option value=\"$level\"$editlevel_selected>$description</option>";
		}
		
		// Render the editor...
		$editor = "
			<form name=\"discuss_editor\" method=\"post\" action=\"" . $this->thisThreadURL . "\">
			<input type=\"hidden\" name=\"discuss_sectionID\" value=\"{$_REQUEST['discuss_sectionID']}\" />
			<input type=\"hidden\" name=\"discuss_forumID\" value=\"{$_REQUEST['discuss_forumID']}\" />
			$thread_info
			<table class=\"setup\">
			<tr class=\"setup\"><th colspan=\"2\"><a class=\"namedanchor\" name=\"thread_editor\"> </a>" . $title . "</th></tr>
			<tr class=\"setup\">
				<td class=\"setup\" width=\"50%\">" . i18n("Title") . ":</td>
				<td class=\"setup\"><input style=\"width: 100%\" type=\"text\" name=\"discussEdit[title]\" value=\"{$thread_data['title']}\" /></td>
			</tr>
			<tr class=\"setup\">
				<td class=\"setup\" width=\"50%\">" . i18n("Description (optional)") . ":</td>
				<td class=\"setup\"><input style=\"width: 100%\" type=\"text\" name=\"discussEdit[description]\" value=\"{$thread_data['description']}\" /></td>
			</tr>
			<tr class=\"setup\">
				<td class=\"setup\" width=\"50%\">" . i18n("Owner") . ":</td>
				<td class=\"setup\"><input style=\"width: 100%\" type=\"text\" name=\"discussEdit[owner]\" value=\"{$thread_data['owner']}\" /></td>
			</tr>
			<tr class=\"setup\">
				<td class=\"setup\" width=\"50%\">" . i18n("The following users can view the thread") . ":</td>
				<td class=\"setup\"><select style=\"width: 100%\" name=\"discussEdit[viewLevel]\">$temp_viewLevel</select></td>
			</tr>
			<tr class=\"setup\">
				<td class=\"setup\" width=\"50%\">" . i18n("The following users can reply to the thread") . ":</td>
				<td class=\"setup\"><select style=\"width: 100%\" name=\"discussEdit[replyLevel]\">$temp_replyLevel</select></td>
			</tr>
			<tr class=\"setup\">
				<td class=\"setup\" width=\"50%\">" . i18n("The following users can edit this post") . ":</td>
				<td class=\"setup\"><select style=\"width: 100%\" name=\"discussEdit[editLevel]\">$temp_editLevel</select></td>
			</tr>";
		if( $isModerator )
			$editor .= "
			<tr class=\"setup\">
				<td class=\"setup\" width=\"50%\">" . i18n("The post is sticky") . ":</td>
				<td class=\"setup\"><label><input type=\"checkbox\" name=\"discussEdit[isSticky]\"$temp_isSticky />" . i18n("Keep the thread first in the forum (sticky)") . "</label></td>
			</tr>";
		$editor .= "
			<tr class=\"setup\">
				<td class=\"setup\" colspan=\"2\">" . i18n("Your post") . ":
			</tr>
			$formatting_toolbar
			<tr class=\"setup\">
				<td class=\"setup\" colspan=\"2\">
				<textarea rows=\"15\" cols=\"70\" style=\"width: 100%;\" id=\"edit\" name=\"discussEdit[content]\">{$thread_data['content']}</textarea>
				</td>
			</tr>
			<tr class=\"setup\">
				<td class=\"setup\"><input type=\"submit\" name=\"discuss_saveThread\" value=\"$commandTitle\" /> <input type=\"submit\" name=\"discuss_preview\" value=\"" . i18n("Preview thread") . "\" /></td>
				<td class=\"setup wikirightalign\"><input type=\"reset\" value=\"" . i18n("Reset") . "\" /></td>
			</tr>
			</table>
			</form>";
			
		return $editor;
	}
	
	/**
	 * Render small block for showing in the forum overview
	 *
	 * @return string The rendered block
	 */
	function renderBlock()
	{
		$data = "<div class=\"discuss_threadHeader\"><a
				href=\"{$this->thisThreadURL}\" class=\"discuss_threadHeader\"><span
				class=\"discuss_threadLink\">" . i18n("Go") . " &raquo;</span><span
				class=\"discuss_threadTitle\">{$this->title}</span><span
				class=\"discuss_threadDescription\">{$this->description}</span></a>";
		
		// Check wether the user wants to show the details or not...
		if( getUserInfo( currentUser(), "discuss_verboseListings" ) != 1 )
		{
			$data .= "<span
				class=\"discuss_threadAuthor\">" . i18n("Posted by") . ": " . parse_profilelinks( "%%{$this->owner}%%" ) . "</span><span
				class=\"discuss_threadNumberOfReplies\">" . i18n("Replies") . ": {$this->repliesCount}</span><span
				class=\"discuss_threadLastChangedBy\">" . i18n("Last change by") . " " . parse_profilelinks( "%%{$this->lastChangedBy}%%" ) . " </span><span
				class=\"discuss_threadLastChanged\">" . i18n("Last updated") . ": " . formatTime( $this->lastChanged ) . "</span>";
		}
		else
		{
			$data .= "<span class=\"discuss_threadUnverbose\">" . i18n("Last change by ##0## on ##1##", array( parse_profilelinks( "%%{$this->lastChangedBy}%%" ), formatTime( $this->lastChanged ) ) ) . " </span>";
		}
		
		$data .= "</div>";
		
		// Render title, owner, number of replies, created date, last change date, last comment by
		return $data;
	}
	
	/**
	 * Render the complete thread
	 *
	 * @return string The rendered thread
	 */
	function render()
	{
		global $subscriptions, $discuss_options;
		if( $this->parentForum->isModerator() && $_REQUEST["discuss_moveThread"] == "true" )
		{
			$commandModerator = "<span class=\"discuss_threadModeratorLink\">" .
				drawCommand( i18n("Abort move"), i18n("Abort move"), $this->thisThreadURL, " discuss_threadModeratorMove" ) .
				"</span>";
			
			foreach( $discuss_options["sectionPositions"] as $position => $sectionID )
			{
				$currentSection = "";
				
				$theSection = new discussSection($sectionID);
				
				foreach( $theSection->forums as $key => $theForum )
				{
					if( $theForum->isModerator() )
					{
						if( $theForum->globalID == $this->parentForum->globalID )
							$default = " default";
						else
							$default = "";
						$currentSection .= "
							<option$default value=\"" . $theForum->globalID . "\">" . $theForum->title . "</option>";
					}
				}
				
				
				if( $currentSection != "" )
					$panel .= "<optgroup label=\"" . $theSection->title . "\">$currentSection</optgroup>";
			}
			$panel = "
				<form action=\"" . $this->thisThreadURL . "\" method=\"post\">
		<table class=\"options\">
		<tr class=\"option\">
			<td class=\"option\">" . i18n("Where do you want to move this thread?") . "<small class=\"comment\">" . i18n("Select the forum where you wish to move this thread. Note that only forums where you are moderator are shown") . "</small></td>
			<td class=\"option\"><select style=\"width: 100%;\" class=\"option_select\" name=\"moveto\">" . $panel . "</select></td>
		</tr>
		<tr class=\"options\">
			<td width=\"50%\" class=\"options-bottomleft\"><input type=\"submit\" class=\"options_save\" name=\"move_thread\" value=\"" .i18n("Move thread") . "\" /></td>
			<td width=\"50%\" class=\"options-bottomright\"><input type=\"reset\" class=\"options_reset\" value=\"" . i18n("Reset") . "\" /></td>
		</tr>
		</table>
				</form>";
			
			$renderedModerator = renderInformationBox( i18n("Move thread?"), $panel, false );
		}
		else if( $this->parentForum->isModerator() )
		{
			$commandModerator = "<span class=\"discuss_threadModeratorLink\">" .
				drawCommand( i18n("Move"), i18n("Move"), $this->thisThreadURL . "&amp;discuss_moveThread=true", " discuss_threadModeratorMove" ) .
				"</span>";
		}
		
		if( $this->userCanEdit() && $_REQUEST["discuss_editThread"] == "true" )
		{
			$commandEdit = "<span class=\"discuss_threadEditLink\">
					" . drawCommand( i18n("Delete"), i18n("Delete"), $this->thisThreadURL . "&amp;discuss_deleteThread=true", " discuss_threadDeleteThread" ) . "
					" . drawCommand( i18n("Abort edit"), i18n("Abort edit"), $this->thisThreadURL . "&amp;discuss_editThread=false", " discuss_threadEditThread" ) . "
				</span>";
			$rendered_thread = $this->renderThreadEditor();
		}
		else if ( $this->userCanEdit() && $_REQUEST["discuss_deleteThread"] == "true" )
		{
			$commandEdit = "<span class=\"discuss_threadEditLink\">
					" . drawCommand( i18n("Abort delete"), i18n("Abort delete"), $this->thisThreadURL . "&amp;discuss_deleteThread=false", " discuss_threadDeleteThread" ) . "
					" . drawCommand( i18n("Edit"), i18n("Edit"), $this->thisThreadURL . "&amp;discuss_editThread=true", " discuss_threadEditThread" ) . "
				</span>";
			$rendered_thread = 
				renderInformationBox(
					i18n("Delete this thread?"),
					i18n("Are you sure you wish to delete this thread? Please note - It will delete all replies, and CANNOT be undone.##0####1## ##2####3##",
						array(
							"<div class=\"wikicenteralign\">",
							drawCommand(
								i18n("Delete"),
								i18n("Accept deletion of this thread and all replies"),
								$this->thisThreadURL . "&amp;discuss_deleteThreadConfirm=true"
							),
							drawCommand(
								i18n("Abort delete"),
								i18n("Click here to accept deleting the thread"),
								$this->thisThreadURL . "&amp;discuss_deleteThread=false"
							),
							"</div>"
						)
					)
				) . "<div class=\"discuss_threadContent\">" . parse_page_data( $this->content ) . "</div>";
		}
		else if ( $this->userCanEdit() )
		{
			$commandEdit = "<span class=\"discuss_threadEditLink\">
					" . drawCommand( i18n("Delete"), i18n("Delete"), $this->thisThreadURL . "&amp;discuss_deleteThread=true", " discuss_threadDeleteThread" ) . "
					" . drawCommand( i18n("Edit"), i18n("Edit"), $this->thisThreadURL . "&amp;discuss_editThread=true", " discuss_threadEditThread" ) . "
				</span>";
			$rendered_thread = "<div class=\"discuss_threadContent\">" . parse_page_data( $this->content ) . "</div>";
		}
		else if( $this->userCanView() )
			$rendered_thread = "<div class=\"discuss_threadContent\">" . parse_page_data( $this->content ) . "</div>";
		else
		{
			// If we've gotten to here... The user isn't allowed to view the thread! Tell them they've been bad
			$rendered_thread = renderInformationBox( i18n("Viewing disallowed"), i18n("You are not allowed to view this thread!") );
		}
		
		// Render header (breadcrumbs trail returns to whatever page we were on in pagination, if applicable)
		$rendered_content = "<span
				class=\"discuss_Breadcrumbs\">
					" . i18n("You are here:") . "
					<a href=\"" . globalIDtoURL("discuss") . "\" class=\"discuss_forumBreadcrumbsToTop\">" . i18n("Forum home") . "</a> &raquo;
					<a href=\"" . globalIDtoURL("discuss/" . $this->sectionID . "/" . $this->forumID ) . "\" class=\"discuss_forumBreadcrumbsForum\">{$this->parentForum->title}</a> &raquo;
					{$this->title}
					</span><span
				class=\"discuss_Navigator\">&nbsp;
				$rendered_navigator
			</span>
			<div class=\"discuss_threadCommands\">
				$commandModerator
				$commandEdit
				" . $subscriptions->renderSubscribeControl( $this->globalID, false, " discussSubscribeControl" ) . "
				<span class=\"discuss_threadAddReply\"><a class=\"command\" href=\"#commentshandler\">[" . i18n("Reply") . "]</a></span>
			</div>";
		
		$avatar = getAvatarThumbnail( $this->owner );
		list( $avatarx, $avatary ) = getimagesize( $avatar );
		$avatar = siteURL(true) . $avatar;
		
		// Render the header
		$rendered_content .= "<div class=\"discuss_threadHeader\"><a href=\"" . globalIDtoURL("user/" . $this->owner) . "\" title=\"" . i18n("##0##'s avatar", array( getUserInfo( $this->owner, "name" ) ) ) . "\"><img
				class=\"comment-avatar\" alt=\"" . i18n("##0##'s avatar", array( getUserInfo( $this->owner, "name" ) ) ) . "\" src=\"$avatar\" width=\"$avatarx\" height=\"$avatary\" /></a><span
				class=\"discuss_threadTitle\">{$this->title}</span><span
				class=\"discuss_threadDescription\">{$this->description}</span></div>";
		
		// Render post (edit or not)
		$rendered_content .= "$renderedModerator<div
			class=\"discuss_threadcontents\">$rendered_thread</div>";
		
		if( $_REQUEST["discuss_editThread"] != "true" && userAllows($this->owner, $this->viewLevel) )
		{
			// Render replies (commentshandler)
			$replies = new commentHandler( $this->replies, $this->owner, $this->replyLevel, $this->editLevel );
			$replies->thisPageURL = $this->thisThreadURL;
			$rendered_content .= $replies->renderComments() . $replies->renderCommentsAdder();
			if( $replies->commentsChanged )
			{
				if( $this->lock() )
				{
					$this->load();
					$this->replies = $replies->comments;
					$this->repliesCount = $replies->numberOfComments();
					$this->lastChangedBy = $this->lastReplyBy = str_replace( "%", "", $replies->lastCommenter ); // Last change is both editing and replying...
					$this->lastReply = $replies->lastComment;
					$this->isChanged = true;
					$this->changeIsReply = true;
					$this->save();
					$this->unlock();
					header( "Location: " . str_replace("&amp;", "&", $this->thisThreadURL ) );
				}
				else
					die( renderInformationBox( i18n("Timeout error!"), i18n("Too many people are attempting to access the forum. Please try again.") ) );
			}
		}
		
		return $rendered_content;
	}
}
?>