<?PHP
/**
 * The discuss_userPosts class, used for keeping track of a user's posting (because we cannot query a
 * database for the information, we need to keep track of it another way)
 */
class discuss_userPosts
{
	/// \private
	var $filename;                     ///<  string    Internal variable - the filename used for saving the information
	/// \public
	var $posts;                        ///<  array     Array containing the actual post data
	
	/**
	 * The constructor
	 * 
	 * @param    username       String containing the username to handle posts for
	 */
	function discuss_userPosts( $username )
	{
		global $userinfo_folder;
		
		$this->filename = "$userinfo_folder/$username/discuss_threads.php";
		
		$userposts = array();
		if( file_exists( $this->filename ) )
			include( $this->filename );
		
		$this->posts = $userposts;
	}
	
	/**
	 * Rotates a post to the front, or adds it if it is a new post
	 *
	 * @param    postdata       An array containing post data to be added
	 *
	 * @return   bool           Wether the rotation was successful
	 */
	function rotate( $postdata )
	{
		// Create globalID from postdata
		$globalID = "discuss/" . $postdata["sectionID"] . "/" . $postdata["forumID"] . "/" . $postdata["threadID"];
		if( $postdata["commentID"] != "" )
			$globalID .= "#" . $postdata["commentID"];
		
		// Check if globalID exists already, and remove the existing position if appropriate
		if( array_key_exists( $globalID, $this->posts ) )
			$this->postDeleted( $globalID );
		
		$tmp = array( $globalID => $postdata );
		
		// This only if it is actually a comment
		if( $postdata["commentID"] != "" )
			$tmp[$globalID]["commentID"] = $postdata["commentID"];
		
		$this->posts = $tmp + $this->posts;
		
		return $this->save();
	}
	
	/**
	 * Deletes a post with a globalID from the list
	 *
	 * @param    globalID       The globalID of the post to remove
	 *
	 * @return   bool           Wether the deletion was successful
	 */
	function postDeleted( $globalID )
	{
		// Remove from list
		unset( $this->posts[$globalID] );
		return $this->save();
	}
	
	/**
	 * Returns the user's post count
	 *
	 * @return   integer        The number of posts the user has made
	 */
	function postcount()
	{
		return count( $this->posts );
	}
	
	/**
	 * Saves the data back to disc
	 *
	 * @return   bool           Wether the data was successfully saved
	 */
	function save()
	{
		if( file_exists( $this->filename ) )
			unlink( $this->filename );
		
		return file_put_contents( $this->filename, array_export( $this->posts, "userposts" ) );
	}
}
?>