<?PHP
/**
 * Event Calendar module for Travelsized CMS
 *
 * @Author	Dan Jensen <admin@leinir.dk>
 *
 * Events directory structure:
 *
 * modules/
 *  events/
 *   XXXX/ (year, ex 2006)
 *    XX/ (month, ex 05)
 *     XX/ (date, ex 21)
 *      $user_id.$timestamp.php (files containing an event data array structure)
 *
 * Event data array structure:
 *
 * array(
 *  "title" => string,
 *  "description => page_data,
 *  "timestamp" => start time,
 *  "duration" => number of seconds,
 *  "creator" => user_id,
 *  "viewlevel" => viewing allowance level,
 *  "participatelevel" => participation allowance level,
 *  "participants" => array(user_id, user_id),
 *  "comments" => comments array (for comment_handler)
 * );
 *
 * CSS classes used for showing data
 *
 * table.event_calendar
 *  tr.event_calendar
 *   td.event_calendar_day_leadin
 *    table.event_calendar_day_leadin
 *     tr.event_calendar_day_top_leadin
 *      td.event_calendar_day_tl_leadin
 *      td.event_calendar_day_tr_leadin
 *     tr.event_calendar_day_bottom_leadin
 *      td.event_calendar_day_b_leadin
 *   td.event_calendar_day
 *   td.event_calendar_day_selected
 *   td.event_calendar_day_ownevent
 *    a.event_calendar_day
 *    a.event_calendar_day_selected
 *    a.event_calendar_day_ownevent
  *    table.event_calendar_day
 *     table.event_calendar_day_selected
 *     table.event_calendar_day_ownevent
 *      tr.event_calendar_day_top
 *      tr.event_calendar_day_top_selected
 *      tr.event_calendar_day_top_ownevent
 *       td.event_calendar_day_tl
 *       td.event_calendar_day_tl_selected
 *       td.event_calendar_day_tl_ownevent
 *       td.event_calendar_day_tr
 *       td.event_calendar_day_tr_selected
 *       td.event_calendar_day_tr_ownevent
 *      tr.event_calendar_day_bottom
 *      tr.event_calendar_day_bottom_selected
 *      tr.event_calendar_day_bottom_ownevent
 *       td.event_calendar_day_b
 *       td.event_calendar_day_b_selected
 *       td.event_calendar_day_b_ownevent
 *    td.event_calendar_day_leadout
 *     table.event_calendar_day_leadout
 *      tr.event_calendar_day_top_leadout
 *       td.event_calendar_day_tl_leadout
 *       td.event_calendar_day_tr_leadout
 *      tr.event_calendar_day_bottom_leadout
 *       td.event_calendar_day_b_leadout
 */

// Register the module versioning information {
	$modules_names[] = "Event Manager";
	$mopules_versions[] = "0.1";
	$modules_descriptions[] = "An events-based calendar";
// Register the module versioning information }

// Register the module functions into the module registry {
	$modules_setup[] = "modulesetup_events";
	
	$modules_globalID[] = "globalID_events";
	
	$profilemodules[] = "profilemodule_events";
	
	$modules[] = "parse_events_calendar";
	$modules[] = "parse_events_events";
	$modules[] = "parse_events_event";
// Register the module functions into the module registry }

// Module help... {
$page_help["&#92;calendar(txt1)"] = "This will show a calendar with events in the categories defined by txt1. txt1 is a space-separated list.";
$page_help["&#92;events(txt1)"] = "This will show the events for the next txt1 days. If txt1 is not present, it will show events for the next 7 days";
$page_help["&#92;event(txt1,txt2)"] = "This will show the event on the timestamp txt2 as defined by the user txt1";
// Module help... }


// Define the events folder name
$events_folder = "$module_folder/events";
// If the module was installed correctly, this /should/ already be there... But just to be safe :)
if ( !file_exists( $events_folder ) )
	RecursiveMkdir($events_folder, 0775);

class eventsEvent {
	var $title;
	var $description;
	var $timestamp; // Start time
	var $duration;
	var $creator;
	var $viewlevel;
	var $editlevel;
	var $commentlevel;
	var $participatelevel;
	var $participants;
	var $comments;
	var $filename;
	
	function eventsEvent( $creator, $timestamp )
	{
		$this->creator = $creator;
		$this->timestamp = $timestamp;
		$this->generateFilename();
		$this->load();
		if( userAllows( $this->creator, $this->editlevel ) )
		{
			if( $_REQUEST["event_creator"] == $this->creator && $_REQUEST["deleteEvent"] == $this->timestamp && $_REQUEST["confirm_delete"] == "yes" )
			{
				$this->delete();
				header("Location: " . thisPageURL(true) . "event_selectedyear=" . date("Y", $this->timestamp) . "&event_selectedmonth=" . date("n", $this->timestamp) . "&event_selectedday=" . date("j", $this->timestamp));
			}
			else if( $_REQUEST["event_creator"] == $this->creator && $_REQUEST["event_timestamp"] == $this->timestamp && $_POST["confirm_delete"] && is_array( $_POST["selectedParticipants"] ) )
			{
				foreach( $_POST["selectedParticipants"] as $key => $value )
					unset( $this->participants[$value] );
			}
		}
		if( userAllows( $this->creator, $this->participatelevel ) )
		{
			if( $_REQUEST["event_creator"] == $this->creator && $_REQUEST["event_timestamp"] == $this->timestamp )
			{
				if( $_REQUEST["participate"] == "yes" )
					$this->addParticipant(currentUser());
				else if( $_REQUEST["participate"] == "no" )
					$this->removeParticipant(currentUser());
				
				if( $_POST["contactConfirm"] && is_array( $_POST["selectedParticipants"] ) )
				{
					$message_success = true;
					foreach( $_POST["selectedParticipants"] as $key => $value )
						if( !sendMessage( $value, $_POST["subject"], $_POST["message"], currentUser() ) )
							$message_success = false;
					
					if( $message_success )
					{
						$year = date( "Y", $this->timestamp );
						$month = date( "n", $this->timestamp );
						$day = date( "j", $this->timestamp );
						header( "Location: " . $thisPageURL(true) . "&event_selectedyear=$year&event_selectedmonth=$month&event_selectedday=$day&event_creator=" . $this->creator . "&event_timestamp=" . $this->timestamp . "#" . $this->creator . "." . $this->timestamp );
					}
				}
			}
		}
	}
	
	/**
	 * Regenerate the filename from the timestamp
	 */
	function generateFilename()
	{
		global $userinfo_folder;
		$year = date( "Y", $this->timestamp );
		$month = date( "n", $this->timestamp );
		$day = date( "j", $this->timestamp );
		$this->filename = "$userinfo_folder/" . $this->creator . "/events/$year/$month/$day/" . $this->timestamp . ".php";
	}
	
	function addParticipant( $user_id )
	{
		$this->participants[$user_id] = time(); // Set the user's participation date
		$this->save(); // Update the event on disk
	}
	
	function removeParticipant( $user_id )
	{
		unset( $this->participants[$user_id] ); // Delete the user from the participants list
		$this->save(); // Update the event on disk
	}
	
	function load()
	{
		global $userinfo_folder;
		if ( file_exists( $this->filename ) )
		{
			include( $this->filename );
		}
		else
		{
			$event["title"] = i18n("No such event!");
			$event["description"] = i18n("No such event! If you found your way here by normal means, the event might have been deleted in the meantime.");
			$event["creator"] = $creator;
			$event["timestamp"] = $timestamp;
			$event["duration"] = 0;
		}
		$this->title = $event["title"];
		$this->description = $event["description"];
		$this->timestamp = $event["timestamp"];
		$this->duration = $event["duration"];
		$this->creator = $event["creator"];
		$this->viewlevel = $event["viewlevel"];
		$this->editlevel = $event["editlevel"];
		$this->commentlevel = $event["commentlevel"];
		$this->participatelevel = $event["participatelevel"];
		$this->participants = $event["participants"];
		$this->comments = $event["comments"];
	}
	
	function save()
	{
		global $userinfo_folder;
		$event["title"] = $this->title;
		$event["description"] = $this->description;
		$event["timestamp"] = $this->timestamp;
		$event["duration"] = $this->duration;
		$event["creator"] = $this->creator;
		$event["viewlevel"] = $this->viewlevel;
		$event["editlevel"] = $this->editlevel;
		$event["commentlevel"] = $this->commentlevel;
		$event["participatelevel"] = $this->participatelevel;
		$event["participants"] = $this->participants;
		$event["comments"] = $this->comments;
		
		$this->generateFilename();
		
		if( !is_dir( dirname( $this->filename ) ) )
			RecursiveMkdir( dirname( $this->filename ) );
		
		if( file_exists( $this->filename ) )
			unlink( $this->filename );
			
		file_put_contents( $this->filename, array_export( $event, "event" ) );
	}
	
	function delete()
	{
		if ( file_exists( $this->filename ) )
			unlink( $this->filename );
		// Check the parent directories and make sure that if they are empty, they too are deleted - if not, we're going to end up with a nasty bunch of empty directories all over the place
	}
	
	function render()
	{
		$commenthandler = new commentHandler($this->comments, $this->creator, $this->commentlevel, $this->editlevel);

		if( userAllows( $this->creator, $this->viewlevel ) )
		{
			$year = date("Y", $this->timestamp);
			$month = date("n", $this->timestamp);
			$day = date("j", $this->timestamp);
			
			$thisEventURL = thisPageURL( false, true, array( "event_selectedyear" => $year, "event_selectedmonth" => $month, "event_selectedday" => $day, "event_creator" => $this->creator, "event_timestamp" => $this->timestamp ) );
			$thisEventURLMarker = "#{$this->creator}.{$this->timestamp}";
			
			if (userAllows($this->creator, $this->participatelevel)) {
				$editAllowed = userAllows($this->creator, $this->editlevel);
				if( $_REQUEST["event_timestamp"] == $this->timestamp && $_REQUEST["event_creator"] == $this->creator && $_REQUEST["event_showparticipants"] == "true" )
				{
					$participateControl = "<a class=\"command\" href=\"$thisEventURL&amp;event_showparticipants=false$thisEventURLMarker\">[" . i18n("Hide") . "]</a> ";
					$renderedParticipants = "
					<form action=\"$thisEventURL&amp;event_showparticipants=true$thisEventURLMarker\" method=\"POST\">
						<ul class=\"event_participants\">";
					if ( count( $this->participants ) > 0 )
					{
						foreach( $this->participants as $key => $value )
						{
							$participantChecked = "";
							foreach ( $_REQUEST["selectedParticipants"] as $key2 => $value )
							{
								if ( $key == $value )
									$participantChecked = " checked";
							}
							
							$renderedParticipants .= "
						<li class=\"event_participant\"><label>
							<input type=\"checkbox\" name=\"selectedParticipants[$key]\"$participantChecked />
							" . i18n("##0## joined on ##1##", array(getUserInfo($key, "name"), formatTime( $value ))) . "
						</label></li>";
						}
					}
					else
					{
						$renderedParticipants .= "
						<li class=\"event_participant\">" . i18n("No participants") . "</li>";
					}
					
					$renderedParticipants .= "
						</ul>";
					
					if ( $editAllowed && $_POST["removeSelected"] ) // Confirm deletion
					{
						$renderedParticipants .= "
						<p>" . i18n("Are you sure you wish to delete the selected participants?") . "</p>
						<input type=\"submit\" name=\"confirm_delete\" value=\"" . i18n("Confirm delete") . "\" />
						<input type=\"submit\" name=\"abort_delete\" value=\"" . i18n("Abort delete") . "\" />";
					}
					else if ( $_POST["contactSelected"] ) // Compose new message
					{
						$renderedParticipants .= "
						<input style=\"width: 100%;\" type=\"text\" name=\"subject\" value=\"" . i18n("Message regarding ##0##", array($this->title)) . "\" />
						<textarea style=\"width: 100%;\" name=\"message\">" . i18n("Write your message here") . "</textarea>
						<input type=\"submit\" name=\"contactConfirm\" value=\"" . i18n("Send message") . "\" />
						<input type=\"submit\" name=\"contactAbort\" value=\"" . i18n("Abort message") . "\" />";
					}
					else if ( $_REQUEST["message_success"] == "true" ) // Was the message sent successfully?
					{
						$renderedParticipants .= "
						<p>" . i18n("The message was successfully sent to the participants") . "</p>";
					}
					else
					{
						if ( $editAllowed )
							$renderedParticipants .= "
						<input type=\"submit\" name=\"removeSelected\" value=\"" . i18n("Remove selected") . "\" />";
					
						$renderedParticipants .= "
						<input type=\"submit\" name=\"contactSelected\" value=\"" . i18n("Contact selected") . "\" />
						<input type=\"reset\" value=\"" . i18n("Reset changes") . "\" />";
					}
					
					$renderedParticipants .= "
					</form>";
				}
				else
				{
					$participateControl = "<a class=\"command\" href=\"$thisEventURL&amp;event_showparticipants=true$thisEventURLMarker\">[" . i18n("Show") . "]</a> ";
					$renderedParticipants = "";
				}
				$participateControl .= i18n("Participants: ##0##", array(count($this->participants))) . " ";
				$isParticipating = false;
				if (is_array($this->participants))
				{
					foreach($this->participants as $key => $value)
						if ($key == currentUser())
							$isParticipating = true;
				}
				if ($isParticipating)
				{
					$participateControl .= "<a class=\"command\" href=\"$thisEventURL&amp;participate=no$thisEventURLMarker\">[" . i18n("Don't participate") . "]</a> |";
				}
				else
				{
					$participateControl .= "<a class=\"command\" href=\"$thisEventURL&amp;participate=yes$thisEventURLMarker\">[" . i18n("Participate") . "]</a> |";
				}
				}
				else
				{
			}
			
			if ( userAllows( $this->creator, $this->editlevel ) )
			{
				$editControl = "<a class=\"command\" href=\"" . globalIDtoURL("user/{$this->creator}/events") . "?event_selectedyear=$year&amp;event_selectedmonth=$month&amp;event_selectedday=$day&amp;event_creator=" . $this->creator . "&amp;event_timestamp=" . $this->timestamp . "&amp;editEvent={$this->timestamp}#editEvent\">[" . i18n("Edit event") . "]</a>";
				
				if( $_REQUEST["deleteEvent"] == $this->timestamp )
				{
					$deleteControlTop = "
					<div class=\"event_confirmdelete\">
						<p class=\"event_confirmdelete\">" . i18n("Please confirm that you wish to delete this event. Warning: This cannot be undone!") . "</p>
						<p class=\"event_confirmdelete\">
							<a class=\"command\" href=\"$thisEventURL&amp;confirm_delete=yes&amp;deleteEvent={$this->timestamp}#" . $this->creator . "." . $this->timestamp . "\">[" . i18n("Confirm delete") . "]</a>
							<a class=\"command\" href=\"$thisEventURL$thisEventURLMarker\">[" . i18n("Abort delete") . "]</a>
						</p>
						<hr />";
					$deleteControlBottom = "
					</div>";
				}
				else
				{
					$deleteControlBottom = " <a class=\"command\" href=\"$thisEventURL&amp;deleteEvent={$this->timestamp}$thisEventURLMarker\">[" . i18n("Delete event") . "]</a>";
				}
			}
			
			if( $_REQUEST["show_comments"] == "true" || $_REQUEST["add_comment"] == "true" )
			{
				$renderedComments = $commenthandler->renderComments();
				$commentControl = " <a class=\"command\" href=\"" . $thisEventURL . "&amp;show_comments=false$thisEventURLMarker\">[" . i18n("Hide comments") . "]</a>";
			}
			else if( $commenthandler->numberOfComments() > 0 )
			{
				$commentControl = " <a class=\"command\" href=\"" . $thisEventURL . "&amp;show_comments=true$thisEventURLMarker\">[" . i18n("Show ##0## comments", array($commenthandler->numberOfComments())) . "]</a>";
			}
			if( userAllows( $this->creator, $this->commentlevel ) )
			{
				if( $_REQUEST["add_comment"] == "true" )
				{
					$renderedComments .= $commenthandler->renderCommentsAdder();
					$commentControl .= " <a class=\"command\" href=\"" . $thisEventURL . "&amp;add_comment=false$thisEventURLMarker\">[" . i18n("Abort add comment") . "]</a>";
				}
				else
				{
					$commentControl .= " <a class=\"command\" href=\"" . $thisEventURL . "&amp;add_comment=true#comment\">[" . i18n("Add comment") . "]</a>";
				}
			}
			else
			{
			}
			if( $commenthandler->commentsChanged )
			{
				$this->comments = $commenthandler->comments;
				$this->save();
			}
			
			$timing = i18n( "Starts: ##0## - Ends: ##1##", array( date( "h:i:s", $this->timestamp ), date( "h:i:s d/n", $this->timestamp + ( $this->duration * 60 ) ) ) );
		
			$data = "
			$deleteControlTop
			<h3 class=\"event_title\"><a class=\"namedanchor\" name=\"" . $this->creator . "." . $this->timestamp . "\">" . $this->title . "</a></h3>
			<p class=\"event_timing\">$timing</p>
			$editControl
			$deleteControlBottom
			" . parse_page_data($this->description) . "
			$renderedComments
			$renderedParticipants
			$participateControl
			$commentControl";
		} else {
			$data = "
			<h3 class=\"event_event\">" . i18n("Access disabled") . "</h3>
			<p>" . i18n("The event's creator has disabled access to this event.") . "</p>";
		}
		
		return $data;
	}
}

class eventsDay
{
	var $year;
	var $month;
	var $day;
	var $events = array();
	var $isSelected;
	var $hasOwnEvent;
	
	function eventsDay( $year, $month, $day )
	{
		$this->year = $year;
		$this->month = $month;
		$this->day = $day;
		$this->load();
	}
	
	function load()
	{
		global $userinfo_folder;
		$i = 0;
		$this_day = $this->year . "/" . $this->month . "/" . $this->day;
		$this->events = array(); // Clear the day...
		$all_users = allUsersArray();
		foreach( $all_users as $key => $value )
		{
			$day_folder = "$userinfo_folder/$value/events/$this_day";
			if( is_dir( $day_folder ) )
			{ // If the folder doesn't exist, there's not much reason in trying to load anything for this day
				if( $f = opendir( $day_folder ) )
				{ // Try and open the folder...
					while( false !== ( $file = readdir( $f ) ) )
					{ // Load all the events for this day
						if( substr($file, 0, 1) != "." )
						{ // As long as the entries are not the current and parent folder...
							// Get the timestamp
							$timestamp = substr( $file, 0, -4 );
							// Get the ID
							$id = "$value.$timestamp";
							// Make a new event at this position
							$this->events[$id] = new eventsEvent( $value, $timestamp );
							// If isAuth(), check the events wether the user owns any of those events
 							if( isAuth() && $this->events[$id]->creator == currentUser() )
 								$this->hasOwnEvent = true;
						}
					}
				}
			}
		}
	}
	
	function addEvent( $title, $description, $timestamp, $duration, $creator, $viewlevel, $editlevel, $commentlevel, $participatelevel, $categories )
	{
		$newkey = "$creator.$timestamp";
		$this->events[$newkey] = new eventsEvent($creator, $timestamp);
		$this->events[$newkey]->title = $title;
		$this->events[$newkey]->description = $description;
		$this->events[$newkey]->timestamp = $timestamp;
		$this->events[$newkey]->duration = $duration;
		$this->events[$newkey]->creator = $creator;
		$this->events[$newkey]->viewlevel = $viewlevel;
		$this->events[$newkey]->editlevel = $editlevel;
		$this->events[$newkey]->commentlevel = $commentlevel;
		$this->events[$newkey]->participatelevel = $participatelevel;
		$this->events[$newkey]->categories = $categories;
		$this->events[$newkey]->save();
	}
	
	function renderEventsList()
	{
		/// TODO: Give events a way to select which page events go on...
		global $language;
		$data = "
		<dl class=\"events_eventlist\">";
		foreach( $this->events as $key => $value )
			$data .= "
			<dt>" . date("H:i", $value->timestamp) . "</dt>
			<dd class=\"events_eventlist\"><a class=\"events_eventlist_event\" href=\"" . thisPageURL( false, true, array( "page_id" => "", "page" => "Events", "language" => $language, "event_selectedyear" => $this->year, "event_selectedmonth" => $this->month, "event_selectedday" => $this->day ) ) . "#{$value->creator}.{$value->timestamp}\">" . $value->title . "</a></li>";
		
		$data .= "
		</dl>";
		return $data;
	}
	
	function render()
	{
		if ($this->isSelected) {
			$additional_css = "_selected";
		} else if ($this->hasOwnEvent) {
			$additional_css = "_ownevent";
		}
		if (count($this->events) == 0) {
			$eventsLabel = "&nbsp;";
		} else if (count($this->events) == 1) {
			$eventsLabel = i18n("##0## event", array(count($this->events)));
		} else {
			$eventsLabel = i18n("##0## events", array(count($this->events)));
		}
		if (date("w", mktime(0, 0, 0, $this->month, $this->day, $this->year)) == "1") $data = "
			<tr class=\"event_calendar_week\">";
		// Maybe putting the list of events into the anchor title would make sense?
		$data .= "
				<td class=\"event_calendar_day$additional_css\">
					<a href=\"" . thisPageURL( false, true, array( "event_selectedyear" => $this->year, "event_selectedmonth" => $this->month, "event_selectedday" => $this->day ) ) . "#events\" class=\"event_calendar_day$additional_css\"><table class=\"event_calendar_day$additional_css\">
						<tr class=\"event_calendar_day_top$additional_css\">
							<td class=\"event_calendar_day_tl$additional_css\">" . i18n(date("D", mktime(0, 0, 0, $this->month, $this->day, $this->year))) . "</td>
							<td class=\"event_calendar_day_tr$additional_css\">" . $this->day . "</td>
						</tr>
						<tr class=\"event_calendar_day_bottom$additional_css\">
							<td colspan=\"2\" class=\"event_calendar_day_b$additional_css\">$eventsLabel</td>
						</tr>
					</table></a>
				</td>";
		if (date("w", mktime(0, 0, 0, $this->month, $this->day, $this->year)) == "0") $data .= "
			</tr>";
		return $data;
	}
}

class eventsMonth {
	var $selectedDay;
	var $year;
	var $month;
	var $categories = "";
	var $days = array();
	
	function eventsMonth($year = 0, $month = 0) {
		if ($_REQUEST["event_selectedyear"] != "") {
			$this->year = $_REQUEST["event_selectedyear"];
		} else {
			$this->year = date("Y");
		}
		if ($_REQUEST["event_selectedmonth"] != "") {
			$this->month = $_REQUEST["event_selectedmonth"];
		} else {
			$this->month = date("n");
		}
		if ($_REQUEST["event_selectedday"] != "") {
			$this->selectedDay = $_REQUEST["event_selectedday"];
		} else {
			$this->selectedDay = date("j");
		}
		if ($year != 0) $this->year = $year;
		if ($month != 0) $this->month = $month;
		
		if ($_POST["event_navigator_prev_year"]) $this->year--;
		if ($_POST["event_navigator_next_year"]) $this->year++;
		if ($_POST["event_navigator_prev_month"]) {
			if ($this->month == 1) {
				$this->month = 12;
				$this->year--;
			} else {
				$this->month--;
			}
		}
		if ($_POST["event_navigator_next_month"]) {
			if ($this->month == 12) {
				$this->month = 1;
				$this->year++;
			} else {
				$this->month++;
			}
		}
		
		if ($_POST["event_navigator_today"]) {
			$this->year = date("Y");
			$this->month = date("n");
			$this->selectedDay = date("j");
		}
		
		$this->load();
	}
	
	/**
	 * Loads data for the current month as defined by $this->year and $this->month
	 *
	 * @return	boolean	Returns true if successfully loaded, and false if it could not load the 
	 */
	function load() {
		$this->days = array(); // Clear the current month
		if ($this->year > 0 && (int)$this->month > 0) {
			$number_of_days = date("t", mktime(0, 0, 0, $this->month, 1, $this->year));
			// Flip through each day of the month $month of the year $year
			for ($i = 1; $i <= $number_of_days; $i++) {
				$this->days[$i] = new eventsDay($this->year, $this->month, $i);
			}
			$this->days[$this->selectedDay]->isSelected = true;
			
			return true;
		} else {
			return false;
		}
	}
	
	function render() {
		$startyear = $this->year - 50;
		for($i = 0; $i < 100; $i++) {
			$theyear = $startyear + $i;
			if ($theyear == $this->year) {
				$selected = " selected";
			} else {
				$selected = "";
			}
			$navigator_year .= "
			<option value=\"$theyear\"$selected>$theyear</option>";
		}
		$allmonths = array(1 => i18n("January"), 2 => i18n("February"), 3 => i18n("March"), 4 => i18n("April"), 5 => i18n("May"), 6 => i18n("June"), 7 => i18n("July"), 8 => i18n("August"), 9 => i18n("September"), 10 => i18n("October"), 11 => i18n("November"), 12 => i18n("December"));
		for($i = 1; $i < 13; $i++) {;
			if ($i == $this->month) {
				$selected = " selected";
			} else {
				$selected = "";
			}
			$navigator_month .= "
			<option value=\"$i\"$selected>" . i18n($allmonths[$i]) . "</option>";
		}
		$navigator = "
		<form class=\"event_navigator\" name=\"event_navigator\" action=\"" . thisPageURL( false, true, array( "event_selectedyear" => $this->year, "event_selectedmonth" => $this->month, "event_selectedday" => $this->selectedDay ) ) . "\" method=\"post\">
		<tr>
			<td class=\"event_navigator\" colspan=\"3\"><input class=\"event_navigator_prev_year\" type=\"submit\" name=\"event_navigator_prev_year\" value=\"&laquo;\"
			/><select onchange=\"document.event_navigator.submit();\" class=\"event_navigator_select_year\" name=\"event_selectedyear\">$navigator_year
			</select><input class=\"event_navigator_next_year\" type=\"submit\" name=\"event_navigator_next_year\" value=\"&raquo;\" /></td>
			<td class=\"event_navigator\" colspan=\"3\"><input class=\"event_navigator_prev_month\" type=\"submit\" name=\"event_navigator_prev_month\" value=\"&laquo;\"
			/><select onchange=\"document.event_navigator.submit();\" class=\"event_navigator_select_month\" name=\"event_selectedmonth\">$navigator_month
			</select><input class=\"event_navigator_next_month\" type=\"submit\" name=\"event_navigator_next_month\" value=\"&raquo;\" /></td>
			<td class=\"event_navigator\"><input class=\"event_navigator_today\" type=\"submit\" name=\"event_navigator_today\" value=\"" . i18n("Go to today") . "\" /></td>
		</tr>
		</form>";
		
		$leadinDays = date("w", mktime(0, 0, 0, $this->month, 1, $this->year)) - 1;
		// The following two if statements make sure that there is always at least one, and up to a whole week, of the previous month showing (for comparison)
		if ($leadinDays == -1) $leadinDays = 6; // Sunday
		else if ($leadinDays == 0) $leadinDays = 7; // Monday
		
		$leadinYear = date("Y", mktime(0, 0, 0, $this->month - 1, 1, $this->year));
		$leadinMonth = date("n", mktime(0, 0, 0, $this->month - 1, 1, $this->year));
		
		$leadinData = "
			<tr class=\"event_calendar_week\">";
		for($i = -$leadinDays; $i < 0; $i++) {
			$thedate = date("j", mktime(0, 0, 0, (int)$this->month, 1+$i, $this->year));
			$leadinData .= "
				<td class=\"event_calendar_day_leadin\">
					<a href=\"" . thisPageURL( false, true, array( "event_selectedyear" => $leadinYear, "event_selectedmonth" => $leadinMonth, "event_selectedday" => $thedate ) ) . "\" class=\"event_calendar_day_leadin$additional_css\"><table class=\"event_calendar_day$additional_css\">
						<table class=\"event_calendar_day_leadin\">
							<tr class=\"event_calendar_day_top_leadin\">
								<td class=\"event_calendar_day_tl_leadin\">&nbsp;</td>
								<td class=\"event_calendar_day_tr_leadin\">$thedate</td>
							</tr>
							<tr class=\"event_calendar_day_bottom_leadin\">
								<td colspan=\"2\" class=\"event_calendar_day_b_leadin\">&nbsp;</td>
							</tr>
						</table>
					</a>
				</td>";
		}
		if ($leadinDays == 7) $leadinData .= "
			</tr>";
		
		$leadoutDays = date("w", mktime(0, 0, 0, (int)$this->month + 1, 1, $this->year));
		// The following if statement makes sure that there is always at least one, and up to a whole week, of the next month showing (for comparison)
		switch ($leadoutDays) {
		case 0: $leadoutDays = 0; break;
		case 1: $leadoutDays = 6; break;
		case 2: $leadoutDays = 5; break;
		case 3: $leadoutDays = 4; break;
		case 4: $leadoutDays = 3; break;
		case 5: $leadoutDays = 2; break;
		case 6: $leadoutDays = 1; break;
		}
		
		$leadoutYear = date("Y", mktime(0, 0, 0, (int)$this->month + 1, 1, $this->year));
		$leadoutMonth = date("n", mktime(0, 0, 0, (int)$this->month + 1, 1, $this->year));
		
		if ($leadoutDays == 6) $leadoutData .="
			<tr class=\"event_calendar_week\">";
		
		for($i = 0; $i <= $leadoutDays; $i++) {
			$thedate = date("j", mktime(0, 0, 0, (int)$this->month, 1+$i, $this->year));
			$leadoutData .= "
				<td class=\"event_calendar_day_leadout\">
					<a href=\"" . thisPageURL( false, true, array( "event_selectedyear" => $leadoutYear, "event_selectedmonth" => $leadoutMonth, "event_selectedday" => $thedate ) ) . "\" class=\"event_calendar_day_leadout$additional_css\"><table class=\"event_calendar_day$additional_css\">
						<table class=\"event_calendar_day_leadout\">
							<tr class=\"event_calendar_day_top_leadout\">
								<td class=\"event_calendar_day_tl_leadout\">&nbsp;</td>
								<td class=\"event_calendar_day_tr_leadout\">$thedate</td>
							</tr>
							<tr class=\"event_calendar_day_bottom_leadout\">
								<td colspan=\"2\" class=\"event_calendar_day_b_leadout\">&nbsp;</td>
							</tr>
						</table>
					</a>
				</td>";
		}
		$leadoutData .= "
			</tr>";
		
		// Render each day...
		foreach($this->days as $key => $value) {
			$renderedDays .= $value->render();
		}
		
		// Render the events for the selected day, if any...
		$renderedEvents = "<h2 class=\"event_calendar_events\">" . i18n("Events for ##0##", array(formatTime( mktime(0, 0, 0, $this->month, $this->selectedDay, $this->year) ) ) ) . "<a class=\"namedanchor\" name=\"events\">&nbsp;</a></h2>";
		if (count($this->days[$this->selectedDay]->events) > 0) {
			foreach($this->days[$this->selectedDay]->events as $key => $value) {
				$renderedEvents .= $value->render();
			}
		} else {
			$renderedEvents .= "<p>" . i18n("There are no events on this date") . "</p>";
		}
		
		$data = "
			<table class=\"event_calendar\">
			$navigator
			$leadinData
			$renderedDays
			$leadoutData
			</table>
			$renderedEvents";
		return $data;
	}
}

/**
 * Returns the url that corresponds to the globalID for an event, if appropriate
 *
 * @param     $splitID       The global ID to check
 *
 * @return    string         A string containing the URL corresponding to the globalID, if existing
 */
function globalID_events( $splitID )
{
	$pageID = null;
	
/*	// Is the globalID directly aimed at this module?
	// Do we know anything about the page the event calendar sits on?
	if( $splitID[0] == "event" )
	{
		$_REQUEST["page_id"] = $foo_pageID;
		$pageID = $foo_pageID;
	}
	else 
	*/
	// Or is it aimed at the module's profile module?
	if( $splitID[0] == "user" && $splitID[2] == "events" )
	{
		$_REQUEST["page_id"] = "13";
		$_REQUEST["user_id"] = $splitID[1];
		$_REQUEST["module"] = "profilemodule_events";
		
		if( array_key_exists( 3, $splitID ) && array_key_exists( 4, $splitID ) )
		{
			$_REQUEST["event_creator"] = $splitID[3];
			$_REQUEST["event_timestamp"] = $splitID[4];
		}
		
		$pageID = "13";
	}
	
	return $pageID;
}

function parse_events_calendar($page_data) {
	global $setup_folder;

	while (true) {
		$begpos = strpos($page_data, "\\calendar("); //where does the first feed begin
		if ($begpos === false) break; //all feeds have been parsed, no need to check more
		$endpos = strpos($page_data, ")", $begpos + 9); //feed end

		$categories = substr( $page_data, $begpos + 10, ($endpos - $begpos) - 10);
		
		$calendar = new eventsMonth();
		$data = $calendar->render();
		
		$page_data = str_replace("\\calendar($categories)", $data, $page_data);
	}
	return $page_data;
}

function parse_events_event( $page_data )
{
	global $setup_folder;

	while( true )
	{
		$begpos = strpos( $page_data, "\\event(" ); //where does the first feed begin
		if( $begpos === false )
			break; //all feeds have been parsed, no need to check more
		$endpos = strpos( $page_data, ")", $begpos + 6 ); //feed end

		$creator_and_timestamp = substr( $page_data, $begpos + 7, ( $endpos - $begpos ) - 7 );
		
		if( $_REQUEST["event_creator"] != "" && $_REQUEST["event_timestamp"] != "" )
			$creator_and_timestamp = $_REQUEST["event_creator"] . ", " . $_REQUEST["event_timestamp"];
		
		if( $creator_and_timestamp != "" )
		{
			list( $creator, $timestamp ) = explode( ",", $user_and_timestamp );
			$creator = trim( $creator );
			$timestamp = trim( $timestamp );
			$theevent = new eventsEvent($creator, $timestamp);
			$data = $theevent->render();
		}
		
		$page_data = str_replace("\\event($creator_and_timestamp)", $data, $page_data);
	}
	return $page_data;
}

function parse_events_events( $page_data )
{
	global $setup_folder;

	while( true )
	{
		$begpos = strpos( $page_data, "\\events(" ); //where does the first feed begin
		if( $begpos === false )
			break; //all feeds have been parsed, no need to check more
		$endpos = strpos( $page_data, ")", $begpos + 7 ); //feed end

		$number_of_days = trim( substr( $page_data, $begpos + 8, ( $endpos - $begpos ) - 8 ) );
		if( $number_of_days == "" )
			$number_of_days = 7; // Show the next seven days...
		
		$theday = time();
		
		for( $i = 0; $i < $number_of_days; $i++ )
		{
			$year = date("Y", $theday);
			$month = date("n", $theday);
			$day = date("j", $theday);
			$theEventsDay = new eventsDay( $year, $month, $day );
			
			if( count( $theEventsDay->events ) > 0 )
				$data .= "
			<div class=\"events_eventlist_day\">" . i18n( date( "l", $theday ) ) . date( " j/n", $theday ) . "</div>" . $theEventsDay	->renderEventsList();

			$theday = mktime(0, 0, 0, $month, $day + 1, $year); // Next day...
		}
		
		if( $data == "" )
			$data = "<div class=\"events_eventlist_empty\">" . i18n("There are no events in the next ##0## days", array( $number_of_days ) ) . "</div>";
		
		$page_data = str_replace("\\events($number_of_days)", $data, $page_data);
	}
	return $page_data;
}

function profilemodule_events() {
	global $tabwidget, $setup_folder, $userinfo_folder, $language, $languages, $permissionlevels_array, $eventcategories, $formatting_toolbar, $viewUser;
	
	$allowEdit = isAllowed("global_user_edit") || ($_REQUEST["user_id"] != "" && $_REQUEST["user_id"] == currentUser());
	
	if ($_POST["event_save_settings"]) {
		saveUserInfo($_REQUEST["user_id"], "event_viewlevel", $_REQUEST["event_viewlevel"]);
		saveUserInfo($_REQUEST["user_id"], "event_editlevel", $_REQUEST["event_editlevel"]);
		saveUserInfo($_REQUEST["user_id"], "event_commentlevel", $_REQUEST["event_commentlevel"]);
		saveUserInfo($_REQUEST["user_id"], "event_participatelevel", $_REQUEST["event_participatelevel"]);
		if (is_array($_REQUEST["event_category"])) {
			foreach ($_REQUEST["event_category"] as $key => $value) {
				$categories_temp[] = $value;
			}
			saveUserInfo($_REQUEST["user_id"], "event_categories", serialize($categories_temp));
		}
	}
	
	$temp_options["viewlevel"] = getUserInfo($_REQUEST["user_id"], "event_viewlevel");
	if ($temp_options["viewlevel"] == i18n("No ##0## information\n", array("event_viewlevel"))) {
		$temp_options["viewlevel"] = 0; // Allow everybody to view by default
		saveUserInfo($_REQUEST["user_id"], "event_viewlevel", 0);
	}
	$temp_options["editlevel"] = getUserInfo($_REQUEST["user_id"], "event_editlevel");
	if ($temp_options["editlevel"] == i18n("No ##0## information\n", array("event_editlevel"))) {
		$temp_options["editlevel"] = 4; // Allow only the user + admin to edit by default
		saveUserInfo($_REQUEST["user_id"], "event_editlevel", 0);
	}
	$temp_options["commentlevel"] = getUserInfo($_REQUEST["user_id"], "event_commentlevel");
	if ($temp_options["commentlevel"] == i18n("No ##0## information\n", array("event_commentlevel"))) {
		$temp_options["commentlevel"] = 2; // Allow only registered users that are not blacklisted to comment by default
		saveUserInfo($_REQUEST["user_id"], "event_commentlevel", 0);
	}
	$temp_options["participatelevel"] = getUserInfo($_REQUEST["user_id"], "event_participatelevel");
	if ($temp_options["participatelevel"] == i18n("No ##0## information\n", array("event_participatelevel"))) {
		$temp_options["participatelevel"] = 2; // Allow only registered users that are not blacklisted to participate by default
		saveUserInfo($_REQUEST["user_id"], "event_participatelevel", 0);
	}
	
	$temp_options["categories"] = getUserInfo($_REQUEST["user_id"], "event_categories");
	if ($temp_options["categories"] == i18n("No ##0## information\n", array("event_categories"))) {
		$temp_options["categories"] = array();
		saveUserInfo($_REQUEST["user_id"], "event_categories", serialize(array()));
	} else {
		$temp_options["categories"] = unserialize($temp_options["categories"]);
	}
	
	$eventcategories_user = getUserInfo($_REQUEST["user_id"], "event_usercategories");
	if ($eventcategories_user == i18n("No ##0## information\n", array("event_usercategories"))) {
		$eventcategories_user = array();
	} else {
		$eventcategories_user = unserialize($eventcategories_user);
	}
	
	if ($_REQUEST["action"] == "options" && $allowEdit) {
		$tabwidget->addCommand(i18n("View calendar"), i18n("View the calendar"), siteURL() . "?page_id=13&amp;user_id={$_REQUEST['user_id']}&amp;profile_mode=module&amp;module=profilemodule_events&amp;event_selectedday={$_REQUEST['event_selectedday']}");

		foreach($permissionlevels_array as $key => $value) {
			if ($temp_options["viewlevel"] == $key) {
				$default = " selected";
			} else {
				$default = "";
			}
			$eventoptions_viewlevel .= "<option value=\"$key\"$default>$value</option>";
		}
		
		foreach($permissionlevels_array as $key => $value) {
			if ($temp_options["editlevel"] == $key) {
				$default = " selected";
			} else {
				$default = "";
			}
			$eventoptions_editlevel .= "<option value=\"$key\"$default>$value</option>";
		}
		
		foreach($permissionlevels_array as $key => $value) {
			if ($temp_options["commentlevel"] == $key) {
				$default = " selected";
			} else {
				$default = "";
			}
			$eventoptions_commentlevel .= "<option value=\"$key\"$default>$value</option>";
		}
		
		foreach($permissionlevels_array as $key => $value) {
			if ($temp_options["participatelevel"] == $key) {
				$default = " selected";
			} else {
				$default = "";
			}
			$eventoptions_participatelevel .= "<option value=\"$key\"$default>$value</option>";
		}
		
		if ($_REQUEST["event_add_category"]) {
			$temp_options["categories"][] = "";
		}
		
		if (is_array($_REQUEST["event_remove_category"])) {
			foreach ($_REQUEST["event_remove_category"] as $key => $value) {
				unset($temp_options["categories"][$key]);
			}
			saveUserInfo($_REQUEST["user_id"], "event_categories", serialize($temp_options["categories"]));
		}
		
		$data = "
		<form action=\"" . thisPageURL() . "\" method=\"post\">
		<table class=\"setup\">
		<tr class=\"setup\">
			<th class=\"setup\" colspan=\"2\">" . i18n("By default new events have the following settings") . "</th>
		</tr>
		<tr class=\"setup\">
			<td class=\"setup\" width=\"50%\">" . i18n("Viewing allowance level") . "</td>
			<td class=\"setup\" width=\"50%\"><select style=\"width: 100%\" name=\"event_viewlevel\">$eventoptions_viewlevel</select></td>
		</tr>
		<tr class=\"setup\">
			<td class=\"setup\" width=\"50%\">" . i18n("Editing allowance level") . "</td>
			<td class=\"setup\" width=\"50%\"><select style=\"width: 100%\" name=\"event_editlevel\">$eventoptions_editlevel</select></td>
		</tr>
		<tr class=\"setup\">
			<td class=\"setup\" width=\"50%\">" . i18n("Commenting allowance level") . "</td>
			<td class=\"setup\" width=\"50%\"><select style=\"width: 100%\" name=\"event_commentlevel\">$eventoptions_commentlevel</select></td>
		</tr>
		<tr class=\"setup\">
			<td class=\"setup\">" . i18n("Participation allowance level") . "</td>
			<td class=\"setup\"><select style=\"width: 100%\" name=\"event_participatelevel\">$eventoptions_participatelevel</select></td>
		</tr>
		<tr class=\"setup\">
			<td class=\"setup\">
			" . i18n("Default categories");
		
		foreach ($temp_options["categories"] as $key => $value) {
			$categories_temp = "<optgroup label=\"" . i18n("Your categories") . "\">";
			foreach($eventcategories_user as $key2 => $value2) {
				if ($value2 == $value) { $default_temp = " selected"; } else { $default_temp = ""; }
				$categories_temp .= "<option value=\"$value2\"$default_temp>" . i18n($value2) . "</option>";
			}
			$categories_temp .= "</optgroup><optgroup label=\"" . i18n("Site-wide categories") . "\">";
			foreach($eventcategories as $key2 => $value2) {
				if ($value2 == $value) { $default_temp = " selected"; } else { $default_temp = ""; }
				$categories_temp .= "<option value=\"$value2\"$default_temp>" . i18n($value2) . "</option>";
			}
			$data .= "<input style=\"float: right;\" type=\"submit\" name=\"event_remove_category[$key]\" value=\"-\" /></td>
			<td class=\"setup\"><select style=\"width: 100%\" name=\"event_category[$key]\">$categories_temp</optgroup></select></td>
		</tr>
		<tr class=\"setup\">
			<td class=\"setup\">";
		}
		
		$data .= "&nbsp;</td>
			<td class=\"setup\"><input type=\"submit\" name=\"event_add_category\" value=\"" . i18n("Add category") . "\" /></td>
		</tr>
		<tr class=\"setup\">
			<td class=\"setup\"><input type=\"submit\" name=\"event_save_settings\" value=\"" . i18n("Save options") . "\" /></td>
			<td class=\"setup wikirightalign\"><input type=\"reset\" value=\"" . i18n("Revert to saved") . "\" /></td>
		</tr>
		</table>
		</form>";
	} else {
		$calendar = new eventsMonth();
		foreach($calendar->days as $key => $value) {
			$calendar->days[$key]->hasOwnEvent = false;
			foreach($calendar->days[$key]->events as $key2 => $value2) {
				if ($value2->creator == $_REQUEST["user_id"]) $calendar->days[$key]->hasOwnEvent = true;
			}
		}
		$data = $calendar->render();
		
		if ($allowEdit) {
			if ($_REQUEST["action"] == "addEvent") {
				$timestamp = mktime(date("h"), date("i"), date("s"), $calendar->month, $calendar->selectedDay, $calendar->year);
				$calendar->days[$calendar->selectedDay]->addEvent(i18n("Untitled event"), i18n("Type your description here"), $timestamp, 0, $_REQUEST["user_id"], $temp_options["viewlevel"], $temp_options["editlevel"], $temp_options["commentlevel"], $temp_options["participatelevel"], $temp_options["categories"]);
				header("Location: " . siteURL() . "?page_id=13&user_id={$_REQUEST['user_id']}&profile_mode=module&module=profilemodule_events&event_selectedyear={$calendar->year}&event_selectedmonth={$calendar->month}&event_selectedday={$calendar->selectedDay}&event_creator={$_REQUEST['user_id']}&editEvent=$timestamp#editEvent");
			} else if ($_REQUEST["editEvent"] != "" && $_REQUEST["event_creator"]) {
				$event = new eventsEvent($_REQUEST["event_creator"], $_REQUEST["editEvent"]);
				
				if ($_POST) {
					$temp_year = $_REQUEST["year"];
					$temp_month = $_REQUEST["month"];
					$temp_date = $_REQUEST["day"];
					$temp_hour = $_REQUEST["hour"];
					$temp_minute = $_REQUEST["minute"];
					$temp_second = $_REQUEST["second"];
					$event->title = htmlspecialchars(stripslashes($_REQUEST["event_title"]));
					$event->description = htmlspecialchars(stripslashes($_REQUEST["event_description"]));
					$event->duration = ($_REQUEST["duration_days"] * 1440) + ($_REQUEST["duration_hours"] * 60) + $_REQUEST["duration_minutes"];
					$event->viewlevel = $_REQUEST["event_viewlevel"];
					$event->editlevel = $_REQUEST["event_editlevel"];
					$event->editlevel = $_REQUEST["event_commentlevel"];
					$event->participatelevel = $_REQUEST["event_participatelevel"];
					$event->categories = $_REQUEST["event_category"];
					if (is_array($_POST["event_remove_category"])) {
						foreach($_POST["event_remove_category"] as $key => $value) {
							unset($event->categories[$key]);
						}
					}
					if ($_POST["event_add_category"]) $event->categories[] = "";
				} else {
					// time is set here only when not submit data...
					$temp_year = date("Y", $event->timestamp);
					$temp_month = date("n", $event->timestamp);
					$temp_date = date("j", $event->timestamp);
					$temp_hour = date("H", $event->timestamp);
					$temp_minute = date("i", $event->timestamp);
					$temp_second = date("s", $event->timestamp);
				}
				
				if ($_POST["save_event"]) {
					$event->delete(); // Delete the event at its current position, or we'll end up with loads of ghosts all over the place!
					$event->timestamp = mktime($temp_hour, $temp_minute, $temp_second, $temp_month, $temp_date, $temp_year); // Set the new timestamp
					$event->save(); // Save the event into its new position
					header("Location: " . globalIDtoURL("user/$userView/events") . "?event_selectedyear=$temp_year&event_selectedmonth=$temp_month&event_selectedday=$temp_date");
				}
				
				for ($i = date("Y") - 50; $i <= date("Y") + 50; $i++) { $allyears[] = $i; }
				foreach ($allyears as $key => $value) {
					if ($value == $temp_year) { $selected = " selected"; } else { $selected = ""; }
					$temp_years .= "<option value=\"$value\"$selected>$value</option>";
				}
				
				$allmonths = array(1 => i18n("January"), 2 => i18n("February"), 3 => i18n("March"), 4 => i18n("April"), 5 => i18n("May"), 6 => i18n("June"), 7 => i18n("July"), 8 => i18n("August"), 9 => i18n("September"), 10 => i18n("October"), 11 => i18n("November"), 12 => i18n("December"));
				foreach ($allmonths as $key => $value) {
					if ($key == $temp_month) { $selected = " selected"; } else { $selected = ""; }
					$temp_months .= "<option value=\"$key\"$selected>$value</option>";
				}
				
				for ($i = 1; $i <= 31; $i++) { $alldates[] = $i; }
				foreach ($alldates as $key => $value) {
					if ($value == $temp_date) { $selected = " selected"; } else { $selected = ""; }
					$temp_days .= "<option value=\"$value\"$selected>$value</option>";
				}
				
				for ($i = 0; $i <= 23; $i++) { if (strlen($i) < 2) { $allhours[] = "0$i"; } else { $allhours[] = $i; } }
				foreach ($allhours as $key => $value) {
					if ($value == $temp_hour) { $selected = " selected"; } else { $selected = ""; }
					$temp_hours .= "<option value=\"$value\"$selected>$value</option>";
				}
				
				for ($i = 0; $i <= 59; $i++) { if (strlen($i) < 2) { $allminutes[] = "0$i"; } else { $allminutes[] = $i; } }
				foreach ($allminutes as $key => $value) {
					if ($value == $temp_minute) { $selected = " selected"; } else { $selected = ""; }
					$temp_minutes .= "<option value=\"$value\"$selected>$value</option>";
				}
				foreach ($allminutes as $key => $value) {
					if ($value == $temp_second) { $selected = " selected"; } else { $selected = ""; }
					$temp_seconds .= "<option value=\"$value\"$selected>$value</option>";
				}
				
				// Duration
				$test_days = floor($event->duration / 1440);
				for($i = 0; $i <= 31; $i++) {
					if ($test_days == $i) {
						$selected = " selected";
					} else {
						$selected = "";
					}
					$temp_duration_days .= "<option value=\"$i\"$selected>" . i18n("##0## days", array($i)) . "</option>";
				}
				$test_hours = floor(($event->duration - ($test_days * 1440)) / 60);
				for($i = 0; $i <= 24; $i++) {
					if ($test_hours == $i) {
						$selected = " selected";
					} else {
						$selected = "";
					}
					$temp_duration_hours .= "<option value=\"$i\"$selected>" . i18n("##0## hours", array($i)) . "</option>";
				}
				$test_minutes = ($event->duration - ($test_days * 1440) - ($test_hours * 60));
				for($i = 0; $i <= 60; $i++) {
					if ($test_minutes == $i) {
						$selected = " selected";
					} else {
						$selected = "";
					}
					$temp_duration_minutes .= "<option value=\"$i\"$selected>" . i18n("##0## minutes", array($i)) . "</option>";
				}
				
				// Viewing allowance level
				foreach($permissionlevels_array as $key => $value) {
					if ($event->viewlevel == $key) {
						$default = " selected";
					} else {
						$default = "";
					}
					$viewlevel .= "<option value=\"$key\"$default>$value</option>";
				}
				// Editing allowance level
				foreach($permissionlevels_array as $key => $value) {
					if ($event->editlevel == $key) {
						$default = " selected";
					} else {
						$default = "";
					}
					$editlevel .= "<option value=\"$key\"$default>$value</option>";
				}
				// Commenting allowance level
				foreach($permissionlevels_array as $key => $value) {
					if ($event->commentlevel == $key) {
						$default = " selected";
					} else {
						$default = "";
					}
					$commentlevel .= "<option value=\"$key\"$default>$value</option>";
				}
				// Participation allowance level
				foreach($permissionlevels_array as $key => $value) {
					if ($event->participatelevel == $key) {
						$default = " selected";
					} else {
						$default = "";
					}
					$participatelevel .= "<option value=\"$key\"$default>$value</option>";
				}
				$data .= "<form action=\"" . thisPageURL( false, true, array( "event_selectedyear" => $calendar->year, "event_selectedmonth" => $calendar->month, "event_selectedday" => $calendar->selectedDay, "event_creator" =>  $event->creator, "editEvent" => $event->timestamp ) ) . "#editEvent\" method=\"post\"><table class=\"setup\">
				<tr class=\"setup\">
					<th colspan=\"2\" class=\"setup\"><a class=\"namedanchor\" name=\"editEvent\">" . i18n("Edit event") . "</a></th>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("Creator:") . "
					<small class=\"comment\">" . i18n("This is always you - this cannot be changed") . "</small></td>
					<td class=\"setup\"><input type=\"hidden\" name=\"event_creator\" value=\"{$_REQUEST["user_id"]}\" />{$_REQUEST['user_id']}</td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("Title:") . "
					<small class=\"comment\">" . i18n("The headline for this event") . "</small></td>
					<td class=\"setup\"><input style=\"width: 99%;\" type=\"text\" name=\"event_title\" value=\"{$event->title}\" /></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("Start time:") . "
					<small class=\"comment\">" . i18n("When does the event begin?") . "</small></td>
					<td class=\"setup\"><div style=\"width: 10%; overflow: hidden; display: inline-block;\">
					" . i18n("On") . "</div><select style=\"width: 30%;\" name=\"year\">$temp_years
					</select><select style=\"width: 40%;\" name=\"month\">$temp_months
					</select><select style=\"width: 20%;\" name=\"day\">$temp_days
					</select><br />
					<div style=\"width: 10%; overflow: hidden; display: inline-block;\">
					" . i18n("At") . "</div><select style=\"width: 30%;\" name=\"hour\">$temp_hours
					</select><select style=\"width: 30%;\" name=\"minute\">$temp_minutes
					</select><select style=\"width: 30%;\" name=\"second\">$temp_seconds
					</select></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("Duration:") . "
					<small class=\"comment\">" . i18n("How long does the event last?") . "</small></td>
					<td class=\"setup\"><select style=\"width: 30%;\" name=\"duration_days\">$temp_duration_days
					</select><select style=\"width: 30%;\" name=\"duration_hours\">$temp_duration_hours
					</select><select style=\"width: 40%;\" name=\"duration_minutes\">$temp_duration_minutes
					</select></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("Viewing allowance level:") . "
					<small class=\"comment\">" . i18n("Who except you are allowed to see this event in the calendar?") . "</small></td>
					<td class=\"setup\"><select style=\"width: 100%\" name=\"event_viewlevel\">$viewlevel</select></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("Editing allowance level:") . "
					<small class=\"comment\">" . i18n("Who except you are allowed to edit this event?") . "</small></td>
					<td class=\"setup\"><select style=\"width: 100%\" name=\"event_editlevel\">$editlevel</select></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("Commenting allowance level:") . "
					<small class=\"comment\">" . i18n("Who except you are allowed to comment on this event?") . "</small></td>
					<td class=\"setup\"><select style=\"width: 100%\" name=\"event_editlevel\">$commentlevel</select></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("Participation allowance level:") . "
					<small class=\"comment\">" . i18n("Who except you are allowed to join this event?") . "</small></td>
					<td class=\"setup\"><select style=\"width: 100%\" name=\"event_participatelevel\">$participatelevel</select></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("Categories") . ":";
					
				if (is_array($event->categories)) {
					foreach ($event->categories as $key => $value) {
						$categories_temp = "<optgroup label=\"" . i18n("Your categories") . "\">";
						foreach($eventcategories_user as $key2 => $value2) {
							if ($value2 == $value) { $default_temp = " selected"; } else { $default_temp = ""; }
							$categories_temp .= "<option value=\"$value2\"$default_temp>" . i18n($value2) . "</option>";
						}
						$categories_temp .= "</optgroup><optgroup label=\"" . i18n("Site-wide categories") . "\">";
						foreach($eventcategories as $key2 => $value2) {
							if (!isAllowed($_REQUEST["user_id"], "global_admin") && $key2 == 0) continue;
							if ($value2 == $value) { $default_temp = " selected"; } else { $default_temp = ""; }
							$categories_temp .= "<option value=\"$value2\"$default_temp>" . i18n($value2) . "</option>";
						}
						$data .= "<input style=\"float: right;\" type=\"submit\" name=\"event_remove_category[$key]\" value=\"-\" /></td>
					<td class=\"setup\"><select style=\"width: 100%\" name=\"event_category[$key]\">$categories_temp</optgroup></select></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">";
					}
				}
				
				$data .= "&nbsp;</td>
					<td class=\"setup\"><input type=\"submit\" name=\"event_add_category\" value=\"" . i18n("Add category") . "\" /></td>
				</tr>
				$formatting_toolbar
				<tr class=\"setup\"><td class=\"setup\" colspan=\"2\">
					<textarea rows=\"15\" cols=\"70\" style=\"width: 100%;\" id=\"edit\" name=\"event_description\">{$event->description}</textarea>
				</td></tr>
				<tr class=\"setup\">
					<td width=\"50%\" class=\"setup\"><input type=\"submit\" name=\"save_event\" value=\"" . i18n("Save event") . "\" /></td>
					<td width=\"50%\"  class=\"setup\ wikirightalign\"><input type=\"reset\" value=\"" . i18n("Reset") . "\" /></td>
				</tr>
				</table></form>";
			} else {
				$tabwidget->addCommand(i18n("Add event"), i18n("Add a new event on this day"), globalIDtoURL("user/$viewUser/events", array( "event_selectedyear" => $calendar->year, "event_selectedmonth" => $calendar->month, "event_selectedday" => $calendar->selectedDay, "action" => "addEvent" ) ) );
			}
			if ($allowEdit) $tabwidget->addCommand(i18n("Options"), i18n("Edit your options"), globalIDtoURL("user/$viewUser/events", array( "event_selectedday" => $_REQUEST['event_selectedday'], "action" => "options" ) ) );
		}
	}
	return $data;
}

function events_savecategories() {
	global $events_folder, $eventcategories;
	natcasesort($eventcategories); // Sort categories alphabetically...
	$file = array_export($eventcategories, "eventcategories");
	$filename = "$events_folder/categories.php";
	if (file_exists($filename)) unlink($filename);
	file_put_contents($filename, $file);
}

function modulesetup_events() {
	global $setup_folder, $eventcategories, $language;
	switch ($_REQUEST["module_action"]) {
	case "category_add":
		if ($_REQUEST["new_category"] != "") {
			$eventcategories[] = str_replace(" ", "_", $_REQUEST["new_category"]);
			events_savecategories();
			header("Location: " . thisPageURL(true));
		} else {
			$data = "
			<form action=\"" . globalIDtoURL("setup/modulesetup/events/na/na/category_add") . "\" method=\"post\">
			<table class=\"setup\">
			<tr class=\"setup\">
				<td class=\"setup\" width=\"50%\">" . i18n("Category name") . ":
				</td>
				<td class=\"setup\" width=\"50%\"><input style=\"width: 100%;\" type=\"text\" name=\"new_category\" /></td>
			</tr>
			<tr class=\"setup\">
				<td class=\"setup\">
				<input type=\"submit\" value=\"" . i18n("Create category") . "\" />
				</td><td class=\"setup\" align=\"right\">
				<input type=\"reset\" value=\"" . i18n("Reset") . "\" />
				</td>
			</tr>
			</table>
			</form>";
		}
		break;
	case "category_remove":
		if ($_REQUEST["category_key"] == 0) {
			$data = "<p>" . i18n("You are not allowed to remove the system category as it is integral to the system. In fact there is not even any GUI to let you do this, so the only way to get this message is to attempt writing the URL yourself. Naughty!") . "</p>";
		} else {
			if ($_REQUEST["confirm_delete"] == "true") {
				$category_name = $eventcategories[$_REQUEST['category_key']];
				unset($eventcategories[$_REQUEST['category_key']]);
				events_savecategories();
				$data = "<p>" . i18n("The &quot;##0##&quot; was deleted, click here to return...", $category_name) . "</a></p>";
				header("Location: " . thisPageURL(true));
			} else {
				$data = "
				<p>" . i18n("You have requested to remove the category &quot;##0##&quot;. Please confirm this choice below. Deleting the category WILL NOT delete posts already created using the category - the category will only not be available for new entries.", array($eventcategories[$_REQUEST["category_key"]])) . "</p>
				<div class=\"wikicenteralign\"><a class=\"command\" href=\"" . thisPageURL( false, true, array( "module_action" => "category_remove", "category_key" => $_REQUEST['category_key'], "confirm_delete" => "true" ) ) . "\">[" . i18n("Comfirm delete") . "]</a>
				<a class=\"command\" href=\"" . thisPageURL() . "\">[" . i18n("Abort") . "]</a></div>";
			}
		}
		break;
	case "category_edit":
		if ($_REQUEST["category_key"] == 0) {
			$data = "<p>" . i18n("You are not allowed to edit the system category as it is integral to the system. In fact there is not even any GUI to let you do this, so the only way to get this message is to attempt writing the URL yourself. Naughty!") . "</p>";
		} else {
			if ($_REQUEST["new_category"] != "") {
				$category_name = $eventcategories[$_REQUEST['category_key']];
				$eventcategories[$_REQUEST['category_key']] = str_replace(" ", "_", $_REQUEST["new_category"]);
				events_savecategories();
				$data = "<p>" . i18n("The &quot;##0##&quot; was changed successfully to &quot;##1##&quot;, click here to return...", array($category_name, $_REQUEST["new_category"])) . "</a></p>";
				header("Location: " . thisPageURL(true));
			} else {
				$data = "
				<form action=\"" . thisPageURL( false, true, array( "module_action" => "category_edit", "category_key" => $_REQUEST["category_key"] ) ) . "\" method=\"post\">
				<table class=\"setup\">
				<tr class=\"setup\">
					<td class=\"setup\" width=\"50%\">" . i18n("Old category name") . ":
					</td>
					<td class=\"setup\" width=\"50%\">" . $eventcategories[$_REQUEST["category_key"]] . "</td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("New category name") . ":
					<small class=\"comment\">" . i18n("Please take note that the changes made to this category will not affect events already created") . "</small>
					</td>
					<td class=\"setup\"><input style=\"width: 100%;\" type=\"text\" name=\"new_category\" value=\"" . $eventcategories[$_REQUEST["category_key"]] . "\" /></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">
					<input type=\"submit\" value=\"" . i18n("Save category") . "\" />
					</td><td class=\"setup\" align=\"right\">
					<input type=\"reset\" value=\"" . i18n("Reset") . "\" />
					</td>
				</tr>
				</table>
				</form>";
			}
		}
		break;
	case "help":
		$help_data = "<div style=\"border: 1px dashed grey;\">" . i18n("To create system entries, sign into a site administrator account and add a blog entry to the System category. Only administrators are allowed to use the System category. Use \\calendar(system_category) to show these entries.") . "</div>";
		$help_link = "<a class=\"command\" href=\"" . thisPageURL() . "\">[" . i18n("Hide help") . "]</a>";
	default:
		if (strlen($help_link) == 0) $help_link = "<a class=\"command\" href=\"" . thisPageURL( false, true, array("module_action" => "help") ) . "\">[" . i18n("Show help") . "]</a>";
		$data .= "
		<div align=\"right\">
		$help_link
		<a class=\"command\" href=\"" . globalIDtoURL("setup/modulesetup/events/na/na/category_add") . "\">[" . i18n("Add category") . "]</a>
		</div>
		$help_data
		<table class=\"setup\">
			<tr class=\"setup\">
				<th class=\"setup\" width=\"50%\">" . i18n("Category") . "</th>
				<th class=\"setup\" width=\"50%\">" . i18n("Functions") . "</th>
			</tr>";
		foreach($eventcategories as $key => $value) {
			$data .= "
			<tr class=\"setup\">
				<td class=\"setup\">" . i18n($value) . "</td>
				<td class=\"setup\" align=\"right\">";
			if ($key == 0) {
				$data .= "<small class=\"comment\">" . i18n("Edit and delete not allowed on system category.") . "</small>";
			} else {
				$data .= "
					<a class=\"command\" href=\"" . thisPageURL( false, true, array( "module_action" => "category_remove", "category_key" => $key ) ) . "\">[" . i18n("Delete") . "]</a>
					<a class=\"command\" href=\"" . thisPageURL( false, true, array( "module_action" => "category_edit", "category_key" => $key ) ) . "\">[" . i18n("Edit") . "]</a>
					";
			}
			$data .= "
				</td>
			</tr>";
		}
		$data .= "
		</table>";
	}
	return $data;
}

$filename = "$events_folder/categories.php";
if (file_exists($filename)) {
	include($filename);
} else {
	$eventcategories = array("system_category");
	events_savecategories();
}

/**
 * Return an array of data about the thumbnail for a global ID
 * As the events don't have thumbnails per se, we simply pass a fitting image directly back
 *
 * @param     splitID        The global ID to check
 *
 * @return    Array containing array
 */
function globalIDThumbnail_events( $splitID )
{
	global $module_folder;
	$data = array( "uri" => siteURL(true) . "$module_folder/events/thumb.png", "width" => 64, "height" => 64 );
	return $data;
}
?>
