<?php
/**
 * New Gallery module for Travelsized CMS
 * Based around the old gallery, but with a new, much improved (and much easier to maintain) code structure
 *
 * @Author	Dan Jensen <admin@leinir.dk>
 * @Version 0.1
 * @Date 3rd Dec, 2006
 *
 *	The structure of a gallery is as follows below:
 *
 * A gallery is a directory with...
 * \arg \b gallery.txt A text file, containing the gallery's title and description, in the form of "title\ndescription" - deleted on first save
 * \arg \b gallery.php A text file, containing the fallery's options and such, descriptions and likewise
 * \arg \b comments/*.jpg/png/gif.inc A text file containing the exported comments data
 * \arg \b images/*.jpg/png/gif The images themselves
 * \arg \b data/*.language.txt Old data files, containing description - deleted on first save
 * \arg \b data/*.language.php New data files, containing all sorts of options
 * \arg \b thumbs/*.jpg/png/gif Thumbnail files
 * \arg \b galleries/* Subgalleries
 */

$modules_names[] = "Gallery2";
$modules_versions[] = "0.1";
$modules_descriptions[] = "A simple but powerful gallery";

$modules_setup[] = "modulesetup_gallery2";

$profilemodules[] = "profilemodule_gallery2";
$modules_globalID[] = "globalID_gallery2";

$modules[] = "parse_gallery2";

$page_help["&#92;gallery(txt1)"] = "This will show the gallery with the global ID txt1.";
$page_help["&#92;galleryimage(txt1,txt2,txt3)"] = "This will show the image thumbnail for the image with the global ID txt1. If txt2 is present the image will be floated to the alignment as described. Accepted values are left, center and right. If you write anything at all as txt3, the shown thumbnail will not be linked to the midsize view. If omitted, the image will be linked.";
$page_help["&#92;profilegallerylist(txt1)"] = "This will show a list of profiles where there is at least one image in the user's profile gallery. txt1 is optional, and is the style of listing. Defaults to three-up.";

// Set up some default options
$gallery_options = array(
	"page_id" => 12, // Default to modules... Refresh to here if the user hasn't assigned a page yet
	"title" => i18n("Travelsized Images"),
	"menu_id" => 100,
	"maxwidth" => 1600,
	"maxheight" => 1200,
	"midwidth" => 600,
	"midheight" => 480,
	"thumbwidth" => 120,
	"thumbheight" => 120,
	"viewType" => 0, // Three-up
	"viewLevel" => 0, // Everybody can view
	"commentLevel" => 1, // Registered users can comment
	"editLevel" => 4, // Admin can edit
	"embedLevel" => 0, // Admin can edit
	);

// This is old - once all old galleries have disappeared, kill this code!
if( file_exists( $module_folder . "/gallery/options.inc" ) )
{
	$temp_options = $gallery_options; // Fill up with the defaults...
	
	include( $module_folder . "/gallery/options.inc" );
	$temp_options["maxwidth"] = $gallery_options["galleryoption_resize_width"];
	$temp_options["maxheight"] = $gallery_options["galleryoption_resize_height"];
	$temp_options["midwidth"] = $gallery_options["galleryoption_midsize_width"];
	$temp_options["midheight"] = $gallery_options["galleryoption_midsize_height"];
	$gallery_options = $temp_options;
	
	file_put_contents( $module_folder . "/gallery/options.php", array_export( $gallery_options, "gallery_options" ) );
	unlink( $module_folder . "/gallery/options.inc" );
}

if( file_exists( $module_folder . "/gallery/options.php" ) )
	include( $module_folder . "/gallery/options.php" );

//! An image, which needs to be contained within a gallery
class gallery2_image extends lockableClass
{
	var $imagename;          ///< The picture's filename
	var $parentGallery;      ///< @var gallery2_gallery Reference to the parent gallery. Used for fetching such things as permission levels
	var $prevGlobalID;       ///< What's the previous image's global ID - null if none
	var $nextGlobalID;       ///< What's the next image's global ID - null if none
	
	//@{
	/** The contents of each of these are generated by the constructor */
	var $thumbURL;           ///< The picture's full thumbnail URL
	var $imageURL;           ///< The picture's full image URL
	var $midsizeURL;         ///< The picture's full midsize image URL
	var $globalID;           ///< The global ID for this image
	var $splitID;            ///< The split ID for this image
	//@}
	
	//@{
	/** The contents of each of these are loaded by the constructor. Make sure to call save() on changing these! */
	var $descriptions;       ///< The description of the picture, array, one for each language
	var $viewCount;          ///< How many times have this image been viewed? User counts are kept in user's home dir, no user data outside there
	var $commentCount;       ///< How many comments exist in this image? Cached so the gallery does not need to load the comments each time, just to check that
	var $width;              ///< The image's width in pixels
	var $height;             ///< The image's height in pixels
	var $weight;             ///< The image's weight in bytes
	var $comments;           ///< Comments data
	//@}
	
	///\private
	var $dataFile;           ///< The data file used for this image
	var $commentFile;        ///< The data file used for this image
	
	///\publicsection
	/**
	 * Constructor for the image
	 *
	 * @param    gallery2_gallery    $parentGallery         A reference to the parent gallery
	 * @param    string              $imagename             A string containing the image's name
	 *
	 * @return   gallery2_image  An instance of the gallery2_image class
	 */
	function gallery2_image( $parentGallery, $imagename )
	{
		global $languages;
		
		$this->parentGallery = &$parentGallery;
		$this->imagename = $imagename;
		
		$this->thumbURL = $this->parentGallery->baseurl . "/thumbs/" . $imagename;
		$this->midsizeURL = $this->parentGallery->baseurl . "/midsize/" . $imagename;
		$this->imageURL = $this->parentGallery->baseurl . "/images/" . $imagename;
		$this->globalID = $this->parentGallery->globalID . "/" . $imagename;
		$this->splitID = $this->parentGallery->splitID;
		$this->splitID[] = $imagename;
		$this->dataFile = $this->parentGallery->basedir . "/data/" . $imagename . ".php";
		$this->commentFile = $this->parentGallery->basedir . "/comments/" . $imagename . ".php";
		$this->lockfile = $this->parentGallery->basedir . "/comments/" . $imagename . ".lock";
		
		if( file_exists( $this->dataFile ) )
		{
			include( $this->dataFile );
			$this->descriptions = $image["descriptions"];
			$this->viewCount = $image["viewCount"];
			$this->commentCount = $image["commentCount"];
			$this->width = $image["width"];
			$this->height = $image["height"];
			$this->weight = $image["weight"];
			
			$comments = array();
			if( file_exists( $this->commentFile ) )
				include( $this->commentFile );
			$this->comments = $comments;
		}
		else
		{
			/// TODO: Make sure this is removed once we're certain no old galleries exist!
			
			// load old data and generate some defaults...
			foreach( $languages as $key => $value )
			{
				if( file_exists( $this->parentgallery->basedir . "/data/" . $this->imagename . ".$value.txt" ) )
				{
					$theData = file_get_contents( $this->parentGallery->basedir . "/data/" . $this->imagename . ".$value.txt" );
					unlink( $this->parentgallery->basedir . "/data/" . $this->imagename . ".$value.txt" );
				}
				$this->descriptions[$value] = $theData;
			}
			
			$this->viewCount = 0;
			list( $this->width, $this->height ) = getimagesize( $this->parentGallery->basedir . "/images/" . $this->imagename );
			$this->weight = fileweight( $this->parentGallery->basedir . "/images/" . $this->imagename );
			
			$usercomments = array();
			if( file_exists( $this->parentGallery->basedir . "/comments/" . $this->imagename . ".inc" ) )
				include( $this->parentGallery->basedir . "/comments/" . $this->imagename . ".inc" );
			$this->comments = $usercomments;
			if( file_exists( $this->parentGallery->basedir . "/comments/" . $this->imagename . ".inc" ) )
				unlink( $this->parentGallery->basedir . "/comments/" . $this->imagename . ".inc" );
			
			$commenthandler = new commentHandler($this->comments, $this->parentGallery->owner, $this->parentGallery->commentLevel, $this->parentGallery->editLevel);
			$this->commentCount = $commenthandler->numberOfComments();
			
			$this->save();
		}
	}
	
	/**
	 * Save the image's data to disc. This includes description, viewersCount, commentsCount and comments
	 * Also throws subscription notifications
	 * 
	 * @param    comment                  If this is different from null, the change was a new comment being added
	 * 
	 * @return   true if successful, or an error message describing the error otherwise (for use with errorHandler)
	 */
	function save($comment = null)
	{
		$image["descriptions"] = $this->descriptions;
		$image["viewCount"] = $this->viewCount;
		$image["commentCount"] = $this->commentCount;
		$image["width"] = $this->width;
		$image["height"] = $this->height;
		$image["weight"] = $this->weight;
		if( file_exists( $this->dataFile ) )
			unlink( $this->dataFile );
		file_put_contents( $this->dataFile, array_export( $image, "image" ) );
		
		if( is_dir( $this->parentGallery->basedir ) && !is_dir( $this->parentGallery->basedir . "/comments" ) )
			RecursiveMkdir( $this->parentGallery->basedir . "/comments" );
	
		$comments = $this->comments;
		if( file_exists( $this->commentFile ) )
			unlink( $this->commentFile );
		file_put_contents( $this->commentFile, array_export( $comments, "comments" ) );
		
		// Subscription subsystem
		global $subscriptions;
		if( $comment == null )
		{
			global $language;
			$subject = i18n( "The image ##0## in the gallery ##1## has a been changed", array( $this->imagename, $this->parentGallery->title ) );
			$message = i18n( "The image was edited. The description of the image is:" ) . "\n" . $this->descriptions[$language];
		}
		else if( $comment[0] != "" || $comment[1] != "" )
		{
			$subject = i18n( "The image ##0## in the gallery ##1## has a new comment", array( $this->imagename, $this->parentGallery->title ) );
			$message .= i18n( "Comment by ##0##:", array( $comment[0] ) ) . "\n" . $comment[1];
		}
		if( $subject != "" )
			$subscriptions->handle( $this->getGlobalID("view"), $this->parentGallery->owner, $this->parentGallery->viewLevel, $subject, $message );
	}
	
	/**
	 * Get the global ID for a particular editing function
	 */
	function getGlobalID( $editingFunction )
	{
		$tempSplitID = $this->splitID;
		if( $this->parentGallery->profileGallery )
			$tempSplitID[3] = "$editingFunction";
		else
			$tempSplitID[1] = "$editingFunction";
		return implode( "/", $tempSplitID );
	}
	
	/**
	 * Renders the actual image, respecting the parent gallery's view setting
	 */
	function render()
	{
		// This is only called when doing Midsize... Rendering gallery views is done by the gallery renderer
		global $language, $gallery_options;
		
		$breadcrumbs = $this->parentGallery->getBreadcrumbs();
		$renderedTrail = $breadcrumbs->render();
		
		if( $this->prevGlobalID != "" )
			$navigationControls = drawCommand( "&laquo;", i18n("Previous image"), globalIDtoURL( $this->prevGlobalID ) );
		else
			$navigationControls = "(&laquo;)";
		
		$navigationControls .= " " . drawCommand( "Gallery", i18n("Return to the gallery"), globalIDtoURL($this->parentGallery->globalID) ) . " ";
		
		if( $this->nextGlobalID != "" )
			$navigationControls .= drawCommand( "&raquo;", i18n("Next image"), globalIDtoURL( $this->nextGlobalID ) );
		else
			$navigationControls .= "(&raquo;)";
		
		if( !is_dir( $this->parentGallery->basedir . "/midsize" ) )
			RecursiveMkdir( $this->parentGallery->basedir . "/midsize" );
		if( !file_exists( $this->parentGallery->basedir . "/midsize/" . $this->imagename ) )
		{
			list($x, $y) = getimagesize( $this->parentGallery->basedir . "/images/" . $this->imagename );
			if( $x > $gallery_options["midwidth"] )
			{
				$resize_factor = $gallery_options["midwidth"] / $x;
				$x = $gallery_options["midwidth"];
				$y = $y * $resize_factor;
			}
			if( $y > $gallery_options["midheight"] )
			{
				$resize_factor = $gallery_options["midheight"] / $y;
				$x = $x * $resize_factor;
				$y = $gallery_options["midheight"];
			}
			scaleImage( $this->parentGallery->basedir . "/images/" . $this->imagename, $this->parentGallery->basedir . "/midsize/" . $this->imagename, $x, $y );
		}
		
		$fullsizeCommand = drawCommand( "&diams;", i18n("Show full size"), $this->imageURL );
		
		// Embedding information
		if( userAllows( $this->parentGallery->owner, $this->parentGallery->embedLevel ) )
		{
			$embedLinks = array(
				"BBCode: " => "[url=" . globalIDtoURL( $this->getGlobalID( "view" ) ) . "][img]" . $this->thumbURL . "[/img][/url]",
				"Travelsized: " => "&#92;galleryimage(" . $this->getGlobalID( "view" ) . ")",
				"HTML: " => htmlentities( "<a href=\"" . globalIDtoURL( $this->getGlobalID( "view" ) ) . "\"><img src=\"" . $this->thumbURL . "\" alt=\"" . $this->imagename . "\"></a>" ) );
			
			$embedData = "";
			foreach( $embedLinks as $name => $text )
				$embedData .= "<div class=\"gallery-embed-item\"><span class=\"gallery-embed-name\">$name</span><input type=\"text\" class=\"gallery-embed-text\" value=\"" . $text . "\"/></div>";
		}
		
		$comments = array();
		if( file_exists( $this->commentFile ) )
			include( $this->commentFile );
		$this->comments = $comments;
		$commenthandler = new commentHandler($this->comments, $this->parentGallery->owner, $this->parentGallery->commentLevel, $this->parentGallery->editLevel);
		$commenthandler->thisPageURL = globalIDtoURL( $this->getGlobalID( "view" ) );
		if( $commenthandler->commentsChanged )
		{
			if( $this->lock() )
			{
				$this->comments = $commenthandler->comments;
				$this->commentCount = $commenthandler->numberOfComments();
				$this->save( array( $commenthandler->lastCommenter, $commenthandler->lastComment ) );
			}
			else
				die( i18n("Too many people attempting to access the same image at the same time!") );
			$this->unlock();
		}
		
		// Subscriptions subsystem
		global $subscriptions;
		$renderedSubscriptionControl = $subscriptions->renderSubscribeControl( $this->globalID, false, " discussSubscribeControl" );
		
		$renderedImage .= "
		<div class=\"wikirightalign\">$renderedSubscriptionControl</div>
		<div class=\"gallery-midsize\">
			<div class=\"gallery-midsize-left\">" . $renderedTrail . "</div>
			<div class=\"gallery-midsize-right\">$navigationControls $fullsizeCommand</div>
			<div class=\"gallery-midsize-image\"><a href=\"" . $this->imageURL . "\"><img border=\"0\" alt=\"" . $this->imagename . "\" src=\"" . $this->midsizeURL . "\" /></a></div>
			<div class=\"gallery-midsize-description-title\">" . i18n("Description") . "</div>
			<div class=\"gallery-midsize-description\">" . parse_page_data( $this->descriptions[$language] ) . "</div>";
		if( userAllows( $this->parentGallery->owner, $this->parentGallery->embedLevel ) )
			$renderedImage .= "
			<div class=\"gallery-embed-title\">" . i18n("Embedding") . "</div>
			<div class=\"gallery-embed\">$embedData</div>";
		$renderedImage .= "
		</div>" . $commenthandler->renderComments() . $commenthandler->renderCommentsAdder();
		return $renderedImage;
	}
}

//! The gallery class itself
class gallery2_gallery
{
	///\publicsection
	var $globalID;           ///< The gallery's global ID
	var $splitID;            ///< The gallery's global ID, split up inty array elements
	var $title;              ///< The gallery's title
	var $description;        ///< The gallery's description
	var $viewType;           ///< Which type of gallery view do you want
	
	var $images;             ///< Instances of gallery_image, in the order of the gallery, with filename as key
	var $subgalleries;       ///< A list of galleries contained within this one
	
	var $owner;              ///< Who owns the gallery - for checking permissions against
	var $viewLevel;          ///< The viewing allowance level
	var $commentLevel;       ///< The commenting allowance level
	var $editLevel;          ///< The editing allowance level
	var $embedLevel;         ///< What type of user is allowed to get embedding information for images in this gallery
	
	var $profileGallery = false; ///< Is the gallery a profile gallery? Default = false
	var $edit = false;       ///< Are we editing the gallery?
	var $editMode = "edit";  ///< What type of editing are we doing?
	var $displayImage = null; ///< The image currently being displayed. Only different from null if an image is actually shown!
	var $error = false;      ///< If this gallery does not exist, we need to tell the user that...
	
	var $basedir;            ///< Filled out automatically by the constructor, used by contained images
	var $baseurl;            ///< Filled out automatically by the constructor, used by contained images
	
	var $imageOrder;         ///< The order of the gallery's images - count of this is the number of images in the gallery... Potentially unsafe if stuff is uploaded outside the UI, but we can expect this happens very rarely
	
	/**
	 * Constructor for the gallery module
	 *
	 * @param    globalID    The gallery's global ID
	 * @param    loadMore    A boolean, describing wether or not to actually load the images' data and the subgalleries. Default is true
	 *
	 * @return   An instance of gallery2_gallery
	 */
	function gallery2_gallery( $globalID, $loadMore = true )
	{
		global $setup_folder, $userinfo_folder, $viewUser, $module_folder, $gallery_options;
		
		$this->splitID = explode("/", $globalID);
		$this->subgalleries = array();
		
		if( strpos( end($this->splitID), "." ) > 0 )
		{
			$this->displayImage = end( $this->splitID );
			$this->splitID = array_slice( $this->splitID, 0, -1 );
			$this->globalID = implode( "/", $this->splitID ); // Reset it - need it below
		}
		else
			$this->globalID = $globalID;
		
		if( $this->splitID[0] == "user")
		{
			// This is all the edit options... Switched through in the editor
			// edit, upload, title, description, delete, new (subgallery), move (physically rename), 
			if(
				array_key_exists( 3, $this->splitID ) &&
					( 	$this->splitID[3] == "edit" || // Show gallery list
						$this->splitID[3] == "thumbnail" || // Thumbnail existing image
						$this->splitID[3] == "moveup" || // Move the image up in the gallery order
						$this->splitID[3] == "movedown" || // Move the image down in the gallery order
						$this->splitID[3] == "upload" || // Upload new image
						$this->splitID[3] == "replace" || // Replace an existing image by uploading a new one...
						$this->splitID[3] == "new" || // Create a new subgallery
						$this->splitID[3] == "rename" || // Rename an image physically
						$this->splitID[3] == "details" || // Title and description
						$this->splitID[3] == "move" || // Delete gallery or image
						$this->splitID[3] == "delete" // Move physically
					)
				)
			{
				$tempID = array_slice( $this->splitID, 4);
				$this->edit = true;
				$this->editMode = $this->splitID[3];
				
			}
			// The "gallery" bit here is backwards compatibility
			else if( array_key_exists( 3, $this->splitID ) && ( $this->splitID[3] == "view" || $this->splitID[3] == "gallery" ) )
				$tempID = array_slice( $this->splitID, 4);
			else
			{
				// Add the view item - we do not want to break stuff if the user is trying to link short to a gallery...
				$this->splitID[] = "view";
				$this->globalID = implode( "/", $this->splitID );
				$tempID = array();
			}
			$this->owner = $this->splitID[1];
			$this->basedir = $userinfo_folder . "/". $this->owner . "/gallery";
			// Trix it... this is nasty, but it works :)
			if( count($tempID) > 0 )
			{
				array_splice( $tempID, 0, 0, array("") );
				$this->basedir .= implode( "/galleries/", $tempID);
			}
			$this->profileGallery = true;
		}
		else
		{
			if( array_key_exists( 1, $this->splitID ) && (
					$this->splitID[1] == "edit" || // Show gallery list
					$this->splitID[1] == "thumbnail" || // Thumbnail existing image
					$this->splitID[1] == "moveup" || // Move the image up in the gallery order
					$this->splitID[1] == "movedown" || // Move the image down in the gallery order
					$this->splitID[1] == "upload" || // Upload new image
					$this->splitID[1] == "replace" || // Replace an existing image by uploading a new one...
					$this->splitID[1] == "new" || // Create a new subgallery
					$this->splitID[1] == "rename" || // Rename an image physically
					$this->splitID[1] == "details" || // Title and description
					$this->splitID[1] == "move" || // Delete gallery or image
					$this->splitID[1] == "delete" // Move physically
					)
				)
			{
				$this->edit = true;
				$this->editMode = $this->splitID[1];
			}
			$tempID = array_slice( $this->splitID, 2 );
			$this->basedir = $module_folder . "/gallery/" . implode( "/galleries/", $tempID );
		}
		
		if( $this->profileGallery )
			$this->baseurl .= $userinfo_folder . "/" . $this->owner . "/gallery/";
		else
			$this->baseurl .= $module_folder . "/gallery/";
		$this->baseurl .= implode( "/galleries/", $tempID );
		$this->baseurl = str_replace( "//", "/", $this->baseurl );
		$this->baseurl = siteURL( true ) . $this->baseurl;
		
		if( is_dir( $this->basedir ) && file_exists( $this->basedir . "/gallery.php" ) )
		{
			include( $this->basedir . "/gallery.php" );
			$this->title = $gallery["title"];
			$this->description = $gallery["description"];
			$this->viewType = $gallery["viewType"];
			if( $this->viewType == "" )
				$this->viewType = "table";
			$this->viewLevel = $gallery["viewLevel"];
			$this->commentLevel = $gallery["commentLevel"];
			$this->editLevel = $gallery["editLevel"];
			$this->embedLevel = $gallery["embedLevel"];
			$this->imageOrder = $gallery["imageOrder"];
			
			// If we have a profile gallery, but there are no subdirectories,
			// Make sure there /is/ a gallery to work on...
			if( $this->profileGallery && count( $this->splitID ) == 4 )
			{
				if( !is_dir($this->basedir . "/images") )
					RecursiveMkdir($this->basedir . "/images");
				if( !is_dir($this->basedir . "/data") )
					RecursiveMkdir($this->basedir . "/data");
				if( !is_dir($this->basedir . "/comments") )
					RecursiveMkdir($this->basedir . "/comments");
				if( !is_dir($this->basedir . "/thumbs") )
					RecursiveMkdir($this->basedir . "/thumbs");
				if( !is_dir($this->basedir . "/galleries") )
					RecursiveMkdir($this->basedir . "/galleries");
			}
		}
		else if( !( is_dir( $this->basedir ) && is_dir( $this->basedir . "/images" ) ) && $this->profileGallery && count($this->splitID) == 4 )
		{
			// If this is the case, we have a new profile gallery... Create the stuff
			RecursiveMkdir( $this->basedir . "/images" );
			RecursiveMkdir( $this->basedir . "/data" );
			RecursiveMkdir( $this->basedir . "/comments" );
			RecursiveMkdir( $this->basedir . "/thumbs" );
			RecursiveMkdir( $this->basedir . "/galleries" );
			
			// First, set up us the default options...
			$this->title = i18n( "##0##'s gallery", array(getUserInfo( $this->owner, "name" )) );
			$this->description = i18n( "No description - change it by editing the gallery's details" );
			$this->viewType = $gallery_options["viewType"];
			$this->viewLevel = $gallery_options["viewLevel"];
			$this->commentLevel = $gallery_options["commentLevel"];
			$this->editLevel = $gallery_options["editLevel"];
			$this->embedLevel = $gallery_options["embedLevel"];
			$this->imageOrder = array();
			$this->images = array();
			$this->save();
		}
		else if( is_dir( $this->basedir ) )
		{
			/// TODO: This is to be removed once we can be sure there are no old galleries around any longer!
			
			// load old data and generate some defaults...
			if( file_exists( $this->basedir . "/gallery.$language.txt" ) )
				$filecontents = file_get_contents( $this->basedir . "/gallery.$language.txt" );
			else if( file_exists( $this->basedir . "/gallery.txt" ) )
				$filecontents = file_get_contents( $this->basedir . "/gallery.txt" );
			else
				$filecontents = "No title\nNo description file found";
			
			$galleryinfobegpos = strpos( $filecontents, "\n" );
			$this->title = substr( $filecontents, 0, $galleryinfobegpos );
			$this->description = substr( $filecontents, $galleryinfobegpos );
			
			// Let's get the values generated and such...
			
			// First, set up us the default options...
			$this->viewType = $gallery_options["viewType"];
			$this->viewLevel = $gallery_options["viewLevel"];
			$this->commentLevel = $gallery_options["commentLevel"];
			$this->editLevel = $gallery_options["editLevel"];
			$this->embedLevel = $gallery_options["embedLevel"];
			$this->imageOrder = array();
			$this->images = array();
			
			// Get all the image names and stick them into the ordering bit - that way we don't need to worry about that later on
			if( file_exists( $this->basedir . "/images" ) )
			{
				$dir_handle = opendir( $this->basedir . "/images" );
				while( $file = readdir( $dir_handle ) )
				{
					if( substr($file, 0, 1) != "."  )
						$this->imageOrder[] = $file;
				}
				closedir($dir_handle);
			}
			
			// Remove all old files... don't need the clutter, and they are nasty and insecure anyway
			unlink_multiple( $this->basedir . "/gallery.*" );
			$this->save();
		}
		else
		{
			// If we've got this far, it is because we have an error. We need to tell the user about this!
			$this->error = true;
			$loadMore = false;
		}
		
		if( $loadMore )
		{
			$this->loadSubgalleries();
			$this->loadImages();
		}
		
		if( $this->edit && !userAllows( $this->owner, $this->editLevel ) )
			$this->edit = false;
	}
	
	/**
	 * Load the subgalleries contained within. You only want this information if you're actually showing the gallery
	 */
	function loadSubgalleries()
	{
		if( file_exists( $this->basedir . "/galleries" ) )
		{
			$fp = opendir( $this->basedir . "/galleries" );
			while ($file = readdir($fp)) {
				if( substr($file, 0, 1) != "." )
					$this->subgalleries[] = new gallery2_gallery( $this->globalID . "/" . $file, false );
			}
			closedir($fp);
		}
	}
	
	/**
	 * If you're simply showing the list of galleries (e.g. subgalleries), you don't want to have loaded the actual images' data. Do that here!
	 */
	function loadImages()
	{
		// List all files in basedir/images into this->images
		$this->images = array();
		if( file_exists( $this->basedir . "/images" ) )
		{
			$dir_handle = opendir( $this->basedir . "/images" );
			while( $file = readdir( $dir_handle ) )
			{
				if( substr($file, 0, 1) != "."  )
					$this->images[$file] = new gallery2_image( &$this, $file );
			}
			closedir($dir_handle);
		}
		
		// Cycle through all the images, and check if they exist in the imageOrder, otherwise add them...
		foreach( $this->images as $key => $image )
		{
			if( ! in_array( $image->imagename, $this->imageOrder ) )
				$this->imageOrder[] = $image->imagename;
		}
		
		// Cycle through this->imageorder and rearrange this->images according to it.
		// Work from the rear end, otherwise we'll end up with them sorted in opposite order
		$tempImageorder = array_reverse( $this->imageOrder, true );
		foreach( $tempImageorder as $key => $value )
		{
			$tempImage = $this->images[$value];
			unset( $this->images[$value] );
			array_splice( $this->images, 0, 0, array($tempImage) );
		}
		
		///\todo FIXME Apparently... the first image doesn't exist... It certainly doesn't get a next image assigned, and the second image doesn't get a previous image assigned..
		$prevImage = null;
		$thisPrev = null;
		foreach( $this->images as $key => $image )
		{
			if( $prevImage != null )
				$this->images[$key]->prevGlobalID = $this->images[$prevImage]->globalID;
			$prevImage = $key;
			if( $thisPrev != null )
				$this->images[$thisPrev]->nextGlobalID = $this->images[$key]->globalID;
			$thisPrev = $key;
		}
		// This is a hack to fix above TODO... There has GOT to be a better way...
		if( count( $this->images ) > 1 )
		{
			$this->images[0]->nextGlobalID = $this->images[1]->globalID;
			$this->images[1]->prevGlobalID = $this->images[0]->globalID;
		}
	}
	
	/**
	 * Save the gallery's options and descriptions and things
	 *
	 * @return   true if successful, or an error message describing the error otherwise (for use with errorHandler)
	 */
	function save()
	{
		$gallery["title"] = $this->title;
		$gallery["description"] = $this->description;
		$gallery["viewType"] = $this->viewType;
		$gallery["viewLevel"] = $this->viewLevel;
		$gallery["commentLevel"] = $this->commentLevel;
		$gallery["editLevel"] = $this->editLevel;
		$gallery["embedLevel"] = $this->embedLevel;
		$gallery["imageOrder"] = $this->imageOrder;
		
		if( file_put_contents( $this->basedir . "/gallery.php", array_export( $gallery, "gallery" ) ) )
			return true;
		else
			return "An error occured while saving the gallery settings!";
	}
	
	/**
	 * Get the global ID for a particular editing function
	 */
	function getGlobalID( $editingFunction )
	{
		$tempSplitID = $this->splitID;
		if( $this->profileGallery )
			$tempSplitID[3] = "$editingFunction";
		else
			$tempSplitID[1] = "$editingFunction";
		return implode( "/", $tempSplitID );
	}
	
	/**
	 * Get the breadcrumbs trail leading up to this gallery
	 *
	 * @return   An instance of the breadcrumbs class
	 */
	function getBreadcrumbs()
	{
		$tempSplitID = $this->splitID;
		
		if( $this->profileGallery )
			$tempSplitID[3] = $this->edit ? "edit" : "view";
		else
			$tempSplitID[1] = $this->edit ? "edit" : "view";
		
		$i = 0; // Set this to 0 to start with
		$firstGallery = true; // We're of course always starting with the parent gallery...
		// Start with filling the tempID out with the top level of the currently viewed gallery
		if( $this->profileGallery )
			$tempID = "user/" . $this->splitID[1] . "/gallery2";
		else
			$tempID = "gallery2/" . $tempSplitID[1];
		foreach( $tempSplitID as $key => $theID )
		{
			// Skip the first couple of items, 4 if it's a profile gallery, 2 if it's not
			// This is because we don't really want to show the top levels of the globalID in the subgallery list
			// First three in profile galleries: user/username/gallery
			// First two in normal galleries: gallery
			if( ( $this->profileGallery && $i < 3 ) || ( $i < 2 ) )
				$i++;
			else
			{
				$tempID .= "/$theID"; // Add the ID to the temporary globalID
				
				if( $tempSplitID[$key] == end( $tempSplitID ) )
				{
					$theTitle = $this->title;
					$imageCount = count($this->imageOrder);
					$thisClass = "-current";
				}
				else
				{
					$parentgallery = new gallery2_gallery( $tempID, false );
					$imageCount = count($parentgallery->imageOrder);
					$theTitle = $parentgallery->title;
					
				}
				
				if( $firstGallery )
					$trail = new breadcrumbs( "$theTitle ($imageCount)", globalIDtoURL($tempID) );
				else
					$trail->addCrumb( "$theTitle ($imageCount)", globalIDtoURL($tempID) );
				
				$firstGallery = false;
			}
		}
		
		if( $this->displayImage )
			$trail->addCrumb( $this->displayImage, globalIDtoURL( $tempID . "/" . $this->displayImage ) );
		
		return $trail;
	}
	

	/**
	* Get a list of all subgalleries contained inside a gallery, listed as options, greyed if not available (for one reason or another)
	*
	* @param     currentGallery The global ID of the current gallery (not selectable, for more or less obvious reasons)
	* @param     level          What level are we at (for indentation purposes)
	*
	* @return    string containing the options listing (for putting into a select element)
	*/
	function optionSubgalleries( $currentGallery = "", $level )
	{
 		if( $this->globalID == $currentGallery )
		{
			$itsHere = i18n(" ##0## Currently here", array("&lt;--") );
			$itsHere2 = " selected";
		}
		
		if( $this->profileGallery && $level == "" )
			$profileName = i18n( "##0##'s profile gallery: ", array( getUserInfo( $this->owner, "name" ) ) );
		
		// Put this gallery into list...
		$renderedList .= "
			<option$itsHere2 value=\"" . $this->globalID . "\">$level$profileName" . $this->title . " (" . count($this->imageOrder) . ")$itsHere</option>";
		
		$this->loadSubgalleries();
		
		// Indent the elements...
		$level .= "&nbsp;&nbsp;";
		
		// Loop through all the galleries, foreach takes care of the no-subgalleries case
		foreach( $this->subgalleries as $key => $theGallery )
			$renderedList .= $theGallery->optionSubgalleries( $currentGallery, $level);
		
		return $renderedList;
	}
	
	/**
	 * Count the amount of images in this and subgalleries
	 *
	 * @return   Integer containing the number of images in this and all subgalleries
	 */
	function countSubImages()
	{
		if( count( $this->images ) == 0 )
			$this->loadImages();
		
		// Put this gallery into list...
		$count += count( $this->images );
		
		if( count( $this->subgalleries ) == 0 )
			$this->loadSubgalleries();
		
		// Loop through all the galleries, foreach takes care of the no-subgalleries case
		foreach( $this->subgalleries as $key => $theGallery )
			$count += $theGallery->countSubImages();
		
		return $count;
	}
	
	/**
	 * Check wether the current user is allowed to edit this gallery
	 *
	 * @return   Boolean value describing wether the user currently logged in is allowed to edit the gallery
	 */
	function userCanEdit()
	{
		if( $this->owner == currentUser() )
			return true;
		
		if( userAllows(currentUser(), $this->editLevel ) )
			return true;
		
		if( ! $this->profileGallery &&  isAllowed( currentUser(), "module_gallery_edit" ) )
			return true;
		
		// Admin can always edit
		if( isAllowed( currentUser(), "global_admin" ) )
			return true;
		
		// Nothing was caught, the user is not allowed to edit...
		return false;
	}
	
	/**
	 * Check wether the current user is allowed to view this gallery
	 *
	 * @return   Boolean value describing wether the user currently logged in is allowed to edit the gallery
	 */
	function userCanView()
	{
		if( $this->owner == currentUser() )
			return true;
		
		if( userAllows(currentUser(), $this->viewLevel ) )
			return true;
		
		if( ! $this->profileGallery &&  isAllowed( currentUser(), "module_gallery_view" ) )
			return true;
		
		// Admin can always edit
		if( isAllowed( currentUser(), "global_admin" ) )
			return true;
		
		// Nothing was caught, the user is not allowed to edit...
		return false;
	}
	
	/**
	 * Render the gallery editor
	 * 
	 * @return   String containing the rendered gallery editing system. This is always the
	 *           verbose table view mode, with extra editing controls
	 */
	function renderEditor()
	{
		global $formatting_toolbar, $languages, $gallery_options;
		$tempSplitID = $this->splitID;
		
		$uploadID = $this->getGlobalID( "upload" );
		$newID = $this->getGlobalID( "new" );
		$detailsID = $this->getGlobalID( "details" );
		$moveID = $this->getGlobalID( "move" );
		$deleteID = $this->getGlobalID( "delete" );
		
		$renderedGallery = "<div class=\"wikirightalign\">";
		
		if (!$this->profileGallery || ($this->profileGallery && endKey($this->splitID) > 3))
			$renderedGallery .= drawCommand( i18n("Rename"), i18n("Rename the gallery physically"), globalIDtoURL($moveID) ) . " ";
		else
			$renderedGallery .= "(" . i18n("Rename") . ") ";
		$renderedGallery .= drawCommand( i18n("Delete"), i18n("Delete this gallery"), globalIDtoURL($deleteID) ) . " ";
		$renderedGallery .= " | ";
		$renderedGallery .= drawCommand( i18n("New Subgallery"), i18n("Create new subgallery"), globalIDtoURL($newID) ) . " ";
		$renderedGallery .= drawCommand( i18n("Edit details"), i18n("Edit the gallery's title and description"), globalIDtoURL($detailsID) ) . " ";
		$renderedGallery .= drawCommand( i18n("Upload"), i18n("Upload a new image"), globalIDtoURL($uploadID) );
		$renderedGallery .= "</div>";
		
		switch( $this->editMode )
		{
		case "thumbnail":
			// Thumbnail an existing image...
			switch( $_REQUEST["thumbtype"] )
			{
			case "cropnresize":
				if (file_exists( $this->basedir . "/images/" . $this->displayImage )) {
					$image = $this->basedir . "/images/" . $this->displayImage; // The image to crop
					$image_url = $this->baseurl . "/images/" . $this->displayImage; // The image's URL
					$dest_image = $this->basedir . "/thumbs/" . $this->displayImage;
					$ext = checkImageMimetype( $image );
				}
				
				///\todo FIXME Move this functionality into a class of its own... We use it in a number of places, and only change a very few things... Might as well abuse that fact
				/**
				 * Cropping method adopted from lixlpixel PHParadise
				 *
				 * category :		image editing
				 *
				 * snippet :		crop image
				 *
				 * downloaded :	11.02.2005 - 16:27
				 *
				 * file URL :		http://www.fundisom.com/phparadise/php/image_editing/crop_image
				 */
				
				$url = globalIDtoURL( $this->globalID . "/" . $this->displayImage, array( "thumbtype" => "cropnresize", "theme" => "empty" ) ); // the url to this wizard
				$margin = 10; // to keep the image and layer in sync
				
				if(!isset($_POST['step'])) $title = '1';
				else $title = $_POST['step'];
				
				$data = '
				<html>
				<head>
					<title>' . i18n("Cropping gallery image - step") . ' ' . $title . '</title>
				</head>
				<body style="margin: '.$margin.'px;">';
				
				if(!isset($_POST['tx']) && !isset($_POST['fx']))
				{
					$data .= '
					<form method="post" action="'.$url.'">
						<input type="image" src="'.$image_url.'"><p>';
					if(!isset($_POST['x']))
					{
						$data .= '
						<input type="hidden" name="step" value="2">
						' . i18n("Click to mark the first corner of the selection rectangle.") . " " . i18n("You will be able to review the result before cropping");
					}else{
						$data .= '
						<input type="hidden" name="step" value="3">
						<input type="hidden" name="fx" value="'.$_POST['x'].'">
						<input type="hidden" name="fy" value="'.$_POST['y'].'">
						' . i18n("Click to mark the second corner of the selection rectangle.") . ' | <a href="'.$url.'">' . i18n("Start over") . '</a> ' . " " . i18n("You will be able to review the result before cropping");
					}
					$data .= '
					</form>
					';
				}
				
				if(isset($_POST['fx']))
				{
					$data .= '
					<form method="post" action="'.$url.'">
						<input type="hidden" name="step" value="4">
						<input type="image" src="'.$image_url.'">
						<input type="hidden" name="tx" value="'.$_POST['fx'].'">
						<input type="hidden" name="ty" value="'.$_POST['fy'].'">
						<input type="hidden" name="width" value="'.($_POST['x']-$_POST['fx']).'">
						<input type="hidden" name="height" value="'.($_POST['y']-$_POST['fy']).'"><p>
						<div style="position: absolute;
									left:'.($_POST['fx']+$margin).'px;
									top: '.($_POST['fy']+$margin).'px;
									width: '.($_POST['x']-$_POST['fx']).'px;
									height: '.($_POST['y']-$_POST['fy']).'px;
									border: 1px solid #fff;">
						</div>
						' . i18n("Click the image to crop. The rectangle shown on the picture is the rectangle that will be cropped.") . '| <a href="'.$url.'">' . i18n("Start over") . '</a>
					</form>';
				}
				
				if(isset($_POST['tx']))
				{
					// Crop the image
					cropImage( $image, $dest_image, $_POST["tx"], $_POST["ty"], $_POST["width"], $_POST["height"] );
					
					// Resize the new thumbnail to the max thumbnail size
					scaleImage( $dest_image, $dest_image, 120, 120 );
					
					header("Location: " . globalIDtoURL( $this->getGlobalID( "edit" ) ) );
				}
				
				// Sort of ugly - but we need the emptiness here!
				return $data;
				break;
			case "fullsize":
				// Resize the new thumbnail to the max thumbnail size
				scaleImage( $this->basedir . "/images/" . $this->displayImage, $this->basedir . "/thumbs/" . $this->displayImage, 120, 120 );
				// Go back to gallery view
				header("Location: " . globalIDtoURL( $this->getGlobalID( "edit" ) ) );
				break;
			}
			$renderedGallery = "";
			break;
		case "moveup":
			// Move the image up in the gallery order
			foreach( $this->imageOrder as $key => $imagename )
			{
				if( $imagename == $this->displayImage )
				{
					$tempImage = $this->imageOrder[$key];
					unset( $this->imageOrder[$key] );
					array_splice( $this->imageOrder, $prevImage, 0, $tempImage );
					break;
				}
				else
					$prevImage = $key;
			}
			$this->save();
			header("Location: " . globalIDtoURL( $this->getGlobalID( "edit" ) ) );
			break;
		case "movedown":
			// Move the image down in the gallery order
			///\todo FIXME: Why the heck does move first image down not work?!
			$prevImage = null;
			foreach( $this->imageOrder as $key => $imagename )
			{
				if( $prevImage != null )
				{
					$tempImage = $this->imageOrder[$prevImage];
					unset( $this->imageOrder[$prevImage] );
					array_splice( $this->imageOrder, $key, 0, $tempImage );
					break;
				}
				else
				{
					if( $imagename == $this->displayImage )
						$prevImage = $key;
				}
			}
			$this->save();
			
			header("Location: " . globalIDtoURL( $this->getGlobalID( "edit" ) ) );
			break;
		case "upload":
			// Upload a new image
			if( $_POST["saveOptions"] )
			{
				// Subscription subsystem
				global $subscriptions;
				$errorHandler = new errorHandler( $this->globalID );
				foreach ($_FILES["uploadedfile"]["name"] as $thekey => $thevalue) {
					$filename = suggestFilename( str_replace("&", "_", str_replace("'", "", stripslashes($_FILES['uploadedfile']['name'][$thekey]))), $this->basedir . "/images" );
					$uploadFile = $this->basedir . "/images/$filename";
					// checkImageMimetype below makes sure that it's actually a supported image that got uploaded
					if( checkImageMimetype( $_FILES['uploadedfile']['tmp_name'][$thekey] ) !== false && move_uploaded_file( $_FILES['uploadedfile']['tmp_name'][$thekey], $uploadFile ) )
					{
						chmod($uploadFile, 0664);
						
						list( $x, $y ) = getimagesize( $uploadFile );
						
						if( is_array($gallery_options) && $x > $gallery_options["maxwidth"] )
						{
							$resize_factor = $gallery_options["maxwidth"] / $x;
							$resized_x = $gallery_options["maxwidth"];
							$resized_y = $y * $resize_factor;
							
							if( $resized_y > $gallery_options["maxheight"] )
							{
								$resize_factor = $gallery_options["maxheight"] / $y;
								$resized_x = $x * $resize_factor;
								$resized_y = $gallery_options["maxheight"];
							}
							
							$errorHandler->handle( i18n( "Resizing ##0##", array( $filename ) ), scaleImage( $uploadFile, $uploadFile, $resized_x, $resized_y ) );
							$x = $resized_x; $y = $resized_y;
						}
						else if( is_array($gallery_options) && $y > $gallery_options["maxheight"] )
						{
							$resize_factor = $gallery_options["maxheight"] / $y;
							$resized_x = $x * $resize_factor;
							$resized_y = $gallery_options["maxheight"];
							$errorHandler->handle( i18n( "Resizing ##0##", array( $filename ) ), scaleImage( $uploadFile, $uploadFile, $resized_x, $resized_y ) );
							$x = $resized_x; $y = $resized_y;
						}
						
						$errorHandler->handle( i18n("Thumbnail ##0##", array( $filename) ) , scaleImage( $uploadFile, $this->basedir . "/thumbs/$filename", $gallery_options["thumbwidth"], $gallery_options["thumbheight"] ) );
						
						foreach($languages as $key => $value)
							$image["descriptions"][$value] = stripslashes($_POST["picture_comment"]);
						$image["viewCount"] = 0;
						$image["commentCount"] = 0;
						$image["width"] = $x;
						$image["height"] = $y;
						$image["weight"] = fileweight($uploadFile);
						
						file_put_contents( $this->basedir . "/data/$filename.php", array_export( $image, "image" ) );
						file_put_contents( $this->basedir . "/comments/$filename.php", array_export( array(), "comments" ) );
						
						$this->imageOrder[] = $filename;
						
						// Subscription subsystem
						$subject = i18n( "New image ##0## uploaded to ##1##", array( $filename, $this->title ) );
						$message = i18n( "A new image was uploaded to the gallery ##0## with the description:", array( $this->title ) ) . "\n" . stripslashes($_POST["picture_comment"]);
						$subscriptions->handle( $this->getGlobalID("view") . "/" . $filename, $this->owner, $this->viewLevel, $subject, $message );
					}
					else
					{
						switch ($_FILES['uploadedfile']['error'][$thekey]) {
						case 0:
							if( checkImageMimetype( $_FILES['uploadedfile']['tmp_name'][$thekey] ) === false )
								$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("Image type not supported or not an image file! Only jpg, gif and png are supported image types") );
							else
								$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  false );
							break;
						case 1:
							$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("The uploaded file exceeds the upload_max_filesize directive in php.ini. Please contact your host and rectify this (or better yet, select a smaller file. Most often, this directive states 2MB as the maximum upload size, however as we have 1MB as the maxium here, they have obviously changed it).") );
							break;
						case 2:
							$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("The uploaded file exceeds 1MB in size, and is too large to upload in this manner.") );
							break;
						case 3:
							$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("The uploaded file was only partially uploaded.") );
							break;
						case 4:
							// This is not an error, it's what happens when all ten file selectors are filled
							break;
						default:
							if( checkImageMimetype( $_FILES['uploadedfile']['tmp_name'][$thekey] ) === false )
								$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("Image type not supported or not an image file! Only jpg, gif and png are supported image types") );
							else
								$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  false );
						}
					}
				}
				
				$renderedGallery .= $errorHandler->render();
				
				if( !$errorHandler->handled )
				{
					$this->save();
					header("Location: " . globalIDtoURL( $this->getGlobalID( "edit" ) ) );
				}
			}
			else
			{
				$panel = new optionsPanel("");
				$panel->saveTitle = i18n("Upload image");
				$panel->resetTitle = i18n("Reset");
				$panel->showHeader = false;
				
				$panel->addOption( i18n("Select picture for upload:"), i18n("Please note, the system supports only images of the format JPG, PNG and GIF."), "", "uploadedfile[]", "file");
				$panel->addOption( i18n("Description:"), "", i18n("Enter a description of the picture here"), "picture_comment", "pagedata");
			
				$theContent = $panel->render();
				
				$furtherContent = "
				<div><input style=\"width: 100%\" id=\"uploadedfile2\" name=\"uploadedfile[]\" type=\"file\" /></div>
				<div><input style=\"width: 100%\" id=\"uploadedfile3\" name=\"uploadedfile[]\" type=\"file\" /></div>
				<div><input style=\"width: 100%\" id=\"uploadedfile4\" name=\"uploadedfile[]\" type=\"file\" /></div>
				<div><input style=\"width: 100%\" id=\"uploadedfile5\" name=\"uploadedfile[]\" type=\"file\" /></div>
				<div><input style=\"width: 100%\" id=\"uploadedfile6\" name=\"uploadedfile[]\" type=\"file\" /></div>
				<div><input style=\"width: 100%\" id=\"uploadedfile7\" name=\"uploadedfile[]\" type=\"file\" /></div>
				<div><input style=\"width: 100%\" id=\"uploadedfile8\" name=\"uploadedfile[]\" type=\"file\" /></div>
				<div><input style=\"width: 100%\" id=\"uploadedfile9\" name=\"uploadedfile[]\" type=\"file\" /></div>
				<div><input style=\"width: 100%\" id=\"uploadedfile0\" name=\"uploadedfile[]\" type=\"file\" /></div>";
				
				$renderedGallery .= renderInformationBox( i18n("Upload a new image"), $theContent . generate_pagehelplink(), false ) . renderInformationBox( i18n("Additional pictures"), i18n("Select up to 9 additional pictures for upload. These pictures will be given the same description as the first picture. If you wish to assign a different description to one picture, do this afterwards.") . $furtherContent, false );
			}
			break;
		case "replace":
			// Replace an existing image by uploading a new one...
			// Retain everything, simply delete image and redo thumbnail
			if( $_POST["saveOptions"] )
			{
				$errorHandler = new errorHandler( $this->globalID );
				$filename = $this->displayImage;
				$uploadFile = $this->basedir . "/images/$filename";
				@unlink( $uploadFile );
				@unlink( $this->basedir . "/midsize/$filename" );
				@unlink( $this->basedir . "/thumbs/$filename" );
				
				// checkImageMimetype below makes sure that it's actually a supported image that got uploaded
				if( checkImageMimetype( $_FILES['uploadedfile']['tmp_name'] ) != false && move_uploaded_file( $_FILES['uploadedfile']['tmp_name'], $uploadFile ) )
				{
					chmod($uploadFile, 0664);
					
					list( $x, $y ) = getimagesize( $uploadFile );
					
					if( is_array($gallery_options) && $x > $gallery_options["maxwidth"] )
					{
						$resize_factor = $gallery_options["maxwidth"] / $x;
						$resized_x = $gallery_options["maxwidth"];
						$resized_y = $y * $resize_factor;
							
						if( $resized_y > $gallery_options["maxheight"] )
						{
							$resize_factor = $gallery_options["maxheight"] / $y;
							$resized_x = $x * $resize_factor;
							$resized_y = $gallery_options["maxheight"];
						}
						
						$errorHandler->handle( i18n( "Resizing ##0##", array( $filename ) ), scaleImage( $uploadFile, $uploadFile, $resized_x, $resized_y ) );
						$x = $resized_x; $y = $resized_y;
					}
					else if( is_array($gallery_options) && $y > $gallery_options["maxheight"] )
					{
						$resize_factor = $gallery_options["maxheight"] / $y;
						$resized_x = $x * $resize_factor;
						$resized_y = $gallery_options["maxheight"];
						$errorHandler->handle( i18n( "Resizing ##0##", array( $filename ) ), scaleImage( $uploadFile, $uploadFile, $resized_x, $resized_y ) );
						$x = $resized_x; $y = $resized_y;
					}
					
					$errorHandler->handle( i18n("Thumbnail ##0##", array( $filename) ) , scaleImage( $uploadFile, $this->basedir . "/thumbs/$filename", $gallery_options["thumbwidth"], $gallery_options["thumbheight"] ) );
					
					include( $this->basedir . "/data/$filename.php" );
					foreach( $image["descriptions"] as $key => $description )
						$image["descriptions"][$key] = $description . "\n" . i18n("Edit by ##0## on ##1##", array("%%" . currentUser() . "%%", formatTime(time()))) . ": " . stripslashes($_POST["comment"]);
					$image["width"] = $x;
					$image["height"] = $y;
					$image["weight"] = fileweight($uploadFile);
					
					file_put_contents( $this->basedir . "/data/$filename.php", array_export( $image, "image" ) );
				}
				else
				{
					switch ($_FILES['uploadedfile']['error']) {
					case 0:
						if( checkImageMimetype( $_FILES['uploadedfile']['tmp_name'] ) == false )
							$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("Image type not supported or not an image file! Only jpg, gif and png are supported image types") );
						else
							$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  false );
						break;
					case 1:
						$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("The uploaded file exceeds the upload_max_filesize directive in php.ini. Please contact your host and rectify this (or better yet, select a smaller file. Most often, this directive states 2MB as the maximum upload size, however as we have 1MB as the maxium here, they have obviously changed it).") );
						break;
					case 2:
						$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("The uploaded file exceeds 1MB in size, and is too large to upload in this manner.") );
						break;
					case 3:
						$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("The uploaded file was only partially uploaded.") );
						break;
					case 4:
						// This is not an error, it's what happens when all ten file selectors are filled
						break;
					default:
						if( checkImageMimetype( $_FILES['uploadedfile']['tmp_name'] ) == false )
							$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("Image type not supported or not an image file! Only jpg, gif and png are supported image types") );
						else
							$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  false );
					}
				}
				
				$renderedGallery .= $errorHandler->render();
				
				if( !$errorHandler->handled )
					header("Location: " . globalIDtoURL( $this->getGlobalID( "edit" ) ) );
			}
			else
			{
				$panel = new optionsPanel("");
				$panel->saveTitle = i18n("Upload image");
				$panel->resetTitle = i18n("Reset");
				$panel->showHeader = false;
				
				$panel->addOption( i18n("Select picture for upload:"), i18n("Please note, the system supports only images of the format JPG, PNG and GIF."), "", "uploadedfile[]", "file");
				$panel->addOption( i18n("Description:"), "", i18n("Enter a description of the picture here"), "picture_comment", "pagedata");
			
				$theContent = $panel->render();
				
				$renderedGallery .= renderInformationBox( i18n("Replace ##0## with a new image", array($this->displayImage)), $theContent, false);
			}
			break;
		case "new":
			global $gallery2_viewTypes, $permissionlevels_array;
			// Create a new subgallery
			$panel = new optionsPanel("");
			$panel->saveTitle = i18n("Create subgallery");
			$panel->resetTitle = i18n("Reset");
			$panel->showHeader = false;
			
			$values["physicalname"] = ( $_POST["physicalname"] != "" ) ? stripslashes($_POST["physicalname"]) : "gallery";
			$values["title"] = ( $_POST["title"] != "" ) ? stripslashes($_POST["title"]) : i18n("Untitled Gallery");
			$values["description"] = ( $_POST["description"] != "" ) ? stripslashes($_POST["description"]) : i18n("Enter the new gallery's description here");
			$values["viewType"] = ( $_POST["viewType"] != "" ) ? stripslashes($_POST["viewType"]) : $gallery_options["viewType"];
			$values["viewLevel"] = ( $_POST["viewLevel"] != "" ) ? stripslashes($_POST["viewLevel"]) : $gallery_options["viewLevel"];
			$values["commentLevel"] = ( $_POST["commentLevel"] != "" ) ? stripslashes($_POST["commentLevel"]) : $gallery_options["commentLevel"];
			$values["editLevel"] = ( $_POST["editLevel"] != "" ) ? stripslashes($_POST["editLevel"]) : $gallery_options["editLevel"];
			
			$panel->addOption( i18n("Physical name:"), i18n("The filename you will find the gallery under (used for linking)"), $values["physicalname"], "physicalname", "text");
			$panel->addOption( i18n("Title:"), i18n("The gallery's human readable name"), $values["title"], "title", "text");
			$panel->addOption( i18n("View type"), i18n("How do you wish to present this gallery to the user by default?"), $values["viewType"], "viewType", "select", $gallery2_viewTypes);
			$panel->addOption( i18n("View Level:"), i18n("Who, except for yourself, is allowed to view the contents of this gallery?"), $values["viewLevel"], "viewLevel", "select", $permissionlevels_array);
			$panel->addOption( i18n("Comment Level:"), i18n("Who, except for yourself, is allowed to comment on the contents of this gallery?"), $values["commentLevel"], "commentLevel", "select", $permissionlevels_array);
			$panel->addOption( i18n("Edit Level:"), i18n("Who, except for yourself, is allowed to edit the contents of this gallery?"), $values["editLevel"], "editLevel", "select", $permissionlevels_array);
			$panel->addOption( i18n("Embedding Level:"), i18n("Who, except for yourself, is allowed to get embedding codes for the contents of this gallery?"), $values["embedLevel"], "embedLevel", "select", $permissionlevels_array);
			$panel->addOption( i18n("Description:"), "", $values["description"], "description", "pagedata");
			
			$newbasedir = $this->basedir . "/galleries/" . str_replace( ".", "-", str_replace( " ", "_", $panel->options["physicalname"]["current"] ) );
			
			if( $_POST["saveOptions"] && file_exists( $newbasedir ) )
				$panel->options["physicalname"]["error"] = i18n("A gallery already exists with this name!");
			
			if( $_POST["saveOptions"] && $panel->options["physicalname"]["current"] != "" && ! file_exists( $newbasedir ) )
			{
				RecursiveMkdir( $newbasedir . "/comments" );
				RecursiveMkdir( $newbasedir . "/data" );
				RecursiveMkdir( $newbasedir . "/galleries" );
				RecursiveMkdir( $newbasedir . "/images" );
				RecursiveMkdir( $newbasedir . "/thumbs" );
				
				$gallery["title"] = $panel->options["title"]["current"];
				$gallery["description"] = $panel->options["description"]["current"];
				$gallery["viewType"] = $gallery2_viewTypes[ $panel->options["viewType"]["current"] ];
				$gallery["viewLevel"] = $panel->options["viewLevel"]["current"];
				$gallery["commentLevel"] = $panel->options["commentLevel"]["current"];
				$gallery["editLevel"] = $panel->options["editLevel"]["current"];
				$gallery["embedLevel"] = $panel->options["embedLevel"]["current"];
				$gallery["imageOrder"] = array();
				if( file_put_contents( $newbasedir . "/gallery.php", array_export( $gallery, "gallery" ) ) )
					header("Location: " . globalIDtoURL( $this->getGlobalID( "edit" ) . "/" . str_replace(" ", "_", $panel->options["physicalname"]["current"]) ) );
				else
					$renderedGallery .= "An error occured while saving the gallery settings!";
			}
			else
				$renderedGallery .= renderInformationBox( i18n("Create a new subgallery under ##0##", array($this->title)), $panel->render(), false);
			
			break;
		case "rename":
			// Rename a sub-gallery physically
			if( $this->displayImage == null )
			{
				// Only possible if it's not the top level profile gallery...$oldName = substr($this->basedir, strrpos( $this->basedir, "/" ) + 1);
				/*if( $_POST["saveOptions"] && $_POST["newName"] != $oldName )
				{
					$theBase = substr($this->basedir, 0, strlen( $this->basedir ) - strlen( $oldName ) );
					rename( $this->basedir, $theBase . $_POST["newName"] );
					$this->basedir = $theBase . $_POST["newName"];
					$this->splitID[end(array_keys($this->splitID))] = $_POST["newName"];
					$this->globalID = implode( "/", $this->splitID );
				}
				else if( $_POST["saveOptions"] && $_POST["newName"] == $oldName )
				{
					header( "Location: " . globalIDtoURL( $this->getGlobalID("edit") ) );
				}
				else
				{
					$panel = new optionsPanel("");
					$panel->saveText = i18n("Rename");
					$panel->showHeader = false;
					
					$panel->addOption(
						i18n("New physical name"),
						i18n("The gallery's new physical name - remember this is used for things like linking, so spaces and other interesting characters shold probably be avoided"),
						$oldName,
						"newName",
						"text");
				
					$data = $panel->render();
				}
				$renderedGallery .= renderInformationBox( i18n("Rename gallery physically"), $data, false );*/
			}
			// Rename an image physically
			else
			{
				$panel = new optionsPanel("");
				$panel->saveTitle = i18n("Rename the image");
				$panel->showHeader = false;
				
				$theFiletitle = substr( $this->displayImage, 0, -4 );
				$theFilesuffix = substr( $this->displayImage, -4 );
				
				$values["filename"] = $_POST["filename"] ? $_POST["filename"] : $theFiletitle;
				
				$panel->addOption(
					i18n("New filename"),
					i18n("The new filename for this image - remember this is used for things like linking, so spaces and other interesting characters shold probably be avoided"),
					$values["filename"],
					"filename",
					"text");
				
				if( $_POST["saveOptions"] && $values["filename"] != $theFiletitle )
				{
					$errorHandler = new errorHandler( globalIDtoURL($this->getGlobalID("rename") . "/" . $this->displayImage) );
				
					$filenames["image file"] = $this->basedir . "/images/" . $this->displayImage;
					$filenames["thumbnail file"] = $this->basedir . "/thumbs/" . $this->displayImage;
					$filenames["midsize file"] = $this->basedir . "/midsize/" . $this->displayImage;
					$filenames["data file"] = $this->basedir . "/data/" . $this->displayImage . ".php";
					$filenames["comments file"] = $this->basedir . "/comments/" . $this->displayImage . ".php";
					
					$newFilenames["image file"] = $this->basedir . "/images/" . $values["filename"] . $theFilesuffix;
					$newFilenames["thumbnail file"] = $this->basedir . "/thumbs/" . $values["filename"] . $theFilesuffix;
					$newFilenames["midsize file"] = $this->basedir . "/midsize/" . $values["filename"] . $theFilesuffix;
					$newFilenames["data file"] = $this->basedir . "/data/" . $values["filename"] . $theFilesuffix . ".php";
					$newFilenames["comments file"] = $this->basedir . "/comments/" . $values["filename"] . $theFilesuffix . ".php";
					
					// Only rename midsizefile if it exists...
					if( !file_exists( $filenames["midsize file"] ) )
						unset( $filenames["midsize file"] );
					
					// Rename image data files
					foreach( $filenames as $key => $value )
						$errorHandler->handle( i18n("Renaming ##0##", array(i18n($key))),  rename($value, $newFilenames[$key]) );
					
					if( !$errorHandler->handled )
					{
						// If image is in the gallery's imageOrder
						$theKey = array_search( $this->displayImage, $this->imageOrder );
						if( is_numeric($theKey) )
						{
							// Rename image in gallery imageOrder
							$this->imageOrder[$theKey] = $values["filename"] . $theFilesuffix;
							// Save gallery
							$this->save();
							
							global $subscriptions;
							$subscriptions->renameGlobalID( $this->getGlobalID("view") . "/" . $this->displayImage, $this->getGlobalID("view") . "/" . $values["filename"] . $theFilesuffix );
						}
						// Go back to editor
						header("Location: " . globalIDtoURL( $this->getGlobalID("edit") ) );
					}
					else
						$renderedGallery = renderInformationBox( i18n("Image rename failed!"), $errorHandler->render() );
				}
				else if( $_POST["saveOptions"] && $values["filename"] == $theFiletitle )
					header("Location: " . globalIDtoURL( $this->getGlobalID("edit") ) );
				else
					$renderedGallery = renderInformationBox( i18n("Rename image physically?"), $panel->render(), false );
			}
			break;
		case "details":
			global $gallery2_viewTypes, $permissionlevels_array, $languages, $languages_names;
			// Edit gallery title and description
			if( $this->displayImage == null )
			{
				$panel = new optionsPanel("");
				$panel->saveTitle = i18n("Save gallery details");
				$panel->resetTitle = i18n("Reset");
				$panel->showHeader = false;
				
				$values["title"] = ( $_POST["title"] != "" ) ? stripslashes($_POST["title"]) : $this->title;
				$values["description"] = ( $_POST["description"] != "" ) ? stripslashes($_POST["description"]) : $this->description;
				$values["viewType"] = ( $_POST["viewType"] != "" ) ? stripslashes($_POST["viewType"]) : $this->viewType;
				$values["viewLevel"] = ( $_POST["viewLevel"] != "" ) ? stripslashes($_POST["viewLevel"]) : $this->viewLevel;
				$values["commentLevel"] = ( $_POST["commentLevel"] != "" ) ? stripslashes($_POST["commentLevel"]) : $this->commentLevel;
				$values["editLevel"] = ( $_POST["editLevel"] != "" ) ? stripslashes($_POST["editLevel"]) : $this->editLevel;
				$values["owner"] = ( $_POST["owner"] != "" ) ? stripslashes($_POST["owner"]) : $this->owner;
				
				$panel->addOption( i18n("Title:"), i18n("The gallery's human readable name"), $values["title"], "title", "text");
				$panel->addOption( i18n("View type"), i18n("How do you wish to present this gallery to the user by default?"), $values["viewType"], "viewType", "select", $gallery2_viewTypes);
				$panel->addOption( i18n("View Level:"), i18n("Who, except for yourself, is allowed to view the contents of this gallery?"), $values["viewLevel"], "viewLevel", "select", $permissionlevels_array);
				$panel->addOption( i18n("Comment Level:"), i18n("Who, except for yourself, is allowed to comment on the contents of this gallery?"), $values["commentLevel"], "commentLevel", "select", $permissionlevels_array);
				$panel->addOption( i18n("Edit Level:"), i18n("Who, except for yourself, is allowed to edit the contents of this gallery?"), $values["editLevel"], "editLevel", "select", $permissionlevels_array);
				$panel->addOption( i18n("Embedding Level:"), i18n("Who, except for yourself, is allowed to get embedding codes for the contents of this gallery?"), $values["embedLevel"], "embedLevel", "select", $permissionlevels_array);
				// Only show this option for galleries that aren't profile galleries, since they're always owned by the user owning that profile
				if( !$this->profileGallery )
					$panel->addOption( i18n("Owner:"), i18n("Who owns the gallery (all permissions are relative to this user)"), $values["owner"], "owner", "select", allUsersArray() );
				$panel->addOption( i18n("Description:"), "", $values["description"], "description", "pagedata");
				
				if( $_POST["saveOptions"] )
				{
					$this->title = $panel->options["title"]["current"];
					$this->description = $panel->options["description"]["current"];
					$this->viewType = $gallery2_viewTypes[ $panel->options["viewType"]["current"] ];
					$this->viewLevel = $panel->options["viewLevel"]["current"];
					$this->commentLevel = $panel->options["commentLevel"]["current"];
					$this->editLevel = $panel->options["editLevel"]["current"];
					$this->embedLevel = $panel->options["embedLevel"]["current"];
					if( array_key_exists( "owner", $panel->options ) )
						$this->owner = $panel->options["owner"]["current"];
					$this->save();
					header( "Location: " . globalIDtoURL( $this->getGlobalID( "edit" ) ) );
				}
				else
					$renderedGallery .= renderInformationBox( i18n( "Edit gallery details", array( $this->getGlobalID( "view" ) ) ), $panel->render(), false );
			}
			// Edit image description
			else
			{
				// Get a reference to the edited image
				foreach( $this->images as $key => $tmpimage )
					if( $tmpimage->imagename == $this->displayImage )
						$image = &$this->images[$key];
				
				$editlanguage = $_REQUEST["editLanguage"];
				// Make sure we're form linked back to the right place...
				$extraReqVars = "&amp;editLanguage=" . $editlanguage;
				foreach( $languages as $key => $value )
					if( $editlanguage == $value )
						$languagename = $languages_names[$key];
				
				$panel = new optionsPanel("");
				$panel->saveTitle = i18n("Save image details");
				$panel->resetTitle = i18n("Reset");
				$panel->showHeader = false;
				
				$panel->addOption( i18n("Description:"), "", $image->descriptions[$editlanguage], "description", "pagedata");
				
				if( $_POST["saveOptions"] )
				{
					$image->descriptions[$editlanguage] = $panel->options["description"]["current"];
					$image->save();
					header( "Location: " . globalIDtoURL( $this->getGlobalID( "edit" ) ) );
				}
				else
					$renderedGallery .= renderInformationBox( i18n("Edit ##0## description for the image ##1##", array( $languagename, $image->getGlobalID("view") ) ), $panel->render(), false );
			}
			break;
		case "move":
			// Physically move gallery
			if( $this->displayImage == null )
			{
				
				$oldName = substr($this->basedir, strrpos( $this->basedir, "/" ) + 1);
				if( $_POST["saveOptions"] && $_POST["newName"] != $oldName )
				{
					$theBase = substr($this->basedir, 0, strlen( $this->basedir ) - strlen( $oldName ) );
					rename( $this->basedir, $theBase . $_POST["newName"] );
					$this->basedir = $theBase . $_POST["newName"];
					$this->splitID[end(array_keys($this->splitID))] = $_POST["newName"];
					$this->globalID = implode( "/", $this->splitID );
				}
				else if( $_POST["saveOptions"] && $_POST["newName"] == $oldName )
				{
					header( "Location: " . globalIDtoURL( $this->getGlobalID("edit") ) );
				}
				else
				{
					$panel = new optionsPanel("");
					$panel->saveText = i18n("Rename");
					$panel->showHeader = false;
					
					$panel->addOption(
						i18n("New physical name"),
						i18n("The gallery's new physical name - remember this is used for things like linking, so spaces and other interesting characters shold probably be avoided"),
						$oldName,
						"newName",
						"text");
				
					$data = $panel->render();
				}
				$renderedGallery .= renderInformationBox( i18n("Rename gallery physically"), $data, false );
			}
			// Physically move image to another subgallery
			else
			{
				if( $_POST["saveOptions"] && $this->getGlobalID("view") != $_POST["whatGallery"] )
				{
					// Get moveto gallery...
					$moveto = new gallery2_gallery($_POST["whatGallery"], false);
					
					// Get filenames from image in current gallery
					$imagefile = $this->basedir . "/images/" . $this->displayImage;
					$thumbfile = $this->basedir . "/thumbs/" . $this->displayImage;
					$midfile = $this->basedir . "/midsize/" . $this->displayImage;
					$datafile = $this->basedir . "/data/" . $this->displayImage . ".php";
					$commentsfile = $this->basedir . "/comments/" . $this->displayImage . ".php";
					
					// Move image files from current gallery to new gallery
					rename( $imagefile, $moveto->basedir . "/images/" . $this->displayImage);
					if( file_exists( $midfile ) )
					{
						if( !is_dir( $moveto->basedir . "/midsize" ) )
							RecursiveMkdir( $moveto->basedir . "/midsize" );
						rename( $midfile, $moveto->basedir . "/midsize/" . $this->displayImage);
					}
					rename( $thumbfile, $moveto->basedir . "/thumbs/" . $this->displayImage);
					
					// Reassign image to new gallery
					foreach( $this->images as $key => $image )
						if( $image->imagename == $this->displayImage )
						{
							$image->parentGallery = &$moveto;
							break;
						}
					
					$image->dataFile = $moveto->basedir . "/data/" . $this->displayImage . ".php";
					$image->commentFile = $moveto->basedir . "/comments/" . $this->displayImage . ".php";
					$image->save();
					
					// Delete image data from current gallery
					unlink( $datafile );
					if( file_exists( $commentsfile ) )
						unlink( $commentsfile );
					foreach( $this->imageOrder as $key => $value )
						if( $value == $this->displayImage )
							unset( $this->imageOrder[$key] );
					$this->save();
					
					global $subscriptions;
					$subscriptions->renameGlobalID( $this->getGlobalID("view") . "/" . $this->displayImage, $moveto->getGlobalID("view") . "/" . $this->displayImage );
					
					// Set location to new gallery, edit mode
					header( "Location: " . globalIDtoURL( $moveto->getGlobalID("edit") ) );
					
				}
				else if( $_POST["saveOptions"] && $this->getGlobalID("view") == $_POST["whatGallery"] )
				{
					header( "Location: " . globalIDtoURL( $this->getGlobalID("edit") ) );
				}
				else
				{
					global $module_folder;
					// First get all site galleries
					$galleries = array();
					$gallerybase = "$module_folder/gallery";
					
					$dir_handle = opendir("$gallerybase");
					while ($folder = readdir($dir_handle)) {
						if( is_dir( "$gallerybase/$folder" ) && substr($folder, 0, 1) != "." ) {
							$theGallery = new gallery2_gallery( "gallery2/view/" . $folder, false );
							if( userAllows( $theGallery->owner, $theGallery->editLevel ) )
								$galleries[] = $theGallery;
						}
					}
					closedir($dir_handle);
					
					// Then get all user galleries
					$allusers = allUsersArray();
					foreach( $allusers as $key => $username )
					{
						$theGallery = new gallery2_gallery( "user/$username/gallery", false );
						if( userAllows( $username, $theGallery->editLevel ) )
							$galleries[] = $theGallery;
					}
					
					// Now cycle through the galleries and list all the subgalleries as options
					foreach( $galleries as $key => $theGallery )
						$renderedList .= $theGallery->optionSubgalleries( $this->getGlobalID("view"), "");
					
					$data = "
					<table class=\"options\">
					<tr class=\"options\">
						<td colspan=\"2\" class=\"options\">
						<p>" . i18n("Select the gallery you wish to move the image to from the drop-down list below") . "</p>
						<select style=\"width: 100%;\" name=\"whatGallery\">$renderedList</select>
					</td>
					<tr class=\"options\">
						<td width=\"50%\" class=\"options-bottomleft\"><input type=\"submit\" class=\"options_save\" name=\"saveOptions\" value=\"" . i18n("Move image") . "\" /></td>
						<td width=\"50%\" class=\"options-bottomright\"><input type=\"reset\" class=\"options_reset\" value=\"" . i18n("Reset") . "\" /></td>
					</tr>
					</table>";
				}
				$renderedGallery .= renderInformationBox( i18n("Move image to another gallery"), $data, false );
			}
			break;
		case "delete":
			// Physically delete a gallery and all subgalleries (not allowed for top level profile galleries)
			if( $this->displayImage == null )
			{
				if( $_REQUEST["deleteAccept"] == "true" )
				{
					$errorhandler = new errorHandler( $this->globalID );
					$errorhandler->handle( i18n("Deleting the gallery and all contents including subgalleries"), rmdirr( $this->basedir ) );
					
					if( $errorhandler->handled )
						$renderedGallery .= $errorhandler->render();
					else
					{
						$tmpID = array_slice( $this->splitID, 0, -1 );
						if( $this->profileGallery )
							$tmpID[3] = "edit";
						else
							$tmpID[1] = "edit";
						
						header( "Location: " . globalIDtoURL( implode( "/", $tmpID ) ) );
					}
				}
				else
					$renderedGallery .= renderInformationBox(
						i18n("Delete the gallery ##0##", array( $this->getGlobalID("view") ) ),
						"<p>" .
						i18n("Please confirm that you wish to delete the gallery by clicking on the Confirm Delete link below. If you do not wish to delete the image, click on Abort Delete in stead to return to the gallery overview") .
						"</p>
						<p>&nbsp;</p>
						<div class=\"wikicenteralign\">" .
						drawCommand( i18n("Confirm Delete"), i18n("Confirm that you wish to delete ##0##", array( $this->getGlobalID("view") ) ), globalIDtoURL( $this->globalID, array( "deleteAccept" => "true" ) ) ) .
						"&nbsp;&nbsp;&nbsp;" .
						drawCommand( i18n("Abort Delete"), i18n("Return to the gallery overview"), globalIDtoURL( $this->getGlobalID( "edit" ) ) ) .
						"<div>",
						false
						);
			}
			// Physically delete an image and all comments and data related to it
			else
			{
				// Get a reference to the image...
				foreach( $this->images as $key => $tmpimage )
					if( $tmpimage->imagename == $this->displayImage )
						$image = &$this->images[$key];
				
				if( $_REQUEST["deleteAccept"] == "true" )
				{
					$errorhandler = new errorHandler( $image->globalID );
					
					$errorhandler->handle( i18n("Deleting image file"), unlink( $this->basedir . "/images/" . $image->imagename ) );
					if( file_exists( $this->basedir . "/midsize/" . $image->imagename ) )
						$errorhandler->handle( i18n("Deleting midsize file"), unlink( $this->basedir . "/midsize/" . $image->imagename ) );
					$errorhandler->handle( i18n("Deleting thumbnail file"), unlink( $this->basedir . "/thumbs/" . $image->imagename ) );
					$errorhandler->handle( i18n("Deleting comments"), unlink( $this->basedir . "/comments/" . $image->imagename . ".php" ) );
					$errorhandler->handle( i18n("Deleting data"), unlink( $this->basedir . "/data/" . $image->imagename . ".php" ) );
					
					global $subscriptions;
					$subscriptions->removeGlobalID( $this->getGlobalID("view") . "/" . $image->imagename );
					
					// Delete the image from the image order
					foreach( $this->imageOrder as $key => $value )
						if( $value == $image->imagename )
							unset( $this->imageOrder[$key] );
					$errorhandler->handle( i18n("Saving new gallery image order"), $this->save() );
					
					if( $errorhandler->handled )
						$renderedGallery .= $errorhandler->render();
					else
						header( "Location: " . globalIDtoURL( $this->getGlobalID( "edit" ) ) );
				}
				else
					$renderedGallery .= renderInformationBox(
						i18n("Delete the image ##0##", array( $image->getGlobalID("view") ) ),
						"<div class=\"wikicenteralign\"><img src=\"" . $image->thumbURL . "\" alt=\"" . $image->getGlobalID("view") . "\" /></div>
						<hr /><p>" .
						i18n("Please confirm that you wish to delete the image by clicking on the Confirm Delete link below. If you do not wish to delete the image, click on Abort Delete in stead to return to the gallery overview") .
						"</p>
						<p>&nbsp;</p>
						<div class=\"wikicenteralign\">" .
						drawCommand( i18n("Confirm Delete"), i18n("Confirm that you wish to delete ##0##", array( $image->getGlobalID("view") ) ), globalIDtoURL( $image->globalID, array( "deleteAccept" => "true" ) ) ) .
						"&nbsp;&nbsp;&nbsp;" .
						drawCommand( i18n("Abort Delete"), i18n("Return to the gallery overview"), globalIDtoURL( $this->getGlobalID( "edit" ) ) ) .
						"<div>",
						false
						);
				}
			break;
		case "edit":
		default:
			// List images in this gallery
			if( count( $this->images ) < 1 )
				$renderedGallery .= renderInformationBox(
						i18n("No images"),
						i18n("There are no images in this gallery. ##0##Please click here to upload your first image##1##.", array( "<a href=\"" . globalIDtoURL($uploadID) . "\">", "</a>") )
					);
			else
			{
				if( $this->viewType == "0" )
					$viewType = "table";
				else
					$viewType = $this->viewType;
				$viewTypeRenderer = "gallery2_viewType_" . $viewType;
				$viewTypeRenderer = new $viewTypeRenderer( &$this );
				$renderedGallery .= $viewTypeRenderer->render();
			}
		}
		
		$tempSplitID = $this->splitID;
		if( $this->profileGallery )
			$tempSplitID[3] = $this->editMode;
		else
			$tempSplitID[1] = $this->editMode;
		
		if( $this->displayImage != null )
			$tempSplitID[] = $this->displayImage;
		
		return "<form enctype=\"multipart/form-data\" action=\"" . globalIDtoURL( implode( "/", $tempSplitID ) ) . "$extraReqVars\" method=\"post\">" . $renderedGallery . "</form>";
	}
	
	/**
	 * Render the gallery view
	 *
	 * @return   String containing the rendered gallery
	 */
	function render()
	{
		global $meta_headers, $module_folder, $permissionlevels_array;
		$meta_headers .= "
	<link rel=\"StyleSheet\" href=\"" . siteURL(true) . "$module_folder/gallery/gallery.css\" type=\"text/css\" />";
		if( $this->userCanView() )
		{
			if( $this->edit )
			{
				$renderedGallery = $this->renderEditor();
				if( $_REQUEST["theme"] == "empty" )
					return $renderedGallery;
			}
			else if( $this->error )
			{
				$renderedGallery = renderInformationBox( i18n("Not found"), i18n("You are attempting to view a gallery which does not exist. Please check the link you attempted to use and tell the person who gave the link to you about this error.") );
			}
			else
			{
				// check if we're showing an image, in that case pass control on to image.
				if( $this->displayImage != null )
				{
					foreach( $this->images as $key => $image )
						if( $image->imagename == $this->displayImage )
							return $image->render();
				}
				else
				{
					if( count( $this->images ) < 1 )
					{
						if( count( $this->subgalleries ) > 0 )
							$renderedGallery = renderInformationBox(
									i18n("No images"),
									i18n("There are no images in this gallery, however there are subgalleries. Please go into one of them by clicking on their name in the list above.")
								);
						else
							$renderedGallery = renderInformationBox(
									i18n("No images"),
									i18n("There are no images in this gallery.")
								);
					}
					else
					{
						if( $this->viewType == "0" )
							$viewType = "table";
						else
							$viewType = $this->viewType;
						$viewTypeRenderer = "gallery2_viewType_" . $viewType;
						$viewTypeRenderer = new $viewTypeRenderer( &$this );
						$renderedGallery = $viewTypeRenderer->render();
					}
					// Subscriptions subsystem
					global $subscriptions;
					$renderedSubscriptionControl = "<div class=\"wikirightalign\">" . $subscriptions->renderSubscribeControl( $this->getGlobalID("view"), false ) . "</div>";
				}
			}
		}
		else
		{
			$renderedGallery = renderInformationBox( i18n("No access"), i18n("You are not allowed to view this gallery. The owner has decided that only users fitting the description **##0##** are allowed to view this.", array( $permissionlevels_array[$this->viewLevel] ) ) );
		}
		
		// Get the breadcrumbs trail for this gallery...
		$trail = $this->getBreadcrumbs();
		
		// List the subgalleries
		foreach( $this->subgalleries as $key => $subgallery )
			$subgalleries .= "<div class=\"gallery-subgallery\"><a class=\"gallery-subgallery\" href=\"" . globalIDtoURL( $subgallery->globalID ) . "\">" . $subgallery->title . " (" . count( $subgallery->imageOrder ) . ")</a></div>";
		
		// And of course...
		if( $subgalleries != "" )
		{
			$subgalleryTitle = "
	<th class=\"gallery-subgalleries\">" . i18n("Subgalleries") . "</th>";
			$subgalleries = "
	<td class=\"gallery-subgalleries\">" . $subgalleries . "</td>";
			$subgallerySpan = " colspan=\"2\"";
		}
		
		$header = "
<table class=\"gallery\">
<tr class=\"gallery-title\">
	<th class=\"gallery-title\">" . $trail->render() . "</th>$subgalleryTitle
</tr>
<tr class=\"gallery-info\">
	<td class=\"gallery-description\">" . parse_page_data( $this->description ) . "</td>$subgalleries
</tr>
<tr class=\"gallery\">
	<td$subgallerySpan class=\"gallery\">";
	
		$footer = "
	</td>
</tr>
</table>";
	
	return $header . $renderedSubscriptionControl . $renderedGallery . $footer;
	}
}

//! The interface definition of the gallery2_viewType
class gallery2_viewType_interface
{
	var $gallery;            ///< Reference to the calling gallery
	
	/**
	 * The constructor for the gallery viewType renderer
	 *
	 * @param    gallery        A reference to the gallery we're supposed to be rendering
	 */
	function gallery2_viewType_interface( $gallery )
	{
		$this->gallery = &$gallery;
	}
	
	///\privatesection
	/**
	 * Render an image according the the viewType
	 *
	 * @param    image          A reference to the image being rendered
	 *
	 * @return   A string containing the rendered image
	 */
	function renderSingleImage( $image )
	{
		$renderedImage = "<img src=\"" . $image->imageURL . "\" />";
		
		if( $this->gallery->edit )
			$renderedImage .= "Insert edit controls";
		
		return $renderedImage;
	}
	
	///\public
	/**
	 * Render the gallery according to the viewType
	 *
	 * @return   A string containing the rendered gallery
	 */
	function render()
	{
		foreach( $this->gallery->images as $key => $image )
			$renderedGallery .= $this->renderSingleImage( &$image );
		
		return $renderedGallery; // Return the rendered gallery
	}
}

// Register the table gallery view type into the registry
$gallery2_viewTypes[] = "table";

//! The Table view-type class for the new gallery
class gallery2_viewType_table
{
	var $gallery;            ///< Reference to the calling gallery
	
	///\private
	var $picsPerRow = 3;     ///< Number of pictures per row
	
	/**
	 * Constructor for the table viewType class
	 */
	function gallery2_viewType_table( $gallery )
	{
		$this->gallery = &$gallery;
	}
	
	///\privatesection
	function renderSingleImage( $key )
	{
		global $setup_folder, $languages, $languages_names;
		$image = &$this->gallery->images[$key];
		
		if( $image->parentGallery->edit )
		{
			///\todo Make much prettier - images plzkthx!
			$rename = drawCommand( "<img src=\"" . siteURL(true) . "$setup_folder/images/edit.png\" border=\"0\" width=\"10\" height=\"10\" alt=\"move back\" />", i18n("Rename the image"), globalIDtoURL($image->getGlobalID( "rename" ) ) );
			$replace = drawCommand( "<img src=\"" . siteURL(true) . "$setup_folder/images/replace.png\" border=\"0\" width=\"10\" height=\"10\" alt=\"move back\" />", i18n("Replace the image with a new one, retaining all comments"), globalIDtoURL($image->getGlobalID( "replace" ) ) );
			$moveGallery = " | " . drawCommand( "<img src=\"" . siteURL(true) . "$setup_folder/images/folder.png\" border=\"0\" width=\"10\" height=\"10\" alt=\"move back\" />", i18n("Move the image to another gallery or subgallery"), globalIDtoURL($image->getGlobalID( "move" ) ) );
			$delete = drawCommand( "<img src=\"" . siteURL(true) . "$setup_folder/images/delete.png\" border=\"0\" width=\"10\" height=\"10\" alt=\"move back\" />", i18n("Delete the image and all associated comments"), globalIDtoURL($image->getGlobalID( "delete" ) ) );
			$moveUp = drawCommand( "<img src=\"" . siteURL(true) . "$setup_folder/images/nav-prev.png\" border=\"0\" width=\"10\" height=\"10\" alt=\"move back\" />", i18n("Move this image one position back in the gallery view order"), globalIDtoURL($image->getGlobalID( "moveup" ) ) );
			$moveDown = drawCommand( "<img src=\"" . siteURL(true) . "$setup_folder/images/nav-next.png\" border=\"0\" width=\"10\" height=\"10\" alt=\"move back\" />", i18n("Move this image one position forward in the gallery view order"), globalIDtoURL($image->getGlobalID( "movedown" ) ) );
			
			$thumbEdit = i18n("Thumbnail:");
			$thumbEdit .= " " . drawCommand( "cr", i18n("Create a thumbnail by selecting a section of the image"), globalIDtoURL($image->getGlobalID( "thumbnail" ), array( "thumbtype" => "cropnresize", "theme" => "empty" ) ) );
			$thumbEdit .= " " . drawCommand( "fs", i18n("Create a thumbnail from the entire image"), globalIDtoURL($image->getGlobalID( "thumbnail" ), array("thumbtype" => "fullsize") ) );
			
			$languageEdit = i18n("Description:");
			foreach ($languages as $key => $value)
				$languageEdit .= " " . drawCommand( i18n($value), i18n("Edit the description in: ##0##", array( $languages_names[$key] ) ), globalIDtoURL($image->getGlobalID( "details" ), array( "editLanguage" => $value ) ) );
			$languageEdit = "<div>$languageEdit</div>";
			
			$break = "<br />";
		}
		else
		{
			$viewLinkTop = "
							<a class=\"gallery-view-table-tile\" href=\"" . globalIDtoURL( $image->globalID ) . "\">";
			$viewLinkBottom = "
							</a>";
		}
		return "
					<td class=\"gallery-view-table-tile\">
						<div class=\"gallery-view-table-tile\">$viewLinkTop
								$thumbEdit $break
								$moveUp<img src=\"" . $image->thumbURL . "\" border=\"0\" class=\"gallery-view-table-tile\" />$moveDown
								$break $rename $replace $moveGallery $delete
								<h6 class=\"gallery-view-table-tile\">" . $image->imagename . "</h6>$viewLinkBottom
								$languageEdit
							<span class=\"gallery-view-table-comments\">" . i18n("Comments") . ": " . $image->commentCount . "<br /></span>
							<span class=\"gallery-view-table-dimensions\">" . $image->width . "&times;" . $image->height . " " . $image->weight . "</span>
						</div>
					</td>";
	}
	
	///\public
	function render()
	{
		$header = "
			<table class=\"gallery-view-table\">";
		$footer = "
			</table>";
		
		$firstinrow = true;
		$lastinrow = false;
		
		foreach( $this->gallery->images as $key => $image )
		{
			// First, render the current image...
			$renderedData .= $this->renderSingleImage( $key );
			
			// Is it first in row?
			if( $firstinrow )
			{
				$firstinrow = false;
				$renderedData = "
				<tr class=\"gallery\">" . $renderedData;
			}
			// Or perhaps last in row?
			else if( $lastinrow )
			{
				$lastinrow = false;
				$firstinrow = true;
				$renderedData .= "
				</tr>";
			}
			// If not, then it's got to be the middle one...
			else
				$lastinrow = true;
		}
		
		if( $lastinrow )
			$renderedData .= "
					<td class=\"gallery-view-table-tile-none\"></td>
				</tr>";
		else if( $lastinrow == false && $firstinrow == false )
			$renderedData .= "
					<td class=\"gallery-view-table-tile-none\"></td>
					<td class=\"gallery-view-table-tile-none\"></td>
				</tr>";
		
		
		return $header . $renderedData . $footer;
	}
}

$gallery2_viewTypes[] = "oldstyle";

//! The interface definition of the gallery2_viewType
class gallery2_viewType_oldstyle
{
	var $gallery;            ///< Reference to the calling gallery
	
	/**
	 * The constructor for the gallery viewType renderer
	 *
	 * @param    gallery        A reference to the gallery we're supposed to be rendering
	 */
	function gallery2_viewType_oldstyle( $gallery )
	{
		$this->gallery = &$gallery;
	}
	
	///\privatesection
	/**
	 * Render an image according the the oldstyle viewType
	 *
	 * @param    image          A reference to the image being rendered
	 *
	 * @return   A string containing the rendered image
	 */
	function renderSingleImage( $image )
	{
		global $language, $languages, $languages_names;
		
		$renderedImage .= "
		<tr class=\"gallery-view-oldstyle\">
			<td class=\"gallery-view-oldstyle-image\">
				<div align=\"center\"><a class=\"gallery-view-oldstyle-image-link\" href=\"" . globalIDtoURL( $image->globalID ) . "\"><img src=\"" . $image->thumbURL . "\" border=\"0\" /><br />" . $image->imagename . "</a></div>
			</td>
			<td class=\"gallery-view-oldstyle-description\">";
		if( $this->gallery->edit )
		{
			foreach ($languages as $key => $value)
				$renderedImage .= parse_page_data( $image->descriptions[$value] ) . drawCommand( i18n($value), i18n("Edit the description in: ##0##", array( $languages_names[$key] ) ), globalIDtoURL($image->getGlobalID( "details" ), array( "editLanguage" => $value ) ) );
		}
		else
			$renderedImage .= parse_page_data( $image->descriptions[$language] );
		
		$renderedImage .= "
			</td>";
		
		if( $this->gallery->edit )
		{
			$renderedImage .= "
			<td class=\"gallery-view-oldstyle-commands\">
				<div align=\"right\">" .
					drawCommand( i18n("Rename"), i18n("Rename the image"), globalIDtoURL($image->getGlobalID( "rename" ) ) ) . " " .
					drawCommand( i18n("Replace"), i18n("Replace the image with a new one, retaining all comments"), globalIDtoURL($image->getGlobalID( "replace" ) ) ) . " " .
					drawCommand( i18n("Move"), i18n("Move the image to another gallery or subgallery"), globalIDtoURL($image->getGlobalID( "move" ) ) ) . " " .
					drawCommand( i18n("Delete"), i18n("Delete the image and all associated comments"), globalIDtoURL($image->getGlobalID( "delete" ) ) ) . "
				</div><hr class=\"gallery-edit-commands\" />
				<div align=\"right\">" .
					drawCommand( i18n("Move up"), i18n("Move this image one position up in the gallery view order"), globalIDtoURL($image->getGlobalID( "moveup" ) ) ) . " " .
					drawCommand( i18n("Move down"), i18n("Move this image one position down in the gallery view order"), globalIDtoURL($image->getGlobalID( "movedown" ) ) ) . "
				</div><hr class=\"gallery-edit-commands\" />
				<div align=\"right\">" . i18n("Redo thumbnail:") . "<br />" .
					drawCommand( i18n("Crop and Resize"), i18n("Create a thumbnail by selecting a section of the image"), globalIDtoURL($image->getGlobalID( "thumbnail" ), array( "thumbtype" => "cropnresize", "theme" => "empty" ) ) ) . " " .
					drawCommand( i18n("Full picture"), i18n("Create a thumbnail from the entire image"), globalIDtoURL($image->getGlobalID( "thumbnail" ), array("thumbtype" => "fullsize") ) ) . "
				</div>
			</td>";
		}
		
		$renderedImage .= "
		</tr>";
		
		return $renderedImage;
	}
	
	///\public
	/**
	 * Render the gallery according to the viewType
	 *
	 * @return   A string containing the rendered gallery
	 */
	function render()
	{
		$renderedGallery .= "
		<table class=\"gallery-view-oldstyle\">";
		
		foreach( $this->gallery->images as $key => $image )
			$renderedGallery .= $this->renderSingleImage( &$image );
		
		$renderedGallery .= "
		</table>";
		
		return $renderedGallery;
	}
}

function parse_gallery2( $page_data )
{
	global $userinfo_folder, $globalID;

	$moduleCommandName = "gallery";
	while( $parameters = spotModuleCommand( $page_data, $moduleCommandName, &$loopcontrol ) )
	{
		$galleryid = $parameters;
		if( $parameters == " " )
			$parameters = "";
		
		// If this is the case... we're dealing with old page data! Add global ID
		$splitID = explode( "/", $galleryid );
		if( $splitID[0] != "gallery" && $splitID[0] != "gallery2"  && $splitID[0] != "user" )
			$galleryid = "gallery2/view/$galleryid";
		
		$splitID = explode( "/", $globalID );
		if( $splitID[0] == "gallery" || $splitID[0] == "gallery2" )
			$galleryid = $globalID;
		
		$theGallery = new gallery2_gallery( $galleryid );
		
		$replaceData = $theGallery->render();
		
		$page_data = str_replace( "\\$moduleCommandName($parameters)", $replaceData, $page_data );
	}
	
	$loopcontrol = 0;
	$moduleCommandName = "profilegallerylist";
	while( $parameters = spotModuleCommand( $page_data, $moduleCommandName, &$loopcontrol ) )
	{
		$viewtype = $parameters;
		if( $parameters == " " )
			$parameters = "";
		
		$userlist = allUsersArray();
		$userswithgalleries = array();
		foreach( $userlist as $key => $value )
		{
			if( is_dir( "$userinfo_folder/$value/gallery" ) )
			{
				$theGallery = new gallery2_gallery("user/$value/gallery", false);
				$count = $theGallery->countSubImages();
				if( $count > 0 )
				{
					$userswithgalleries[] = $value;
					$gallerycounts[] = $count;
				}
			}
		}
		if( count($userswithgalleries) > 0 )
		{
			$replaceData = parse_userlists( "\userlist($viewtype)", false, $userswithgalleries, "gallery2" );
		
			foreach($userswithgalleries as $key => $value)
				$replaceData = str_replace("($value)", "($value) " . $gallerycounts[$key], $replaceData);
		}
		else
			$replaceData = renderInformationBox( i18n("No gallery profiles"), i18n("Sorry, there were found no profiles with at least one picture in the profile gallery") );
		
		$page_data = str_replace( "\\$moduleCommandName($parameters)", $replaceData, $page_data );
	}
	
	$loopcontrol = 0;
	$moduleCommandName = "galleryimage";
	while( $parameters = spotModuleCommand( $page_data, $moduleCommandName, &$loopcontrol ) )
	{
		list( $galleryid, $float, $dontlink ) = explode( ",", $parameters );
		
		$theGallery = new gallery2_gallery( $galleryid );
		
		foreach( $theGallery->images as $key => $image )
		{
			if( $image->imagename == $theGallery->displayImage )
			{
				$theImage = $image;
				break;
			}
		}
		
		if( $float == "right" )
			$floatit = " style=\"float: right;\"";
		else if( $float == "center" )
			$floatit = " style=\"float: center;\"";
		else if( $float == "left" )
			$floatit = " style=\"float: left;\"";
		
		if( $dontlink == "" )
		{
			$link = "<a href=\"" . globalIDtoURL( $image->globalID ) . "\">";
			$link2 = "</a>";
		}
		
		list( $x, $y ) = getimagesize( $image->parentGallery->basedir . "/thumbs/" . $image->imagename );
		$replaceData = "$link<img$floatit border=\"0\" src=\"" . $image->parentGallery->baseurl . "/thumbs/" . $image->imagename . "\" width=\"$x\" height=\"$y\" alt=\"" . $image->imagename . "\" />$link2";
		
		$page_data = str_replace( "\\$moduleCommandName($parameters)", $replaceData, $page_data );
	}

	return $page_data;
}

function profilemodule_gallery2()
{
	global $tabwidget, $viewUser, $globalID;
	
	$gallery = $globalID;
	
	$theGallery = new gallery2_gallery( $gallery );
	
	if( userAllows( $theGallery->owner, $theGallery->editLevel) && $theGallery->edit )
	{
		$tabwidget->addCommand( i18n("Upload"), i18n("Upload an image to the gallery"), globalIDtoURL( $theGallery->getGlobalID( "upload" ) ) );
		$tabwidget->addCommand( i18n("View"), i18n("View the gallery"), globalIDtoURL( $theGallery->getGlobalID( "view" ) ) );
	}
	else if( userAllows( $theGallery->owner, $theGallery->editLevel) )
	{
		$tabwidget->addCommand( i18n("Upload"), i18n("Upload an image to the gallery"), globalIDtoURL( $theGallery->getGlobalID( "upload" ) ) );
		$tabwidget->addCommand( i18n("Edit"), i18n("Edit the gallery"), globalIDtoURL( $theGallery->getGlobalID( "edit" ) ) );
	}
	return $theGallery->render();
}

function modulesetup_gallery2()
{
	global $setup_folder, $module_folder, $page_folder, $menu_folder, $recent_file, $pagectl, $default_language, $language, $meta_headers, $theModuleSection, $theModuleSubsection, $theModuleAction, $gallery_options, $gallery2_viewTypes, $permissionlevels_array;
	$meta_headers .= "
	<link rel=\"StyleSheet\" href=\"" . siteURL(true) . "$module_folder/gallery/gallery.css\" type=\"text/css\" />";
	
	$gallerySetup = new TabWidget;
	$gallerySetup->tabbar->addTab(i18n("Gallery Setup"), globalIDtoURL("setup/modulesetup/gallery2/setup"));
	$gallerySetup->tabbar->addTab(i18n("Options"), globalIDtoURL("setup/modulesetup/gallery2/options"));
	
	switch( $theModuleSection )
	{
	case "options":
		if( $theModuleAction == "create_gallery_page" )
		{
			// Create the page to save the forum to!
			$edit_page["id"] = $pagectl->getFreePageID($page_folder, $pagectl);
			$edit_page["title"] = $gallery_options["title"];
			$edit_page["menu"] = $gallery_options["menu_id"];
			$edit_page["language"] = $default_language;
			$edit_page["content"] = "\\\\gallery()";
			$edit_page["author"] = getUserInfo(currentUser(), "name");
			$edit_page["email"] = getUserInfo(currentUser(), "email");
			
			unlink_multiple( "$page_folder/{$edit_page['id']}.*" );
			
			if ($pagectl->savePageData($page_folder, $edit_page["id"], $edit_page["language"], $edit_page["author"], $edit_page["email"], $gallery_options["menu_id"], $edit_page["title"], $edit_page["content"])) {
				update_getpage_id($page_folder, $setup_folder, $pagectl);
				rotate_recent($edit_page["id"], $edit_page["title"], $edit_page["language"], date("j/n/y"), $recent_file);
				
				$gallery_options["page_id"] = $edit_page["id"];
				if( file_exists( $module_folder . "/gallery/options.php" ) )
					unlink( $module_folder . "/gallery/options.php" );
				file_put_contents( $module_folder . "/gallery/options.php", array_export( $gallery_options, "gallery_options" ) );
				
				header("Location: " . globalIDtoURL("setup/modulesetup/gallery2/options"));
			} else {
				$error_description = parse_page_data(i18n("The gallery page could not be saved!"));
			}
		}
		if( $_POST["saveOptions"] && $_POST["gallery"]["page_id"] != 12 && ( $gallery_options["page_id"] != $_POST["gallery"]["page_id"] || $gallery_options["menu_id"] != $_POST["gallery"]["menu_id"] || $gallery_options["title"] != $_POST["gallery"]["title"] ) )
		{
			$thePage = $pagectl->fetchPageData($_POST["page_id"]);
			unlink_multiple( "$page_folder/{$gallery_options['page_id']}.*" );
			if ($pagectl->savePageData($page_folder, $_POST["gallery"]["page_id"], $thePage["language"], $thePage["author"], $thePage["email"], $_POST["gallery"]["menu_id"], $_POST["gallery"]["title"], "\\\\gallery()"))
			{
				update_getpage_id($page_folder, $setup_folder, $pagectl);
				rotate_recent($gallery_options["page_id"], $gallery_options["title"], $thePage["language"], date("j/n/y"), $recent_file);
			}
		}
		$gallerySetup->tabbar->setCurrent( i18n( "Options" ) );
		$panel = new optionsPanel("gallery");
		
		$values["page_id"] = $_POST["saveOptions"] ? $_POST["gallery"]["page_id"] : $gallery_options["page_id"] ;
		$values["title"] = $_POST["saveOptions"] ? $_POST["gallery"]["title"] : $gallery_options["title"] ;
		$values["menu_id"] = $_POST["saveOptions"] ? $_POST["gallery"]["menu_id"] : $gallery_options["menu_id"] ;
		
		$values["maxwidth"] = $_POST["saveOptions"] ? $_POST["gallery"]["maxwidth"] : $gallery_options["maxwidth"] ;
		$values["maxheight"] = $_POST["saveOptions"] ? $_POST["gallery"]["maxheight"] : $gallery_options["maxheight"] ;
		$values["midwidth"] = $_POST["saveOptions"] ? $_POST["gallery"]["midwidth"] : $gallery_options["midwidth"] ;
		$values["midheight"] = $_POST["saveOptions"] ? $_POST["gallery"]["midheight"] : $gallery_options["midheight"] ;
		$values["thumbwidth"] = $_POST["saveOptions"] ? $_POST["gallery"]["thumbwidth"] : $gallery_options["thumbwidth"] ;
		$values["thumbheight"] = $_POST["saveOptions"] ? $_POST["gallery"]["thumbheight"] : $gallery_options["thumbheight"] ;
		
		$values["viewType"] = $_POST["saveOptions"] ? $_POST["gallery"]["viewType"] : $gallery_options["viewType"] ;
		$values["viewLevel"] = $_POST["saveOptions"] ? $_POST["gallery"]["viewLevel"] : $gallery_options["viewLevel"] ;
		$values["commentLevel"] = $_POST["saveOptions"] ? $_POST["gallery"]["commentLevel"] : $gallery_options["commentLevel"] ;
		$values["editLevel"] = $_POST["saveOptions"] ? $_POST["gallery"]["editLevel"] : $gallery_options["editLevel"] ;
		$values["embedLevel"] = $_POST["saveOptions"] ? $_POST["gallery"]["embedLevel"] : $gallery_options["embedLevel"] ;
		
		$panel->addHeader( i18n("Gallery page"), "thepage" );
		
		$pagelist = $pagectl->fetchPageList($page_folder);
		$temp_pagelist[12] = i18n("Please select a page...");
		foreach( $pagelist as $key => $value )
		{
			$temp_pagelist[$key] = $value["title"][0];
		}
		$panel->addOption(
			i18n("Gallery page"),
			i18n("This is the page that will be used to show the gallery images on when linking. If you want to create a new page to put the information into, ##0##simply click here##1## and this will be done for you.", array("<a href=\"" . globalIDtoURL("setup/modulesetup/gallery2/options/na/create_gallery_page") . "\">", "</a>")),
			$values["page_id"],
			"page_id",
			"select",
			$temp_pagelist
		);
		
		$panel->addOption(
			i18n("Title"),
			i18n("The title of the gallery page"),
			$values["title"],
			"title",
			"text"
		);
		
		$menulist = $pagectl->fetchMenuMenuList($menu_folder);
		foreach( $menulist as $key => $value )
		{
			$temp_menulist[$value["id"]] = $value["title"][0];
		}
		$panel->addOption(
			i18n("Gallery menu"),
			i18n("This is the menu that will be shown on the gallery page"),
			$values["menu_id"],
			"menu_id",
			"select",
			$temp_menulist
		);
		
		$panel->addHeader( i18n("Image resize sizes"), "sizes" );
		$panel->addOption(
			i18n("Resize width"),
			i18n("Images will be resized if they are larger than this width when uploading to galleries"),
			$values["maxwidth"],
			"maxwidth",
			"text");
		$panel->addOption(
			i18n("Resize height"),
			i18n("Images will be resized if they are larger than this height when uploading to galleries"),
			$values["maxheight"],
			"maxheight",
			"text");
		$panel->addOption(
			i18n("Midsize width"),
			i18n("When viewing gallery images, they will be resized to this width if required"),
			$values["midwidth"],
			"midwidth",
			"text");
		$panel->addOption(
			i18n("Midsize height"),
			i18n("When viewing gallery images, they will be resized to this height if required"),
			$values["midheight"],
			"midheight",
			"text");
		$panel->addOption(
			i18n("Thumbnail width"),
			i18n("The maximum width for thumbnails"),
			$values["thumbwidth"],
			"thumbwidth",
			"text");
		$panel->addOption(
			i18n("Thumbnail height"),
			i18n("The maximum height for thumbnails"),
			$values["thumbheight"],
			"thumbheight",
			"text");
		
		$panel->addHeader( i18n("Default gallery options"), "defaults" );
		
		$panel->addOption(
			i18n("View type"),
			i18n("The view type of newly created galleries"),
			$values["viewType"],
			"viewType",
			"select",
			$gallery2_viewTypes);
		$panel->addOption(
			i18n("View level"),
			i18n("Apart from the gallery owner, who is allowed to view the gallery's contents?"),
			$values["viewLevel"],
			"viewLevel",
			"select",
			$permissionlevels_array);
		$panel->addOption(
			i18n("Comment level"),
			i18n("Apart from the gallery owner, who is allowed to comment on the gallery's contents?"),
			$values["commentLevel"],
			"commentLevel",
			"select",
			$permissionlevels_array);
		$panel->addOption(
			i18n("Edit level"),
			i18n("Apart from the gallery owner, who is allowed to edit the gallery?"),
			$values["editLevel"],
			"editLevel",
			"select",
			$permissionlevels_array);
		$panel->addOption(
			i18n("Embedding level"),
			i18n("Apart from the gallery owner, who is allowed to get embedding codes for the contents of the gallery?"),
			$values["embedLevel"],
			"embedLevel",
			"select",
			$permissionlevels_array);
		
		if( $_POST["saveOptions"] )
		{
			$gallery_options["page_id"] = $values["page_id"];
			$gallery_options["menu_id"] = $values["menu_id"];
			$gallery_options["title"] = $values["title"];
			
			$gallery_options["maxwidth"] = $values["maxwidth"];
			$gallery_options["maxheight"] = $values["maxheight"];
			$gallery_options["midwidth"] = $values["midwidth"];
			$gallery_options["midheight"] = $values["midheight"];
			$gallery_options["thumbwidth"] = $values["thumbwidth"];
			$gallery_options["thumbheight"] = $values["thumbheight"];
			$gallery_options["viewType"] = $values["viewType"];
			$gallery_options["viewLevel"] = $values["viewLevel"];
			$gallery_options["commentLevel"] = $values["commentLevel"];
			$gallery_options["editLevel"] = $values["editLevel"];
			$gallery_options["embedLevel"] = $values["embedLevel"];
			
			if( file_exists( $module_folder . "/gallery/options.php" ) )
				unlink( $module_folder . "/gallery/options.php" );
			file_put_contents( $module_folder . "/gallery/options.php", array_export( $gallery_options, "gallery_options" ) );
			
			header( "Location: " . globalIDtoURL( "setup/modulesetup/gallery2/options" ) );
		}
		else
			$data = $panel->render();
		
		$gallerySetup->contents = "<form action=\"" . globalIDtoURL("setup/modulesetup/gallery2/options") . "\" method=\"POST\">" . $data . "</form>";
		break;
	case "edit":
	case "thumbnail":
	case "moveup":
	case "movedown":
	case "upload":
	case "replace":
	case "new":
	case "rename":
	case "details":
	case "move":
	case "delete":
		global $gallery2_gallery;
		$theGallery = new gallery2_gallery( $gallery2_gallery );
		$gallerySetup->contents = $theGallery->render();
		break;
	case "setup":
	default:
		$gallerySetup->tabbar->setCurrent( i18n( "Gallery Setup" ) );
		$gallerySetup->addCommand( i18n("New gallery"), i18n("Create a new global gallery"), globalIDtoURL( "setup/modulesetup/gallery2/setup/new" ) );
		
		switch( $theModuleSubsection )
		{
		case "new":
			$panel = new optionsPanel("");
			$panel->saveTitle = i18n("Create subgallery");
			$panel->resetTitle = i18n("Reset");
			$panel->showHeader = false;
			
			$values["physicalname"] = ( $_POST["physicalname"] != "" ) ? stripslashes($_POST["physicalname"]) : "gallery";
			$values["title"] = ( $_POST["title"] != "" ) ? stripslashes($_POST["title"]) : i18n("Untitled Gallery");
			$values["description"] = ( $_POST["description"] != "" ) ? stripslashes($_POST["description"]) : i18n("Enter the new gallery's description here");
			$values["viewType"] = ( $_POST["viewType"] != "" ) ? stripslashes($_POST["viewType"]) : $gallery_options["viewType"];
			$values["viewLevel"] = ( $_POST["viewLevel"] != "" ) ? stripslashes($_POST["viewLevel"]) : $gallery_options["viewLevel"];
			$values["commentLevel"] = ( $_POST["commentLevel"] != "" ) ? stripslashes($_POST["commentLevel"]) : $gallery_options["commentLevel"];
			$values["editLevel"] = ( $_POST["editLevel"] != "" ) ? stripslashes($_POST["editLevel"]) : $gallery_options["editLevel"];
			$values["embedLevel"] = ( $_POST["embedLevel"] != "" ) ? stripslashes($_POST["embedLevel"]) : $gallery_options["embedLevel"];
			
			$panel->addOption( i18n("Physical name:"), i18n("The filename you will find the gallery under (used for linking)"), $values["physicalname"], "physicalname", "text");
			$panel->addOption( i18n("Title:"), i18n("The gallery's human readable name"), $values["title"], "title", "text");
			$panel->addOption( i18n("View type"), i18n("How do you wish to present this gallery to the user by default?"), $values["viewType"], "viewType", "select", $gallery2_viewTypes);
			$panel->addOption( i18n("View Level:"), i18n("Who, except for yourself, is allowed to view the contents of this gallery?"), $values["viewLevel"], "viewLevel", "select", $permissionlevels_array);
			$panel->addOption( i18n("Comment Level:"), i18n("Who, except for yourself, is allowed to comment on the contents of this gallery?"), $values["commentLevel"], "commentLevel", "select", $permissionlevels_array);
			$panel->addOption( i18n("Edit Level:"), i18n("Who, except for yourself, is allowed to edit the contents of this gallery?"), $values["editLevel"], "editLevel", "select", $permissionlevels_array);
			$panel->addOption( i18n("Description:"), "", $values["description"], "description", "pagedata");
			
			$newbasedir = $module_folder . "/gallery/" . str_replace( ".", "-", str_replace( " ", "_", $panel->options["physicalname"]["current"] ) ) ;
			
			if( $_POST["saveOptions"] && file_exists( $newbasedir ) )
				$panel->options["physicalname"]["error"] = i18n("A gallery already exists with this name!");
			
			if( $_POST["saveOptions"] && $panel->options["physicalname"]["current"] != "" && ! file_exists( $newbasedir ) )
			{
				RecursiveMkdir( $newbasedir . "/comments" );
				RecursiveMkdir( $newbasedir . "/data" );
				RecursiveMkdir( $newbasedir . "/galleries" );
				RecursiveMkdir( $newbasedir . "/images" );
				RecursiveMkdir( $newbasedir . "/thumbs" );
				
				$gallery["title"] = $panel->options["title"]["current"];
				$gallery["description"] = $panel->options["description"]["current"];
				$gallery["viewType"] = $gallery2_viewTypes[ $panel->options["viewType"]["current"] ];
				$gallery["viewLevel"] = $panel->options["viewLevel"]["current"];
				$gallery["commentLevel"] = $panel->options["commentLevel"]["current"];
				$gallery["editLevel"] = $panel->options["editLevel"]["current"];
				$gallery["embedLevel"] = $panel->options["embedLevel"]["current"];
				$gallery["imageOrder"] = array();
				$gallery["owner"] = currentUser();
				
				if( file_put_contents( $newbasedir . "/gallery.php", array_export( $gallery, "gallery" ) ) )
					header("Location: " . globalIDtoURL( "gallery2/edit/" . $panel->options["physicalname"]["current"] ) );
				else
					$renderedGallery .= "An error occured while saving the gallery settings!";
			}
			else
				$renderedGallery .= renderInformationBox( i18n("Create a new gallery"), $panel->render(), false);
			
			$data = "<form action=\"" . globalIDtoURL("setup/modulesetup/gallery2/setup/new") . "\" method=\"POST\">" . $renderedGallery . "</form>";
			break;
		case "gallery_galleries":
		default:
			$galleries = array();
			$gallerybase = "$module_folder/gallery";
			
			$dir_handle = opendir("$gallerybase");
			while ($folder = readdir($dir_handle)) {
				if( is_dir( "$gallerybase/$folder" ) && substr($folder, 0, 1) != "." ) {
					$galleries[] = new gallery2_gallery( "gallery2/edit/" . $folder, false );
				}
			}
			closedir($dir_handle);
			
			if( count( $galleries ) == 0 )
			{
				$data = renderInformationBox( i18n("No galleries"), i18n("There are currently no galleries on the system. If you wish, you can create one by choosing the New gallery command link above.") );
			}
			else
			{
				foreach( $galleries as $key => $theGallery )
				{
					if( userAllows( $theGallery->owner, $theGallery->editLevel ) )
						$data .= "
						<div class=\"gallery-global-list\">
							<a class=\"gallery-global-header\" href=\"" . globalIDtoURL( $theGallery->getGlobalID("edit") ) . "\">
								<span class=\"gallery-global-go\">" . i18n("Edit ##0##", array( "&raquo;" ) ) . "</span>
								<h3 class=\"gallery-global-header\">" . $theGallery->title . " (" . $theGallery->countSubImages() . ")</h3>
							</a>
							<div class=\"gallery-global-description\">" . parse_page_data( $theGallery->description ) . "</div>
						</div>";
					else
						$data .= "
						<div class=\"gallery-global-list\">
								<span class=\"gallery-global-go\">" . i18n("Edit not allowed" ) . "</span>
								<h3 class=\"gallery-global-header\">" . $theGallery->title . " (" . $theGallery->countSubImages() . ")</h3>
							<div class=\"gallery-global-description\">" . parse_page_data( $theGallery->description ) . "</div>
						</div>";
				}
			}
		}
		$gallerySetup->contents = $data;
	}
	return $gallerySetup->renderTabWidget();
}

/**
 * Returns the url that corresponds to the globalID corresponding to a gallery item if appropriate
 *
 * @param     splitID        The global ID to check
 *
 * @return    string         A string containing the URL corresponding to the globalID, if existing
 */
function globalID_gallery2( $splitID )
{
	global $gallery2_gallery, $gallery_options;
	$pageID = null;
	
	// Is the globalID directly aimed at this module?
	// This doesn't exist... yet :P To be implemented once the gallery module is less... broken
	if( $splitID[0] == "gallery2" || $splitID[0] == "gallery" )
	{
		if( $splitID[1] == "view" || $splitID[1] == "" )
		{
			$pageID = $gallery_options["page_id"]; // Put the gallery's page in here...
		}
		else
		{
			$pageID = 12; // Module setup...
			global $theModule, $theModuleSection;
			$theModule = "gallery2";
			$theModuleSection = $splitID[1];
			$gallery2_gallery = implode( "/", $splitID );
		}
	}
	// Or is it aimed at the module's profile module?
	else if( $splitID[0] == "user" && ( $splitID[2] == "gallery2" || $splitID[2] == "gallery" ))
	{
		// Make sure we know it's this module
		$_REQUEST["module"] = "profilemodule_gallery2";
		// Set the gallery...
		$gallery2_gallery = implode( "/", $splitID );
		// Set the page ID to user profile
		$pageID = 13;
	}
	return $pageID;
}

/**
 * Return an array of data about the thumbnail for a global ID
 *
 * @param     splitID        The global ID to check
 *
 * @return    Array containing array( "uri" => the fully qualified URI for the thumbnail, "width" => the width in pixels, "height" => the height in pixels )
 */
function globalIDThumbnail_gallery2( $splitID )
{
	$theGallery = new gallery2_gallery( join( $splitID, "/" ) );
	// Get a reference to the image...
	foreach( $theGallery->images as $key => $tmpimage )
	{
		if( $tmpimage->imagename == $theGallery->displayImage )
		{
			$image = &$theGallery->images[$key];
			break;
		}
	}
	
	list( $width, $height ) = getimagesize($theGallery->basedir . "/thumbs/" . $tmpimage->imagename);
	$data = array( "uri" => $tmpimage->thumbURL, "width" => $width, "height" => $height );
	return $data;
}

function globalIDThumbnail_gallery( $splitID ) { return globalIDThumbnail_gllery2( $splitID ); }
?>