<?PHP 
/**
* BloggerMod for Travelsized CMS
* @author	Dan Jensen <admin@leinir.dk>
* @date		2005-07-17
* @version	0.2
*
* Blog entry files structure
* <?PHP
* $entry = array();
* ?>
*
* Blog array structure
* array(
*	"name" => username,
*	"timestamp" => unix timestamp,
*	"title" => the blog title,
*	"content" => the blog entry (pagedata),
*	"categories" => array("category1", "category2"),
*	"personal" => boolean, defining wether the entry is personal or not,
*	"viewlevel" => integer, describing the viewing allowance level,
*	"commentlevel" => integer, describing the commenting allowance level,
*	"comments" => array(
*		"name" => Name of commenter,
*		"email" => Commenter's email address,
*		"timestamp" => Timestamp when the comment was made,
*		"comment" => Comment text,
*		"comments" => array())
*	)
*/

// Register module human name, version and description.
$modules_names[] = "BloggerMod";
$mopules_versions[] = "0.2";
$modules_descriptions[] = "Category based blogging module - one blog per user, with both global and personal categories";

// Register the module into module settings
$modules_setup[] = "modulesetup_blogger";

// Register the module into profile modules
$profilemodules[] = "profilemodule_blogger";

// Register the module into globalID
$modules_globalID[] = "globalID_blogger";

// Register the module's parsing function
$modules[] = "parse_blogger";
$modules[] = "parse_blogger_all";
$modules[] = "parse_blogger_feed";

// Give help to using the whole blogging module system
$page_help["--- in blog entry"] = "This will cause BloggerMod to cut the entry here when not showing the full entry - it will still be parsed as a horizontal line normally";
$page_help["&#92;blogall(txt1)"] = "Will show you a list of blog entries by all users in the system, optionally with the space-delimited category list txt1";
$page_help["&#92;blog(txt1, txt2, txt3)"] = "Will show you a list of blog entries by the user txt1, limited to the categories defined by the space-delimited list txt2. If txt3 is present, it is the amount of blog items to show.";
$page_help["&#92;blog(txt1)"] = "Will show you a list of blog entries by the user txt1 (txt1 can also be a space-delimited list of usernames)";

/**
* blogger_allowViewPersonal is a version of the profile system's allowView function fiddled with to handle blogger view allowance in stead
* It returns wether the currently logged in user is allowed to view a user $username's personal blog entries
*/
function blogger_userAllowsViewPersonal($username) {
	$viewlevel = getUserInfo($username, "blogger_allowsviewpersonal");
	return userAllows($username, $viewlevel);
}

/**
* blogger_allowView is a version of the profile system's allowView function fiddled with to handle blogger view allowance in stead
* It returns wether the currently logged in user is allowed to view a user $username's blog entry with viewing allowance level $viewlevel
*/
function blogger_userAllowsView($username, $viewlevel) {
	return userAllows($username, $viewlevel);
}

/**
* blogger_allowView is a version of the profile system's allowView function fiddled with to handle blogger view allowance in stead
* It returns wether the currently logged in user is allowed to view a user $username's blog entry with viewing allowance level $viewlevel
*/
function blogger_userAllowsCommenting($username, $viewlevel) {
	return userAllows($username, $viewlevel);
}

/**
* blogger_getblogs will get you an array containing the blogs for a particular
* user, and if categories are defined only the blogs with those categories
*
* @param	$username	The username of the user to return blog entries for
* @param	$categories	A space-separated list of categories
*
* @return	array	An array containing the blog entries, empty if no entries exist
*/
function blogger_getblogs($username = "", $categories = "") {
	global $setup_folder, $userinfo_folder, $language;
	
	if ($categories != "") {
		if (strpos($categories, " ") === true) {
			$categories = explode(" ", $categories);
		} else {
			$categories = array($categories);
		}
	}
	
	if ($username == "") {
		$username = allUsersArray();
	} else if (!is_array($username)) {
		if (strpos($username, " ") === true) {
			$username = explode(" ", $username);
		} else {
			$username = array($username);
		}
	}
	
	$blog = "";
	
	foreach($username as $key => $value) {
		if (($fp = @opendir("$userinfo_folder/$value/blogger")) === false) {
			// Do naaathink!
		} else {
			while ($file = readdir($fp)) {
				if (($value != "" || substr($file, 0, strlen($value)) == $value) && substr($file, 0, 1) != "." ) {
					include("$userinfo_folder/$value/blogger/$file");
					if ($entry["personal"]) { // Personal view setting overrides ALL other view settings
						if (blogger_userAllowsViewPersonal($value)) $blog[$entry["timestamp"] . $entry["name"]] = $entry;
					} else {
						if (is_array($categories)) {
							$showentry = false;
							if (is_array($entry["categories"])) {
								foreach($entry["categories"] as $key2 => $value2) {
									if ($value2 == "system_category") {
										$showentry = true;
									} else {
										foreach ($categories as $key3 => $value3) {
											if ($value2 == $value3 && blogger_userAllowsView($value, $entry["viewlevel"])) {
												$showentry = true;
												break;
											}
										}
									}
									if ($showentry) break;
								}
							}
							if ($showentry) $blog[$entry["timestamp"] . $entry["name"]] = $entry;
						} else {
							$blog[$entry["timestamp"] . $entry["name"]] = $entry;
						}
					}
				}
			}
			closedir($fp);
		}
	}
	
	if (is_array($blog)) {
		krsort($blog);
		return $blog;
	}
	return array();
}

/**
 * blogger_navigator will create a navigation bar for blogger...
 *
 * @param    $shownEntries    int the amount of blogs actually shown
 * @param    $currentPage     int the current page
 * @param    $lowerBound      int the lowest blog entry on the page
 * @param    $upperBound      int the highest blog entry on the page
 * @param    $entriesPerPage  int the amount of entries per page
 *
 * @return   string           The div containing the navigator
 */
function blogger_navigator($shownEntries, $currentPage, $lowerBound, $upperBound, $entriesPerPage)
{
	if( $upperBound + 1 > $shownEntries )
		$upperBound = $shownEntries - 1;
	
	$blog_navigate = "";
	
	if( $currentPage <= ($upperBound / $entriesPerPage) - 1 )
		$blog_navigate_previous .= "<a class=\"command\" href=\"" . thisPageURL() . "&amp;blogger_currentpage=" . ($currentPage + 1) . "\">[ &laquo; " . i18n("Prevous ##0##", array($entriesPerPage) ) . "]</a> ";
	
	if( $currentPage > 0 )
		$blog_navigate_next .= " <a class=\"command\" href=\"" . thisPageURL() . "&amp;blogger_currentpage=" . ($currentPage - 1) . "\">[" . i18n("Next ##0##", array($entriesPerPage) ) . " &raquo; ]</a>";
	
	if( $shownEntries > $entriesPerPage ) // Then there are actually pages...
		$blog_navigate = "
		<div class=\"blogger_navigator\">
			$blog_navigate_previous
			" .i18n("Showing entries number ##0## to ##1## of ##2##", array($shownEntries - $upperBound, $shownEntries - $lowerBound, $shownEntries ) ) . "
			$blog_navigate_next
		</div>";
		
	return $blog_navigate;
}

/**
* blogger_showentry takes a username and timestamp, then shows the full entry with comments and such
*
* @param	$username	The username to get the blog entry from
* @param	$timestamp	The entry made at this timestamp
*
* @return	string	The full entry
*/
function blogger_showentry($username, $timestamp)
{
	global $setup_folder, $userinfo_folder, $page_id, $language, $permissionlevels_array, $formatting_toolbar;
	if (file_exists("$userinfo_folder/$username/blogger/$timestamp.php")) include("$userinfo_folder/$username/blogger/$timestamp.php");
	if (is_array($entry)) {
		$listid = 0; // Must define this, to be able to pass by reference
		$commenthandler = new commentHandler($entry["comments"], $entry["name"], $entry["commentlevel"], 0);
		$commenthandler->thisPageURL = thisPageURL() . "&amp;blogger_entry={$entry['name']}.{$entry['timestamp']}";
		$blog_data .= "
		<div class=\"blogger\">
			<h2 class=\"blogger\">" . $entry["title"] . "</h2>
			<h6 class=\"blogger\">" . parse_profilelinks("%%" . $entry["name"] . "%%") . " - " . formatTime( $entry["timestamp"]) . "</h6>
			<div class=\"blogger-content\">" . parse_page_data($entry["content"]) . "</div>
			<h3 class=\"blogger\">" . i18n("Comments") . "</h3>
		" . $commenthandler->renderComments() . $commenthandler->renderCommentsAdder() . "
		</div>";
		
		if( $commenthandler->commentsChanged && $commenthandler->lastCommenter != "" )
		{ // If the comments have changed...
			$entry["comments"] = $commenthandler->comments; // Get the comments back from the comments handler
			file_put_contents("$userinfo_folder/$username/blogger/$timestamp.php", array_export($entry, "entry")); // Save the comments
			
			global $subscriptions;
			$subject = i18n( "The blog entry ##0## by ##1## has a new comment", array( $entry["title"], $username ) );
			$theMessage .= "\n\n---\n" . i18n( "Comment by: ##0##", array( parse_profilelinks( "%%" . $commenthandler->lastCommenter . "%%" ) ) ) . "\nMessage as follows:\n" . $commenthandler->lastComment;
			$message = i18n( "The change in ##0## is as follows:", array( $entry["title"] ) ) . $theMessage;
			$subscriptions->handle( "user/$username/blogger/{$entry['name']}.{$entry['timestamp']}", $username, $entry["viewlevel"], $subject, $message );
		}
		
		return $blog_data;
	} else {
		return "<p>" . i18n("Sorry, there is no blog entry with this username and timestamp.") . "</p>";
	}
}

/**
* parse_blogger takes in pagedata and parses it, looking for \\blog()
*
* @param	$page_data	The unparsed pagedata
*
* @return	string	The parsed pagedata
*/
function parse_blogger($page_data) {
	global $setup_folder, $meta_headers, $page_id, $page_title, $language;
	while (true) {
		$begpos = strpos($page_data, "\\blog("); //where does the first feed begin
		if ($begpos === false) break; //all feeds have been parsed, no need to check more
		$endpos = strpos($page_data, ")", $begpos + 5); //feed end
		$bloginfo = substr( $page_data, $begpos + 6, ($endpos - $begpos) - 6);
		$meta_headers .= "<link rel=\"alternate\" type=\"application/rss+xml\" title=\"$page_title RSS\" href=\"" . siteURL() . "?module_function=blogfeed($bloginfo)\" />";
		if ($_REQUEST["blogger_entry"] != "") {
			list($username, $timestamp) = explode(".", $_REQUEST["blogger_entry"]);
			$blog_data = "<a class=\"command\" style=\"float: right;\" href=\"" . thisPageURL() . "\">[" . i18n("Back") . "]</a>" . blogger_showentry($username, $timestamp);
		} else {
			if (strpos($bloginfo, ",") === true) {
				list($username, $categories, $count) = explode(",", $bloginfo);
			} else {
				$username = $bloginfo;
				$categories = "";
			}
			$blogs = blogger_getblogs($username, $categories);
			if (count($blogs) > 0) {
				$blog_data = "";
				
				$entriesPerPage = ($count > 0) ? $count : 10; // This should really be user-settable...
				$currentPage = $_REQUEST["blogger_currentpage"];
				$lowerBound = $entriesPerPage * $currentPage;
				$upperBound = $lowerBound + $entriesPerPage;
				$currentEntry = 0; // We start at entry number 1...
				
				foreach($blogs as $key => $value) {
					if ($value["published"] == true) {
						if( $lowerBound <= $currentEntry && $currentEntry < $upperBound )
						{
							$blog_data .= "
							<h2 class=\"blogger\">" . $value["title"] . "</h2>
							<h6 class=\"blogger\">" . parse_profilelinks( "%%" . $value["name"] . "%%" ) . " - " . formatTime( $value["timestamp"] ) . "</h6>
							";
							if (strpos($value["content"], "---") > 0) {
								$blog_data .= parse_page_data(substr($value["content"], 0, strpos($value["content"], "---"))) . "<small class=\"comment\">" . i18n("(entry cut - view to see full entry)") . "</small>";
							} else {
								$blog_data .= parse_page_data($value["content"]);
							}
							$commenthandler = new commentHandler($value["comments"], $value["name"], $value["commentlevel"], 0);
							$blog_data .= "
							<div class=\"blogger_commands\">
								<div style=\"float: right;\">
									<a class=\"command\" href=\"" . thisPageURL() . "&amp;blogger_entry={$value['name']}.{$value['timestamp']}\">[" . i18n("View") . "]</a>
									<a class=\"command\" href=\"" . thisPageURL() . "&amp;blogger_entry={$value['name']}.{$value['timestamp']}#comment\">[" . i18n("Comment") . "]</a>
								</div>
								<div class=\"blogger\">" . i18n("Comments") . ": " . $commenthandler->numberOfComments() . "</div>
							</div>";
						}
						$currentEntry++;
					}
				}
				
				$blog_data = blogger_navigator($currentEntry, $currentPage, $lowerBound, $upperBound, $entriesPerPage) . $blog_data;
				
			} else {
				$blog_data = renderInformationBox(
						i18n("No blog entries"),
						i18n("No blog entries were found")
					);
			}
		}
		$page_data = str_replace("\\blog($bloginfo)", $blog_data, $page_data);
	}
	return $page_data;
}
/**
* parse_blogger takes in pagedata and parses it, looking for \\blogall()
*
* @param	$page_data	The unparsed pagedata
*
* @return	string	The parsed pagedata
*/
function parse_blogger_all($page_data) {
	global $setup_folder, $meta_headers, $page_title;
	while (true) {
		$begpos = strpos($page_data, "\\blogall("); //where does the first feed begin
		if ($begpos === false) break; //all feeds have been parsed, no need to check more
		$endpos = strpos($page_data, ")", $begpos + 8); //feed end
		$categories = substr( $page_data, $begpos + 9, ($endpos - $begpos) - 9);
		$blogs = blogger_getblogs("", $categories);
		$meta_headers .= "<link rel=\"alternate\" type=\"application/rss+xml\" title=\"$page_title RSS\" href=\"" . siteURL() . "?module_function=blogfeedall($categories)\" />";
		
		if ($_REQUEST["blogger_entry"] != "") {
			list($username, $timestamp) = explode(".", $_REQUEST["blogger_entry"]);
			$blog_data = "<a class=\"command\" style=\"float: right;\" href=\"" . thisPageURL() . "\">[" . i18n("Back") . "]</a>" . blogger_showentry($username, $timestamp);
		} else {
			if (count($blogs) > 0) {
				$blog_data = "";
				
				$entriesPerPage = 10; // This should really be user-settable...
				$currentPage = $_REQUEST["blogger_currentpage"];
				$lowerBound = $entriesPerPage * $currentPage;
				$upperBound = $lowerBound + $entriesPerPage;
				$currentEntry = 0; // We start at entry number 1...
				
				foreach($blogs as $key => $value) {
					if ($value["published"] == true) {
						if( $lowerBound <= $currentEntry && $currentEntry < $upperBound )
						{
							$blog_data .= "
							<h2 class=\"blogger\">" . $value["title"] . "</h2>
							<h6 class=\"blogger\">" . parse_profilelinks( "%%{$value['name']}%%" ) . " - " . formatTime( $value["timestamp"] ) . "</h6>
							";
							if (strpos($value["content"], "---") > 0) {
								$blog_data .= parse_page_data(substr($value["content"], 0, strpos($value["content"], "---"))) . "<small class=\"comment\">" . i18n("(entry cut - view to see full entry)") . "</small>";
							} else {
								$blog_data .= parse_page_data($value["content"]);
							}
							$commenthandler = new commentHandler($value["comments"], $value["name"], $value["commentlevel"], 0);
							$blog_data .= "
							<div class=\"blogger_commands\">
								<div style=\"float: right;\">
									<a class=\"command\" href=\"" . thisPageURL() . "&amp;blogger_entry={$value['name']}.{$value['timestamp']}\">[" . i18n("View") . "]</a>
									<a class=\"command\" href=\"" . thisPageURL() . "&amp;blogger_entry={$value['name']}.{$value['timestamp']}#comment\">[" . i18n("Comment") . "]</a>
								</div>
								<div class=\"blogger\">" . i18n("Comments") . ": " . $commenthandler->numberOfComments() . "</div>
							</div>";
						}
						$currentEntry++;
					}
				}
				
				$blog_data = blogger_navigator($currentEntry, $currentPage, $lowerBound, $upperBound, $entriesPerPage) . $blog_data;
				
			} else {
				$blog_data = "<p>" . i18n("No blog entries were found") . "</p>";
			}
		}
		$page_data = str_replace("\\blogall($categories)", $blog_data, $page_data);
	}
	return $page_data;
}

/**
* parse_blogger_feed takes in pagedata and parses it, looking for \\blogfeed()
*
* @param	$page_data	The unparsed pagedata
*
* @return	string	The parsed pagedata
*/
function parse_blogger_feed($page_data) {
	global $setup_folder, $owner_email, $page_title;
	
	$feed_header = "<?xml version=\"1.0\" encoding=\"iso-8859-1\" ?>
<rss version=\"2.0\" xmlns:slash=\"http://purl.org/rss/1.0/modules/slash/\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:annotate=\"http://purl.org/rss/1.0/modules/annotate/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">
<channel>
	<title>";
	$feed_header2 = "</title>
	<link>" . thisPageURL(false, true) . "</link>
	<description></description>
	<managingEditor>$owner_email</managingEditor>
	<docs>http://blogs.law.harvard.edu/tech/rss</docs>
	<generator>BloggerMod 0.1</generator>
	<language>en</language>
	<lastBuildDate>" . date("r") . "</lastBuildDate>
";
	$feed_footer = "
</channel>
</rss>";
	while (true) {
		$begpos = strpos($page_data, "\\blogfeedall("); //where does the first feed begin
		if ($begpos === false) break; //all feeds have been parsed, no need to check more
		$endpos = strpos($page_data, ")", $begpos + 12); //feed end
		$categories = substr( $page_data, $begpos + 13, ($endpos - $begpos) - 13);
		$blogs = blogger_getblogs("", $categories);
		
		$blog_data = "";
		$category_data = "";
		foreach($blogs as $key => $value) {
			if ($value["published"] == true) {
				$commenthandler = new commentHandler($value["comments"], $value["name"], $value["commentlevel"], 0);
				foreach($value["categories"] as $key2 => $value2) {
					if ($category_data != "") $separator = ", ";
					$category_data .= i18n($value2) . $separator;
				}
				if ($category_data == "") $category_data = i18n("No categories defined");
	
				$blog_data .= "
	<item>
		<title>{$value['title']}</title>
		<link>" . thisPageURL() . "&amp;blogger_entry={$value['name']}.{$value['timestamp']}</link>
		<pubDate>" . date("r", $value["timestamp"]) . "</pubDate>
		<guid isPermaLink=\"true\">" . thisPageURL() . "&amp;blogger_entry={$value['name']}.{$value['timestamp']}</guid>
		<description>
		" . i18n("Posted on ##0## by ##1##, in the categories: ##2##", array(date("r", $value["timestamp"]), "&lt;a href=&quot;" . siteURL() . "?user_id=" . $value['name'] . "&quot;&gt;" . getUserInfo($value['name'], "name") . "&lt;/a&gt;", $category_data)) . "
		&lt;hr /&gt;";
		if (strpos($value["content"], "---") > 0) {
			$blog_data .= str_replace("<", "&lt;", str_replace(">", "&gt;", parse_page_data(substr($value["content"], 0, strpos($value["content"], "---"))))) . i18n("(entry cut - view to see full entry)");
		} else {
			$blog_data .= str_replace("<", "&lt;", str_replace(">", "&gt;", parse_page_data($value["content"])));
		}
		$blog_data .= "
		</description>
		<dc:creator>" . getUserInfo($value['name'], "name") . "</dc:creator>
		<dc:subject>{$value['title']}</dc:subject>
		<slash:comments>" . $commenthandler->numberOfComments() . "</slash:comments>
		<annotate:reference rdf:resource=\"" . thisPageURL() . "&amp;blogger_entry={$value['name']}.{$value['timestamp']}\" />
		<comments>" . thisPageURL() . "&amp;blogger_entry={$value['name']}.{$value['timestamp']}#comments</comments>
	</item>";
			}
		}
		
		
		$blog_data = "$feed_header$page_title$feed_header2$blog_data$feed_footer";
		
		$page_data = str_replace("\\blogfeedall($categories)", $blog_data, $page_data);
	}
	while (true) {
		$begpos = strpos($page_data, "\\blogfeed("); //where does the first feed begin
		if ($begpos === false) break; //all feeds have been parsed, no need to check more
		$endpos = strpos($page_data, ")", $begpos + 9); //feed end
		$bloginfo = substr( $page_data, $begpos + 10, ($endpos - $begpos) - 10);
		
		if (strpos($bloginfo, ",") === true) {
			list($username, $categories) = explode(",", $bloginfo);
		} else {
			$username = $bloginfo;
			$categories = "";
		}
		$blogs = blogger_getblogs($username, $categories);
		
		$blog_data = "";
		$category_data = "";
		foreach($blogs as $key => $value) {
			$commenthandler = new commentHandler($value["comments"], $value["name"], $value["commentlevel"], 0);
			if ($value["published"] == true) {
				$blog_data .= "
	<item>
		<title>{$value['title']}</title>
		<link>" . thisPageURL() . "&amp;blogger_entry={$value['name']}.{$value['timestamp']}</link>
		<pubDate>" . date("r", $value["timestamp"]) . "</pubDate>
		<guid isPermaLink=\"true\">" . thisPageURL() . "&amp;blogger_entry={$value['name']}.{$value['timestamp']}</guid>
		<description>
		" . i18n("Posted on ##0## by ##1##, in the categories: ##2##", array(date("r", $value["timestamp"]), "&lt;a href=&quot;" . siteURL() . "?user_id=" . $value['name'] . "&quot;&gt;" . getUserInfo($value['name'], "name") . "&lt;/a&gt;", $category_data)) . "
		&lt;hr /&gt;";
		if (strpos($value["content"], "---") > 0) {
			$blog_data .= str_replace("<", "&lt;", str_replace(">", "&gt;", parse_page_data(substr($value["content"], 0, strpos($value["content"], "---"))))) . i18n("(entry cut - view to see full entry)");
		} else {
			$blog_data .= str_replace("<", "&lt;", str_replace(">", "&gt;", parse_page_data($value["content"])));
		}
		$blog_data .= "
		</description>
		<dc:creator>" . getUserInfo($value['name'], "name") . "</dc:creator>
		<dc:subject>{$value['title']}</dc:subject>
		<slash:comments>" . $commenthandler->numberOfComments() . "</slash:comments>
		<annotate:reference rdf:resource=\"" . thisPageURL() . "&amp;blogger_entry={$value['name']}.{$value['timestamp']}\" />
		<comments>" . thisPageURL() . "&amp;blogger_entry={$value['name']}.{$value['timestamp']}#comments</comments>
	</item>";
			}
		}
		
		$thetitle = i18n("##0##'s blog", array(getUserInfo($username, "name")));
		$blog_data = "$feed_header$thetitle$feed_header2$blog_data$feed_footer";
		
		$page_data = str_replace("\\blogfeed($bloginfo)", $blog_data, $page_data);
	}

	return $page_data;
}

/**
 * Returns the url that corresponds to the globalID corresponding to a blog item if appropriate
 *
 * @param     $splitID       The global ID to check
 *
 * @return    string         A string containing the URL corresponding to the globalID, if existing
 */
function globalID_blogger( $splitID )
{
	$pageID = null;
/*	
	// Is the globalID directly aimed at this module?
	// Not yet implemented - blogger mod does not know which page the blog is on...
	if( $splitID[0] == "blogger" )
	{
		$_REQUEST["page_id"] = $foo_pageID;
		$pageID = $foo_pageID;
	}
	else
	*/
	// Or is it aimed at the module's profile module?
	if( $splitID[0] == "user" && $splitID[2] == "blogger" )
	{
		$_REQUEST["page_id"] = "13";
		$_REQUEST["user_id"] = $splitID[1];
		$_REQUEST["module"] = "profilemodule_blogger";
		
		if( array_key_exists( 3, $splitID ) )
			$_REQUEST["blogger_entry"] = $splitID[3];
		
		$pageID = "13";
	}
	
	return $pageID;
}

function profilemodule_blogger() {
	global $globalID, $tabwidget, $setup_folder, $userinfo_folder, $language, $languages, $permissionlevels_array, $blogcategories, $formatting_toolbar, $viewUser;
	
	$splitID = explode( "/", $globalID );
	
	$bloggerbase = "$userinfo_folder/$viewUser/blogger/";
	if (!file_exists($bloggerbase)) mkdir($bloggerbase, 0766);
	
	if ($_POST["blogger_save_settings"]) {
		saveUserInfo($viewUser, "blogger_viewlevel", $_REQUEST["blogger_viewlevel"]);
		saveUserInfo($viewUser, "blogger_commentlevel", $_REQUEST["blogger_commentlevel"]);
		saveUserInfo($viewUser, "blogger_allowsviewpersonal", $_REQUEST["blogger_allowsviewpersonal"]);
		if ($_REQUEST["blogger_personal"]) {
			saveUserInfo($viewUser, "blogger_personal", 1);
		} else {
			saveUserInfo($viewUser, "blogger_personal", 0);
		}
		if (is_array($_REQUEST["blogger_category"])) {
			foreach ($_REQUEST["blogger_category"] as $key => $value) {
				$categories_temp[] = $value;
			}
			saveUserInfo($viewUser, "blogger_categories", serialize($categories_temp));
		}
	}

	$temp_options["viewlevel"] = getUserInfo($viewUser, "blogger_viewlevel");
	if ($temp_options["viewlevel"] == i18n("No ##0## information\n", array("blogger_viewlevel"))) {
		$temp_options["viewlevel"] = 0;
		saveUserInfo($viewUser, "blogger_viewlevel", 0);
	}
	$temp_options["commentlevel"] = getUserInfo($viewUser, "blogger_commentlevel");
	if ($temp_options["commentlevel"] == i18n("No ##0## information\n", array("blogger_commentlevel"))) {
		$temp_options["commentlevel"] = 0;
		saveUserInfo($viewUser, "blogger_commentlevel", 0);
	}
	$temp_options["personal"] = getUserInfo($viewUser, "blogger_personal");
	if ($temp_options["personal"] == i18n("No ##0## information\n", array("blogger_personal"))) {
		$temp_options["personal"] = false;
		saveUserInfo($viewUser, "blogger_personal", false);
	}
	$temp_options["allowsviewpersonal"] = getUserInfo($viewUser, "blogger_allowsviewpersonal");
	if ($temp_options["allowsviewpersonal"] == i18n("No ##0## information\n", array("blogger_allowsviewpersonal"))) {
		$temp_options["allowsviewpersonal"] = 4;
		saveUserInfo($viewUser, "blogger_allowsviewpersonal", 4);
	}
	
	$temp_options["categories"] = getUserInfo($viewUser, "blogger_categories");
	if ($temp_options["categories"] == i18n("No ##0## information\n", array("blogger_categories"))) {
		$temp_options["categories"] = array();
		saveUserInfo($viewUser, "blogger_categories", serialize(array()));
	} else {
		$temp_options["categories"] = unserialize($temp_options["categories"]);
	}
	
	$blogcategories_user = getUserInfo($viewUser, "blogger_usercategories");
	if ($blogcategories_user == i18n("No ##0## information\n", array("blogger_usercategories"))) {
		$blogcategories_user = array();
	} else {
		$blogcategories_user = unserialize($blogcategories_user);
	}
	
	if ($splitID[3] == "edit" && (isAllowed("global_user_edit") || ($viewUser != "" && $viewUser == currentUser()))) {
		$tabwidget->addCommand(i18n("New entry"), i18n("Create a new blog entry"), globalIDtoURL("user/$viewUser/blogger/edit", array( "subaction" => "add" ) ) );
		$tabwidget->addCommand(i18n("Options"), i18n("View and edit options for ##0##'s blog", array($viewUser)), globalIDtoURL("user/$viewUser/blogger/options") );
		$tabwidget->addCommand(i18n("Edit blog"), i18n("Edit the blog"), globalIDtoURL("user/$viewUser/blogger/edit") );
		$tabwidget->addCommand(i18n("View blog"), i18n("View the blog"), globalIDtoURL("user/$viewUser/blogger") );
		if ($_REQUEST["subaction"] == "add" && $_REQUEST["addaction"] == "") {
			$entry = array(
				"name" => $_REQUEST["user_id"],
				"timestamp" => time(),
				"title" => i18n("Entry title"),
				"content" => i18n("Write your content here"),
				"categories" => $temp_options["categories"],
				"personal" => $temp_options["personal"],
				"viewlevel" => $temp_options["viewlevel"],
				"commentlevel" => $temp_options["commentlevel"],
				"comments" => array()
				);
			$entrycontents = "<?PHP\n\$entry=" . var_export($entry, true) . ";?>";
			$entryfile = $bloggerbase . $entry["timestamp"] . ".php";
			if (!file_exists($entryfile)) {
				file_put_contents($entryfile, $entrycontents);
				chmod($entryfile, 0664);// make sure the file is actually accessible...
				header("Location: " . thisPageURL(true) . "&addaction=entrycreated&timestamp=" . $entry["timestamp"]);
			} else {
				$blog_data = "<p>" . i18n("An entry already exists for this moment in time. If you wish to add to this entry, edit it, otherwise delete it and try again. ##0##Click here to return...##1## or click refresh to try again.", array("<a href=\"" . thisPageURL() . "&amp;addaction=failed\">", "</a>")) . "</p>";
			}
		} else if (($_REQUEST["subaction"] == "edit" && $_REQUEST["timestamp"] != "" ) || $_REQUEST["addaction"] == "entrycreated") {
			include($bloggerbase . "/" . $_REQUEST["timestamp"] . ".php");
			if ($_POST) {
				$temp_year = $_REQUEST["year"];
				$temp_month = $_REQUEST["month"];
				$temp_date = $_REQUEST["day"];
				$temp_hour = $_REQUEST["hour"];
				$temp_minute = $_REQUEST["minute"];
				$temp_second = $_REQUEST["second"];
				$entry["title"] = stripslashes($_REQUEST["title"]);
				$entry["content"] = stripslashes($_REQUEST["new_content"]);
				$entry["personal"] = $_REQUEST["personal"] ? true : false;
				$entry["viewlevel"] = $_REQUEST["viewlevel"];
				$entry["commentlevel"] = $_REQUEST["commentlevel"];
				$entry["categories"] = $_REQUEST["blogger_category"];
				if (is_array($_POST["blogger_remove_category"])) {
					foreach($_POST["blogger_remove_category"] as $key => $value) {
						unset($entry["categories"][$key]);
					}
				}
				if ($_POST["blogger_add_category"]) $entry["categories"][] = "";
			} else {
				// time is set here only when not submit data...
				$temp_year = date("Y", $entry["timestamp"]);
				$temp_month = date("m", $entry["timestamp"]);
				$temp_date = date("j", $entry["timestamp"]);
				$temp_hour = date("H", $entry["timestamp"]);
				$temp_minute = date("i", $entry["timestamp"]);
				$temp_second = date("s", $entry["timestamp"]);
			}
			
			if ($_POST["blogger_abortedit"]) {
				header("Location: " . thisPageURL(true));
			} else if ($_POST["blogger_saveentry"] || $_POST["blogger_saveandpublish"] ) {
				$entry["timestamp"] = mktime($temp_hour, $temp_minute, $temp_second, $temp_month, $temp_date, $temp_year);
				$entrycontents = "<?PHP\n\$entry=" . var_export($entry, true) . ";?>";
				$entryfile = $bloggerbase . $entry["timestamp"] . ".php";
				$oldentryfile = $bloggerbase . $_REQUEST["timestamp"] . ".php";
				if (file_exists($oldentryfile)) unlink($oldentryfile);
				if (!file_exists($entryfile)) {
					file_put_contents($entryfile, $entrycontents);
					chmod($entryfile, 0664);// make sure the file is actually accessible...
					if( $_POST["blogger_saveandpublish"] )
						header("Location: " . thisPageURL(true) . "&timestamp={$entry['timestamp']}&subaction=publish" );
					else
						header("Location: " . thisPageURL(true));
				} else {
					$blog_data = "<p>" . i18n("An entry already exists for this moment in time. If you wish to replace this entry, please delete it first, then create a new one or save an old one in its place.") . "</p>";
				}
			} else {
				
				for ($i = date("Y") - 50; $i <= date("Y") + 50; $i++) { $allyears[] = $i; }
				foreach ($allyears as $key => $value) {
					if ($value == $temp_year) { $selected = " selected"; } else { $selected = ""; }
					$temp_years .= "<option value=\"$value\"$selected>$value</option>";
				}
				
				$allmonths = array("01" => "January", "02" => "February", "03" => "March", "04" => "April", "05" => "May", "06" => "June", "07" => "July", "08" => "August", "09" => "September", "10" => "October", "11" => "November", "12" => "December");
				foreach ($allmonths as $key => $value) {
					if ($key == $temp_month) { $selected = " selected"; } else { $selected = ""; }
					$temp_months .= "<option value=\"$key\"$selected>$value</option>";
				}
				
				for ($i = 1; $i <= 31; $i++) { $alldates[] = $i; }
				foreach ($alldates as $key => $value) {
					if ($value == $temp_date) { $selected = " selected"; } else { $selected = ""; }
					$temp_days .= "<option value=\"$value\"$selected>$value</option>";
				}
				
				for ($i = 0; $i <= 23; $i++) { if (strlen($i) < 2) { $allhours[] = "0$i"; } else { $allhours[] = $i; } }
				foreach ($allhours as $key => $value) {
					if ($value == $temp_hour) { $selected = " selected"; } else { $selected = ""; }
					$temp_hours .= "<option value=\"$value\"$selected>$value</option>";
				}
				
				for ($i = 0; $i <= 59; $i++) { if (strlen($i) < 2) { $allminutes[] = "0$i"; } else { $allminutes[] = $i; } }
				foreach ($allminutes as $key => $value) {
					if ($value == $temp_minute) { $selected = " selected"; } else { $selected = ""; }
					$temp_minutes .= "<option value=\"$value\"$selected>$value</option>";
				}
				foreach ($allminutes as $key => $value) {
					if ($value == $temp_second) { $selected = " selected"; } else { $selected = ""; }
					$temp_seconds .= "<option value=\"$value\"$selected>$value</option>";
				}
				
				if ($entry["personal"] == true) {
					$ispersonal = " checked";
				} else {
					$ispersonal = "";
				}
				
				foreach($permissionlevels_array as $key => $value) {
					if ($entry["viewlevel"] == $key) {
						$default = " selected";
					} else {
						$default = "";
					}
					$viewlevel .= "<option value=\"$key\"$default>$value</option>";
				}
				
				foreach($permissionlevels_array as $key => $value) {
					if ($entry["commentlevel"] == $key) {
						$default = " selected";
					} else {
						$default = "";
					}
					$commentlevel .= "<option value=\"$key\"$default>$value</option>";
				}
				
				if ($_POST["blogger_previewentry"]) {
					$blog_data .= "<div class=\"blogger-content\">" . parse_page_data($entry["content"]) . "</div>";
				}
				
				$blog_data .= "
				<form action=\"" . thisPageURL( false, true, array("subaction" => "edit", "timestamp" => $_REQUEST['timestamp']) ) . "\" method=\"post\">
				<table class=\"setup\">
				<tr class=\"setup\">
					<td class=\"setup\" width=\"40%\">" . i18n("Title") . ":</td>
					<td class=\"setup\" width=\"60%\"><input type=\"text\" style=\"width: 100%;\" name=\"title\" value=\"" . $entry["title"] . "\" /></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("Post time") . ":</td>
					<td class=\"setup\"><div style=\"width: 10%; overflow: hidden; display: inline-block;\">
					" . i18n("On") . "</div><select style=\"width: 30%;\" name=\"year\">$temp_years
					</select><select style=\"width: 40%;\" name=\"month\">$temp_months
					</select><select style=\"width: 20%;\" name=\"day\">$temp_days
					</select><br />
					<div style=\"width: 10%; overflow: hidden; display: inline-block;\">
					" . i18n("at") . "</div><select style=\"width: 30%;\" name=\"hour\">$temp_hours
					</select><select style=\"width: 30%;\" name=\"minute\">$temp_minutes
					</select><select style=\"width: 30%;\" name=\"second\">$temp_seconds
					</select></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("Personal post") . ":</td>
					<td class=\"setup\"><label><input type=\"checkbox\" name=\"personal\"$ispersonal />" . i18n("This is a personal post") . "</label></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("Viewable by") . ":</td>
					<td class=\"setup\"><select style=\"width: 100%\" name=\"viewlevel\">$viewlevel</select></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("Commentable by") . ":</td>
					<td class=\"setup\"><select style=\"width: 100%\" name=\"commentlevel\">$commentlevel</select></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("Categories") . ":";
					
		
				foreach ($entry["categories"] as $key => $value) {
					$categories_temp = "<optgroup label=\"" . i18n("Your categories") . "\">";
					foreach($blogcategories_user as $key2 => $value2) {
						if ($value2 == $value) { $default_temp = " selected"; } else { $default_temp = ""; }
						$categories_temp .= "<option value=\"$value2\"$default_temp>" . i18n($value2) . "</option>";
					}
					$categories_temp .= "</optgroup><optgroup label=\"" . i18n("Site-wide categories") . "\">";
					foreach($blogcategories as $key2 => $value2) {
						if (!isAllowed($_REQUEST["user_id"], "global_admin") && $key2 == 0) continue;
						if ($value2 == $value) { $default_temp = " selected"; } else { $default_temp = ""; }
						$categories_temp .= "<option value=\"$value2\"$default_temp>" . i18n($value2) . "</option>";
					}
					$blog_data .= "<input style=\"float: right;\" type=\"submit\" name=\"blogger_remove_category[$key]\" value=\"-\" /></td>
					<td class=\"setup\"><select style=\"width: 100%\" name=\"blogger_category[$key]\">$categories_temp</optgroup></select></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">";
				}
				
				$blog_data .= "&nbsp;</td>
					<td class=\"setup\"><input type=\"submit\" name=\"blogger_add_category\" value=\"" . i18n("Add category") . "\" /></td>
				</tr>
				$formatting_toolbar
				<tr class=\"setup\"><td class=\"setup\" colspan=\"2\">
				<textarea rows=\"15\" cols=\"70\" style=\"width: 100%;\" id=\"edit\" name=\"new_content\">{$entry['content']}</textarea>
				</td></tr>
				<tr class=\"setup\">
					<td class=\"setup\" colspan=\"2\">
					<input style=\"float: right;\" type=\"submit\" name=\"blogger_abortedit\" value=\"" . i18n("Abort edit") . "\" />
					<input type=\"submit\" name=\"blogger_saveentry\" value=\"" . i18n("Save entry") . "\" />
					<input type=\"submit\" name=\"blogger_saveandpublish\" value=\"" . i18n("Save &amp; Publish") . "\" />
					<input type=\"submit\" name=\"blogger_previewentry\" value=\"" . i18n("Preview entry") . "\" />
					</td>
				</tr>
				</table>
				" . generate_pagehelplink() . "
				</form>";
			}
		} else if ($_REQUEST["subaction"] == "delete") {
			$entryfile = "$bloggerbase/{$_REQUEST['timestamp']}.php";
			if (file_exists($entryfile)) {
				if ($_REQUEST["confirm_delete"] == "true") {
					if (unlink($entryfile)) {
						$blog_data = "<p>" . i18n("The entry was deleted correctly!") . "</p>";
						global $subscriptions;
						$subscriptions->removeGlobalID("user/$viewUser/blogger/" . $_REQUEST["timestamp"]);
						header("Location: " . thisPageURL(true));
					} else {
						$blog_data = "<p>" . i18n("The entry could not be deleted! Please check your permissions and make sure that you are allowed access to the file. The file in question is ##0##", array($blogfile)) . "</p>";
					}
				} else {
					include($entryfile);
					$blog_data = "<p>" . i18n("You have selected the entry titled &quot;##0##&quot; (posted for ##1##) to be deleted. Please confirm that you wish to delete this by clicking on Confirm delete below.", array( $entry["title"], formatTime( $entry["timestamp"] ) ) ) . "</p>
					<div class=\"wikicenteralign\">
					<a href=\"" . thisPageURL( false, true, array("subaction" => "delete", "timestamp" => $_REQUEST['timestamp'], "confirm_delete" => "true" ) ) . "\" class=\"command\" />" . i18n("Confirm delete") . "</a>
					<a href=\"" . thisPageURL() . "\" class=\"command\" />" . i18n("Abort delete") . "</a>
					</div>";
				}
			} else {
				$blog_data = "<p>" . i18n("The entry you have selected to delete does not exist! Did someone else delete it before you got here? Use your browser's back button to return to the previous page.") . "</p>";
			}
		} else if ($_REQUEST["subaction"] == "publish") {
			$entryfile = "$bloggerbase/{$_REQUEST['timestamp']}.php";
			if (file_exists($entryfile)) {
				include($entryfile);
				$entry["published"] = true;
				$entrycontents = "<?PHP\n\$entry=" . var_export($entry, true) . ";?>";
				if (unlink($entryfile)) {
					file_put_contents($entryfile, $entrycontents);
					chmod($entryfile, 0664);// make sure the file is actually accessible...
					
					global $subscriptions;
					$subject = i18n( "The blog entry ##0## by ##1## has been published", array( $entry["title"], $entry['name'] ) );
					$theMessage .= "\n\n---\n" . i18n( "Comment by: ##0##", array( parse_profilelinks( "%%" . $commentshandler->lastCommenter . "%%" ) ) ) . "\nMessage as follows:\n" . $commentshandler->lastComment;
					$message = i18n( "The change in ##0## is as follows:", array( $entry["title"] ) ) . $theMessage;
					$subscriptions->handle( "user/$username/blogger/{$entry['name']}.{$entry['timestamp']}", $entry['name'], $entry["viewlevel"], $subject, $message );
					
					header("Location: " . thisPageURL(true));
				} else {
						$blog_data = "<p>" . i18n("The entry could not be published! Please check your permissions and make sure that you are allowed access to the file. The file in question is ##0##", array($blogfile)) . "</p>";
				}
			} else {
				$blog_data = "<p>" . i18n("The entry you have selected to publish does not exist! Did someone else delete it before you got here? Use your browser's back button to return to the previous page.") . "</p>";
			}
		} else if ($_REQUEST["subaction"] == "unpublish") {
			$entryfile = "$bloggerbase/{$_REQUEST['timestamp']}.php";
			if (file_exists($entryfile)) {
				include($entryfile);
				$entry["published"] = false;
				$entrycontents = "<?PHP\n\$entry=" . var_export($entry, true) . ";?>";
				if (unlink($entryfile)) {
					file_put_contents($entryfile, $entrycontents);
					chmod($entryfile, 0664);// make sure the file is actually accessible...
					global $subscriptions;
					$subscriptions->removeGlobalID("user/$viewUser/blogger/" . $_REQUEST["timestamp"]);
					header("Location: " . thisPageURL(true));
				} else {
						$blog_data = "<p>" . i18n("The entry could not be published! Please check your permissions and make sure that you are allowed access to the file. The file in question is ##0##", array($blogfile)) . "</p>";
				}
			} else {
				$blog_data = "<p>" . i18n("The entry you have selected to unpublish does not exist! Did someone else delete it before you got here? Use your browser's back button to return to the previous page.") . "</p>";
			}
		} else {
			$allblogs = blogger_getblogs($_REQUEST["user_id"]);
			if (count($allblogs) > 0) {
				$blog_data = "<table class=\"setup\">
						<tr class=\"setup\">
							<th class=\"setup\" width=\"60%\">" . i18n("Entries") . "</th>
							<th class=\"setup\" width=\"40%\">" . i18n("Functions") . "</th>
						</tr>";
				foreach ($allblogs as $key => $value) {
					// delete, edit, publish/unpublish
					if ($value["published"] == true) {
						$publishit = "unpublish";
						$publishit_text = i18n("Unpublish");
					} else {
						$publishit = "publish";
						$publishit_text = i18n("Publish");
					}
					$blog_data .= "
						<tr class=\"setup\">
							<td class=\"setup\">" . $value["title"] . "<small class=\"comment\">" . formatTime( $value["timestamp"] ) . "</small></td>
							<td class=\"setup\" align=\"right\">
							<a class=\"command\" href=\"" . thisPageURL( false, true, array( "timestamp" => $value["timestamp"],  "subaction" => "delete" ) ) . "\">[" . i18n("Delete") . "]</a>
							<a class=\"command\" href=\"" . thisPageURL( false, true, array( "timestamp" => $value["timestamp"],  "subaction" => $publishit ) ) . "\">[" . i18n($publishit_text) . "]</a>
							<a class=\"command\" href=\"" . thisPageURL( false, true, array( "timestamp" => $value["timestamp"],  "subaction" => "edit" ) ) . "\">[" . i18n("Edit") . "]</a>
							</td>
						</tr>";
				}
				$blog_data .= "</table>";
			} else {
				$blog_data .= "<p>" . i18n("You have not yet made any blog entries. ##0##Click here to add your first entry##1##.", array("<a href=\"" . thisPageURL() . "&amp;subaction=add\">", "</a>")) . "</p>";
			}
		}
	} else if ($splitID[3] == "categories" && (isAllowed("global_user_edit") || ($viewUser != "" && $viewUser == currentUser()))) {
		$tabwidget->addCommand(i18n("New category"), i18n("Add a category"), globalIDtoURL("user/$viewUser/blogger/categories", array("subaction" => "add" ) ) );
		$tabwidget->addCommand(i18n("Options"), i18n("View and edit options for ##0##'s blog", array($viewUser)), globalIDtoURL("user/$viewUser/blogger/options") );
		$tabwidget->addCommand(i18n("Edit blog"), i18n("Edit the blog"), globalIDtoURL("user/$viewUser/blogger/edit") );
		$tabwidget->addCommand(i18n("View blog"), i18n("View the blog"), globalIDtoURL("user/$viewUser/blogger") );
		switch ($_REQUEST["subaction"]) {
		case "add":
			if ($_REQUEST["new_category"] != "") {
				$blogcategories_user[] = str_replace(" ", "_", htmlspecialchars($_REQUEST["new_category"]));
				saveUserInfo($_REQUEST["user_id"], "blogger_usercategories", serialize($blogcategories_user));
				header("Location: " . thisPageURL(true));
			} else {
				$blog_data = "
				<form action=\"" . thisPageURL() . "&amp;subaction=add\" method=\"post\">
				<table class=\"setup\">
				<tr class=\"setup\">
					<td class=\"setup\" width=\"50%\">" . i18n("Category name") . ":
					</td>
					<td class=\"setup\" width=\"50%\"><input style=\"width: 100%;\" type=\"text\" name=\"new_category\" /></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">
					<input type=\"submit\" value=\"" . i18n("Create category") . "\" />
					</td><td class=\"setup\" align=\"right\">
					<input type=\"reset\" value=\"" . i18n("Reset") . "\" />
					</td>
				</tr>
				</table>
				</form>";
			}
			break;
		case "edit":
			if ($_REQUEST["new_category"] != "") {
				$category_name = $blogcategories_user[$_REQUEST['category_key']];
				$blogcategories_user[$_REQUEST['category_key']] = $_REQUEST["new_category"];
				saveUserInfo($_REQUEST["user_id"], "blogger_usercategories", serialize($blogcategories_user));
				$blog_data = "<p>" . i18n("The &quot;##0##&quot; was changed successfully to &quot;##1##&quot;, click here to return...", array($category_name, $_REQUEST["new_category"])) . "</a></p>";
				header("Location: " . thisPageURL(true));
			} else {
				$blog_data = "
				<form action=\"" . thisPageURL() . "&amp;subaction=edit&amp;category_key={$_REQUEST["category_key"]}\" method=\"post\">
				<table class=\"setup\">
				<tr class=\"setup\">
					<td class=\"setup\" width=\"50%\">" . i18n("Old category name") . ":
					</td>
					<td class=\"setup\" width=\"50%\">" . $blogcategories_user[$_REQUEST["category_key"]] . "</td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("New category name") . ":
					<small class=\"comment\">" . i18n("Please take note that the changes made to this category will not affect blog entries already created") . "</small>
					</td>
					<td class=\"setup\"><input style=\"width: 100%;\" type=\"text\" name=\"new_category\" value=\"" . $blogcategories_user[$_REQUEST["category_key"]] . "\" /></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">
					<input type=\"submit\" value=\"" . i18n("Save category") . "\" />
					</td><td class=\"setup\" align=\"right\">
					<input type=\"reset\" value=\"" . i18n("Reset") . "\" />
					</td>
				</tr>
				</table>
				</form>";
			}
			break;
		case "remove":
			if ($_REQUEST["confirm_delete"] == "true") {
				$category_name = $blogcategories_user[$_REQUEST['category_key']];
				unset($blogcategories_user[$_REQUEST['category_key']]);
				saveUserInfo($_REQUEST["user_id"], "blogger_usercategories", serialize($blogcategories_user));
				$blog_data = "<p>" . i18n("The &quot;##0##&quot; was deleted, click here to return...", $category_name) . "</a></p>";
				header("Location: " . thisPageURL(true));
			} else {
				$blog_data = "
				<p>" . i18n("You have requested to remove the category &quot;##0##&quot;. Please confirm this choice below. Deleting the category WILL NOT delete posts already created using the category - the category will only not be available for new entries.", array($blogcategories_user[$_REQUEST["category_key"]])) . "</p>
				<div class=\"wikicenteralign\"><a class=\"command\" href=\"" . thisPageURL() . "&amp;subaction=remove&amp;category_key={$_REQUEST['category_key']}&amp;confirm_delete=true\">[" . i18n("Comfirm delete") . "]</a>
				<a class=\"command\" href=\"" . thisPageURL() . "\">[" . i18n("Abort") . "]</a></div>";
			}
			break;
		default: //list
			$blog_data .= "
			<table class=\"setup\">
				<tr class=\"setup\">
					<th width=\"50%\">" . i18n("Category") . "</th>
					<th width=\"50%\">" . i18n("Actions") . "</th>
				</tr>";
			if (count($blogcategories_user) == 0) {
				$blog_data .= "
					<tr class=\"setup\">
						<td class=\"setup\" colspan=\"2\">" . i18n("There are not yet any user categories defined. ##0##Click here to begin creating one.##1##", array("<a href=\"" . thisPageURL() . "&amp;subaction=add\">", "</a>")) . "</td>
					</tr>
				";
			} else {
				foreach($blogcategories_user as $key => $value) {
					$blog_data .= "
					<tr class=\"setup\">
						<td class=\"setup\">$value</td>
						<td class=\"setup\" align=\"right\">
							<a class=\"command\" href=\"" . thisPageURL() . "&amp;subaction=remove&amp;category_key=$key\">[" . i18n("Delete") . "]</a>
							<a class=\"command\" href=\"" . thisPageURL() . "&amp;subaction=edit&amp;category_key=$key\">[" . i18n("Edit") . "]</a>
						</td>
					</tr>";
				}
			}
			$blog_data .= "
			</table>";
		}
	} else if ($splitID[3] == "options" && (isAllowed("global_user_edit") || ($viewUser != "" && $viewUser == currentUser()))) {
		$tabwidget->addCommand(i18n("Categories"), i18n("View and edit the userdefined categories"), globalIDtoURL("user/$viewUser/blogger/categories") );
		$tabwidget->addCommand(i18n("Edit blog"), i18n("Edit the blog"), globalIDtoURL("user/$viewUser/blogger/edit") );
		$tabwidget->addCommand(i18n("View blog"), i18n("View the blog"), globalIDtoURL("user/$viewUser/blogger") );
		
		foreach($permissionlevels_array as $key => $value) {
			if ($temp_options["viewlevel"] == $key) {
				$default = " selected";
			} else {
				$default = "";
			}
			$bloggeroptions_viewlevel .= "<option value=\"$key\"$default>$value</option>";
		}
		
		foreach($permissionlevels_array as $key => $value) {
			if ($temp_options["commentlevel"] == $key) {
				$default = " selected";
			} else {
				$default = "";
			}
			$bloggeroptions_commentlevel .= "<option value=\"$key\"$default>$value</option>";
		}
		
		if ($temp_options["personal"] == true) {
			$bloggeroptions_personal = " checked";
		} else {
			$bloggeroptions_personal = "";
		}
		foreach($permissionlevels_array as $key => $value) {
			if ($temp_options["allowsviewpersonal"] == $key) {
				$default = " selected";
			} else {
				$default = "";
			}
			$bloggeroptions_allowsviewpersonal .= "<option value=\"$key\"$default>$value</option>";
		}
		
		if ($_REQUEST["blogger_add_category"]) {
			$temp_options["categories"][] = "";
		}
		
		if (is_array($_REQUEST["blogger_remove_category"])) {
			foreach ($_REQUEST["blogger_remove_category"] as $key => $value) {
				unset($temp_options["categories"][$key]);
			}
			saveUserInfo($_REQUEST["user_id"], "blogger_categories", serialize($temp_options["categories"]));
		}
		
		$blog_data = "
		<form action=\"" . thisPageURL() . "\" method=\"post\">
		<table class=\"setup\">
		<tr class=\"setup\">
			<th class=\"setup\" colspan=\"2\">" . i18n("By default new entries have the following settings") . "</th>
		</tr>
		<tr class=\"setup\">
			<td class=\"setup\" width=\"50%\">" . i18n("Viewing allowance level") . "</td>
			<td class=\"setup\" width=\"50%\"><select style=\"width: 100%\" name=\"blogger_viewlevel\">$bloggeroptions_viewlevel</select></td>
		</tr>
		<tr class=\"setup\">
			<td class=\"setup\">" . i18n("Commenting allowance level") . "</td>
			<td class=\"setup\"><select style=\"width: 100%\" name=\"blogger_commentlevel\">$bloggeroptions_commentlevel</select></td>
		</tr>
		<tr class=\"setup\">
			<td class=\"setup\">" . i18n("Personal post") . "</td>
			<td class=\"setup\"><label><input type=\"checkbox\" name=\"blogger_personal\"$bloggeroptions_personal>" . i18n("This is a personal post") . "</label></td>
		</tr>
		<tr class=\"setup\">
			<td class=\"setup\">
			" . i18n("Default categories");
		
		foreach ($temp_options["categories"] as $key => $value) {
			$categories_temp = "<optgroup label=\"" . i18n("Your categories") . "\">";
			foreach($blogcategories_user as $key2 => $value2) {
				if ($value2 == $value) { $default_temp = " selected"; } else { $default_temp = ""; }
				$categories_temp .= "<option value=\"$value2\"$default_temp>" . i18n($value2) . "</option>";
			}
			$categories_temp .= "</optgroup><optgroup label=\"" . i18n("Site-wide categories") . "\">";
			foreach($blogcategories as $key2 => $value2) {
				if ($value2 == $value) { $default_temp = " selected"; } else { $default_temp = ""; }
				$categories_temp .= "<option value=\"$value2\"$default_temp>" . i18n($value2) . "</option>";
			}
			$blog_data .= "<input style=\"float: right;\" type=\"submit\" name=\"blogger_remove_category[$key]\" value=\"-\" /></td>
			<td class=\"setup\"><select style=\"width: 100%\" name=\"blogger_category[$key]\">$categories_temp</optgroup></select></td>
		</tr>
		<tr class=\"setup\">
			<td>";
		}
		
		$blog_data .= "&nbsp;</td>
			<td class=\"setup\"><input type=\"submit\" name=\"blogger_add_category\" value=\"" . i18n("Add category") . "\" /></td>
		</tr>
		<tr class=\"setup\">
			<th class=\"setup\" colspan=\"2\">" . i18n("Allowance levels") . "</th>
		</tr>
		<tr class=\"setup\">
			<td class=\"setup\">" . i18n("View personal posts") . "</td>
			<td class=\"setup\"><select style=\"width: 100%\" name=\"blogger_allowsviewpersonal\">$bloggeroptions_allowsviewpersonal</select></td>
		</tr>
		<tr class=\"setup\">
			<td class=\"setup\"><input type=\"submit\" name=\"blogger_save_settings\" value=\"" . i18n("Save options") . "\" /></td>
			<td class=\"setup wikirightalign\"><input type=\"reset\" value=\"" . i18n("Revert to saved") . "\" /></td>
		</tr>
		</table>
		</form>";
	} else {
		if (isAllowed("global_user_edit") || ($viewUser != "" && $viewUser == currentUser()))
		{
			$tabwidget->addCommand(i18n("New entry"), i18n("Create a new blog entry"), globalIDtoURL("user/$viewUser/blogger/edit", array( "subaction" => "add" ) ) );
			$tabwidget->addCommand(i18n("Edit blog"), i18n("Edit the blog"), globalIDtoURL("user/$viewUser/blogger/edit") );
		}
		$blog_data = parse_blogger("\blog(" . $viewUser . ")");
	}
	return $blog_data;
}

function blogger_savecategories() {
	global $setup_folder, $blogcategories;
	natcasesort($blogcategories); // Sort categories alphabetically...
	$file = "<?PHP\n\$blogcategories = " . var_export($blogcategories, true) . ";\n?>";
	$filename = "$setup_folder/modules/blogger/categories.php";
	if (file_exists($filename)) unlink($filename);
	file_put_contents($filename, $file);
	chmod($filename, 0664);// make sure the file is actually accessible...
}

function modulesetup_blogger() {
	global $setup_folder, $blogcategories, $language, $theModuleSection, $theModuleSubsection, $theModuleAction;
	switch ($theModuleSection) {
	case "category_add":
		if ($_REQUEST["new_category"] != "") {
			$blogcategories[] = $_REQUEST["new_category"];
			blogger_savecategories();
			header("Location: " . globalIDtoURL("setup/modulesetup/blogger") );
		} else {
			$data = "
			<form action=\"" . globalIDtoURL("setup/modulesetup/blogger/category_add") . "\" method=\"post\">
			<table class=\"setup\">
			<tr class=\"setup\">
				<td class=\"setup\" width=\"50%\">" . i18n("Category name") . ":
				</td>
				<td class=\"setup\" width=\"50%\"><input style=\"width: 100%;\" type=\"text\" name=\"new_category\" /></td>
			</tr>
			<tr class=\"setup\">
				<td class=\"setup\">
				<input type=\"submit\" value=\"" . i18n("Create category") . "\" />
				</td><td class=\"setup\" align=\"right\">
				<input type=\"reset\" value=\"" . i18n("Reset") . "\" />
				</td>
			</tr>
			</table>
			</form>";
		}
		break;
	case "category_remove":
		if ($_REQUEST["category_key"] == 0) {
			$data = "<p>" . i18n("You are not allowed to remove the system category as it is integral to the system. In fact there is not even any GUI to let you do this, so the only way to get this message is to attempt writing the URL yourself. Naughty!") . "</p>";
		} else {
			if ($theModuleAction == "confirm") {
				$category_name = $blogcategories[$theModuleSubsection];
				unset($blogcategories[$_REQUEST['category_key']]);
				blogger_savecategories();
				$data = "<p>" . i18n("The &quot;##0##&quot; was deleted, click here to return...", $category_name) . "</a></p>";
				header("Location: " . globalIDtoURL("setup/modulesetup/blogger") );
			} else {
				$data = "
				<p>" . i18n("You have requested to remove the category &quot;##0##&quot;. Please confirm this choice below. Deleting the category WILL NOT delete posts already created using the category - the category will only not be available for new entries.", array($blogcategories[$_REQUEST["category_key"]])) . "</p>
				<div class=\"wikicenteralign\">" .
					drawCommand(i18n("Comfirm delete") ,i18n("Comfirm delete") , globalIDtoURL("setup/modulesetup/blogger/category_remove/$theModuleSubsection/confirm") ) . " " .
					drawCommand(i18n("Abort"), i18n("Abort"), globalIDtoURL("setup/modulesetup/blogger") ) .
				"</div>";
			}
		}
		break;
	case "category_edit":
		if ($_REQUEST["category_key"] == 0) {
			$data = "<p>" . i18n("You are not allowed to edit the system category as it is integral to the system. In fact there is not even any GUI to let you do this, so the only way to get this message is to attempt writing the URL yourself. Naughty!") . "</p>";
		} else {
			if ($_POST["new_category"] != "") {
				$category_name = $blogcategories[$theModuleSubsection];
				$blogcategories[$theModuleSubsection] = $_POST["new_category"];
				blogger_savecategories();
				$data = "<p>" . i18n("The &quot;##0##&quot; was changed successfully to &quot;##1##&quot;, click here to return...", array($category_name, $POST["new_category"])) . "</a></p>";
				header("Location: " . globalIDtoURL("setup/modulesetup/blogger"));
			} else {
				$data = "
				<form action=\"" . globalIDtoURL("setup/modulesetup/blogger/category_edit/$theModuleSubsection") . "\" method=\"post\">
				<table class=\"setup\">
				<tr class=\"setup\">
					<td class=\"setup\" width=\"50%\">" . i18n("Old category name") . ":
					</td>
					<td class=\"setup\" width=\"50%\">" . $blogcategories[$theModuleSubsection] . "</td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">" . i18n("New category name") . ":
					<small class=\"comment\">" . i18n("Please take note that the changes made to this category will not affect blog entries already created") . "</small>
					</td>
					<td class=\"setup\"><input style=\"width: 100%;\" type=\"text\" name=\"new_category\" value=\"" . $blogcategories[$theModuleSubsection] . "\" /></td>
				</tr>
				<tr class=\"setup\">
					<td class=\"setup\">
					<input type=\"submit\" value=\"" . i18n("Save category") . "\" />
					</td><td class=\"setup\" align=\"right\">
					<input type=\"reset\" value=\"" . i18n("Reset") . "\" />
					</td>
				</tr>
				</table>
				</form>";
			}
		}
		break;
	case "help":
		$help_data = "<div style=\"border: 1px dashed grey;\">" . i18n("To create system entries, sign into a site administrator account and add a blog entry to the System category. Only administrators are allowed to use the System category. Use \\blogall(system_category) to show these entries.") . "</div>";
		$help_link = drawCommand( i18n("Hide help") ,i18n("Hide help") , globalIDtoURL("setup/modulesetup/blogger") );
	default:
		if (strlen($help_link) == 0) $help_link = drawCommand(i18n("Show help") ,i18n("Show help") , globalIDtoURL("setup/modulesetup/blogger/help") );
		$data .= "
		<div align=\"right\">
		$help_link
		" . drawCommand( i18n("Add category"), i18n("Add category"), globalIDtoURL("setup/modulesetup/blogger/category_add") ) . "
		</div>
		$help_data
		<table class=\"setup\">
			<tr class=\"setup\">
				<th class=\"setup\" width=\"50%\">" . i18n("Category") . "</th>
				<th class=\"setup\" width=\"50%\">" . i18n("Functions") . "</th>
			</tr>";
		foreach($blogcategories as $key => $value) {
			$data .= "
			<tr class=\"setup\">
				<td class=\"setup\">" . i18n($value) . "</td>
				<td class=\"setup\" align=\"right\">";
			if ($key == 0) {
				$data .= "<small class=\"comment\">" . i18n("Edit and delete not allowed on system category. Used for providing site updates - only available to website admins") . "</small>";
			} else {
				$data .= "
					" . drawCommand( i18n("Delete"), i18n("Delete the category"), globalIDtoURL("setup/modulesetup/blogger/category_remove/$key") ) . "
					" . drawCommand( i18n("Edit"), i18n("Edit the category"), globalIDtoURL("setup/modulesetup/blogger/category_edit/$key") ) . "
					";
			}
			$data .= "
				</td>
			</tr>";
		}
		$data .= "
		</table>";
	}
	return $data;
}

$filename = "$setup_folder/modules/blogger/categories.php";
if (file_exists($filename)) {
	include($filename);
} else {
	$blogcategories = array("system_category");
	blogger_savecategories();
}

/**
 * Return an array of data about the thumbnail for a global ID
 * As the blog entries don't have thumbnails per se, we simply pass a fitting image directly back
 *
 * @param     splitID        The global ID to check
 *
 * @return    Array containing array
 */
function globalIDThumbnail_blogger( $splitID )
{
	global $module_folder;
	$data = array( "uri" => siteURL(true) . "$module_folder/blogger/thumb.png", "width" => 64, "height" => 64 );
	return $data;
}
?>
