<?PHP
/**
 * Gallery module for Travelsized CMS
 *
 * @Author	Dan Jensen <admin@leinir.dk>
 *
 *	The structure of a gallery is as follows below:
 *
 * Folders with...
 * 	 gallery.txt
 * 		containing "title\ndescription"
 * 	images/*.jpg/png/gif
 * 	data/*.language.txt
 * 		containing description
 * 	thumbs/*.jpg/png/gif
 */

// Register gallery human name with description.

#$modules_names[] = "Gallery";
#$mopules_versions[] = "0.2";
#$modules_descriptions[] = "A simple but powerful gallery script";

// Register the gallery into module settings
//$modules_setup[] = "modulesetup_gallery";

// Register the gallery into modules
#$modules[] = "parse_gallery_profile_list";
#$modules[] = "parse_gallery_profile";
#$modules[] = "parse_gallery_image_right";
#$modules[] = "parse_gallery_image";
#$modules[] = "parse_gallery";

// Register the gallery into the profile modules
//$profilemodules[] = "profilemodule_gallery";

// Register the gallery into the globalID backwards lookup
#$modules_globalID[] = "globalID_gallery";

// Module help... {
$page_help["&#92;gallery(txt1,txt2)"] = "This will show the gallery with the physical name txt1, with the style txt2. If you omit txt2 it will default to table. Possible options for txt2 are: table (a three-up view of the gallery), simplelist (a simple thumbnail-less list of the gallery entries) and oldstyle (a verbose table style, with one item per row, and the image description shown)";
$page_help["&#92;galleryimage(txt1,txt2)"] = "This will show the thumbnail for the image txt2 in the gallery txt1, presenting a link to the full image. Note that this does not include a midsize view.";
$page_help["&#92;profilegallerylist(txt1)"] = "This will show a list of profiles where there is at least one image in the user's profile gallery. txt1 is optional, and is the style of listing. Defaults to three-up.";
$page_help["&#92;profilegallery(txt1)"] = "This will show the profile gallery for the user txt1";
// Module help... }


$modulegalleryurl = thisPageURL( false, true, array( "page_id" => "12", "module" => "modulesetup_gallery", "language" => $language ) );
$modulegalleryurlheader = thisPageURL( true, true, array( "page_id" => "12", "module" => "modulesetup_gallery", "language" => $language ) );
$gallerybase = "$setup_folder/modules/gallery/";

if (file_exists($gallerybase . "options.inc")) include($gallerybase . "options.inc");

// All internal gallery function are names modulename_functionname

function gallery_imageweight($file) {
	//thanks to dave at birko dot cjb dot net on the php.net manual site for this bit :)
	$a = array("B", "KB", "MB", "GB", "TB", "PB");
	$pos = 0;
	$size = filesize($file);
	while ($size >= 1024) {
		$size /= 1024;
		$pos++;
	}
	return round($size,2) . " " . $a[$pos];
}

function gallery_build_image($gallerybase, $gallery, $img, $numberofpictures, $thispicturenumber, $style, $arg1, $arg2)
{
	global $language, $pagectl, $page_id, $gallery_option, $viewUser;

	$fullimage = "$gallerybase$gallery/images/$img";
	$thumbimage = "$gallerybase$gallery/thumbs/$img";
	$descimage = "$gallerybase$gallery/data/$img.$language.txt";
	if (file_exists("$gallerybase$gallery/comments/$img.inc")) include ("$gallerybase$gallery/comments/$img.inc");
	$commenthandler = new commentHandler($usercomments, "nouser", 0, 0); // Don't assign any allowance levels - we don't need them here :)
	$numberofcomments = $commenthandler->numberOfComments();
	
	list($x, $y) = getimagesize($fullimage);
	$imgweight = gallery_imageweight($fullimage);
	list($x2, $y2) = getimagesize($thumbimage);
	
	$globalID = "user/$viewUser/gallery/" . str_replace("/galleries/", "/", $gallery ) . "/$img";
	$linkhere = globalIDtoURL($globalID);
	//$linkhere = str_replace(" ", "%20", globalIDtoURL( );

	if ($gallery_options["galleryoption_midsize_enabled"])
	{
		$fullimage = thisPageURL() . "&amp;gallery_midsize=$img";
		if (strpos($gallery, "/galleries/")) $fullimage .= "&amp;gallery_subgallery=" . str_replace("/galleries/", "/", $gallery);
	}
	if (file_exists($descimage)) {$comment = parse_page_data(file_get_contents($descimage));} else {$comment = "<em class=\"comment\">(" . i18n("No description was found") . ")</em>";}

	switch ($style)
	{
	case "oldstyle":
		if ($arg1 == "" || $arg1 == 0) $arg1 = 99999; // Pictures per page (0 == all == default) - unused here!
		$tempdata = "<tr class=\"gallery\"><td colspan=\"3\" class=\"gallery\"><table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" class=\"gallery_image\">
			<tr class=\"gallery\">
			<td class=\"gallery gallery_image_thumbnail\" rowspan=\"3\">
				<div style=\"text-align: center;\"><a href=\"$linkhere\" class=\"gallery_image_thumbnail\"><img src=\"" . siteURL(true) . "$thumbimage\" width=\"$x2\" height=\"$y2\" border=\"0\" class=\"gallery_image_thumbnail\" /></a></div>
			</td>
			<td class=\"gallery gallery_image_title\" colspan=\"2\" valign=\"top\">
				<h6 class=\"gallery gallery_image_title\"><a href=\"$linkhere\" class=\"gallery_image_title\">$img</a></h6>
				<div class=\"gallery gallery_image_title\">$comment</div>
			</td>
			</tr>
			<tr class=\"gallery\">
			<td class=\"gallery gallery_image_comments_left\">
				" . i18n("Comments") . ": " . $numberofcomments . "
			</td>
			<td class=\"gallery gallery_image_comments_right\">";
		if (isAllowed("modules_gallery_gallery_" . $gallery . "_comment") || ($_REQUEST["page_id"] == 13 && userAllowsContact($viewUser)))
		{
			$tempdata .= "
					<div class=\"wikirightalign\"><a href=\"" . thisPageURL() . "&amp;gallery_midsize=$img#comment\" class=\"command\">[" . i18n("Add") . "]</a></div>";
		}
		$tempdata .= "
			</td>
			</tr>
			<tr class=\"gallery\">
			<td class=\"gallery gallery_image_dimensions\" style=\"height: 1.5em;\">
				<div class=\"gallery gallery_image_dimensions\">$x x $y</div>
			</td>
			<td class=\"gallery gallery_image_weight\">
				<div class=\"gallery gallery_image_weight\">$imgweight</div>
			</td>
			</tr></table>\n</td></tr>";
		break;
	case "simplelist":
		if ($arg1 == "" || $arg1 == 0) $arg1 = 99999; // Pictures per page (0 == all == default) - unused here!
		
		$tempdata .= "
			<dl>
			<dt><a href=\"$linkhere\" class=\"gallery_image_tile\" style=\"width: $width%;\"><h6 class=\"gallery gallery_image_title\">$img</h6></a></dt>
				<dd>$x x $y - " . i18n("Comments") . ": " . $numberofcomments . "</dd>
			</dl>
		";
		break;
	case "table":
	default:
		if ($arg1 == "") $arg1 = 3; // Pics per row
		if ($arg2 == "" || $arg2 == 0) $arg2 = 99999; // Rows per page (0 == all == default) - unused here!
		
		$firstinrow = 1;
		for ($i = 0; $i < 99999; $i++) {
			if ((($i * $arg1) + 1) < $thispicturenumber) {
				$firstinrow += $arg1;
			} else {
				break;
			}
		}
		
		if ($firstinrow == $thispicturenumber) { $isfirstinrow = true; } else { $isfirstinrow = false; }
		
		$islastinrow = false;
		if ($thispicturenumber == ($firstinrow + $arg1)) $islastinrow = true;
		
		$width = floor(100 / $arg1);
		if ($islastinrow) $width = $width + (100 - ($arg1 * floor(100 / $arg1)));
		
		if ($numberofpictures == 1) {
			$firstinrow = true;
			$islastinrow = true;
			$width = "100";
		} else if ($numberofpictures == 2) {
			$width = "50";
			if ($thispicturenumber == 2) $islastinrow = true;
		}
		
		
		if ($isfirstinrow) $tempdata = "<tr class=\"gallery\">";
		
		$tempdata .= "
			<td width=\"$width%\" class=\"gallery\">
				<div style=\"text-align: center;\" class=\"gallery_image_tile\">
					<a href=\"$linkhere\" class=\"gallery_image_tile\" style=\"width: $width%;\">
						<img src=\"" . siteURL(true) . "$thumbimage\" width=\"$x2\" height=\"$y2\" border=\"0\" class=\"gallery_image_tile\" />
						<h6 class=\"gallery gallery_image_title\">$img</h6>
					</a>
					" . i18n("Comments") . ": " . $numberofcomments . "<br />
					$x x $y
				</div>
			</td>
		";
		
		if ($islastinrow) $tempdata .= "</tr>";
		
	}
	return $tempdata;
}

// This function takes care of showing the single, full version of an image, with comments when appropriate
function gallery_build_midsize($gallery, $img, $subgallery, $nextimg, $previmg, $profile_gallery = false, $user_id = "") {
	global $language, $gallery_options, $page_id, $modulegalleryurl, $userinfo_folder, $viewUser;
	
	if ( $profile_gallery )
	{
		if($_REQUEST["user_id"] == "")
		{
			$gallerybase = "$userinfo_folder/$user_id/";
			$galleryowner = $user_id;
			if ( $subgallery == "" )
				$linkhere = globalIDtoURL("user/$user_id/gallery/gallery/$img");
			else
				$linkhere = globalIDtoURL("user/$user_id/gallery/$subgallery/$img");
		}
		else
		{
			$gallerybase = "$userinfo_folder/$viewUser/";
			$galleryowner = $viewUser;
			if( $subgallery == "" )
				$linkhere = globalIDtoURL("user/$viewUser/gallery/gallery/$img");
			else
				$linkhere = globalIDtoURL("user/$viewUser/gallery/$subgallery/$img");
		}
		$gallerycancomment = getUserInfo($galleryowner, "profiles_contact");
		if ($gallerycancomment  == i18n("No") . " profiles_contact " . i18n("information")) $gallerycancomment  = $profile_options["profiles_contact"];
		$gallerycanedit = 4;
	} else {
		global $gallerybase;
		$galleryowner = "admin";
		if( isAllowed("modules_gallery_gallery_" . $gallery . "_comment") )
			$gallerycancomment = 0;
		else
			$gallerycancomment = 4;
			
		if( isAllowed("modules_gallery_gallery_{$gallery}_edit") )
			$gallerycanedit = 0;
		else
			$gallerycanedit = 4;
			
		$linkhere = thisPageURL( false, true, array("gallery_subgallery" => $_REQUEST['gallery_subgallery'], "gallery_midsize" => $img) );
	}
	$linkhere = str_replace(" ", "%20", $linkhere);
	
	list($x, $y) = getimagesize("$gallerybase$gallery/images/$img");
	if( $x > $gallery_options["galleryoption_midsize_width"] )
	{
		$resize_factor = $gallery_options["galleryoption_midsize_width"] / $x;
		$x = $gallery_options["galleryoption_midsize_width"];
		$y = $y * $resize_factor;
	}
	if( $y > $gallery_options["galleryoption_midsize_height"] )
	{
		$resize_factor = $gallery_options["galleryoption_midsize_height"] / $y;
		$x = $x * $resize_factor;
		$y = $gallery_options["galleryoption_midsize_height"];
	}
	$descimage = "$gallerybase$gallery/data/$img.$language.txt";
	if (file_exists($descimage))
		$comment = parse_page_data(file_get_contents($descimage));
	else
		$comment = "<em class=\"comment\">(" . i18n("No description was found") . ")</em>";
	$tempdata = "
				<tr class=\"gallery_midsize_header\">
					<th class=\"gallery_midsize_title\">$img <a href=\"$linkhere\">#</a></th>
					<td class=\"gallery_midsize_navigator wikirightalign\">";
	
	if ($previmg != "")
		$tempdata .= "<a href=\"" . thisPageURL() . "&amp;gallery_midsize=$previmg&amp;gallery_subgallery=$subgallery\" class=\"command\">[&lt;]</a>";
	else
		$tempdata .= "[&lt]";
	
	$tempdata .= "<a href=\"" . thisPageURL() . "&amp;gallery_subgallery=$subgallery\">" . i18n("Back") . "</a>";
	
	if ($nextimg != "")
		$tempdata .= "<a href=\"" . thisPageURL() . "&amp;gallery_midsize=$nextimg&amp;gallery_subgallery=$subgallery\" class=\"command\">[&gt;]</a>";
	else
		$tempdata .= "[&gt;]";
	
	$tempdata .= "
					</td>
				</tr>
				<tr class=\"gallery_midsize_image\">
					<td class=\"gallery_midsize_image\" colspan=\"2\">
					<div class=\"wikicenteralign\">
						<a href=\"" . siteURL(true) . "$gallerybase$gallery/images/$img\"><img border=\"0\" src=\"" . siteURL(true) . "$gallerybase$gallery/images/$img\" width=\"$x\" height=\"$y\" alt=\"$img\" /></a>
					</div>
					$comment
					</td>
				</tr>";

	if( file_exists("$gallerybase$gallery/comments/$img.inc") )
		include("$gallerybase$gallery/comments/$img.inc");
	$commenthandler = new commentHandler($usercomments, $galleryowner, $gallerycancomment, $gallerycanedit);
	$commenthandler->thisPageURL = thisPageURL() . "&amp;gallery_subgallery={$_REQUEST['gallery_subgallery']}&amp;gallery_midsize=$img";
	if( userAllows($commenthandler->commentsOwner, $commenthandler->commentLevel) || (!userAllows($commenthandler->commentsOwner, $commenthandler->commentLevel) && $commenthandler->numberOfComments() > 0) ) {
		$tempdata .= "
				<tr class=\"gallery_midsize_comments\">
					<th>" . i18n("Comments") . "</th>
					<td class=\"gallery_midsize_comments wikirightalign\">";

		if( userAllows($commenthandler->commentsOwner, $commenthandler->commentLevel) )
			$tempdata .= "<a href=\"#comment\" class=\"command\">[" . i18n("Add") . "]</a>";
	
		$tempdata .= "</td>
				</tr>
				<tr class=\"gallery_midsize_comments\"><td class=\"gallery_midsize_comments\" colspan=\"2\">
				" . $commenthandler->renderComments();
	
		if( userAllows($commenthandler->commentsOwner, $commenthandler->commentLevel) )
			$tempdata .= $commenthandler->renderCommentsAdder();
	
		$tempdata .= "
				</td></tr>";
		
		if( $commenthandler->commentsChanged )
			file_put_contents("$gallerybase$gallery/comments/$img.inc", array_export($commenthandler->comments, "usercomments"));
	}

	return $tempdata;
}


function gallery_build_gallery($gallery, $style, $arg1, $arg2, $profile_gallery = false, $user_id = "") {
	global $setup_folder, $userinfo_folder, $language, $page_id, $viewUser;
	
	if ($profile_gallery == true) {
		if ($page_id == 13) {
			$modulegalleryurl = thisPageURL( false, true, array( "page_id" => "13", "profile_mode" => "module", "module" => "profilemodule_gallery", "user_id" => $viewUser ) );
		} else {
			global $modulegalleryurl;
			if( isAllowed("modules_gallery_gallery_{$_REQUEST['gallery_name']}_edit") )
				$editlink = "
				<div style=\"float: right;\">
				<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name=" . $_REQUEST["gallery_name"] . "&amp;gallery_subgallery=" . $_REQUEST["gallery_subgallery"] . "\">" . i18n("Edit") . "</a>
				<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name=" . $_REQUEST["gallery_name"] . "&amp;gallery_subgallery=" . $_REQUEST["gallery_subgallery"] . "&amp;gallery_command=picture_upload\">" . i18n("Upload") . "</a>
				</div>";
		}
		if ($_REQUEST["gallery_subgallery"] == "") {
			$gallery = "gallery";
		} else {
			$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_subgallery"]);
		}
		$gallerybase = "$userinfo_folder/$user_id/";
	} else {
		global $modulegalleryurl, $gallerybase;
	}
	
	$gallerystyle = $style;
	if (isAuth() && getUserInfo(currentUser(), "gallery_style") != i18n("No ##0## information\n", array("gallery_style"))) $gallerystyle = getUserInfo(currentUser(), "gallery_style");
	if ($_REQUEST["gallery_style"] != "") {
		$gallerystyle = $_REQUEST["gallery_style"];
		if (isAuth()) saveUserInfo(currentUser(), "gallery_style", $gallerystyle);
	}
	
	switch ($gallerystyle) {
	case "oldstyle":
		$styleoldstyle = " selected";
		break;
	case "simplelist":
		$stylesimplelist = "selected";
		break;
	case "table":
	default:
		$styletable = " selected";
	}
	
	$gallery_data = "<table class=\"gallery\">";
	$picturecount = 0;
	if (!file_exists("$gallerybase$gallery")) {
		$gallery_data .= "<tr class=\"gallery\"><td class=\"gallery\">" . i18n("The ##0## gallery does not exist.", array("&quot;$gallery&quot;")) . "</td></tr>";
		if ($_REQUEST["gallery_subgallery"] != "") {
			$gallery_data .= "<tr class=\"gallery\"><td class=\"gallery\"><a href=\"" . thisPageURL() . "\">" . i18n("Try outside of the subgallery") . "</a></td></tr>";
		}
	} else {
		if (file_exists("$gallerybase$gallery/images")) {
			$dir_handle = opendir("$gallerybase$gallery/images");
			while ($file = readdir($dir_handle)) {
				if( substr($file, 0, 1) != "." ) {
					$pictures[] = $file;
					$picturecount = $picturecount + 1;
				}
			}
			closedir($dir_handle);
		}
		if (is_array($pictures)) sort($pictures);
		
		if ($_REQUEST["gallery_midsize"] != "") {
			foreach($pictures as $key => $value) {
				if ($value == $_REQUEST["gallery_midsize"]) {
					$previmg = $key - 1;
					if ($previmg >= 0) { $previmg = $pictures[$previmg]; } else { $previmg = ""; }
					$nextimg = $key + 1;
					if ($nextimg < count($pictures)) { $nextimg = $pictures[$nextimg]; } else { $nextimg = ""; }
				}
			}
			$gallery_data .= gallery_build_midsize($gallery, $_REQUEST["gallery_midsize"], $_REQUEST["gallery_subgallery"], $nextimg, $previmg, $profile_gallery, $user_id);
		} else {
			// Find subgalleries {
			$parent_pos = 0;
			$parentgallery = str_replace("/galleries/", "/", $gallery);
			if (strpos($gallery, "/galleries/")) {
				while (true) {
					if (strpos($parentgallery, "/", $parent_pos + 1)) {
						$parent_pos = strpos($parentgallery, "/", $parent_pos + 1);
						$subgalleries .= "
						<div class=\"gallery\">$gallerylevel<a href=\"" . siteURL(true) . "user/$viewUser/gallery/" . substr($parentgallery, 0, $parent_pos) . "\">#</a> <a href=\"" . thisPageURL( false, true, array("gallery_subgallery" => substr($parentgallery, 0, $parent_pos) ) ) . "\">" . substr($parentgallery, $oldparent_pos, ($parent_pos - $oldparent_pos)) . "</a></div>";
						$gallerylevel .= "&nbsp;";
						$oldparent_pos = $parent_pos + 1;
					} else {
						break;
					}
				}
			}
			$subgalleries .= "
						<div class=\"gallery\">$gallerylevel<a href=\"" . siteURL(true) . "user/$viewUser/gallery/$parentgallery\">#</a> " . substr($parentgallery, $oldparent_pos) . "</div>";
			$gallerylevel .= "&nbsp;";
			if (file_exists("$gallerybase$gallery/galleries")) {
				$subgalleriesgallery = str_replace("/galleries/", "/", $gallery);
				$fp = opendir("$gallerybase$gallery/galleries/");
				while ($file = readdir($fp)) {
					if( substr($file, 0, 1) != "." ) {
						$subgalleries .= "
						<div class=\"gallery\">$gallerylevel<a href=\"" . siteURL(true) . "user/$viewUser/gallery/$subgalleriesgallery/$file\">#</a> <a href=\"" . thisPageURL( false, true, array("gallery_subgallery" => "$subgalleriesgallery/$file") ) . "\">$file</a></div>";
					}
				}
				closedir($fp);
			} else {
				$subgalleries .= "<div class=\"gallery\">$gallerylevel(" . i18n("no subgalleries") . ")</div>";
			}
			// Find subgalleries }
		
			// Show gallery information {
			if (file_exists("$gallerybase$gallery/gallery.$language.txt")) {
				$contents = file_get_contents("$gallerybase$gallery/gallery.$language.txt");
			} else if (file_exists("$gallerybase$gallery/gallery.en.txt")) {
				$contents = file_get_contents("$gallerybase$gallery/gallery.en.txt");
			} else if (file_exists("$gallerybase$gallery/gallery.txt")) {
				$contents = file_get_contents("$gallerybase$gallery/gallery.txt");
			} else {
				$contents = "Untitled gallery\nNo description";
			}
			$galleryinfobegpos = strpos($contents, "\n");
			$gallery_title = substr($contents, 0, $galleryinfobegpos);
			$gallery_description = substr($contents, $galleryinfobegpos);
			$gallery_data .= "
			<tr class=\"gallery\">
				<th class=\"gallery\">$gallery_title</th>
				<th style=\"width: 33%\" class=\"gallery\">" . i18n("Subgalleries") . "</th>
			</tr>
			<tr class=\"gallery\">
				<td class=\"gallery_description\"><div>" . parse_page_data($gallery_description) . "</div></td>
				<td class=\"gallery_subgalleries\">$subgalleries</td>
			</tr>
			<tr class=\"gallery\"><td class=\"gallery_list\" colspan=\"2\">$editlink<table class=\"gallery_list\">
			";
			// Show gallery information }

			if (is_array($pictures)) {
				$numberofpictures = count($pictures);
				foreach($pictures as $key => $value) {
					$gallery_data .= gallery_build_image($gallerybase, $gallery, $value, $numberofpictures, ($key + 1), $gallerystyle, $arg1, $arg2);
				}
			} else {
				$gallery_data .= "<tr class=\"gallery\"><td class=\"gallery\">
				" . renderInformationBox(
						i18n("No images"),
						i18n("There are no images in this gallery, however there are subgalleries. Please go into one of them by clicking on their name in the list above.")
					) . "
				</td></tr>";
			}
			
			$gallery_data .="</table>
			</td></td>
			<tr class=\"gallery\"><td class=\"gallery\"><form action=\"" . thispageurl() . "\" method=\"POST\" class=\"gallery_settings_form\">
			<div class=\"gallery_settings\">
			" . i18n("Gallery view style") . ": 
			<select name=\"gallery_style\" class=\"gallery_style_select\">
			<option value=\"table\"$styletable>" . i18n("Table") . "</option>
			<option value=\"simplelist\"$stylesimplelist>" . i18n("Simple list") . "</option>
			<option value=\"oldstyle\"$styleoldstyle>" . i18n("Verbose table (old style)") . "</option>
			</select>
			$arg1select
			$arg2select
			<input type=\"submit\" name=\"gallery_settings\" class=\"gallery_settings_apply\" value=\"" . i18n("Apply") . "\" />
			</div></form>
			</td>
			
			";
			
			if ($picturecount == 0) {
				$gallery_data .= "
			<td class=\"gallery\"><div>" . i18n("This gallery is empty") . "<div></td></tr>";
			} else if ($picturecount == 1) {
				$gallery_data .= "
			<td class=\"gallery\"><div>" . i18n("There is 1 picture in this gallery") . "<div></td></tr>";
			} else {
				$gallery_data .= "
			<td class=\"gallery\"><div>" . i18n("There are ##0## pictures in this gallery", array($picturecount)) . "</div></td></tr>";
			}
		}
	}
	$gallery_data .= "</table>";
	return $gallery_data;
}

/**
 * Returns the url that corresponds to the globalID corresponding to a gallery item if appropriate
 *
 * @param     $splitID       The global ID to check
 *
 * @return    string         A string containing the URL corresponding to the globalID, if existing
 */
function globalID_gallery( $splitID )
{
	$pageID = null;
	
	// Is the globalID directly aimed at this module?
	// This doesn't exist... yet :P To be implemented once the gallery module is less... broken
	/*if( $splitID[0] == "gallery" )
	{
		$extraReqVars["page_id"] = 0;
		$pageID = thisPageURL( true, true, $extraReqVars );
	}
	else*/
	// Or is it aimed at the module's profile module?
	if( $splitID[0] == "user" && $splitID[2] == "gallery" )
	{
		$_REQUEST["module"] = "profilemodule_gallery";
		if( strpos( end( $splitID ), "." ) === false )
		{
			$_REQUEST["gallery_subgallery"] = array_slice( $splitID, 3, count( $splitID ) - 3 );
			$_REQUEST["gallery_subgallery"] = implode( "/", $_REQUEST["gallery_subgallery"] );
		}
		else
		{
			$_REQUEST["gallery_subgallery"] = array_slice( $splitID, 3, count( $splitID ) - 4 );
			$_REQUEST["gallery_subgallery"] = implode( "/", $_REQUEST["gallery_subgallery"] );
			$_REQUEST["gallery_midsize"] = end( $splitID );
		}
		$pageID = 13;
	}
	
	return $pageID;
}

// The main function of a module is parse_modulename($page_data)

function parse_gallery($page_data) {
	global $modulegalleryurl, $setup_folder, $language, $gallerybase, $page_id;
	
	while (true) {
		$begpos = strpos($page_data, "\gallery("); //where does the first gallery begin
		if ($begpos === false) break; //all galleries have been created, no need to check more
		$endpos = strpos($page_data, ")", $begpos + 8); //gallery end

		$thegallery = substr( $page_data, $begpos + 9, ($endpos - $begpos) - 9);
		
		if (strpos($thegallery, ",") === false) {
			$tempgallery = $thegallery;
		} else {
			list($tempgallery, $style) = explode(",", $thegallery);
		}
		
		if ($_REQUEST["gallery_subgallery"] != "") $gallery = $_REQUEST["gallery_subgallery"];
		if (strpos($gallery, "/")) {
			$gallery = str_replace("/", "/galleries/", $gallery);
		} else {
			$gallery = $tempgallery;
		}

		$page_data = str_replace("\gallery($thegallery)", gallery_build_gallery($gallery, $style, $arg1, $arg2), $page_data);

		// loop control {
		$loopcontrol++;
		if ($loopcontrol > 1000) {
			return "<p>Loop control has found more than 1000 loops through this page. This probably means an error occured while parsing the \galleryimage tags, which probably means you've forgotten to end a tag somewhere. Please review the output below:</p><hr />" . $page_data;
		}
		// loop control }
	}
	return $page_data;
}


// The function that parses for \galleryimageright(Gallery name, filename.ext)
function parse_gallery_image_right($page_data) {
	global $setup_folder, $gallerybase;
	
	while (true) {
		$begpos = strpos($page_data, "\galleryimageright("); //where does the first galleryimage begin
		if ($begpos === false) break; //all galleryimages have been created, no need to check more
		$midpos = strpos($page_data, ",", $begpos + 18); //find where the two arguments are split
		$endpos = strpos($page_data, ")", $begpos + 18); //galleryimage end

		if ($endpos < $midpos) { // This means that the \galleryimage() was not written properly (i.e. no gallery or no picture defined)
			return "<p>" . i18n("An error occured during the parsing of \galleryimage tags. This means that you missed something when defining one or more of them. The tag with the error is:") . substr($page_data, $begpos, ($endpos - $begpos) + 1) . "</p><hr />" . $page_data;
}
		
		$gallery = trim(substr($page_data, $begpos + 19, ($midpos - $begpos) - 19));
		if (strpos($gallery, "/")) $gallery = str_replace($gallery, "/", "/galleries/");
		$picture = trim(substr($page_data, $midpos + 1, ($endpos - $midpos) - 1));
		list($thumbx, $thumby) = getimagesize("$setup_folder/modules/gallery/$gallery/thumbs/$picture");
		$galleryimage_data = "\n<a href=\"$setup_folder/modules/gallery/$gallery/images/$picture\" target=\"_blank\"><img src=\"$setup_folder/modules/gallery/$gallery/thumbs/$picture\" alt=\"" . i18n("##0## in ##1##", array($picture, $gallery)) . "\" width=\"$thumbx\" height=\"$thumby\" border=\"0\" style=\"float: right;\" /></a>";
		$page_data = str_replace(substr($page_data, $begpos, ($endpos - $begpos) + 1), $galleryimage_data, $page_data);

		// loop control {
		$loopcontrol++;
		if ($loopcontrol > 1000) {
			return "<p>Loop control has found more than 1000 loops through this page. This probably means an error occured while parsing the \galleryimage tags, which probably means you've forgotten to end a tag somewhere. Please review the output below:</p><hr />" . $page_data;
}
		// loop control }
}
	return $page_data;
}


// The function that parses for \galleryimage(Gallery name, filename.ext)
function parse_gallery_image($page_data) {
	global $setup_folder, $gallerybase;
	
	while (true) {
		$begpos = strpos($page_data, "\galleryimage("); //where does the first galleryimage begin
		if ($begpos === false) break; //all galleryimages have been created, no need to check more
		$midpos = strpos($page_data, ",", $begpos + 13); //find where the two arguments are split
		$endpos = strpos($page_data, ")", $begpos + 13); //galleryimage end

		if ($endpos < $midpos) { // This means that the \galleryimage() was not written properly (i.e. no gallery or no picture defined)
			return "<p>" . i18n("An error occured during the parsing of \galleryimage tags. This means that you missed something when defining one or more of them. The tag with the error is:") . substr($page_data, $begpos, ($endpos - $begpos) + 1) . "</p><hr />" . $page_data;
		}
		
		$gallery = trim(substr($page_data, $begpos + 14, ($midpos - $begpos) - 14));
		if (strpos($gallery, "/")) $gallery = str_replace($gallery, "/", "/galleries/");
		$picture = trim(substr($page_data, $midpos + 1, ($endpos - $midpos) - 1));
		list($thumbx, $thumby) = getimagesize("$setup_folder/modules/gallery/$gallery/thumbs/$picture");
		$galleryimage_data = "\n<a href=\"$setup_folder/modules/gallery/$gallery/images/$picture\" target=\"_blank\"><img src=\"$setup_folder/modules/gallery/$gallery/thumbs/$picture\" width=\"$thumbx\" height=\"$thumby\" border=\"0\"></a>";
		$page_data = str_replace(substr($page_data, $begpos, ($endpos - $begpos) + 1), $galleryimage_data, $page_data);

		// loop control {
		$loopcontrol++;
		if ($loopcontrol > 1000) {
			return "<p>Loop control has found more than 1000 loops through this page. This probably means an error occured while parsing the \galleryimage tags, which probably means you've forgotten to end a tag somewhere. Please review the output below:</p><hr />" . $page_data;
		}
		// loop control }
	}
	return $page_data;
}

function gallery_countimages($folder, $countfrom = 0) {
	$files = array();
	$count = $countfrom;
	
	if ($dir_open = @ opendir("$folder/images")) {
		while (($dir_content = readdir($dir_open)) !== false) {
			if ( substr($dir_content, 0, 1) != "." ) $count++;
		}
		closedir($dir_open);
	}
	
	if ($dir_open = @ opendir("$folder/galleries")) {
		while (($dir_content = readdir($dir_open)) !== false)
			if( substr($dir_content, 0, 1) != "." )
				$count = gallery_countimages("$folder/galleries/$dir_content", $count);
		
		closedir($dir_open);
	}
	
	return $count;
}

function parse_gallery_profile_list($page_data) {
	global $userinfo_folder;
	while (true) {
		$begpos = strpos($page_data, "\profilegallerylist("); //where does the first gallery begin
		if ($begpos === false) break; //all galleries have been created, no need to check more
		$endpos = strpos($page_data, ")", $begpos + 19); //gallery end

		$viewstyle = substr( $page_data, $begpos + 20, ($endpos - $begpos) - 20);
		
		$userlist = allUsersArray();
		foreach ($userlist as $key => $value) {
			if (is_dir("$userinfo_folder/$value/gallery")) {
				$count = gallery_countimages("$userinfo_folder/$value/gallery");
				if ($count > 0) {
					$userswithgalleries[] = $value;
					$gallerycounts[] = $count;
				}
			}
		}
		
		$data = parse_userlists("\userlist($viewstyle)", false, $userswithgalleries);
		
		foreach($userswithgalleries as $key => $value) {
			$data = str_replace("($value)", "($value) " . $gallerycounts[$key], $data);
		}
		
		$page_data = str_replace("\profilegallerylist($viewstyle)", $data, $page_data);
		// loop control {
		$loopcontrol++;
		if ($loopcontrol > 1000) {
			return "<p>Loop control has found more than 1000 loops through this page. This probably means an error occured while parsing the \profilegallery tags, which probably means you've forgotten to end a tag somewhere. Please review the output below:</p><hr />" . $page_data;
		}
		// loop control }
	}
	return $page_data;
}

function parse_gallery_profile($page_data) {
	while (true) {
		$begpos = strpos($page_data, "\profilegallery("); //where does the first gallery begin
		if ($begpos === false) break; //all galleries have been created, no need to check more
		$endpos = strpos($page_data, ")", $begpos + 15); //gallery end

		$thegallery = substr( $page_data, $begpos + 16, ($endpos - $begpos) - 16);
		if ($_REQUEST["gallery_subgallery"] != "") $gallery = $_REQUEST["gallery_subgallery"];
		if (strpos($gallery, "/")) {
			$gallery = str_replace("/", "/galleries/", $gallery);
		} else {
			$gallery = $thegallery;
		}
		$page_data = str_replace("\profilegallery($thegallery)", gallery_build_gallery($gallery, $style, $arg1, $arg2, true, "$thegallery/"), $page_data);
		// loop control {
		$loopcontrol++;
		if ($loopcontrol > 1000) {
			return "<p>Loop control has found more than 1000 loops through this page. This probably means an error occured while parsing the \profilegallery tags, which probably means you've forgotten to end a tag somewhere. Please review the output below:</p><hr />" . $page_data;
		}
		// loop control }
	}
	return $page_data;
}


/**
* Function for displaying all galleries and subgalleries in a structure, adding a link to them
* Edited from an original by de kibo niels in the php manual on php.net
*/
function gallery_displaydir ($path, $depth, $linkurl, $parentpath) {
	if (substr($path, -1) == "/") $path = substr($path, 0, strlen($path) - 1);
	if ($depth == 0 && substr($path, 0, 10) == "galleries/") $path = substr($path, 10);
	
	if ($_REQUEST["page_id"] == 13 && $depth == 0) {
		$dir_data = "<a href=\"" . substr($linkurl, 0 , strlen($linkurl) - 1) . "//\">{$_REQUEST["user_id"]}</a><br />";
		$depth++; $depth++;
	}
	
	if (($d = @opendir($path)) === false) {
		$dir_data = "(" . i18n("Could not open path") . ": $path)";
	} else {
		while ($f = readdir($d)) {
			if ( substr($f, 0, 1) != "." && $f != "comments" && $f != "data" && $f != "images" && $f != "thumbs") {
				if (is_dir($path . "/" . $f)) {
					for ($i = 0; $i < $depth - 1; $i++) $dir_data .= "&nbsp;";
					if ($f != "galleries") {
						if (strpos($parentpath, "/") == 1) $parentpath = strstr($parentpath, 1);
						$dir_data .= "<a href=\"$linkurl" . str_replace("galleries/", "", str_replace("/galleries", "", str_replace("/galleries/", "/", $parentpath)));
						$dir_data .= "$f\">$f</a><br />";
					}
					$depth++;
					$dir_data .= gallery_displaydir($path . "/" . $f, $depth, $linkurl, "$parentpath$f/");
					$depth--;
				}
			}
		}
		closedir ($d);
	}
	return $dir_data;
}

function profilemodule_gallery() {
	global $gallery_options, $tabwidget, $setup_folder, $userinfo_folder, $language, $languages, $viewUser;
	$gallerybase = "$userinfo_folder/$viewUser/gallery/";
	if (!file_exists($gallerybase)) {
		mkdir($gallerybase, 0777);
		mkdir("$gallerybase/comments", 0777);
		mkdir("$gallerybase/data", 0777);
		mkdir("$gallerybase/galleries", 0777);
		mkdir("$gallerybase/images", 0777);
		mkdir("$gallerybase/thumbs", 0777);
	}
	
	$modulegalleryurl = thisPageURL( false, true, array( "page_id" => "13", "user_id" => $viewUser, "profile_mode" => "module", "module" => "profilemodule_gallery", "action" => "edit", "language" => $language ) );
	$modulegalleryurlheader = thisPageURL( true, true, array( "page_id" => "13", "user_id" => $viewUser, "profile_mode" => "module", "module" => "profilemodule_gallery", "action" => "edit", "language" => $language ) );
	
	if( $_REQUEST["action"] == "edit" )
	{
		if( $_REQUEST["gallery_subgallery"] == "" )
			$modulegallerylinksubgallery = "&amp;gallery_subgallery=gallery" . $_REQUEST["gallery_name"];
		else
			$modulegallerylinksubgallery = "&amp;gallery_subgallery=" . $_REQUEST["gallery_subgallery"];
		$modulegallerylinkgallery = "";
	}
	else
	{
		$modulegallerylinksubgallery = "";
		if( $_REQUEST["gallery_name"] == "" )
			$modulegallerylinkgallery = "&amp;gallery_name=" . substr( $_REQUEST["gallery_subgallery"], 7);
		else
			$modulegallerylinkgallery = "&amp;gallery_name=" . $_REQUEST["gallery_name"];
	}
		
	if ($_REQUEST["action"] == "edit") {
		if (isAllowed("global_user_edit") || ($_REQUEST["user_id"] != "" && $_REQUEST["user_id"] == currentUser())) {
			$tabwidget->addCommand(i18n("View gallery"), i18n("View the gallery"), siteURL() . "?page_id=13&amp;user_id=$viewUser&amp;profile_mode=module&amp;module=profilemodule_gallery$modulegallerylinkgallery$modulegallerylinksubgallery");
			if ($_REQUEST["gallery_command"] == "resizethumb") {
				$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
				$original = "$gallerybase$gallery/images/" . stripslashes($_REQUEST["picture_name"]);
				$thumbnail = "$gallerybase$gallery/thumbs/" . stripslashes($_REQUEST["picture_name"]);
				
				if( file_exists( $thumbnail ) )
					unlink( $thumbnail );
				
				scaleImage( $original, $thumbnail, 120, 120 );
				
				header("Location: $modulegalleryurlheader&gallery_name=" . $_REQUEST["gallery_name"]);
			} else if ($_REQUEST["gallery_command"] == "cropnresize") {
				$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
				$picture = stripslashes($_REQUEST["picture_name"]);
				if (file_exists("$gallerybase/$gallery/images/$picture")) {
					$image = "$gallerybase/$gallery/images/$picture"; // the image to crop
					$dest_image = "$gallerybase/$gallery/thumbs/$picture";
					$ext = checkImageMimetype( $image );
				}
				
				/**
				 * Cropping method adopted from lixlpixel PHParadise
				 *
				 * category :		image editing
				 *
				 * snippet :		crop image
				 *
				 * downloaded :	11.02.2005 - 16:27
				 *
				 * file URL :		http://www.fundisom.com/phparadise/php/image_editing/crop_image
				 */
				
				$url = "$modulegalleryurl&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name={$_REQUEST['picture_name']}&amp;theme=empty&amp;gallery_command=cropnresize"; // the url to this wizard
				$margin = 10; // to keep the image and layer in sync
				
				if(!isset($_POST['step'])) $title = '1';
				else $title = $_POST['step'];
				
				$data = '
				<html>
				<head>
					<title>' . i18n("Cropping gallery image - step") . ' ' . $title . '</title>
				</head>
				<body style="margin: '.$margin.'px;">';
				
				if(!isset($_POST['tx']) && !isset($_POST['fx']))
				{
					$data .= '
					<form method="post" action="'.$url.'">
						<input type="image" src="'.$image.'"><p>';
					if(!isset($_POST['x']))
					{
						$data .= '
						<input type="hidden" name="step" value="2">
						' . i18n("Click to mark the first corner of the selection rectangle.") . " " . i18n("You will be able to review the result before cropping");
					}else{
						$data .= '
						<input type="hidden" name="step" value="3">
						<input type="hidden" name="fx" value="'.$_POST['x'].'">
						<input type="hidden" name="fy" value="'.$_POST['y'].'">
						' . i18n("Click to mark the second corner of the selection rectangle.") . ' | <a href="'.$url.'">' . i18n("Start over") . '</a> ' . " " . i18n("You will be able to review the result before cropping");
					}
					$data .= '
					</form>
					';
				}
				
				if(isset($_POST['fx']))
				{
					$data .= '
					<form method="post" action="'.$url.'">
						<input type="hidden" name="step" value="4">
						<input type="image" src="'.$image.'">
						<input type="hidden" name="tx" value="'.$_POST['fx'].'">
						<input type="hidden" name="ty" value="'.$_POST['fy'].'">
						<input type="hidden" name="width" value="'.($_POST['x']-$_POST['fx']).'">
						<input type="hidden" name="height" value="'.($_POST['y']-$_POST['fy']).'"><p>
						<div style="position: absolute;
									left:'.($_POST['fx']+$margin).'px;
									top: '.($_POST['fy']+$margin).'px;
									width: '.($_POST['x']-$_POST['fx']).'px;
									height: '.($_POST['y']-$_POST['fy']).'px;
									border: 1px solid #fff;">
						</div>
						' . i18n("Click the image to crop. The rectangle shown on the picture is the rectangle that will be cropped.") . '| <a href="'.$url.'">' . i18n("Start over") . '</a>
					</form>';
				}
				
				if(isset($_POST['tx']))
				{
					// Crop the image
					cropImage( $image, $dest_image, $_POST["tx"], $_POST["ty"], $_POST["width"], $_POST["height"] );
					
					// Resize the new thumbnail to the max thumbnail size
					scaleImage( $dest_image, $dest_image, 120, 120 );
					
					header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
				}
			} else if ($_REQUEST["gallery_command"] == "picture_upload") {
				$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
				if (!is_array($_FILES['uploadedfile'])) {
					$data = "
					<form enctype=\"multipart/form-data\" action=\"$modulegalleryurl&amp;gallery_command=picture_upload&amp;gallery_name={$_REQUEST['gallery_name']}\" method=\"post\">
					<table class=\"setup\">
					<tr class=\"setup\">
						<td class=\"setup\" width=\"25%\"><div><label for=\"uploadedfile[]\" accesskey=\"1\">" . i18n("Select picture for upload") . "</label></div></td>
						<td class=\"setup\" width=\"75%\"><div><input style=\"width: 100%\" id=\"uploadedfile1\" name=\"uploadedfile[]\" type=\"file\" /></div></td>
					</tr>
					<tr class=\"setup\">
						<td class=\"setup\"><div><label for=\"edit\" accesskey=\"c\">" . i18n("Comment") . "</label></div></td>
						<td class=\"setup\"><div><textarea rows=\"10\" cols=\"70\" style=\"width: 100%;\" id=\"edit\" name=\"picture_comment\">{$_REQUEST['picture_comment']}</textarea></div></td>
					</tr>
					</table>
					<div align=\"center\"><input accesskey=\"s\" type=\"submit\" value=\"" . i18n("Upload") . "\" /></div>
					<table class=\"setup\">
					<tr class=\"setup\">
						<td class=\"setup\" width=\"25%\"><div><label for=\"uploadedfile[]\" accesskey=\"2\">" . i18n("Additional pictures") . "</label></div><small class=\"comment\">" . i18n("Select up to 9 additional pictures for upload. These pictures will be given the same comment as the first picture. If you wish to assign a different comment to one picture, do this afterwards.") . "</small></td>
						<td class=\"setup\" width=\"75%\">
							<div><input style=\"width: 100%\" id=\"uploadedfile2\" name=\"uploadedfile[]\" type=\"file\" /></div>
							<div><input style=\"width: 100%\" id=\"uploadedfile3\" name=\"uploadedfile[]\" type=\"file\" /></div>
							<div><input style=\"width: 100%\" id=\"uploadedfile4\" name=\"uploadedfile[]\" type=\"file\" /></div>
							<div><input style=\"width: 100%\" id=\"uploadedfile5\" name=\"uploadedfile[]\" type=\"file\" /></div>
							<div><input style=\"width: 100%\" id=\"uploadedfile6\" name=\"uploadedfile[]\" type=\"file\" /></div>
							<div><input style=\"width: 100%\" id=\"uploadedfile7\" name=\"uploadedfile[]\" type=\"file\" /></div>
							<div><input style=\"width: 100%\" id=\"uploadedfile8\" name=\"uploadedfile[]\" type=\"file\" /></div>
							<div><input style=\"width: 100%\" id=\"uploadedfile9\" name=\"uploadedfile[]\" type=\"file\" /></div>
							<div><input style=\"width: 100%\" id=\"uploadedfile0]\" name=\"uploadedfile[]\" type=\"file\" /></div></td>
					</tr>
					</table>
					</form>
					" . generate_pagehelplink();
				} else {
					$errorHandler = new errorHandler( "$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}" );
					foreach ($_FILES["uploadedfile"]["name"] as $thekey => $thevalue) {
						$filename = str_replace("&", "_", str_replace("'", "", stripslashes($_FILES['uploadedfile']['name'][$thekey])));
						$uploadFile = "$gallerybase$gallery/images/$filename";
						// checkImageMimetype below makes sure that it's actually a supported image that got uploaded
						if( checkImageMimetype( $_FILES['uploadedfile']['tmp_name'][$thekey] ) != false && move_uploaded_file( $_FILES['uploadedfile']['tmp_name'][$thekey], $uploadFile ) )
						{
							chmod($uploadFile, 0664);
							
							if ($x > $gallery_options["galleryoption_resize_width"] && is_array($gallery_options)) {
								$resize_factor = $gallery_options["galleryoption_resize_width"] / $x;
								$resized_x = $gallery_options["galleryoption_resize_width"];
								$resized_y = $y * $resize_factor;
								$errorHandler->handle( i18n( "Resizing ##0##", array( $filename ) ), scaleImage( $uploadFile, $uploadFile, $resized_x, $resized_y ) );
							} else if ($y > $gallery_options["galleryoption_resize_height"] && is_array($gallery_options)) {
								$resize_factor = $gallery_options["galleryoption_resize_height"] / $y;
								$resized_x = $x * $resize_factor;
								$resized_y = $gallery_options["galleryoption_resize_height"];
								$errorHandler->handle( i18n( "Resizing ##0##", array( $filename ) ), scaleImage( $uploadFile, $uploadFile, $resized_x, $resized_y ) );
							}
							
							$errorHandler->handle( i18n("Thumbnail ##0##", array( $filename) ) , scaleImage( $uploadFile, "$gallerybase$gallery/thumbs/$filename", 120, 120 ) );
							
							file_put_contents("$gallerybase$gallery/comments/$filename.inc", "<?PHP\n\$usercomments=array();\n?>");
							
							foreach($languages as $key => $value) {
								file_put_contents("$gallerybase$gallery/data/$filename.$value.txt", stripslashes($_REQUEST["picture_comment"]));
							}
						}
						else
						{
							switch ($_FILES['uploadedfile']['error'][$thekey]) {
							case 0:
								if( checkImageMimetype( $_FILES['uploadedfile']['tmp_name'][$thekey] ) == false )
									$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("Image type not supported or not an image file! Only jpg, gif and png are supported image types") );
								else
									$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  false );
								break;
							case 1:
								$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("The uploaded file exceeds the upload_max_filesize directive in php.ini. Please contact your host and rectify this (or better yet, select a smaller file. Most often, this directive states 2MB as the maximum upload size, however as we have 1MB as the maxium here, they have obviously changed it).") );
								break;
							case 2:
								$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("The uploaded file exceeds 1MB in size, and is too large to upload in this manner.") );
								break;
							case 3:
								$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("The uploaded file was only partially uploaded.") );
								break;
							case 4:
								// This is not an error, it's what happens when all ten file selectors are filled
								break;
							default:
								if( checkImageMimetype( $_FILES['uploadedfile']['tmp_name'][$thekey] ) == false )
									$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("Image type not supported or not an image file! Only jpg, gif and png are supported image types") );
								else
									$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  false );
							}
						}
					}
					
					$data .= $errorHandler->render();
					
					if( !$errorHandler->handled )
						header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
				}
			} else if ($_REQUEST["gallery_command"] == "picture_rename") {
				$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
				if ($_REQUEST["renameto"] == "") {
					$data = "
					<form action=\"$modulegalleryurl&amp;gallery_command=picture_rename&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=" . stripslashes($_REQUEST['picture_name']) . "\" method=\"post\">
					<table class=\"setup\">
					<tr class=\"setup\">
					<td class=\"setup\"><div><label for=\"renameto\" accesskey=\"r\">" . i18n("Rename the picture to:") . "</label></div></td>
					<td class=\"setup\"><div><input style=\"width: 100%\" type=\"text\" id=\"renameto\" name=\"renameto\" value=\"" . stripslashes($_REQUEST['picture_name']) . "\" /></div></td>
					</tr>
					</table>
					<div align=\"center\"><input accesskey=\"s\" type=\"submit\" value=\"Submit\" /></div>
					</form>
					";
				} else {
					$renameto = str_replace("'", "", stripslashes($_REQUEST['renameto']));
					$picture_name = stripslashes($_REQUEST['picture_name']);
					if (file_exists("$gallerybase$gallery/images/$renameto")) { // make sure picture doesn't already exists
						if ($_REQUEST['overwrite'] != "true") {
							$data = "<p>" . i18n("You are attempting to rename a picture, but a picture with this name already exists. Do you wish to replace the old with the new, or do you wish to abort the move?") . "
							<div align=\"center\">
							<a href=\"$modulegalleryurl&amp;gallery_command=picture_rename&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$picture_name&amp;moveto=$renameto&amp;overwrite=true\">[" . i18n("Overwrite destination") . "]</a>
							<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">[" . i18n("Abort") . "]</a>
							</div>
							";
						} else {
							if (unlink_multiple("$gallerybase$gallery/data/$renameto*") and unlink("$gallerybase$gallery/images/$renameto") and unlink("$gallerybase$gallery/thumbs/$renameto")) {
								$renamesuccess = true;
								if (file_exists("$gallerybase$gallery/comments/$renameto.inc")) {
									if (!unlink("$gallerybase$gallery/comments/$renameto.inc")) {
										$renamesuccess = false;
									}
								}
								foreach ($languages as $key => $value) {
									if( file_exists("$gallerybase$gallery/data/$picture_name.$value.txt") )
										if (!rename("$gallerybase$gallery/data/$picture_name.$value.txt", "$gallerybase$gallery/data/$renameto.$value.txt"))
											$renamesuccess = false;
								}
								if (	$renamesuccess and
									file_exists( "$gallerybase$gallery/images/$picture_name" ) and
									rename("$gallerybase$gallery/images/$picture_name", "$gallerybase$gallery/images/$renameto") and
									file_exists( "$gallerybase$gallery/thumbs/$picture_name" ) and
									rename("$gallerybase$gallery/thumbs/$picture_name", "$gallerybase$gallery/thumbs/$renameto")
									) {
									if (file_exists("$gallerybase$gallery/comments/$picture_name.inc")) {
										if (rename("$gallerybase$gallery/comments/$picture_name.inc", "$gallerybase$gallery/comments/$renameto.inc")) {
											$data = "<p>" . i18n("The picture was successfully renamed.") . " <a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">" . i18n("Click here to go back") . "</a>.</p>";
											header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
										} else {
											$data = "<p>" . i18n("An error occured during the attempt to move the picture and associated data. This probably means that you do not have write permissions to either of the galleries and the data folders. Please rectify this and try again.") . "</p>";
										}
									} else {
										$data = "<p>" . i18n("The picture was successfully renamed.") . " <a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">" . i18n("Click here to go back") . "</a>.</p>";
										header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
									}
								} else {
									$data = "<p>" . i18n("An error occured during the attempt to move the picture and associated data. This probably means that you do not have write permissions to either of the galleries and the data folders. Please rectify this and try again.") . "</p>";
								}
							} else {
								$data = "<p>" . i18n("An error occured during the deletion of the existing picture. This is probably because you do not have writing permissions in the target gallery folders or to any one of the target gallery picture files. Please rectify this and try again.") . "</p>";
							}
						}
					} else {
						$renamesuccess = true;
						foreach ($languages as $key => $value) {
							if( file_exists("$gallerybase$gallery/data/$picture_name.$value.txt") )
								if (!rename("$gallerybase$gallery/data/$picture_name.$value.txt", "$gallerybase$gallery/data/$renameto.$value.txt"))
									$renamesuccess = false;
						}
						if (	$renamesuccess and
							file_exists( "$gallerybase$gallery/images/$picture_name" ) and
							rename("$gallerybase$gallery/images/$picture_name", "$gallerybase$gallery/images/$renameto") and
							file_exists( "$gallerybase$gallery/thumbs/$picture_name" ) and
							rename("$gallerybase$gallery/thumbs/$picture_name", "$gallerybase$gallery/thumbs/$renameto") and
							file_exists( "$gallerybase$gallery/comments/$picture_name.inc" ) and
							rename("$gallerybase$gallery/comments/$picture_name.inc", "$gallerybase$gallery/comments/$renameto.inc")
							) {
							$data = "<p>" . i18n("The picture was successfully renamed.") . " <a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">" . i18n("Click here to go back") . "</a>.</p>";
							header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
						} else {
							$data = "<p>" . i18n("An error occured during the attempt to move the picture and associated data. This probably means that you do not have write permissions to either of the galleries and the data folders. Please rectify this and try again.") . "</p>";
						}
					}
				}
			} else if ($_REQUEST["gallery_command"] == "picture_comment") {
				$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
				$picture_name = stripslashes($_REQUEST["picture_name"]);
				if ($_REQUEST["picture_comment"] == "") {
					if (file_exists("$gallerybase$gallery/data/$picture_name.{$_REQUEST['picture_language']}.txt")) {
						$comment = file_get_contents("$gallerybase$gallery/data/$picture_name.{$_REQUEST['picture_language']}.txt");
					} else if (file_exists("$gallerybase$gallery/data/$picture_name.txt")) {
						$comment = file_get_contents("$gallerybase$gallery/data/$picture_name.txt");
					} else {
						$comment = "No description file found";
					}
					$data = "
					<form action=\"$modulegalleryurl&amp;gallery_command=picture_comment&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_language={$_REQUEST['picture_language']}&amp;picture_name=$picture_name\" method=\"POST\">
					<div><textarea rows=\"10\" cols=\"70\" style=\"width: 100%;\" id=\"edit\" name=\"picture_comment\">$comment</textarea></div>
					<div align=\"center\"><input accesskey=\"s\" type=\"submit\" value=\"" . i18n("Save") . "\" /></div>
					</form>
					" . generate_pagehelplink();
				} else {
					$picture_comment = stripslashes($_REQUEST["picture_comment"]);
					if (file_exists("$gallerybase$gallery/data/$picture_name.{$_REQUEST['picture_language']}.txt")) {
						if (unlink("$gallerybase$gallery/data/$picture_name.{$_REQUEST['picture_language']}.txt")) {
							if (file_put_contents("$gallerybase$gallery/data/$picture_name.{$_REQUEST['picture_language']}.txt", $picture_comment)) {
								$data = "<p>" . i18n("Save complete.") . " <a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">" . i18n("Click here to return") . "</a>.</p>";
								header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
							} else {
								$data = "<p>" . i18n("Save failed!") . "</p>";
							}
						} else {
							$data = "<p>" . i18n("File could not be deleted. Check permissions!") . "</p>";
						}
					} else {
						if (file_put_contents("$gallerybase$gallery/data/$picture_name.{$_REQUEST['picture_language']}.txt", $picture_comment)) {
							$data = "<p>" . i18n("Save complete.") . " <a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">" . i18n("Click here to return") . "</a>.</p>";
							header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
						} else {
							$data = "<p>" . i18n("Save failed!") . "</p>";
						}
					}
				}
			} else if ($_REQUEST["gallery_command"] == "picture_move") {
				$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
				$moveto = str_replace("/", "/galleries/", $_REQUEST["moveto"]);
				$picture_name = stripslashes($_REQUEST["picture_name"]);
				if ($_REQUEST["moveto"] == "//") $moveto = "";
				if ($_REQUEST["moveto"] == "") {
					$data = "
					<table width=\"100%\">
					<tr>
					<td width=\"50%\">
						<div><label for=\"moveto\" accesskey=\"m\">" . i18n("Move the picture to the gallery:") . "</label></div>
					</td>
					<td>" . gallery_displaydir($gallerybase, 0, "$modulegalleryurl&amp;gallery_command=picture_move&amp;picture_name=$picture_name&amp;gallery_name={$_REQUEST['gallery_name']}&amp;moveto=/", "") . "</td>
					</tr>
					</table>
					<div align=\"center\"><a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\" class=\"command\">[" . i18n("Return to gallery overview") . "]</a></div>
					";
				} else {
					if( $gallery == $moveto ) // This happens if the user simply clicks submit, ignore this and return home
						header("Location: $modulegalleryurlheader&gallery_command=gallery_list");
					else if (file_exists("$gallerybase$moveto/images/$picture_name")) { // make sure picture doesn't already exists
						if ($_REQUEST['overwrite'] != "true") {
							$data = "<p>" . i18n("You are attempting to move the picture to a gallery, where a picture with this name already exists. Do you wish to replace the old with the new, or do you wish to abort the move?") . "
							<div align=\"center\">
							<a href=\"$modulegalleryurl&amp;gallery_command=picture_move&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$picture_name&amp;moveto={$_REQUEST['moveto']}&amp;overwrite=true\">[" . i18n("Overwrite destination") . "]</a>
							<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">[" . i18n("Abort") . "]</a>
							</div>
							";
						} else {
							if (unlink_multiple("$gallerybase$moveto/data/$picture_name*") and unlink("$gallerybase$moveto/images/$picture_name") and unlink("$gallerybase$moveto/thumbs/$picture_name")) {
								if (	rename_multiple("$gallerybase$gallery/data/", "$gallerybase$moveto/data/", $picture_name) and
									rename("$gallerybase$gallery/images/$picture_name", "$gallerybase$moveto/images/$picture_name") and
									rename("$gallerybase$gallery/thumbs/$picture_name", "$gallerybase$moveto/thumbs/$picture_name")
									) {
									if (file_exists("$gallerybase$gallery/comments/$picture_name.inc")) {
										if (rename("$gallerybase$gallery/comments/$picture_name.inc", "$gallerybase$moveto/comments/$picture_name.inc")) {
											if ($moveto == "") {
												$data = "<p>" . i18n("The picture was successfully moved to the ##0## gallery", array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list\">{$_REQUEST['user_id']}", "</a>")) . ". <a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">" . i18n("Click here to go back") . "</a>.</p>";
												header("Location: $modulegalleryurlheader&gallery_command=gallery_list");
											} else {
												$data = "<p>" . i18n("The picture was successfully moved to the ##0## gallery", array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['moveto']}\">{$_REQUEST['moveto']}", "</a>")) . ". <a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">" . i18n("Click here to go back") . "</a>.</p>";
												header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name=" . $_REQUEST["moveto"]);
											}
										} else {
											$data = "<p>" . i18n("An error occured during the attempt to move the picture and associated data. This probably means that you do not have write permissions to either of the galleries and the data folders. Please rectify this and try again.") . "</p>";
										}
									} else {
										if ($moveto == "") {
											$data = "<p>" . i18n("The picture was successfully moved to the ##0## gallery", array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list\">{$_REQUEST['user_id']}", "</a>")) . ". <a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">" . i18n("Click here to go back") . "</a>.</p>";
											header("Location: $modulegalleryurlheader&gallery_command=gallery_list");
										} else {
											$data = "<p>" . i18n("The picture was successfully moved to the ##0## gallery", array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['moveto']}\">{$_REQUEST['moveto']}", "</a>")) . ". <a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">" . i18n("Click here to go back") . "</a>.</p>";
											header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name=" . $_REQUEST["moveto"]);
										}
									}
								} else {
									$data = "<p>" . i18n("An error occured during the attempt to move the picture and associated data. This probably means that you do not have write permissions to either of the galleries and the data folders. Please rectify this and try again.") . "</p>";
								}
							} else {
								$data = "<p>" . i18n("An error occured during the deletion of the existing picture. This is probably because you do not have writing permissions in the target gallery folders or to any one of the target gallery picture files. Please rectify this and try again.") . "</p>";
							}
						}
					} else {
						if (	rename_multiple("$gallerybase$gallery/data/", "$gallerybase$moveto/data/", $picture_name) and
							rename_multiple("$gallerybase$gallery/comments/", "$gallerybase$moveto/comments/", $picture_name) and
							rename("$gallerybase$gallery/images/$picture_name", "$gallerybase$moveto/images/$picture_name") and
							rename("$gallerybase$gallery/thumbs/$picture_name", "$gallerybase$moveto/thumbs/$picture_name")
							) {
							if ($moveto == "") {
								$data = "<p>" . i18n("The picture was successfully moved to the ##0## gallery", array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list\">{$_REQUEST['user_id']}", "</a>")) . ". <a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">" . i18n("Click here to go back") . "</a>.</p>";
								header("Location: $modulegalleryurlheader&gallery_command=gallery_list");
							} else {
								$data = "<p>" . i18n("The picture was successfully moved to the ##0## gallery", array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['moveto']}\">{$_REQUEST['moveto']}", "</a>")) . ". <a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">" . i18n("Click here to go back") . "</a>.</p>";
								header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name=" . $_REQUEST["moveto"]);
							}
						} else {
							$data = "<p>" . i18n("An error occured during the attempt to move the picture and associated data. This probably means that you do not have write permissions to either of the galleries and the data folders. Please rectify this and try again.") . "</p>";
						}
					}
				}
			} else if ($_REQUEST["gallery_command"] == "picture_delete") {
				$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
				$picture_name = stripslashes($_REQUEST["picture_name"]);
				if ($_REQUEST["confirm_delete"] != "true") {
					$data = "<p>" . i18n("Are you sure you wish to delete the picture ##0## from the gallery ##1##? WARNING! The picture and associated data will be irreversibly lost after this. If you wish to back it up, please do so before continuing.", array("&quot;$picture_name&quot;", "&quot;{$_REQUEST['gallery_name']}&quot;")) . "</p>
					<div align=\"center\">
					<a href=\"$modulegalleryurl&amp;gallery_command=picture_delete&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$picture_name&amp;confirm_delete=true\">[" . i18n("Confirm delete") . "]</a>
					<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">[" . i18n("Abort delete") . "]</a>
					</div>
					";
				} else {
					if (unlink_multiple("$gallerybase$gallery/data/$picture_name*") and unlink("$gallerybase$gallery/images/$picture_name") and unlink("$gallerybase$gallery/thumbs/$picture_name")) {
						if (file_exists("$gallerybase$gallery/comments/$picture_name.inc")) {
							if (unlink("$gallerybase$gallery/comments/$picture_name.inc")) {
							$data = "<p>" . i18n("The picture was successfully deleted.") . " <a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">" . i18n("Click here to return to the gallery") . "</a>.</p>";
							header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
							}
						} else {
							$data = "<p>" . i18n("An error occured during the deletion of the picture. This is probably because you do not have writing permissions in the gallery folders or to any one of the gallery picture files. Please rectify this and try again.") . "</p>";
						}
					} else {
						$data = "<p>" . i18n("An error occured during the deletion of the picture. This is probably because you do not have writing permissions in the gallery folders or to any one of the gallery picture files. Please rectify this and try again.") . "</p>";
					}
				}
			} else if ($_REQUEST["gallery_command"] == "gallery_delete") {
				$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
				if ($_REQUEST["gallery_delete"] != "true") {
					$data = "<p>" . i18n("Are you sure you wish to delete the ##0## gallery? This cannot be undone, and if you regret your decision, you will <i>not</i> be able to retrieve the images. If you wish to save of the images, click on Abort and back up the gallery now.", array("&quot;{$_REQUEST['gallery_name']}&quot;")) . "</p>
					<div align=\"center\">
					<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_delete&amp;gallery_name={$_REQUEST['gallery_name']}&amp;gallery_delete=true\">[" . i18n("Confirm delete") . "]</a>
					<a class=\"command\" href=\"$modulegalleryurl\">[" . i18n("Abort delete") . "]</a>
					</div>";
				} else {
					if (rmdirr("$gallerybase$gallery")) {
						$data = "<p>" . i18n("The gallery was deleted.") . " <a href=\"$modulegalleryurl\">" . i18n("Click here to return") . "</a>.</p>";
						header("Location: $modulegalleryurlheader");
					} else {
						$data = "<p>" . i18n("An error occured during the deletion of the gallery. Please check file permissions and try again.") . "</p>";
					}
				}
			} else if ($_REQUEST["gallery_command"] == "gallery_new") {
				$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
				if ($_REQUEST["gallery_name"] == "" or $_REQUEST["gallery_humanname"] == "" or $_REQUEST["gallery_comment"] == "") {
					if ($_REQUEST["gallery_name"] == "") $_REQUEST["gallery_name"] = "galleryname";
					if ($_REQUEST["gallery_humanname"] == "") $_REQUEST["gallery_humanname"] = i18n("Human readable name");
					if ($_REQUEST["gallery_comment"] == "") $_REQUEST["gallery_comment"] = i18n("Type in any comment in this field");
					$data = "
					<form action=\"$modulegalleryurl&amp;gallery_command=gallery_new\" method=\"post\">
					<table class=\"setup\">
					<tr class=\"setup\">
						<td class=\"setup\"><div><label for=\"gallery_name\" accesskey=\"n\">" . i18n("Physical name") . ":</label></div></td>
						<td class=\"setup\"><div><input style=\"width: 100%\" type=\"text\" id=\"gallery_name\" name=\"gallery_name\" value=\"{$_REQUEST["gallery_name"]}\" /></div></td>
					</tr>
					<tr class=\"setup\">
						<td class=\"setup\"><div><label for=\"gallery_humanname\" accesskey=\"h\">" . i18n("Human name") . ":</label></div></td>
						<td class=\"setup\"><div><input style=\"width: 100%\" type=\"text\" id=\"gallery_humanname\" name=\"gallery_humanname\" value=\"{$_REQUEST["gallery_humanname"]}\" /></div></td>
					</tr>
					<tr class=\"setup\">
						<td class=\"setup\"><div><label for=\"gallery_description\" accesskey=\"d\">" . i18n("Description") . ":</label></div></td>
						<td class=\"setup\"><div><textarea rows=\"10\" cols=\"70\" style=\"width: 100%;\" id=\"gallery_description\" name=\"gallery_comment\">{$_REQUEST["gallery_comment"]}</textarea></div></td>
					</tr>
					</table>
					<div align=\"center\">
					<input accesskey=\"s\" type=\"submit\" value=\"" . i18n("Create gallery") . "\" />
					</div>
					</form>
					" . generate_pagehelplink();
				} else {
					if (file_exists("$gallerybase$gallery")) {
						$data = "<p>" . i18n("A gallery with the name ##0## already exists. Please either delete this and try again or go back and change the name of the gallery you are attempting to create.", array($_REQUEST['gallery_name'])) . "</p>";
					} else {
						if (RecursiveMkdir("$gallerybase$gallery")) {
							if (mkdir("$gallerybase$gallery/comments", 0775) and mkdir("$gallerybase$gallery/data", 0775) and mkdir("$gallerybase$gallery/images", 0775) and mkdir("$gallerybase$gallery/thumbs", 0775)) {
								$commentsfile = stripslashes($_REQUEST["gallery_humanname"]) . "\n" . stripslashes($_REQUEST["gallery_comment"]);
								if (file_exists("$gallerybase$gallery/gallery.txt")) unlink("$gallerybase$gallery/gallery.txt");
								if (file_put_contents("$gallerybase$gallery/gallery.txt", $commentsfile)) {
									$data = "<p>" . i18n("The gallery was created successfully. ##0##Click here to upload the first picture##1##, ##2##here to view the gallery##3## or ##4##here to go back to the overview##5##.", array("<a accesskey=\"u\" href=\"$modulegalleryurl&amp;gallery_command=picture_upload&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>", "<a accesskey=\"l\" href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>", "<a accesskey=\"b\" href=\"$modulegalleryurl\">", "</a>")) . "</p>";
								} else {
									$data = "<p>" . i18n("An error occured during the saving of the gallery description file. This probably means that you have run out of space on your webhost. Please rectify this and try again.") . "</p>";
								}
							} else {
								$data = "<p>" . i18n("An error occured during the saving of the gallery description file. This probably means that you have run out of space on your webhost. Please rectify this and try again.") . "</p>";
							}
						} else {
							$data = "<p>" . i18n("An error occured during the saving of the gallery description file. This probably means that you have run out of space on your webhost. Please rectify this and try again.") . "</p>";
						}
					}
				}
			} else if ($_REQUEST["gallery_command"] == "gallery_move") {
				$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
				$gallerymoveto = str_replace("/", "/galleries/", $_REQUEST['moveto']);
				if ($_REQUEST["moveto"] == "") {
					$data = "<p>" . i18n("Move the gallery ##0## physically (rename the folder containing the gallery) to the following. This is only used internally, as the user never sees this. If you wish to change the name the user sees, ##1##please go back to the main page##2## and choose Rename in the corresponding language.", array($_REQUEST['gallery_name'], "<a href=\"$modulegalleryurl\">", "</a>")) . "</p>
					<form action=\"$modulegalleryurl&amp;gallery_command=gallery_move&amp;gallery_name={$_REQUEST['gallery_name']}\" method=\"post\">
					<input type=\"text\" name=\"moveto\" value=\"{$_REQUEST['gallery_name']}\" style=\"width: 100%\" />
					<div align=\"center\"><input accesskey=\"s\" type=\"submit\" value=\"Submit\" /></div>
					</form>
					";
				} else {
					if( $gallery == $gallerymoveto ) // This happens if the user simply clicks submit before... just ignore that!
						header("Location: $modulegalleryurlheader");
					else if( file_exists("$gallerybase$gallerymoveto") )
					{
						if ($_REQUEST["move_overwrite"] != "true") {
							if (rmdirr("$gallerybase$gallerymoveto")) {
								if (rename("$gallerybase$gallery", "$gallerybase$gallerymoveto")) {
									$data = "<p>" . i18n("The move was successful. ##0##Click here to return##1##.", array("<a href=\"$modulegalleryurl\">", "</a>")) . "</p>";
								} else {
									$data = "<p>" . i18n("An error occured during the attempt at moving the gallery. This probably means that you do not have write permissions to the parent folder. Please rectify this and try again.") . "</p>";
								}
							} else {
								$data = "<p>" . i18n("An error occured during the attempt at deleting the existing gallery with this name. This probably means that you do not have the right permissions to delete that folder. Please rectify this and try again.") . "</p>";
							}
						} else {
							$data = "
							<p>" . i18n("A gallery with the name ##0## already exists. Are you sure you wish to overwrite this gallery and replace it with ##1##? WARNING! This cannot be undone, and if you regret your decision, you will <i>not</i> be able to retrieve the images. If you wish to save of the images, click on Abort and back up the gallery now.", array($_REQUEST['moveto'], $_REQUEST['gallery_name'])) . "</p>
							<div align=\"center\">
							<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_move&amp;gallery_name={$_REQUEST['gallery_name']}&amp;moveto={$_REQUEST['moveto']}&amp;move_overwrite=true\">[" . i18n("Confirm overwrite") . "]</a>
							<a class=\"command\" href=\"$modulegalleryurl\">[" . i18n("Abort") . "]</a>
							</div>
							";
						}
					}
					else
					{
						if (rename("$gallerybase$gallery", "$gallerybase$gallerymoveto")) {
							$data = "<p>" . i18n("The move was successful. ##0##Click here to return##1##.", array("<a href=\"$modulegalleryurl\">", "</a>")) . "</p>";
							header("Location: $modulegalleryurlheader");
						} else {
							$data = "<p>" . i18n("An error occured during the attempt at moving the gallery. This probably means that you do not have write permissions to the parent folder. Please rectify this and try again.") . "</p>";
						}
					}
				}
			} else if ($_REQUEST["gallery_command"] == "gallery_rename") {
				$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
				if ($_REQUEST["gallery_title_new"] == "") {
					if (file_exists("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt")) {
						$comment = file_get_contents("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt");
					} else if (file_exists("$gallerybase$gallery/gallery.txt")) {
						$comment = file_get_contents("$gallerybase$gallery/gallery.txt");
					} else {
						$comment = "No title\nNo description file found";
					}
					$title = substr($comment, 0, strpos($comment, "\n"));
					$comment = substr($comment, strpos($comment, "\n") + 1);
					$data = "
					<form action=\"$modulegalleryurl&amp;gallery_command=gallery_rename&amp;gallery_name={$_REQUEST['gallery_name']}&amp;gallery_language={$_REQUEST['gallery_language']}\" method=\"POST\">
					<input type=\"hidden\" name=\"gallery_comment\" value=\"$comment\" />
					<input type=\"text\" style=\"width: 100%\" name=\"gallery_title_new\" value=\"$title\" />
					<div align=\"center\"><input accesskey=\"s\" type=\"submit\" value=\"" . i18n("Save") . "\" /></div>
					</form>
					";
				} else {
					if (file_exists("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt")) {
						if (unlink("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt")) {
							if (file_put_contents("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt", stripslashes($_REQUEST["gallery_title_new"]) . "\n" . stripslashes($_REQUEST["gallery_comment"]))) {
								$data = "<p>" . i18n("Save complete. ##0##Click here to return##1##.", array("<a href=\"$modulegalleryurl\">", "</a>")) . "</p>";
								header("Location: $modulegalleryurlheader");
							} else {
								$data = "<p>" . i18n("Save failed!") . "</p>";
							}
						} else {
							$data = "<p>" . i18n("File could not be deleted. Check permissions!") . "</p>";
						}
					} else {
						if (file_put_contents("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt", stripslashes($_REQUEST["gallery_title_new"]) . "\n" . stripslashes($_REQUEST["gallery_comment"]))) {
							$data = "<p>" . i18n("Save complete. ##0##Click here to return##1##.", array("<a href=\"$modulegalleryurl\">", "</a>")) . "</p>";
							header("Location: $modulegalleryurlheader");
						} else {
							$data = "<p>" . i18n("Save failed!") . "</p>";
						}
					}
				}
			} else if ($_REQUEST["gallery_command"] == "gallery_description") {
				$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
				if ($_REQUEST["gallery_comment_new"] == "") {
					if (file_exists("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt")) {
						$comment = file_get_contents("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt");
					} else if (file_exists("$gallerybase$gallery/gallery.txt")) {
						$comment = file_get_contents("$gallerybase$gallery/gallery.txt");
					} else {
						$comment = "No title\nNo description file found";
					}
					$title = substr($comment, 0, strpos($comment, "\n"));
					$comment = substr($comment, strpos($comment, "\n") + 1);
					$data = "
					<form action=\"$modulegalleryurl&amp;gallery_command=gallery_description&amp;gallery_name={$_REQUEST['gallery_name']}&amp;gallery_language={$_REQUEST['gallery_language']}\" method=\"POST\">
					<input type=\"hidden\" name=\"gallery_title\" value=\"$title\" />
					<textarea rows=\"10\" cols=\"70\" style=\"width: 100%;\" id=\"edit\" name=\"gallery_comment_new\">$comment</textarea>
					<div align=\"center\"><input accesskey=\"s\" type=\"submit\" value=\"" . i18n("Save") . "\" /></div>
					</form>
					" . generate_pagehelplink();
				} else {
					if (file_exists("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt")) {
						if (unlink("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt")) {
							$gallery_comment_new = $_REQUEST["gallery_comment_new"];
							$gallery_comment_new = stripslashes($gallery_comment_new);
							if (file_put_contents("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt", stripslashes($_REQUEST["gallery_title"]) . "\n" . $gallery_comment_new)) {
								$data = "<p>" . i18n("Save complete. ##0##Click here to return##1##.", array("<a href=\"$modulegalleryurl\">", "</a>")) . "</p>";
								header("Location: $modulegalleryurlheader");
							} else {
								$data = "<p>" . i18n("Save failed!") . "</p>";
							}
						} else {
							$data = "<p>" . i18n("File could not be deleted. Check permissions!") . "</p>";
						}
					} else {
						$gallery_comment_new = $_REQUEST["gallery_comment_new"];
						$gallery_comment_new = stripslashes($gallery_comment_new);
						if (file_put_contents("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt", stripslashes($_REQUEST["gallery_title_new"]) . "\n" . $gallery_comment_new)) {
							$data = "<p>" . i18n("Save complete. ##0##Click here to return##1##.", array("<a href=\"$modulegalleryurl\">", "</a>")) . "</p>";
							header("Location: $modulegalleryurlheader");
						} else {
							$data = "<p>" . i18n("Save failed!") . "</p>";
						}
					}
				}
			}  else {
				// Find subgalleries {
				$parentgallery = $_REQUEST["gallery_name"];
				$parent_pos = 0;
				$oldparent_pos = 1;
				$gallery = str_replace("/", "/galleries/", $parentgallery);
				if (strpos($gallery, "/galleries/") !== false) {
					$subgalleries .= "
					<div><a href=\"$modulegalleryurl&amp;gallery_command=gallery_list\">{$_REQUEST['user_id']}</a></div>";
					$gallerylevel .= "&nbsp;";
					while (true) {
						if (strpos($parentgallery, "/", $parent_pos + 1)) {
							$parent_pos = strpos($parentgallery, "/", $parent_pos + 1);
							$subgalleries .= "
							<div>$gallerylevel<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name=" . substr($parentgallery, 0, $parent_pos) . "\">" . substr($parentgallery, $oldparent_pos, ($parent_pos - $oldparent_pos)) . "</a></div>";
							$gallerylevel .= "&nbsp;";
							$oldparent_pos = $parent_pos + 1;
						} else {
							break;
						}
					}
				}
				if (trim(substr($parentgallery, $oldparent_pos)) == "") {
					$subgalleries .= "
								<div>$gallerylevel{$_REQUEST['user_id']} <a href=\"$modulegalleryurl&amp;gallery_command=gallery_new&amp;gallery_name={$_REQUEST['gallery_name']}/galleryname\" class=\"command\">[" . i18n("New subgallery") . "]</a></div>";
				} else {
					$subgalleries .= "
								<div>$gallerylevel" . substr($parentgallery, $oldparent_pos) . "
									<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_delete&amp;gallery_name=$parentgallery\">[" . i18n("Delete") . "]</a>
									<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_move&amp;gallery_name=$parentgallery\">[" . i18n("Rename") . "]</a></div>";
				}
				$gallerylevel .= "&nbsp;";
				if (file_exists("$gallerybase$gallery/galleries")) {
					$subgalleriesgallery = str_replace("/galleries/", "/", $gallery);
					$fp = opendir("$gallerybase$gallery/galleries/");
					while ($file = readdir($fp)) {
						if( substr($file, 0, 1) != "." ) {
							$subgalleries .= "
							<div>$gallerylevel<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name=$subgalleriesgallery/$file\">$file</a>
								<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_delete&amp;gallery_name=$subgalleriesgallery/$file\">[" . i18n("Delete") . "]</a>
								<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_move&amp;gallery_name=$subgalleriesgallery/$file\">[" . i18n("Rename") . "]</a></div>";
						}
					}
					closedir($fp);
				} else {
					$subgalleries .= "<div>$gallerylevel(" . i18n("no subgalleries") . ")</div>";
				}
				$subgalleries .= "<div>$gallerylevel<a href=\"$modulegalleryurl&amp;gallery_command=gallery_new&amp;gallery_name={$_REQUEST['gallery_name']}/galleryname\">(" . i18n("New subgallery") . ")</a></div>";
				// Find subgalleries }
				
				
				if (file_exists("$gallerybase$gallery/gallery.$language.txt")) {
					$filecontents = file_get_contents("$gallerybase$gallery/gallery.$language.txt");
				} else if (file_exists("$gallerybase$gallery/gallery.txt")) {
					$filecontents = file_get_contents("$gallerybase$gallery/gallery.txt");
				} else {
					$filecontents = "No title\nNo description file found";
				}
				$galleryinfobegpos = strpos($filecontents, "\n");
				$gallerytitle = substr($filecontents, 0, $galleryinfobegpos);
				$gallerydescription = substr($filecontents, $galleryinfobegpos);
				
				foreach($languages as $key => $value) {
					$title_changes .= "<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_rename&amp;gallery_name=$parentgallery&amp;gallery_language=$value\">[$value]</a>";
					$desc_changes .= "<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_description&amp;gallery_name=$parentgallery&amp;gallery_language=$value\">[$value]</a>";
				}
				
				$data = "
				<table class=\"gallery\">
				<tr class=\"gallery\">
					<td class=\"gallery_description\" colspan=\"2\">
					<em>$gallerytitle</em> $title_changes<br />
					" . parse_page_data($gallerydescription) . " $desc_changes
					</td>
					<td class=\"gallery_subgalleries\">" . i18n("Subgalleries") . "
					$subgalleries
					</td>
				</tr>
				<tr class=\"gallery\">
					<th class=\"gallery\" style=\"width: 30%;\">
						<div>" . i18n("Picture") . "</div>
					</th>
					<th class=\"gallery\" style=\"width: 33%;\">
						<div>" . i18n("Comment" ) . "</div>
					</th>
					<th class=\"gallery\" style=\"width: 37%;\">
						<div align=\"right\">
						<a accesskey=\"u\" class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=picture_upload&amp;gallery_name={$_REQUEST['gallery_name']}\">[" . i18n("Upload") . "]</a>
						</div>
					</th>
				<tr>
				";
				
				$galleryfolder = "$gallerybase$gallery/images";
				if (file_exists($galleryfolder)) {
					$dir_handle = opendir($galleryfolder);
					while ($file = readdir($dir_handle)) {
						if( substr($file, 0, 1) != "." and strstr($file, -4) != ".inc" ) {
							$data .= "
								<tr class=\"gallery\">
								<td class=\"gallery\">
									<div align=\"center\"><img src=\"$gallerybase$gallery/thumbs/$file\" border=\"0\" /><br />$file</div>
								</td>
								<td class=\"gallery\">
									<div>";
							$data .= parse_page_data(file_get_contents("$gallerybase$gallery/data/$file.$language.txt"));
							foreach ($languages as $key => $value) {
								$data .= " <a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=picture_comment&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$file&amp;picture_language=$value\">[$value]</a>";
							}
							$data .= "</div>
								</td>
								<td class=\"gallery\">
									<div align=\"right\">
									<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=picture_move&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$file\">[" . i18n("Move") . "]</a>
									<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=picture_rename&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$file\">[" . i18n("Rename") . "]</a>
									<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=picture_delete&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$file\">[" . i18n("Delete") . "]</a>
									</div><hr />
									<div align=\"right\">
									<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=cropnresize&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$file&amp;theme=empty\">[" . i18n("Create thumbnail with Crop and Resize") . "]</a><br />
									<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=resizethumb&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$file\">[" . i18n("Thumbnail full size picture") . "]</a>
									</div>";
							
							if (file_exists($gallerybase . $gallery . "/comments/$file.inc")) include($gallerybase . $gallery . "/comments/$file.inc");
							$data .= "
								</td>
								</tr>
								";
							$picturecount = $picturecount + 1;
							$usercomments = array();
						}
					}
					closedir($dir_handle);
					if ($picturecount < 1) $data .= "<tr class=\"gallery\"><td class=\"gallery\" colspan=\"3\">" . renderInformationBox( i18n("No images"), i18n("This gallery contains no images. ##0##Click here to upload one##1##.", array("<a href=\"$modulegalleryurl&amp;gallery_command=picture_upload&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>")) ) . "</td></tr>";
				}
				
				$data .= "
				</table>";
			}
			return $data;
		} else {
			return "<p>" . i18n("You are not authorised to view this page. Please log in and try again.") . "</p>";
		}
	} else {
		if (isAllowed("global_user_edit") || ($viewUser != "" && $viewUser == currentUser()))
		{
			$tabwidget->addCommand(i18n("Upload"), i18n("Upload a picture to this gallery"), siteURL() . "?page_id=13&amp;user_id=$viewUser&amp;profile_mode=module&amp;module=profilemodule_gallery&amp;action=edit&amp;gallery_command=picture_upload$modulegallerylinkgallery$modulegallerylinksubgallery");
			$tabwidget->addCommand(i18n("Edit gallery"), i18n("Edit the gallery"), siteURL() . "?page_id=13&amp;user_id=$viewUser&amp;profile_mode=module&amp;module=profilemodule_gallery&amp;action=edit$modulegallerylinkgallery$modulegallerylinksubgallery");
		}
		return gallery_build_gallery($viewUser, $_REQUEST["gallery_style"], $_REQUEST["gallery_arg1"], $_REQUEST["gallery_arg2"], true, $viewUser);
	}
}

/*
Overall:
 New gallery

Operations per gallery: $_REQUEST["gallery_gallery"]
 Rename
 Delete (type gallery name for confirmation)
 Edit description (per language)

Operations per item: $_REQUEST["gallery_item"]
 Upload (upload file, generate thumbnail, make empty comment files)
 Rename
 Move
 Delete
 Edit comment (per language)

*/

function modulesetup_gallery() {
	global $language, $languages, $module_titles, $module_commands, $gallerybase, $modulegalleryurl, $modulegalleryurlheader;
	if ($_REQUEST["gallery_command"] == "resizethumb") {
		$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
		$original = "$gallerybase$gallery/images/" . stripslashes($_REQUEST["picture_name"]);
		$thumbnail = "$gallerybase$gallery/thumbs/" . stripslashes($_REQUEST["picture_name"]);
		
		if( file_exists( $thumbnail ) )
			unlink( $thumbnail );
		
		scaleImage( $original, $thumbnail, 120, 120 );
		
		header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name=" . $_REQUEST["gallery_name"]);
	} else if ($_REQUEST["gallery_command"] == "cropnresize") {
		global $modulegalleryurl, $modulegalleryurlheader, $gallerybase;
		$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
		$picture = stripslashes($_REQUEST["picture_name"]);
		if (file_exists("$gallerybase/$gallery/images/$picture")) {
			$image = "$gallerybase/$gallery/images/$picture"; // the image to crop
			$dest_image = "$gallerybase/$gallery/thumbs/$picture";
			$ext = checkImageMimetype( $image );
		}
		
		/**
		 * Cropping method adopted from lixlpixel PHParadise
		 *
		 * category :		image editing
		 *
		 * snippet :		crop image
		 *
		 * downloaded :	11.02.2005 - 16:27
		 *
		 * file URL :		http://www.fundisom.com/phparadise/php/image_editing/crop_image
		 */
		
		$url = "$modulegalleryurl&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name={$_REQUEST['picture_name']}&amp;theme=empty&amp;gallery_command=cropnresize"; // the url to this wizard
		$margin = 10; // to keep the image and layer in sync
		
		if(!isset($_POST['step'])) $title = '1';
		else $title = $_POST['step'];
		
		$data = '
		<html>
		<head>
			<title>' . i18n("Cropping gallery image - step") . ' ' . $title . '</title>
		</head>
		<body style="margin: '.$margin.'px;">';
		
		if(!isset($_POST['tx']) && !isset($_POST['fx']))
		{
			$data .= '
			<form method="post" action="'.$url.'">
				<input type="image" src="'.$image.'"><p>';
			if(!isset($_POST['x']))
			{
				$data .= '
				<input type="hidden" name="step" value="2">
				' . i18n("Click to mark the first corner of the selection rectangle.") . " " . i18n("You will be able to review the result before cropping");
			}else{
				$data .= '
				<input type="hidden" name="step" value="3">
				<input type="hidden" name="fx" value="'.$_POST['x'].'">
				<input type="hidden" name="fy" value="'.$_POST['y'].'">
				' . i18n("Click to mark the second corner of the selection rectangle.") . ' | <a href="'.$url.'">' . i18n("Start over") . '</a> ' . " " . i18n("You will be able to review the result before cropping");
			}
			$data .= '
			</form>
			';
		}
		
		if(isset($_POST['fx']))
		{
			$data .= '
			<form method="post" action="'.$url.'">
				<input type="hidden" name="step" value="4">
				<input type="image" src="'.$image.'">
				<input type="hidden" name="tx" value="'.$_POST['fx'].'">
				<input type="hidden" name="ty" value="'.$_POST['fy'].'">
				<input type="hidden" name="width" value="'.($_POST['x']-$_POST['fx']).'">
				<input type="hidden" name="height" value="'.($_POST['y']-$_POST['fy']).'"><p>
				<div style="position: absolute;
							left:'.($_POST['fx']+$margin).'px;
							top: '.($_POST['fy']+$margin).'px;
							width: '.($_POST['x']-$_POST['fx']).'px;
							height: '.($_POST['y']-$_POST['fy']).'px;
							border: 1px solid #fff;">
				</div>
				' . i18n("Click the image to crop. The rectangle shown on the picture is the rectangle that will be cropped.") . '| <a href="'.$url.'">' . i18n("Start over") . '</a>
			</form>';
		}
		
		if(isset($_POST['tx']))
		{
			// Crop the image
			cropImage( $image, $dest_image, $_POST["tx"], $_POST["ty"], $_POST["width"], $_POST["height"] );
			
			// Resize the new thumbnail to the max thumbnail size
			scaleImage( $dest_image, $dest_image, 120, 120 );
			
			header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
		}
	} else if ($_REQUEST["gallery_command"] == "picture_upload") {
		global $modulegalleryurl, $modulegalleryurlheader;
		$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
		if (!is_array($_FILES['uploadedfile'])) {
			$data = "
			<form enctype=\"multipart/form-data\" action=\"$modulegalleryurl&amp;gallery_command=picture_upload&amp;gallery_name={$_REQUEST['gallery_name']}\" method=\"post\">
			<table class=\"setup\">
			<tr class=\"setup\">
				<td class=\"setup\" width=\"25%\"><div><label for=\"uploadedfile[]\" accesskey=\"1\">" . i18n("Select picture for upload") . "</label></div></td>
				<td class=\"setup\" width=\"75%\"><div><input style=\"width: 100%\" id=\"uploadedfile1\" name=\"uploadedfile[]\" type=\"file\" /></div></td>
			</tr>
			<tr class=\"setup\">
				<td class=\"setup\"><div><label for=\"edit\" accesskey=\"c\">" . i18n("Comment") . "</label></div></td>
				<td class=\"setup\"><div><textarea rows=\"10\" cols=\"70\" style=\"width: 100%;\" id=\"edit\" name=\"picture_comment\">{$_REQUEST['picture_comment']}</textarea></div></td>
			</tr>
			</table>
			<div align=\"center\"><input accesskey=\"s\" type=\"submit\" value=\"" . i18n("Upload") . "\" /></div>
			<table class=\"setup\">
			<tr class=\"setup\">
				<td style=\"setup\" width=\"25%\"><div><label for=\"uploadedfile[]\" accesskey=\"2\">" . i18n("Additional pictures") . "</label></div><small class=\"comment\">" . i18n("Select up to 9 additional pictures for upload. These pictures will be given the same comment as the first picture. If you wish to assign a different comment to one picture, do this afterwards.") . "</small></td>
				<td style=\"setup\" width=\"75%\">
					<div><input style=\"width: 100%\" id=\"uploadedfile2\" name=\"uploadedfile[]\" type=\"file\" /></div>
					<div><input style=\"width: 100%\" id=\"uploadedfile3\" name=\"uploadedfile[]\" type=\"file\" /></div>
					<div><input style=\"width: 100%\" id=\"uploadedfile4\" name=\"uploadedfile[]\" type=\"file\" /></div>
					<div><input style=\"width: 100%\" id=\"uploadedfile5\" name=\"uploadedfile[]\" type=\"file\" /></div>
					<div><input style=\"width: 100%\" id=\"uploadedfile6\" name=\"uploadedfile[]\" type=\"file\" /></div>
					<div><input style=\"width: 100%\" id=\"uploadedfile7\" name=\"uploadedfile[]\" type=\"file\" /></div>
					<div><input style=\"width: 100%\" id=\"uploadedfile8\" name=\"uploadedfile[]\" type=\"file\" /></div>
					<div><input style=\"width: 100%\" id=\"uploadedfile9\" name=\"uploadedfile[]\" type=\"file\" /></div>
					<div><input style=\"width: 100%\" id=\"uploadedfile0]\" name=\"uploadedfile[]\" type=\"file\" /></div></td>
			</tr>
			</table>
			</form>
			" . generate_pagehelplink();
		} else {
			global $modulegalleryurl, $modulegalleryurlheader, $gallery_options;
			$errorHandler = new errorHandler( "$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}" );
			foreach ($_FILES["uploadedfile"]["name"] as $thekey => $thevalue) {
				$filename = str_replace("&", "_", str_replace("'", "", stripslashes($_FILES['uploadedfile']['name'][$thekey])));
				$uploadFile = "$gallerybase$gallery/images/$filename";
				if ( checkImageMimetype( $_FILES['uploadedfile']['tmp_name'][$thekey] ) != false && move_uploaded_file($_FILES['uploadedfile']['tmp_name'][$thekey], $uploadFile) ) {
					chmod($uploadFile, 0664);
					if ($x > $gallery_options["galleryoption_resize_width"] && is_array($gallery_options)) {
						$resize_factor = $gallery_options["galleryoption_resize_width"] / $x;
						$resized_x = $gallery_options["galleryoption_resize_width"];
						$resized_y = $y * $resize_factor;
						$errorHandler->handle( i18n( "Resizing ##0##", array( $filename ) ), scaleImage( $uploadFile, $uploadFile, $resized_x, $resized_y ) );
					} else if ($y > $gallery_options["galleryoption_resize_height"] && is_array($gallery_options)) {
						$resize_factor = $gallery_options["galleryoption_resize_height"] / $y;
						$resized_x = $x * $resize_factor;
						$resized_y = $gallery_options["galleryoption_resize_height"];
						$errorHandler->handle( i18n( "Resizing ##0##", array( $filename ) ), scaleImage( $uploadFile, $uploadFile, $resized_x, $resized_y ) );
					}
					
					$errorHandler->handle( i18n("Creating thumbnail of ##0##", array( $filename) ) , scaleImage( $uploadFile, "$gallerybase$gallery/thumbs/$filename", 120, 120 ) );
					
					file_put_contents("$gallerybase$gallery/comments/$filename.inc", "<?PHP\n\$usercomments=array();\n?>");
					
					foreach($languages as $key => $value) {
						file_put_contents("$gallerybase$gallery/data/$filename.$value.txt", stripslashes($_REQUEST["picture_comment"]));
					}
				} else {
					switch ($_FILES['uploadedfile']['error'][$thekey]) {
					case 0:
						if( checkImageMimetype( $_FILES['uploadedfile']['tmp_name'][$thekey] ) == false )
							$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("Image type not supported or not an image file! Only jpg, gif and png are supported image types") );
						else
							$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  false );
						break;
					case 1:
						$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("The uploaded file exceeds the upload_max_filesize directive in php.ini. Please contact your host and rectify this (or better yet, select a smaller file. Most often, this directive states 2MB as the maximum upload size, however as we have 1MB as the maxium here, they have obviously changed it).") );
						break;
					case 2:
						$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("The uploaded file exceeds 1MB in size, and is too large to upload in this manner.") );
						break;
					case 3:
						$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("The uploaded file was only partially uploaded.") );
						break;
					case 4:
						// This is not an error, it's what happens when all ten file selectors are filled
						break;
					default:
						if( checkImageMimetype( $_FILES['uploadedfile']['tmp_name'][$thekey] ) == false )
							$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  i18n("Image type not supported or not an image file! Only jpg, gif and png are supported image types") );
						else
							$errorHandler->handle( i18n("Uploading image ##0##", array($filename) ),  false );
					}
				}
			}
			$data .= $errorHandler->render();
			
			if( !$errorHandler->handled )
				header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
		}
	} else if ($_REQUEST["gallery_command"] == "picture_rename") {
		global $modulegalleryurl, $modulegalleryurlheader;
		$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
		$picture_name = stripslashes($_REQUEST["picture_name"]);
		$renameto = str_replace("'", "", stripslashes($_REQUEST["renameto"]));
		if ($_REQUEST["renameto"] == "") {
			$data = "
			<form action=\"$modulegalleryurl&amp;gallery_command=picture_rename&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$picture_name\" method=\"post\">
			<table class=\"setup\">
			<tr class=\"setup\">
			<td class=\"setup\"><div><label for=\"renameto\" accesskey=\"r\">" . i18n("Rename the picture to:") . "</label></div></td>
			<td class=\"setup\"><div><input style=\"width: 100%\" type=\"text\" id=\"renameto\" name=\"renameto\" value=\"$picture_name\" /></div></td>
			</tr>
			</table>
			<div align=\"center\"><input accesskey=\"s\" type=\"submit\" value=\"Submit\" /></div>
			</form>
			";
		} else {
			if (file_exists("$gallerybase$gallery/images/$renameto")) { // make sure picture doesn't already exists
				if ($_REQUEST['overwrite'] != "true") {
					$data = "<p>" . i18n("You are attempting to rename a picture, but a picture with this name already exists. Do you wish to replace the old with the new, or do you wish to abort the move?") . "
					<div align=\"center\">
					<a href=\"$modulegalleryurl&amp;gallery_command=picture_rename&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$picture_name&amp;moveto=$renameto&amp;overwrite=true\">[" . i18n("Overwrite destination") . "]</a>
					<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">[" . i18n("Abort") . "]</a>
					</div>
					";
				} else {
					if (unlink_multiple("$gallerybase$gallery/data/$renameto*") and unlink("$gallerybase$gallery/images/$renameto") and unlink("$gallerybase$gallery/thumbs/$renameto")) {
						$renamesuccess = true;
						if (file_exists("$gallerybase$gallery/comments/$renameto.inc")) {
							if (!unlink("$gallerybase$gallery/comments/$renameto.inc")) {
								$renamesuccess = false;
							}
						}
						foreach ($languages as $key => $value) {
							if (!rename("$gallerybase$gallery/data/$picture_name.$value.txt", "$gallerybase$gallery/data/$renameto.$value.txt")) $renamesuccess = false;
						}
						if (	$renamesuccess and
							rename("$gallerybase$gallery/images/$picture_name", "$gallerybase$gallery/images/$renameto") and
							rename("$gallerybase$gallery/thumbs/$picture_name", "$gallerybase$gallery/thumbs/$renameto")
							) {
							if (file_exists("$gallerybase$gallery/comments/$picture_name.inc")) {
								if (rename("$gallerybase$gallery/comments/$picture_name.inc", "$gallerybase$gallery/comments/$renameto.inc")) {
									$data = "<p>" . i18n("The picture was successfully renamed. ##0##Click here to go back##1##.", array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>")) . "</p>";
									header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
								} else {
									$data = "<p>" . i18n("An error occured during the attempt to move the picture and associated data. This probably means that you do not have write permissions to either of the galleries and the data folders. Please rectify this and try again.") . "</p>";
								}
							} else {
								$data = "<p>" . i18n("The picture was successfully renamed. ##0##Click here to go back##1##.", array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>")) . "</p>";
								header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
							}
						} else {
							$data = "<p>" . i18n("An error occured during the attempt to move the picture and associated data. This probably means that you do not have write permissions to either of the galleries and the data folders. Please rectify this and try again.") . "</p>";
						}
					} else {
						$data = "<p>" . i18n("An error occured during the attempt to move the picture and associated data. This probably means that you do not have write permissions to either of the galleries and the data folders. Please rectify this and try again.") . "</p>";
					}
				}
			} else {
				$renamesuccess = true;
				foreach ($languages as $key => $value) {
					if (!rename("$gallerybase$gallery/data/$picture_name.$value.txt", "$gallerybase$gallery/data/$renameto.$value.txt")) $renamesuccess = false;
				}
				if (	$renamesuccess and
					rename("$gallerybase$gallery/images/$picture_name", "$gallerybase$gallery/images/$renameto") and
					rename("$gallerybase$gallery/thumbs/$picture_name", "$gallerybase$gallery/thumbs/$renameto")
					) {
					$data = "<p>" . i18n("The picture was successfully renamed. ##0##Click here to go back##1##.", array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>")) . "</p>";
					header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
				} else {
					$data = "<p>" . i18n("An error occured during the attempt to move the picture and associated data. This probably means that you do not have write permissions to either of the galleries and the data folders. Please rectify this and try again.") . "</p>";
				}
			}
		}
	} else if ($_REQUEST["gallery_command"] == "picture_comment") {
		global $modulegalleryurl, $modulegalleryurlheader;
		$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
		$picture_name = stripslashes($_REQUEST["picture_name"]);
		if ($_REQUEST["picture_comment"] == "") {
			if (file_exists("$gallerybase$gallery/data/$picture_name.{$_REQUEST['picture_language']}.txt")) {
				$comment = file_get_contents("$gallerybase$gallery/data/$picture_name.{$_REQUEST['picture_language']}.txt");
			} else if (file_exists("$gallerybase$gallery/data/$picture_name.txt")) {
				$comment = file_get_contents("$gallerybase$gallery/data/$picture_name.txt");
			} else {
				$comment = i18n("No description file found");
			}
			$data = "
			<form action=\"$modulegalleryurl&amp;gallery_command=picture_comment&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_language={$_REQUEST['picture_language']}&amp;picture_name=$picture_name\" method=\"POST\">
			<div><textarea rows=\"10\" cols=\"70\" style=\"width: 100%;\" id=\"edit\" name=\"picture_comment\">$comment</textarea></div>
			<div align=\"center\"><input accesskey=\"s\" type=\"submit\" value=\"" . i18n("Save") . "\" /></div>
			</form>
			" . generate_pagehelplink();
		} else {
			$picture_comment = stripslashes($_REQUEST["picture_comment"]);
			if (file_exists("$gallerybase$gallery/data/$picture_name.{$_REQUEST['picture_language']}.txt")) {
				if (unlink("$gallerybase$gallery/data/$picture_name.{$_REQUEST['picture_language']}.txt")) {
					if (file_put_contents("$gallerybase$gallery/data/$picture_name.{$_REQUEST['picture_language']}.txt", $picture_comment)) {
						$data = "<p>" . i18n("Save complete. ##0##Click here to return##1##." , array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>")) . "</p>";
						header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
					} else {
						$data = "<p>" . i18n("Save failed!") . "</p>";
					}
				} else {
					$data = "<p>" . i18n("File could not be deleted. Check permissions!") . "</p>";
				}
			} else {
				if (file_put_contents("$gallerybase$gallery/data/$picture_name.{$_REQUEST['picture_language']}.txt", $picture_comment)) {
					$data = "<p>" . i18n("Save complete. ##0##Click here to return##1##." , array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>")) . "</p>";
					header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
				} else {
					$data = "<p>" . i18n("Save failed!") . "</p>";
				}
			}
		}
	} else if ($_REQUEST["gallery_command"] == "picture_move") {
		global $modulegalleryurl, $modulegalleryurlheader;
		$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
		$moveto = str_replace("/", "/galleries/", $_REQUEST["moveto"]);
		$picture_name = stripslashes($_REQUEST["picture_name"]);
		if ($_REQUEST["moveto"] == "") {
			$data = "
			<table class=\"setup\">
			<tr class=\"setup\">
			<td class=\"setup\" width=\"50%\">
				<div><label for=\"moveto\" accesskey=\"m\">" . i18n("Move the picture to the gallery:") . "</label></div>
			</td>
			<td class=\"setup\">" . gallery_displaydir($gallerybase, 0, "$modulegalleryurl&amp;gallery_command=picture_move&amp;picture_name=$picture_name&amp;gallery_name={$_REQUEST['gallery_name']}&amp;moveto=", "") . "</td>
			</tr>
			</table>
			";
		} else {
			if (file_exists("$gallerybase$moveto/images/$picture_name")) { // make sure picture doesn't already exists
				if ($_REQUEST['overwrite'] != "true") {
					$data = "<p>" . i18n("You are attempting to move the picture to a gallery, where a picture with this name already exists. Do you wish to replace the old with the new, or do you wish to abort the move?") . "
					<div align=\"center\">
					<a href=\"$modulegalleryurl&amp;gallery_command=picture_move&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$picture_name&amp;moveto={$_REQUEST['moveto']}&amp;overwrite=true\">[" . i18n("Overwrite destination") . "]</a>
					<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">[" . i18n("Abort") . "]</a>
					</div>
					";
				} else {
					if (unlink_multiple("$gallerybase$moveto/data/$picture_name*") and unlink("$gallerybase$moveto/images/$picture_name") and unlink("$gallerybase$moveto/thumbs/$picture_name")) {
						if (	rename_multiple("$gallerybase$gallery/data/", "$gallerybase$moveto/data/", $_REQUEST["picture_name"]) and
							rename("$gallerybase$gallery/images/$picture_name", "$gallerybase$moveto/images/$picture_name") and
							rename("$gallerybase$gallery/thumbs/$picture_name", "$gallerybase$moveto/thumbs/$picture_name")
							) {
							if (file_exists("$gallerybase$gallery/comments/$picture_name.inc")) {
								if (rename("$gallerybase$gallery/comments/$picture_name.inc", "$gallerybase$moveto/comments/$picture_name.inc")) {
									$data = "<p>" . i18n("The picture was successfully moved to the ##0## gallery##1##. ##2##Click here to go back##3##.", array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['moveto']}\">{$_REQUEST['moveto']}", "</a>", "<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>")) . "</p>";
								} else {
									$data = "<p>" . i18n("An error occured during the attempt to move the picture and associated data. This probably means that you do not have write permissions to either of the galleries and the data folders. Please rectify this and try again.") . "</p>";
								}
							} else {
								$data = "<p>" . i18n("The picture was successfully moved to the ##0## gallery##1##. ##2##Click here to go back##3##.", array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['moveto']}\">{$_REQUEST['moveto']}", "</a>", "<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>")) . "</p>";
							}
						} else {
							$data = "<p>" . i18n("An error occured during the attempt to move the picture and associated data. This probably means that you do not have write permissions to either of the galleries and the data folders. Please rectify this and try again.") . "</p>";
						}
					} else {
						$data = "<p>" . i18n("An error occured during the deletion of the existing picture. This is probably because you do not have writing permissions in the target gallery folders or to any one of the target gallery picture files. Please rectify this and try again.") . "</p>";
					}
				}
			} else {
				if (	rename_multiple("$gallerybase$gallery/data/", "$gallerybase$moveto/data/", $picture_name) and
					rename("$gallerybase$gallery/images/$picture_name", "$gallerybase$moveto/images/$picture_name") and
					rename("$gallerybase$gallery/thumbs/$picture_name", "$gallerybase$moveto/thumbs/$picture_name")
					) {
					$data = "<p>" . i18n("The picture was successfully moved to the ##0## gallery##1##. ##2##Click here to go back##3##.", array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['moveto']}\">{$_REQUEST['moveto']}", "</a>", "<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>")) . "</p>";
				} else {
					$data = "<p>" . i18n("An error occured during the attempt to move the picture and associated data. This probably means that you do not have write permissions to either of the galleries and the data folders. Please rectify this and try again.") . "</p>";
				}
			}
		}
	} else if ($_REQUEST["gallery_command"] == "picture_delete") {
		global $modulegalleryurl, $modulegalleryurlheader;
		$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
		$picture_name = stripslashes($_REQUEST["picture_name"]);
		if ($_REQUEST["confirm_delete"] != "true") {
			$data = "<p>" . i18n("Are you sure you wish to delete the picture ##0## from the gallery ##1##? WARNING! The picture and associated data will be irreversibly lost after this. If you wish to back it up, please do so before continuing.", array("&quot;$picture_name&quot;", "&quot;{$_REQUEST['gallery_name']}&quot;")) . "</p>
			<div align=\"center\">
			<a href=\"$modulegalleryurl&amp;gallery_command=picture_delete&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$picture_name&amp;confirm_delete=true\">[" . i18n("Confirm delete") . "]</a>
			<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">[" . i18n("Abort delet") . "]</a>
			</div>
			";
		} else {
			if (unlink_multiple("$gallerybase$gallery/data/$picture_name*") and unlink("$gallerybase$gallery/images/$picture_name") and unlink("$gallerybase$gallery/thumbs/$picture_name")) {
				if (file_exists("$gallerybase$gallery/comments/$picture_name.inc")) {
					if (unlink("$gallerybase$gallery/comments/$picture_name.inc")) {
					$data = "<p>" . i18n("The picture was successfully deleted. ##0##Click here to return to the gallery##1##.", array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>")) . "</p>";
					header("Location: $modulegalleryurlheader&gallery_command=gallery_list&gallery_name={$_REQUEST['gallery_name']}");
					}
				} else {
					$data = "<p>" . i18n("An error occured during the deletion of the picture. This is probably because you do not have writing permissions in the gallery folders or to any one of the gallery picture files. Please rectify this and try again.") . "</p>";
				}
			} else {
				$data = "<p>" . i18n("An error occured during the deletion of the picture. This is probably because you do not have writing permissions in the gallery folders or to any one of the gallery picture files. Please rectify this and try again.") . "</p>";
			}
		}
	} else if ($_REQUEST["gallery_command"] == "gallery_new") {
		global $modulegalleryurl, $modulegalleryurlheader;
		$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
		if ($_REQUEST["gallery_name"] == "" or $_REQUEST["gallery_humanname"] == "" or $_REQUEST["gallery_comment"] == "") {
			if ($_REQUEST["gallery_name"] == "") $_REQUEST["gallery_name"] = "galleryname";
			if ($_REQUEST["gallery_humanname"] == "") $_REQUEST["gallery_humanname"] = i18n("Human readable name");
			if ($_REQUEST["gallery_comment"] == "") $_REQUEST["gallery_comment"] = i18n("Type in any comment in this field");
			$data = "
			<form action=\"$modulegalleryurl&amp;gallery_command=gallery_new\" method=\"post\">
			<table class=\"setup\">
			<tr class=\"setup\">
				<td class=\"setup\"><div><label for=\"gallery_name\" accesskey=\"n\">Physical name:</label></div></td>
				<td class=\"setup\"><div><input style=\"width: 100%\" type=\"text\" id=\"gallery_name\" name=\"gallery_name\" value=\"{$_REQUEST["gallery_name"]}\" /></div></td>
			</tr>
			<tr class=\"setup\">
				<td class=\"setup\"><div><label for=\"gallery_humanname\" accesskey=\"h\">Human name:</label></div></td>
				<td class=\"setup\"><div><input style=\"width: 100%\" type=\"text\" id=\"gallery_humanname\" name=\"gallery_humanname\" value=\"{$_REQUEST["gallery_humanname"]}\" /></div></td>
			</tr>
			<tr class=\"setup\">
				<td class=\"setup\"><div><label for=\"gallery_description\" accesskey=\"d\">Description:</label></div></td>
				<td class=\"setup\"><div><textarea rows=\"10\" cols=\"70\" style=\"width: 100%;\" id=\"gallery_description\" name=\"gallery_comment\">{$_REQUEST["gallery_comment"]}</textarea></div></td>
			</tr>
			</table>
			<div align=\"center\">
			<input accesskey=\"s\" type=\"submit\" value=\"Create gallery\" />
			</div>
			</form>
			" . generate_pagehelplink();
		} else {
			if (file_exists("$gallerybase$gallery")) {
				$data = "<p>" . i18n("A gallery with the name ##0## already exists. Please either delete this and try again or go back and change the name of the gallery you are attempting to create.", array($_REQUEST['gallery_name'])) . "</p>";
			} else {
				if (RecursiveMkdir("$gallerybase$gallery")) {
 					if (mkdir("$gallerybase$gallery/comments", 0775) and mkdir("$gallerybase$gallery/data", 0775) and mkdir("$gallerybase$gallery/images", 0775) and mkdir("$gallerybase$gallery/thumbs", 0775)) {
						$commentsfile = stripslashes($_REQUEST["gallery_humanname"]) . "\n" . stripslashes($_REQUEST["gallery_comment"]);
						if (file_put_contents("$gallerybase$gallery/gallery.txt", $commentsfile)) {
							$data = "<p>" . i18n("The gallery was created successfully. ##0##Click here to upload the first picture##1##, ##2##here to view the gallery##3## or ##4##here to go back to the overview##5##.", array("<a accesskey=\"u\" href=\"$modulegalleryurl&amp;gallery_command=picture_upload&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>", "<a accesskey=\"l\" href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>", "<a accesskey=\"b\" href=\"$modulegalleryurl\">", "</a>")) . "</p>";
						} else {
							$data = "<p>" . i18n("An error occured during the saving of the gallery description file. This probably means that you have run out of space on your webhost. Please rectify this and try again.") . "</p>";
						}
					} else {
						$data = "<p>" . i18n("An error occured during the creation of the gallery structure. This probably means that you have run out of space on your webhost. Please rectify this and try again.") . "</p>";
					}
				} else {
					$data = "<p>" . i18n("An error occured during the creation of the gallery folder.  This probably means that you do not have the right permissions to write to the gallery base folder. Please rectify this and try again.") . "</p>";
				}
			}
		}
	} else if ($_REQUEST["gallery_command"] == "gallery_move") {
		global $modulegalleryurl, $modulegalleryurlheader;
		$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
		if ($_REQUEST["moveto"] == "") {
			$data = "<p>" . i18n("Move the gallery ##0## physically (rename the folder containing the gallery) to the following. This is only used internally, as the user never sees this. If you wish to change the name the user sees, ##1##please go back to the main page##2## and choose Rename in the corresponding language.", array($_REQUEST['gallery_name'], "<a href=\"$modulegalleryurl\">", "</a>")) . "</p>
			<form action=\"$modulegalleryurl&amp;gallery_command=gallery_move&amp;gallery_name={$_REQUEST['gallery_name']}\" method=\"post\">
			<input type=\"text\" name=\"moveto\" value=\"{$_REQUEST['gallery_name']}\" style=\"width: 100%\" />
			<div align=\"center\"><input accesskey=\"s\" type=\"submit\" value=\"Submit\" /></div>
			</form>
			";
		} else {
			if (file_exists("$gallerybase{$_REQUEST['moveto']}")) {
				if ($_REQUEST["move_overwrite"] != "true") {
					if (rmdirr("$gallerybase{$_REQUEST['moveto']}")) {
						if (rename("$gallerybase$gallery", "$gallerybase{$_REQUEST['moveto']}")) {
							$data = "<p>" . i18n("The move was successful. ##0##Click here to return##1##.", array("<a href=\"$modulegalleryurl\">", "</a>")) . "</p>";
						} else {
							$data = "<p>" . i18n("An error occured during the attempt at moving the gallery. This probably means that you do not have write permissions to the parent folder. Please rectify this and try again.") . "</p>";
						}
					} else {
						$data = "<p>" . i18n("An error occured during the attempt at deleting the existing gallery with this name. This probably means that you do not have the right permissions to delete that folder. Please rectify this and try again.") . "</p>";
					}
				} else {
					$data = "
					<p>" . i18n("A gallery with the name ##0## already exists. Are you sure you wish to overwrite this gallery and replace it with ##1##? WARNING! This cannot be undone, and if you regret your decision, you will <i>not</i> be able to retrieve the images. If you wish to save of the images, click on Abort and back up the gallery now.", array($_REQUEST['moveto'], $_REQUEST['gallery_name'])) . "</p>
					<div align=\"center\">
					<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_move&amp;gallery_name={$_REQUEST['gallery_name']}&amp;moveto={$_REQUEST['moveto']}&amp;move_overwrite=true\">[" . i18n("Confirm overwrite") . "]</a>
					<a class=\"command\" href=\"$modulegalleryurl\">[" . i18n("Abort") . "]</a>
					</div>
					";
				}
			} else {
				if (rename("$gallerybase$gallery", "$gallerybase{$_REQUEST['moveto']}")) {
					$data = "<p>" . i18n("The move was successful. ##0##Click here to return##1##.", array("<a href=\"$modulegalleryurl\">", "</a>")) . "</p>";
					header("Location: $modulegalleryurlheader");
				} else {
					$data = "<p>" . i18n("An error occured during the attempt at moving the gallery. This probably means that you do not have write permissions to the parent folder. Please rectify this and try again.") . "</p>";
				}
			}
		}
	} else if ($_REQUEST["gallery_command"] == "gallery_delete") {
		global $modulegalleryurl, $modulegalleryurlheader;
		$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
		if ($_REQUEST["gallery_delete"] != "true") {
			$data = "<p>" . i18n("Are you sure you wish to delete the ##0## gallery? WARNING! This cannot be undone, and if you regret your decision, you will <i>not</i> be able to retrieve the images. If you wish to save of the images, click on Abort and back up the gallery now.", array("&quot;{$_REQUEST['gallery_name']}&quot;")) . "</p>
			<div align=\"center\">
			<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_delete&amp;gallery_name={$_REQUEST['gallery_name']}&amp;gallery_delete=true\">[" . i18n("Delete gallery") . "]</a>
			<a class=\"command\" href=\"$modulegalleryurl\">[" . i18n("Abort delete") . "]</a>
			</div>";
		} else {
			if (rmdirr("$gallerybase$gallery")) {
				$data = "<p>" . i18n("The gallery was deleted.") . " <a href=\"$modulegalleryurl\">" . i18n("Click here to return") . "</a>.</p>";
				header("Location: $modulegalleryurlheader");
			} else {
				$data = "<p>" . i18n("An error occured during the deletion of the gallery. Please check file permissions and try again.") . "</p>";
			}
		}
	} else if ($_REQUEST["gallery_command"] == "gallery_rename") {
		global $modulegalleryurl, $modulegalleryurlheader;
		$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
		if ($_REQUEST["gallery_title_new"] == "") {
			if (file_exists("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt")) {
				$comment = file_get_contents("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt");
			} else if (file_exists("$gallerybase$gallery/gallery.txt")) {
				$comment = file_get_contents("$gallerybase$gallery/gallery.txt");
			} else {
				$comment = "No title\nNo description file found";
			}
			$title = substr($comment, 0, strpos($comment, "\n"));
			$comment = substr($comment, strpos($comment, "\n") + 1);
			$data = "
			<form action=\"$modulegalleryurl&amp;gallery_command=gallery_rename&amp;gallery_name={$_REQUEST['gallery_name']}&amp;gallery_language={$_REQUEST['gallery_language']}\" method=\"POST\">
			<input type=\"hidden\" name=\"gallery_comment\" value=\"$comment\" />
			<input type=\"text\" style=\"width: 100%\" name=\"gallery_title_new\" value=\"$title\" />
			<div align=\"center\"><input accesskey=\"s\" type=\"submit\" value=\"" . i18n("Save") . "\" /></div>
			</form>
			";
		} else {
			if (file_exists("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt")) {
				if (unlink("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt")) {
					if (file_put_contents("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt", stripslashes($_REQUEST["gallery_title_new"]) . "\n" . stripslashes($_REQUEST["gallery_comment"]))) {
						$data = "<p>" . i18n("Save complete. ##0##Click here to return##1##.", array("<a href=\"$modulegalleryurl\">", "</a>")) . "</p>";
						header("Location: $modulegalleryurlheader");
					} else {
						$data = "<p>" . i18n("Save failed!") . "</p>";
					}
				} else {
					$data = "<p>" . i18n("File could not be deleted. Check permissions!") . "</p>";
				}
			} else {
				if (file_put_contents("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt", stripslashes($_REQUEST["gallery_title_new"]) . "\n" . stripslashes($_REQUEST["gallery_comment"]))) {
					$data = "<p>" . i18n("Save complete. ##0##Click here to return##1##.", array("<a href=\"$modulegalleryurl\">", "</a>")) . "</p>";
					header("Location: $modulegalleryurlheader");
				} else {
					$data = "<p>" . i18n("Save failed!") . "</p>";
				}
			}
		}
	} else if ($_REQUEST["gallery_command"] == "gallery_description") {
		global $modulegalleryurl, $modulegalleryurlheader;
		$gallery = str_replace("/", "/galleries/", $_REQUEST["gallery_name"]);
		if ($_REQUEST["gallery_comment_new"] == "") {
			if (file_exists("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt")) {
				$comment = file_get_contents("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt");
			} else if (file_exists("$gallerybase$gallery/gallery.txt")) {
				$comment = file_get_contents("$gallerybase$gallery/gallery.txt");
			} else {
				$comment = "No title\nNo description file found";
			}
			$title = substr($comment, 0, strpos($comment, "\n"));
			$comment = substr($comment, strpos($comment, "\n") + 1);
			$data = "
			<form action=\"$modulegalleryurl&amp;gallery_command=gallery_description&amp;gallery_name={$_REQUEST['gallery_name']}&amp;gallery_language={$_REQUEST['gallery_language']}\" method=\"POST\">
			<input type=\"hidden\" name=\"gallery_title\" value=\"$title\" />
			<textarea rows=\"10\" cols=\"70\" style=\"width: 100%;\" id=\"edit\" name=\"gallery_comment_new\">$comment</textarea>
			<div align=\"center\"><input accesskey=\"s\" type=\"submit\" value=\"" . i18n("Save") . "\" /></div>
			</form>
			" . generate_pagehelplink();
		} else {
			if (file_exists("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt")) {
				if (unlink("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt")) {
					if (file_put_contents("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt", stripslashes($_REQUEST["gallery_title"]) . "\n" . stripslashes($_REQUEST["gallery_comment_new"]))) {
						$data = "<p>" . i18n("Save complete. ##0##Click here to return##1##.", array("<a href=\"$modulegalleryurl\">", "</a>")) . "</p>";
						header("Location: $modulegalleryurlheader");
					} else {
						$data = "<p>" . i18n("Save failed!") . "</p>";
					}
				} else {
					$data = "<p>" . i18n("File could not be deleted. Check permissions!") . "</p>";
				}
			} else {
				if (file_put_contents("$gallerybase$gallery/gallery.{$_REQUEST['gallery_language']}.txt", stripslashes($_REQUEST["gallery_title"]) . "\n" . stripslashes($_REQUEST["gallery_comment_new"]))) {
					$data = "<p>" . i18n("Save complete. ##0##Click here to return##1##.", array("<a href=\"$modulegalleryurl\">", "</a>")) . "</p>";
					header("Location: $modulegalleryurlheader");
				} else {
					$data = "<p>" . i18n("Save failed!") . "</p>";
				}
			}
		}
	} else if ($_REQUEST["gallery_command"] == "gallery_list") {
		global $modulegalleryurl, $modulegalleryurlheader, $languages;
		// Find subgalleries {
		$parentgallery = $_REQUEST["gallery_name"];
		$parent_pos = 0;
		$gallery = str_replace("/", "/galleries/", $parentgallery);
		if (strpos($gallery, "/galleries/")) {
			while (true) {
				if (strpos($parentgallery, "/", $parent_pos + 1)) {
					$parent_pos = strpos($parentgallery, "/", $parent_pos + 1);
					$subgalleries .= "
					<div>$gallerylevel<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name=" . substr($parentgallery, 0, $parent_pos) . "\">" . substr($parentgallery, $oldparent_pos, ($parent_pos - $oldparent_pos)) . "</a></div>";
					$gallerylevel .= "&nbsp;";
					$oldparent_pos = $parent_pos + 1;
				} else {
					break;
				}
			}
		}
		$subgalleries .= "
					<div>$gallerylevel" . substr($parentgallery, $oldparent_pos) . " <a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_delete&amp;gallery_name=$parentgallery\">[" . i18n("Delete") . "]</a> <a href=\"$modulegalleryurl&amp;gallery_command=gallery_new&amp;gallery_name={$_REQUEST['gallery_name']}/galleryname\" class=\"command\">[" . i18n("New subgallery") . "]</a></div>";
		$gallerylevel .= "&nbsp;";
		if (file_exists("$gallerybase$gallery/galleries")) {
			$subgalleriesgallery = str_replace("/galleries/", "/", $gallery);
			$fp = opendir("$gallerybase$gallery/galleries/");
			while ($file = readdir($fp))
				if( substr($file, 0, 1) != "." )
					$subgalleries .= "
					<div>$gallerylevel<a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name=$subgalleriesgallery/$file\">$file</a> <a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_delete&amp;gallery_name=$subgalleriesgallery/$file\">[" . i18n("Delete") . "]</a></div>";
			
			closedir($fp);
		} else {
			$subgalleries .= "<div>$gallerylevel(" . i18n("no subgalleries") . ")</div>";
		}
		// Find subgalleries }
		
		
		if (file_exists("$gallerybase$gallery/gallery.$language.txt")) {
			$filecontents = file_get_contents("$gallerybase$gallery/gallery.$language.txt");
		} else if (file_exists("$gallerybase$gallery/gallery.txt")) {
			$filecontents = file_get_contents("$gallerybase$gallery/gallery.txt");
		} else {
			$filecontents = "No title\nNo description file found";
		}
		$galleryinfobegpos = strpos($filecontents, "\n");
		$gallerytitle = substr($filecontents, 0, $galleryinfobegpos);
		$gallerydescription = substr($filecontents, $galleryinfobegpos);
		
		foreach($languages as $key => $value) {
			$title_changes .= "<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_rename&amp;gallery_name={$_REQUEST['gallery_name']}&amp;gallery_language=$value\">[$value]</a>";
			$desc_changes .= "<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_description&amp;gallery_name={$_REQUEST['gallery_name']}&amp;gallery_language=$value\">[$value]</a>";
		}
		
		$data = "
		<table class=\"gallery\">
		<tr class=\"gallery\">
			<td class=\"gallery_description\" colspan=\"2\">
			<em>$gallerytitle</em> $title_changes<br />
			" . parse_page_data($gallerydescription) . " $desc_changes
			</td>
			<td class=\"gallery_subgalleries\">" . i18n("Subgalleries") . "
			$subgalleries
			</td>
		</tr>
		<tr class=\"gallery\">
			<th class=\"gallery\" style=\"width: 30%;\">
				<div>{$module_titles['picture']}</div>
			</th>
			<th class=\"gallery\" style=\"width: 33%;\">
				<div>{$module_titles['comment']}</div>
			</th>
			<th class=\"gallery\" style=\"width: 37%;\">
				<div align=\"right\">
				<a accesskey=\"u\" class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=picture_upload&amp;gallery_name={$_REQUEST['gallery_name']}\">[" . i18n("Upload") . "]</a>
				</div>
			</th>
		<tr>
		";
		
		$galleryfolder = "$gallerybase$gallery/images";
		if (file_exists($galleryfolder)) {
			$dir_handle = opendir($galleryfolder);
			while ($file = readdir($dir_handle)) {
				if( substr($file, 0, 1) != "." and strstr($file, -4) != ".inc") {
					$data .= "
						<tr class=\"gallery\">
						<td class=\"gallery\">
							<div align=\"center\"><img src=\"$gallerybase$gallery/thumbs/$file\" border=\"0\" /><br />$file</div>
						</td>
						<td class=\"gallery\">
							<div>";
					$data .= parse_page_data(file_get_contents("$gallerybase$gallery/data/$file.$language.txt"));
					$data .= "</div>
						</td>
						<td class=\"gallery\">
							<div align=\"right\">
							<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=picture_move&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$file\">[" . i18n("Move") . "]</a>
							<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=picture_rename&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$file\">[" . i18n("Rename") . "]</a>
							<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=picture_delete&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$file\">[" . i18n("Delete") . "]</a>
							</div><hr />
							<div align=\"right\">
 							<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$file&amp;theme=empty&amp;gallery_command=cropnresize\">[" . i18n("Create thumbnail with Crop and Resize") . "]</a>
 							<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$file&amp;theme=empty&amp;gallery_command=resizethumb\">[" . i18n("Create thumbnail from full size picture") . "]</a>
							</div>
							<div align=\"right\">
							" . i18n("Comments") . ": ";
							foreach ($languages as $key => $value) {
								$data .= " <a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=picture_comment&amp;gallery_name={$_REQUEST['gallery_name']}&amp;picture_name=$file&amp;picture_language=$value\">[$value]</a>";
							}
					
					if (file_exists($gallerybase . $gallery . "/comments/$file.inc")) include($gallerybase . $gallery . "/comments/$file.inc");
					$data .= "
							</div>
						</td>
						</tr>
						";
					$picturecount = $picturecount + 1;
					$usercomments = array();
				}
			}
			closedir($dir_handle);
			if ($picturecount < 1) $data .= "<tr style=\"gallery\"><td style=\"gallery\" colspan=\"3\">" . renderInformationBox( i18n("No images"), i18n("This gallery contains no images. ##0##Click here to upload one##1##.", array("<a href=\"$modulegalleryurl&amp;gallery_command=picture_upload&amp;gallery_name={$_REQUEST['gallery_name']}\">", "</a>")) ) . "</td></tr>";
		}
		
		$data .= "
		</table>";
	} else if ($_REQUEST["gallery_command"] == "gallery_options") {
		global $modulegalleryurl, $modulegalleryurlheader, $gallerybase, $gallery_options;
		if ($_REQUEST["save_options"]) {
			$gallery_options["galleryoption_resize_enabled"] = false;
			$gallery_options["galleryoption_midsize_enabled"] = false;
			foreach ($_REQUEST as $key => $value) {
				global $gallery_options;
				if (substr($key, 0, 14) == "galleryoption_") $gallery_options[$key] = $value;
				if ($key == "galleryoption_resize_enabled" or $key == "galleryoption_midsize_enabled") $gallery_options[$key] = true;
				
			}
			$filename = "$gallerybase/options.inc";
			$contents = "<?PHP\n\$gallery_options = " . var_export($gallery_options, true) . "\n?>";
			if (file_exists($filename)) unlink($filename);
			file_put_contents($filename, $contents);
		}
		if ($gallery_options["galleryoption_resize_enabled"]) $resize_checked = "checked";
		if ($gallery_options["galleryoption_midsize_enabled"]) $midsize_checked = "checked";
		$data = "
		<form action=\"$modulegalleryurl&amp;gallery_command=gallery_options\" method=\"post\">
		<table class=\"options\">
		<tr class=\"option\">
			<th class=\"options\" width=\"50%\">
				<div>" . i18n("Option") . "</div>
			</th>
			<th class=\"options\" width=\"25%\">
				<div>" . i18n("Value") . "</div>
			</th>
			<th class=\"options\" width=\"25%\">
				<div align=\"right\">
				<a accesskey=\"c\" class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_options\">[" . i18n("Options") . "]</a>
				<a accesskey=\"n\" class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_new\">[" . i18n("New") . "]</a>
				</div>
			</th>
		</tr>
		<tr class=\"option\">
			<td class=\"option\">" . i18n("Resize uploaded images") . ":<br />
				<small class=\"comment\">" . i18n("Check this to resize large images when uploading new pictures") . "</small></td>
			<td class=\"option\" colspan=\"2\"><label><input type=\"checkbox\" $resize_checked name=\"galleryoption_resize_enabled\" /> " . i18n("Enable resizing of uploaded images") . "</td>
		</tr>
		<tr class=\"option\">
			<td class=\"option\">" . i18n("Resized dimensions") . ":<br />
				<small class=\"comment\">" . i18n("The width and height in pixels for the resized images") . "</small></td>
			<td class=\"option\" colspan=\"2\">
			" . i18n("Width") . ": <input type=\"text\" style=\"width: 30%;\" name=\"galleryoption_resize_width\" value=\"{$gallery_options['galleryoption_resize_width']}\" />
			" . i18n("Height") . ": <input type=\"text\" style=\"width: 30%;\" name=\"galleryoption_resize_height\" value=\"{$gallery_options['galleryoption_resize_height']}\" />
			</td>
		</tr>
		<tr class=\"option\">
			<td class=\"option\">" . i18n("Maximum user comment length") . ":<br />
				<small class=\"comment\">" . i18n("Characters") . "</small></td>
			<td class=\"option\" colspan=\"2\"><input type=\"text\" style=\"width: 99%\" name=\"galleryoption_commentlength\" value=\"{$gallery_options['galleryoption_commentlength']}\" /></td>
		</tr>
		<tr class=\"option\">
			<td class=\"option\">" . i18n("Midsize showing") . ":<br />
				<small class=\"comment\">" . i18n("Check this to enable the mid-size system when viewing gallery images. When clicked the image will replace the gallery view and take up the maximum dimensions stated below. If this is disabled, the full picture will open immediately when the thumbnail is clicked. If you disable midsize showing, it will still be used for user comments, if these are enabled.") . "</small></td>
			<td class=\"option\" colspan=\"2\"><label><input type=\"checkbox\" $midsize_checked name=\"galleryoption_midsize_enabled\" /> " . i18n("Enable midsize showing") . "</label></td>
		</tr>
		<tr class=\"option\">
			<td class=\"option\">" . i18n("Middle size dimensions") . ":<br />
				<small class=\"comment\">" . i18n("The width and height in pixels for images in mid-size showing") . "</small></td>
			<td class=\"option\" colspan=\"2\">
			" . i18n("Width") . ": <input type=\"text\" style=\"width: 30%;\" name=\"galleryoption_midsize_width\" value=\"{$gallery_options['galleryoption_midsize_width']}\" />
			" . i18n("Height") . ": <input type=\"text\" style=\"width: 30%;\" name=\"galleryoption_midsize_height\" value=\"{$gallery_options['galleryoption_midsize_height']}\" />
			</td>
		</tr>
		<tr class=\"option\">
			<td class=\"options-bottomleft\"><input type=\"submit\" name=\"save_options\" value=\"" . i18n("Save settings") . "\" /></td>
			<td class=\"options-bottomright\" colspan=\"2\"><input type=\"reset\" value=\"" . i18n("Revert to saved") . "\" /></td>
		</tr>
		</table>
		</form>";
		
	} else { // No action to be taken, show list of galleries
		global $modulegalleryurl, $modulegalleryurlheader, $gallerybase;
		$data = "
		<table class=\"setup\">
		<tr class=\"setup\">
			<th class=\"setup\">
				<div>" . i18n("Gallery") . "</div>
			</th>
			<th class=\"setup\">
				<div>" . i18n("Description") . "</div>
			</th>
			<th class=\"setup\">
				<div align=\"right\">
				<a accesskey=\"c\" class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_options\">[" . i18n("Options") . "]</a>
				<a accesskey=\"n\" class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_new\">[" . i18n("New") . "]</a>
				</div>
			</th>
		</tr>";
		
		//Get name and description of each of the galleries
		
		$dir_handle = opendir("$gallerybase");
		while ($folder = readdir($dir_handle)) {
			if( substr($folder, 0, 1) != "." && $folder != "options.inc" && substr($folder, 0, 5) != "i18n.") {
				$gallerynum++;
				$gallerynames[$gallerynum] = $folder;
				$file_handle = opendir("$gallerybase/$folder");
				while ($file = readdir($file_handle)) {
					if( substr($file, 0, 1) != "." && $file != "data" && $file != "images" && $file != "thumbs") {
						$filecontents = file_get_contents("$gallerybase/$folder/$file");
						$galleryinfobegpos = strpos($filecontents, "\n");
						$gallerylanguage = substr($file, 8, 2);
						$gallerytitles[$gallerynum][$gallerylanguage] = substr($filecontents, 0, $galleryinfobegpos);
						$gallerydescriptions[$gallerynum][$gallerylanguage] = parse_page_data(substr($filecontents, $galleryinfobegpos));
					}
				}
			}
		}
		closedir($dir_handle);
		
		if (is_array($gallerytitles)) {
			foreach ($gallerytitles as $key => $value) {
				global $language, $languages;
				if ($gallerydescriptions[$key][$language] == "" && $value[$language] == "") {
					$value[$language] = $value["tx"];
					$gallerydescriptions[$key][$language] = $gallerydescriptions[$key]["tx"];
				}
				$data .= "
		<tr class=\"setup\">
			<td class=\"setup\">
				<div><a href=\"$modulegalleryurl&amp;gallery_command=gallery_list&amp;gallery_name={$gallerynames[$key]}\">{$value[$language]}</a><br />\gallery({$gallerynames[$key]})</div>
			</td>
			<td class=\"setup\">
				<div>{$gallerydescriptions[$key][$language]}</div>
			</td>
			<td class=\"setup\">
				<div align=\"right\">
				<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_access&amp;gallery_name={$gallerynames[$key]}\">[" . i18n("Access control") . "]</a>
				<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_move&amp;gallery_name={$gallerynames[$key]}\">[" . i18n("Move") . "]</a>
				<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_delete&amp;gallery_name={$gallerynames[$key]}\">[" . i18n("Delete") . "]</a></div>
				<div align=\"right\">
				" . i18n("Rename") . ": ";
				foreach ($languages as $key2 => $value2) {
					$data .= "<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_rename&amp;gallery_name={$gallerynames[$key]}&amp;gallery_language=$value2\">[$value2]</a>";
				}
				$data .= "</div><div align=\"right\">
				" . i18n("Description") . ": ";
				foreach ($languages as $key2 => $value2) {
					$data .= "<a class=\"command\" href=\"$modulegalleryurl&amp;gallery_command=gallery_description&amp;gallery_name={$gallerynames[$key]}&amp;gallery_language=$value2\">[$value2]</a>";
				}
				$data .= "</div>
			</td>
		</tr>";
			}
		} else {
			$data .= "
			<tr class=\"setup\"><td class=\"setup\" colspan=\"3\">" . renderInformationBox( i18n("No galleries"), i18n("No galleries found. ##0##Click here to create one##1##.", array("<a href=\"$modulegalleryurl&amp;gallery_command=gallery_new\">", "</a>")) ) . "</td></tr>
			";
		}
		$data .= "
		</table>";
	}
	return $data;
}

// User permissions {

/**
 Renaming - set new permissions with old permissions, unset old permissions
 Remove - unset permissions
 New - set permissions to default
*/

$dir_handle = opendir("$gallerybase");
$gallerynum = 0;
while ($folder = readdir($dir_handle)) {
	if( substr($folder, 0, 1) != "." && substr($folder, -4) != ".inc") {
		$gallerynum++;
		$gallerynames[$gallerynum] = $folder;
	}
}
closedir($dir_handle);

if (is_array($gallerynames)) {
	foreach ($gallerynames as $key => $value) {
		global $usergroups_global_default;
		$usergroups_global_default["modules_gallery_gallery_{$value}_view"] = true;
		$usergroups_global_default["modules_gallery_gallery_{$value}_edit"] = false;
		$usergroups_global_default["modules_gallery_gallery_{$value}_comment"] = true;
	}
}
$usergroups_global_default["modules_gallery_view_default"] = true;
$usergroups_global_default["modules_gallery_edit_default"] = false;
// User permissions }
?>